@@ -69,6 +69,7 @@
  */
 strong_alias(create_buf,	slurm_create_buf);
 strong_alias(create_mmap_buf,	slurm_create_mmap_buf);
+strong_alias(create_shadow_buf,	slurm_create_shadow_buf);
 strong_alias(free_buf,		slurm_free_buf);
 strong_alias(grow_buf,		slurm_grow_buf);
 strong_alias(init_buf,		slurm_init_buf);
@@ -119,12 +120,13 @@ buf_t *create_buf(char *data, uint32_t size)
 		return NULL;
 	}
 
-	my_buf = xmalloc_nz(sizeof(*my_buf));
+	my_buf = xmalloc(sizeof(*my_buf));
 	my_buf->magic = BUF_MAGIC;
 	my_buf->size = size;
 	my_buf->processed = 0;
 	my_buf->head = data;
 	my_buf->mmaped = false;
+	my_buf->shadow = false;
 
 	return my_buf;
 }
@@ -158,18 +160,24 @@ buf_t *create_mmap_buf(const char *file)
 		return NULL;
 	}
 
-	my_buf = xmalloc_nz(sizeof(*my_buf));
-	my_buf->magic = BUF_MAGIC;
-	my_buf->size = f_stat.st_size;
-	my_buf->processed = 0;
-	my_buf->head = data;
-	my_buf->mmaped = true;
+	my_buf = create_buf(data, f_stat.st_size);
+	if (my_buf)
+		my_buf->mmaped = true;
 
 	debug3("%s: loaded file `%s` as buf_t", __func__, file);
 
 	return my_buf;
 }
 
+extern buf_t *create_shadow_buf(char *data, uint32_t size)
+{
+	buf_t *my_buf = create_buf(data, size);
+
+	if (my_buf)
+		my_buf->shadow = true;
+
+	return my_buf;
+}
 
 /* free_buf - release memory associated with a given buffer */
 void free_buf(buf_t *my_buf)
@@ -179,7 +187,7 @@ void free_buf(buf_t *my_buf)
 	xassert(my_buf->magic == BUF_MAGIC);
 	if (my_buf->mmaped)
 		munmap(my_buf->head, my_buf->size);
-	else
+	else if (!my_buf->shadow)
 		xfree(my_buf->head);
 
 	xfree(my_buf);
@@ -188,16 +196,52 @@ void free_buf(buf_t *my_buf)
 /* Grow a buffer by the specified amount */
 void grow_buf(buf_t *buffer, uint32_t size)
 {
+	uint64_t new_size = (uint64_t) size + buffer->size;
+
+	xassert(buffer->magic == BUF_MAGIC);
+
 	if (buffer->mmaped)
 		fatal_abort("attempt to grow mmap()'d buffer not supported");
-	if ((buffer->size + size) > MAX_BUF_SIZE) {
-		error("%s: Buffer size limit exceeded (%u > %u)",
-		      __func__, (buffer->size + size), MAX_BUF_SIZE);
-		return;
+	if (buffer->shadow)
+		fatal_abort("attempt to grow shadow buffer not supported");
+	if (new_size > MAX_BUF_SIZE)
+		fatal_abort("%s: Buffer size limit exceeded (%"PRIu64" > %u)",
+			    __func__, new_size, MAX_BUF_SIZE);
+
+	buffer->size = new_size;
+	xrealloc_nz(buffer->head, buffer->size);
+}
+
+extern int try_grow_buf(buf_t *buffer, uint32_t size)
+{
+	uint64_t new_size = ((uint64_t) size) + buffer->size;
+
+	xassert(buffer->magic == BUF_MAGIC);
+
+	if (buffer->mmaped || buffer->shadow)
+		return EINVAL;
+	if (new_size > MAX_BUF_SIZE) {
+		error("%s: Buffer size limit exceeded (%"PRIu64" > %u)",
+		      __func__, new_size, MAX_BUF_SIZE);
+		return ESLURM_DATA_TOO_LARGE;
 	}
 
-	buffer->size += size;
-	xrealloc_nz(buffer->head, buffer->size);
+	if (!try_xrealloc(buffer->head, new_size))
+		return ENOMEM;
+
+	buffer->size = new_size;
+
+	return SLURM_SUCCESS;
+}
+
+extern int try_grow_buf_remaining(buf_t *buffer, uint32_t size)
+{
+	xassert(buffer->magic == BUF_MAGIC);
+
+	if (remaining_buf(buffer) < size)
+		return try_grow_buf(buffer, size);
+
+	return SLURM_SUCCESS;
 }
 
 /* init_buf - create an empty buffer of the given size */
@@ -205,22 +249,57 @@ buf_t *init_buf(uint32_t size)
 {
 	buf_t *my_buf;
 
-	if (size > MAX_BUF_SIZE) {
-		error("%s: Buffer size limit exceeded (%u > %u)",
-		      __func__, size, MAX_BUF_SIZE);
-		return NULL;
-	}
+	if (size > MAX_BUF_SIZE)
+		fatal_abort("%s: Buffer size limit exceeded (%u > %u)",
+			    __func__, size, MAX_BUF_SIZE);
+
 	if (size <= 0)
 		size = BUF_SIZE;
-	my_buf = xmalloc_nz(sizeof(*my_buf));
+	my_buf = xmalloc(sizeof(*my_buf));
 	my_buf->magic = BUF_MAGIC;
 	my_buf->size = size;
 	my_buf->processed = 0;
 	my_buf->head = xmalloc(size);
 	my_buf->mmaped = false;
+	my_buf->shadow = false;
 	return my_buf;
 }
 
+extern buf_t *try_init_buf(uint32_t size)
+{
+	buf_t *buf;
+
+	if (!size)
+		size = BUF_SIZE;
+
+	if (size > MAX_BUF_SIZE) {
+		error("%s: Buffer size limit exceeded (%u > %u)",
+		      __func__, size, MAX_BUF_SIZE);
+		return NULL;
+	}
+
+	if (!(buf = try_xmalloc(sizeof(*buf)))) {
+		error("%s: Unable to allocate memory for %zu bytes",
+		      __func__, sizeof(*buf));
+		return NULL;
+	}
+
+	if (!(buf->head = try_xmalloc(size))) {
+		error("%s: Unable to allocate memory for %u bytes",
+		      __func__, size);
+		buf->magic = ~BUF_MAGIC;
+		xfree(buf);
+		return NULL;
+	}
+
+	buf->magic = BUF_MAGIC;
+	buf->size = size;
+	buf->processed = 0;
+	buf->mmaped = false;
+	buf->shadow = false;
+	return buf;
+}
+
 /* xfer_buf_data - return a pointer to the buffer's data and release the
  * buffer's structure */
 void *xfer_buf_data(buf_t *my_buf)
@@ -231,12 +310,33 @@ void *xfer_buf_data(buf_t *my_buf)
 
 	if (my_buf->mmaped)
 		fatal_abort("attempt to xfer mmap()'d buffer not supported");
+	if (my_buf->shadow)
+		fatal_abort("attempt to xfer shadow buffer not supported");
 
 	data_ptr = (void *) my_buf->head;
 	xfree(my_buf);
 	return data_ptr;
 }
 
+extern int swap_buf_data(buf_t *x, buf_t *y)
+{
+	if (!x || !y)
+		return EINVAL;
+
+	xassert(x->magic == BUF_MAGIC);
+	xassert(y->magic == BUF_MAGIC);
+	xassert(xsize(x->head) >= 0);
+	xassert(xsize(y->head) >= 0);
+
+	SWAP(x->head, y->head);
+	SWAP(x->processed, y->processed);
+	SWAP(x->size, y->size);
+	SWAP(x->mmaped, y->mmaped);
+	SWAP(x->shadow, y->shadow);
+
+	return SLURM_SUCCESS;
+}
+
 /*
  * Given a time_t in host byte order, promote it to int64_t, convert to
  * network byte order, store in buffer and adjust buffer acc'd'ngly
@@ -245,16 +345,8 @@ void pack_time(time_t val, buf_t *buffer)
 {
 	int64_t n64 = HTON_int64((int64_t) val);
 
-	if (remaining_buf(buffer) < sizeof(n64)) {
-		if ((buffer->size + BUF_SIZE) > MAX_BUF_SIZE) {
-			error("%s: Buffer size limit exceeded (%u > %u)",
-			      __func__, (buffer->size + BUF_SIZE),
-			      MAX_BUF_SIZE);
-			return;
-		}
-		buffer->size += BUF_SIZE;
-		xrealloc_nz(buffer->head, buffer->size);
-	}
+	if (try_grow_buf_remaining(buffer, sizeof(n64)))
+		return;
 
 	memcpy(&buffer->head[buffer->processed], &n64, sizeof(n64));
 	buffer->processed += sizeof(n64);
@@ -294,16 +386,9 @@ void packdouble(double val, buf_t *buffer)
 	 */
 	uval.d =  (val * FLOAT_MULT);
 	nl =  HTON_uint64(uval.u);
-	if (remaining_buf(buffer) < sizeof(nl)) {
-		if ((buffer->size + BUF_SIZE) > MAX_BUF_SIZE) {
-			error("%s: Buffer size limit exceeded (%u > %u)",
-			      __func__, (buffer->size + BUF_SIZE),
-			      MAX_BUF_SIZE);
-			return;
-		}
-		buffer->size += BUF_SIZE;
-		xrealloc_nz(buffer->head, buffer->size);
-	}
+
+	if (try_grow_buf_remaining(buffer, sizeof(nl)))
+		return;
 
 	memcpy(&buffer->head[buffer->processed], &nl, sizeof(nl));
 	buffer->processed += sizeof(nl);
@@ -430,16 +515,8 @@ void pack64(uint64_t val, buf_t *buffer)
 {
 	uint64_t nl =  HTON_uint64(val);
 
-	if (remaining_buf(buffer) < sizeof(nl)) {
-		if ((buffer->size + BUF_SIZE) > MAX_BUF_SIZE) {
-			error("%s: Buffer size limit exceeded (%u > %u)",
-			      __func__, (buffer->size + BUF_SIZE),
-			      MAX_BUF_SIZE);
-			return;
-		}
-		buffer->size += BUF_SIZE;
-		xrealloc_nz(buffer->head, buffer->size);
-	}
+	if (try_grow_buf_remaining(buffer, sizeof(nl)))
+		return;
 
 	memcpy(&buffer->head[buffer->processed], &nl, sizeof(nl));
 	buffer->processed += sizeof(nl);
@@ -469,16 +546,8 @@ void pack32(uint32_t val, buf_t *buffer)
 {
 	uint32_t nl = htonl(val);
 
-	if (remaining_buf(buffer) < sizeof(nl)) {
-		if ((buffer->size + BUF_SIZE) > MAX_BUF_SIZE) {
-			error("%s: Buffer size limit exceeded (%u > %u)",
-			      __func__, (buffer->size + BUF_SIZE),
-			      MAX_BUF_SIZE);
-			return;
-		}
-		buffer->size += BUF_SIZE;
-		xrealloc_nz(buffer->head, buffer->size);
-	}
+	if (try_grow_buf_remaining(buffer, sizeof(nl)))
+		return;
 
 	memcpy(&buffer->head[buffer->processed], &nl, sizeof(nl));
 	buffer->processed += sizeof(nl);
@@ -663,16 +732,8 @@ void pack16(uint16_t val, buf_t *buffer)
 {
 	uint16_t ns = htons(val);
 
-	if (remaining_buf(buffer) < sizeof(ns)) {
-		if ((buffer->size + BUF_SIZE) > MAX_BUF_SIZE) {
-			error("%s: Buffer size limit exceeded (%u > %u)",
-			      __func__, (buffer->size + BUF_SIZE),
-			      MAX_BUF_SIZE);
-			return;
-		}
-		buffer->size += BUF_SIZE;
-		xrealloc_nz(buffer->head, buffer->size);
-	}
+	if (try_grow_buf_remaining(buffer, sizeof(ns)))
+		return;
 
 	memcpy(&buffer->head[buffer->processed], &ns, sizeof(ns));
 	buffer->processed += sizeof(ns);
@@ -701,16 +762,8 @@ int unpack16(uint16_t *valp, buf_t *buffer)
  */
 void pack8(uint8_t val, buf_t *buffer)
 {
-	if (remaining_buf(buffer) < sizeof(uint8_t)) {
-		if ((buffer->size + BUF_SIZE) > MAX_BUF_SIZE) {
-			error("%s: Buffer size limit exceeded (%u > %u)",
-			      __func__, (buffer->size + BUF_SIZE),
-			      MAX_BUF_SIZE);
-			return;
-		}
-		buffer->size += BUF_SIZE;
-		xrealloc_nz(buffer->head, buffer->size);
-	}
+	if (try_grow_buf_remaining(buffer, sizeof(uint8_t)))
+		return;
 
 	memcpy(&buffer->head[buffer->processed], &val, sizeof(uint8_t));
 	buffer->processed += sizeof(uint8_t);
@@ -770,16 +823,8 @@ extern void packbuf(buf_t *source, buf_t *buffer)
 	if (!size_val)
 		return;
 
-	if (remaining_buf(buffer) < size_val) {
-		if ((buffer->size + size_val) > MAX_BUF_SIZE) {
-			error("%s: Buffer size limit exceeded (%u > %u)",
-			      __func__, (buffer->size + size_val),
-			      MAX_BUF_SIZE);
-			return;
-		}
-		buffer->size += size_val;
-		xrealloc_nz(buffer->head, buffer->size);
-	}
+	if (try_grow_buf_remaining(buffer, size_val))
+		return;
 
 	memcpy(&buffer->head[buffer->processed], get_buf_data(source), size_val);
 	buffer->processed += size_val;
@@ -799,16 +844,9 @@ extern void packmem(void *valp, uint32_t size_val, buf_t *buffer)
 		      __func__, size_val, MAX_PACK_MEM_LEN);
 		return;
 	}
-	if (remaining_buf(buffer) < (sizeof(ns) + size_val)) {
-		if ((buffer->size + size_val + BUF_SIZE) > MAX_BUF_SIZE) {
-			error("%s: Buffer size limit exceeded (%u > %u)",
-			      __func__, (buffer->size + size_val + BUF_SIZE),
-			      MAX_BUF_SIZE);
-			return;
-		}
-		buffer->size += (size_val + BUF_SIZE);
-		xrealloc_nz(buffer->head, buffer->size);
-	}
+
+	if (try_grow_buf_remaining(buffer, (sizeof(ns) + size_val)))
+		return;
 
 	memcpy(&buffer->head[buffer->processed], &ns, sizeof(ns));
 	buffer->processed += sizeof(ns);
@@ -821,7 +859,7 @@ extern void packmem(void *valp, uint32_t size_val, buf_t *buffer)
 
 
 /*
- * Given a buffer containing a network byte order 16-bit integer,
+ * Given a buffer containing a network byte order 32-bit integer,
  * and an arbitrary data string, return a pointer to the
  * data string in 'valp'.  Also return the sizes of 'valp' in bytes.
  * Adjust buffer counters.
@@ -856,7 +894,7 @@ unpack_error:
 }
 
 /*
- * Given a buffer containing a network byte order 16-bit integer,
+ * Given a buffer containing a network byte order 32-bit integer,
  * and an arbitrary data string, copy the data string into the location
  * specified by valp.  Also return the sizes of 'valp' in bytes.
  * Adjust buffer counters.
@@ -893,7 +931,7 @@ unpack_error:
 }
 
 /*
- * Given a buffer containing a network byte order 16-bit integer,
+ * Given a buffer containing a network byte order 32-bit integer,
  * and an arbitrary data string, copy the data string into the location
  * specified by valp.  Also return the sizes of 'valp' in bytes.
  * Adjust buffer counters.
@@ -931,7 +969,7 @@ unpack_error:
 }
 
 /*
- * Given a buffer containing a network byte order 16-bit integer,
+ * Given a buffer containing a network byte order 32-bit integer,
  * and an arbitrary char string, copy the data string into the location
  * specified by valp and escape ' and \ to be database safe.
  * Also return the sizes of 'valp' in bytes.
@@ -1010,16 +1048,8 @@ void packstr_array(char **valp, uint32_t size_val, buf_t *buffer)
 	int i;
 	uint32_t ns = htonl(size_val);
 
-	if (remaining_buf(buffer) < sizeof(ns)) {
-		if ((buffer->size + BUF_SIZE) > MAX_BUF_SIZE) {
-			error("%s: Buffer size limit exceeded (%u > %u)",
-			      __func__, (buffer->size + BUF_SIZE),
-			      MAX_BUF_SIZE);
-			return;
-		}
-		buffer->size += BUF_SIZE;
-		xrealloc_nz(buffer->head, buffer->size);
-	}
+	if (try_grow_buf_remaining(buffer, sizeof(ns)))
+		return;
 
 	memcpy(&buffer->head[buffer->processed], &ns, sizeof(ns));
 	buffer->processed += sizeof(ns);
@@ -1066,16 +1096,8 @@ unpack_error:
  */
 void packmem_array(char *valp, uint32_t size_val, buf_t *buffer)
 {
-	if (remaining_buf(buffer) < size_val) {
-		if ((buffer->size + size_val + BUF_SIZE) > MAX_BUF_SIZE) {
-			error("%s: Buffer size limit exceeded (%u > %u)",
-			      __func__, (buffer->size + size_val + BUF_SIZE),
-			      MAX_BUF_SIZE);
-			return;
-		}
-		buffer->size += (size_val + BUF_SIZE);
-		xrealloc_nz(buffer->head, buffer->size);
-	}
+	if (try_grow_buf_remaining(buffer, size_val))
+		return;
 
 	memcpy(&buffer->head[buffer->processed], valp, size_val);
 	buffer->processed += size_val;
