@@ -44,19 +44,20 @@
 #include <dlfcn.h>
 #include <glob.h>
 #include <libgen.h>
+#include <limits.h>
 #include <stdlib.h>
 #include <string.h>
 
+#include "src/common/env.h"
+#include "src/common/job_options.h"
+#include "src/common/optz.h"
 #include "src/common/plugin.h"
-#include "src/common/xmalloc.h"
-#include "src/common/xstring.h"
-#include "src/common/xassert.h"
-#include "src/common/strlcpy.h"
 #include "src/common/read_config.h"
 #include "src/common/spank.h"
-#include "src/common/optz.h"
-#include "src/common/job_options.h"
-#include "src/common/env.h"
+#include "src/common/strlcpy.h"
+#include "src/common/xassert.h"
+#include "src/common/xmalloc.h"
+#include "src/common/xstring.h"
 
 #include "src/slurmd/slurmstepd/slurmstepd_job.h"
 
@@ -68,6 +69,9 @@
 
 static bool has_prolog = false;
 static bool has_epilog = false;
+static bool has_user_init = false;
+static bool has_task_post_fork = false;
+static bool has_task_exit = false;
 
 struct spank_plugin_operations {
 	spank_f *init;
@@ -139,24 +143,6 @@ enum spank_context_type {
 	S_TYPE_JOB_SCRIPT,      /* JOB_SCRIPT == prolog/epilog*/
 };
 
-/*
- *  SPANK plugin hook types:
- */
-typedef enum step_fn {
-	SPANK_INIT = 0,
-	SPANK_JOB_PROLOG = 2,
-	SPANK_INIT_POST_OPT,
-	LOCAL_USER_INIT,
-	STEP_USER_INIT,
-	STEP_TASK_INIT_PRIV,
-	STEP_USER_TASK_INIT,
-	STEP_TASK_POST_FORK,
-	STEP_TASK_EXIT,
-	SPANK_JOB_EPILOG,
-	SPANK_SLURMD_EXIT,
-	SPANK_EXIT
-} step_fn_t;
-
 /*
  *  Job information in prolog/epilog context:
  */
@@ -201,8 +187,9 @@ static int _spank_option_register(struct spank_plugin *p,
 static int _spank_stack_load (struct spank_stack *stack, const char *file);
 static void _spank_plugin_destroy (struct spank_plugin *);
 static void _spank_plugin_opt_destroy (struct spank_plugin_opt *);
-static void _spank_stack_get_remote_options(struct spank_stack *, List);
-static void _spank_stack_get_remote_options_env(struct spank_stack *, char **);
+static void _spank_stack_get_remote_options(struct spank_stack *, List, List);
+static void _spank_stack_get_remote_options_env(struct spank_stack *, char **,
+						List);
 static void _spank_stack_set_remote_options_env(struct spank_stack *stack);
 static int dyn_spank_set_job_env (const char *var, const char *val, int ovwt);
 static char *_opt_env_name(struct spank_plugin_opt *p, char *buf, size_t siz);
@@ -253,7 +240,7 @@ static int plugin_in_list (List l, struct spank_plugin *sp)
 {
 	int rc = 0;
 	struct spank_plugin *p;
-	ListIterator i = list_iterator_create (l);
+	list_itr_t *i = list_iterator_create(l);
 	while ((p = list_next (i))) {
 		if (p->fq_path == sp->fq_path) {
 			rc = 1;
@@ -344,11 +331,11 @@ static struct spank_plugin *_spank_plugin_create(struct spank_stack *stack,
 {
 	struct spank_plugin *plugin;
 	plugin_handle_t p;
-	plugin_err_t e;
+	int e;
 	struct spank_plugin_operations ops;
 
-	if ((e = plugin_load_from_file(&p, path)) != EPLUGIN_SUCCESS) {
-		error ("spank: %s: %s", path, plugin_strerror(e));
+	if ((e = plugin_load_from_file(&p, path)) != SLURM_SUCCESS) {
+		error("spank: %s: %s", path, slurm_strerror(e));
 		return NULL;
 	}
 
@@ -402,7 +389,7 @@ void _spank_plugin_destroy(struct spank_plugin *sp)
 static char *
 _spank_plugin_find (const char *path, const char *file)
 {
-	char dir [4096];
+	char dir[PATH_MAX];
 	char *p, *entry;
 	int pathlen = strlen (path);
 
@@ -427,7 +414,7 @@ _spank_plugin_find (const char *path, const char *file)
 			xstrcatchar (fq_path, '/');
 		xstrcat (fq_path, file);
 
-		if (plugin_peek(fq_path, NULL, 0, NULL) == EPLUGIN_SUCCESS)
+		if (plugin_peek(fq_path, NULL, 0) == SLURM_SUCCESS)
 			return (fq_path);
 
 		xfree (fq_path);
@@ -472,6 +459,12 @@ spank_stack_plugin_valid_for_context (struct spank_stack *stack,
 		 *   always loaded in these contexts, so continue
 		 *   to do so
 		 */
+		if (p->ops.user_init)
+			has_user_init = true;
+		if (p->ops.task_post_fork)
+			has_task_post_fork = true;
+		if (p->ops.task_exit)
+			has_task_exit = true;
 		return (1);
 	default:
 		return (0);
@@ -554,7 +547,7 @@ static int _spank_stack_load(struct spank_stack *stack, const char *path)
 {
 	int rc = 0;
 	int line;
-	char buf[4096];
+	char buf[PATH_MAX];
 	int fd;
 	FILE *fp;
 
@@ -666,17 +659,17 @@ static const char *_step_fn_name(step_fn_t type)
 		return ("job_prolog");
 	case SPANK_INIT_POST_OPT:
 		return ("init_post_opt");
-	case LOCAL_USER_INIT:
+	case SPANK_LOCAL_USER_INIT:
 		return ("local_user_init");
-	case STEP_USER_INIT:
+	case SPANK_STEP_USER_INIT:
 		return ("user_init");
-	case STEP_TASK_INIT_PRIV:
+	case SPANK_STEP_TASK_INIT_PRIV:
 		return ("task_init_privileged");
-	case STEP_USER_TASK_INIT:
+	case SPANK_STEP_USER_TASK_INIT:
 		return ("task_init");
-	case STEP_TASK_POST_FORK:
+	case SPANK_STEP_TASK_POST_FORK:
 		return ("task_post_fork");
-	case STEP_TASK_EXIT:
+	case SPANK_STEP_TASK_EXIT:
 		return ("task_exit");
 	case SPANK_JOB_EPILOG:
 		return ("job_epilog");
@@ -699,17 +692,17 @@ static spank_f *spank_plugin_get_fn (struct spank_plugin *sp, step_fn_t type)
 		return (sp->ops.job_prolog);
 	case SPANK_INIT_POST_OPT:
 		return (sp->ops.init_post_opt);
-	case LOCAL_USER_INIT:
+	case SPANK_LOCAL_USER_INIT:
 		return (sp->ops.local_user_init);
-	case STEP_USER_INIT:
+	case SPANK_STEP_USER_INIT:
 		return (sp->ops.user_init);
-	case STEP_TASK_INIT_PRIV:
+	case SPANK_STEP_TASK_INIT_PRIV:
 		return (sp->ops.task_init_privileged);
-	case STEP_USER_TASK_INIT:
+	case SPANK_STEP_USER_TASK_INIT:
 		return (sp->ops.user_task_init);
-	case STEP_TASK_POST_FORK:
+	case SPANK_STEP_TASK_POST_FORK:
 		return (sp->ops.task_post_fork);
-	case STEP_TASK_EXIT:
+	case SPANK_STEP_TASK_EXIT:
 		return (sp->ops.task_exit);
 	case SPANK_JOB_EPILOG:
 		return (sp->ops.job_epilog);
@@ -728,7 +721,7 @@ static int _do_call_stack(struct spank_stack *stack,
 	step_fn_t type, void * job, int taskid)
 {
 	int rc = SLURM_SUCCESS;
-	ListIterator i;
+	list_itr_t *i;
 	struct spank_plugin *sp;
 	struct spank_handle spank[1];
 	const char *fn_name;
@@ -794,15 +787,18 @@ int _spank_init(enum spank_context_type context, stepd_step_rec_t *step)
 static int spank_stack_post_opt (struct spank_stack * stack,
 				 stepd_step_rec_t *step)
 {
+	List found_opts = job_options_create();
+
 	/*
 	 *  Get any remote options from job launch message:
 	 */
-	_spank_stack_get_remote_options(stack, step->options);
+	_spank_stack_get_remote_options(stack, step->options, found_opts);
 
 	/*
 	 *  Get any remote option passed thru environment
 	 */
-	_spank_stack_get_remote_options_env(stack, step->env);
+	_spank_stack_get_remote_options_env(stack, step->env, found_opts);
+	list_destroy(found_opts);
 
 	/*
 	 * Now clear any remaining options passed through environment
@@ -862,35 +858,37 @@ int spank_init_post_opt (void)
 
 int spank_user(stepd_step_rec_t *step)
 {
-	return (_do_call_stack(global_spank_stack, STEP_USER_INIT, step, -1));
+	return (_do_call_stack(global_spank_stack, SPANK_STEP_USER_INIT,
+			       step, -1));
 }
 
 int spank_local_user(struct spank_launcher_job_info *job)
 {
-	return (_do_call_stack(global_spank_stack, LOCAL_USER_INIT, job, -1));
+	return (_do_call_stack(global_spank_stack, SPANK_LOCAL_USER_INIT,
+			       job, -1));
 }
 
 int spank_task_privileged(stepd_step_rec_t *step, int taskid)
 {
-	return (_do_call_stack(global_spank_stack, STEP_TASK_INIT_PRIV,
+	return (_do_call_stack(global_spank_stack, SPANK_STEP_TASK_INIT_PRIV,
 			       step, taskid));
 }
 
 int spank_user_task(stepd_step_rec_t *step, int taskid)
 {
-	return (_do_call_stack(global_spank_stack, STEP_USER_TASK_INIT,
+	return (_do_call_stack(global_spank_stack, SPANK_STEP_USER_TASK_INIT,
 			       step, taskid));
 }
 
 int spank_task_post_fork(stepd_step_rec_t *step, int taskid)
 {
-	return (_do_call_stack(global_spank_stack, STEP_TASK_POST_FORK,
+	return (_do_call_stack(global_spank_stack, SPANK_STEP_TASK_POST_FORK,
 			       step, taskid));
 }
 
 int spank_task_exit(stepd_step_rec_t *step, int taskid)
 {
-	return (_do_call_stack(global_spank_stack, STEP_TASK_EXIT,
+	return (_do_call_stack(global_spank_stack, SPANK_STEP_TASK_EXIT,
 			       step, taskid));
 }
 
@@ -1105,7 +1103,7 @@ struct option *spank_option_table_create(const struct option *orig)
 {
 	struct spank_plugin_opt *spopt;
 	struct option *opts = NULL;
-	ListIterator i = NULL;
+	list_itr_t *i = NULL;
 
 	List option_cache = get_global_option_cache();
 	if (option_cache == NULL)
@@ -1194,12 +1192,12 @@ extern int spank_process_option(int optval, const char *arg)
 	return (0);
 }
 
-extern int spank_process_env_options()
+extern int spank_process_env_options(void)
 {
 	char var[1024];
 	const char *arg;
 	struct spank_plugin_opt *option;
-	ListIterator i;
+	list_itr_t *i;
 	List option_cache = get_global_option_cache();
 	int rc = 0;
 
@@ -1361,14 +1359,14 @@ _spank_opt_print(struct spank_option *opt, FILE * fp, int left_pad, int width)
 	return;
 }
 
-int spank_print_options(FILE * fp, int left_pad, int width)
+void spank_print_options(FILE * fp, int left_pad, int width)
 {
 	struct spank_plugin_opt *p;
-	ListIterator i;
+	list_itr_t *i;
 	List option_cache = get_global_option_cache();
 
 	if ((option_cache == NULL) || (list_count(option_cache) == 0))
-		return (0);
+		return;
 
 	fprintf(fp, "\nOptions provided by plugins:\n");
 
@@ -1380,7 +1378,7 @@ int spank_print_options(FILE * fp, int left_pad, int width)
 	}
 	list_iterator_destroy(i);
 
-	return (0);
+	return;
 }
 
 #define OPT_TYPE_SPANK 0x4400
@@ -1462,7 +1460,7 @@ static int _option_setenv (struct spank_plugin_opt *option)
 static void _spank_stack_set_remote_options_env(struct spank_stack *stack)
 {
 	struct spank_plugin_opt *p;
-	ListIterator i;
+	list_itr_t *i;
 	List option_cache;
 
 	if (stack == NULL)
@@ -1481,34 +1479,34 @@ static void _spank_stack_set_remote_options_env(struct spank_stack *stack)
 	list_iterator_destroy(i);
 }
 
-int spank_set_remote_options(List opts)
+void spank_set_remote_options(List opts)
 {
 	struct spank_plugin_opt *p;
-	ListIterator i;
+	list_itr_t *i;
 	List option_cache;
 
 	if (global_spank_stack == NULL)
-		return (0);
+		return;
 	option_cache = global_spank_stack->option_cache;
 
 	if ((option_cache == NULL) || (list_count(option_cache) == 0))
-		return (0);
+		return;
 
 	i = list_iterator_create(option_cache);
 	while ((p = list_next(i))) {
-		char optstr[1024];
+		char *optstr;
 
 		if (!p->found)
 			continue;
 
-		snprintf(optstr, sizeof(optstr), "%s:%s",
-			 p->opt->name, p->plugin->name);
+		optstr = xstrdup_printf("%s:%s", p->opt->name, p->plugin->name);
 
 		job_options_append(opts, OPT_TYPE_SPANK, optstr,
 				   p->optarg);
+		xfree(optstr);
 	}
 	list_iterator_destroy(i);
-	return (0);
+	return;
 }
 
 struct opt_find_args {
@@ -1531,18 +1529,15 @@ spank_stack_find_option_by_name(struct spank_stack *stack, const char *str)
 {
 	struct spank_plugin_opt *opt = NULL;
 	struct opt_find_args args;
-	char buf[256];
+	char *buf;
 	char *name;
 	List option_cache = stack->option_cache;
 
-	if (strlcpy(buf, str, sizeof(buf)) >= sizeof(buf)) {
-		error("plugin option \"%s\" too big. Ignoring.", str);
-		return (NULL);
-	}
-
-	if (!(name = strchr(buf, ':'))) {
+	buf = xstrdup(str);
+	if (!(name = xstrchr(buf, ':'))) {
 		error("Malformed plugin option \"%s\" received. Ignoring",
 		      str);
+		xfree(buf);
 		return (NULL);
 	}
 
@@ -1554,17 +1549,16 @@ spank_stack_find_option_by_name(struct spank_stack *stack, const char *str)
 	if (option_cache) {
 		opt = list_find_first(option_cache, (ListFindF) _opt_find,
 				      &args);
-		if (opt == NULL) {
+		if (opt == NULL)
 			warning("SPANK plugin \"%s\" option \"%s\" not found",
 				name, buf);
-			return (NULL);
-		}
 	} else {
 		warning("no SPANK plugin found to process option \"%s\"",
 			name);
-		return (NULL);
 	}
 
+	/* opt is NULL on warning */
+	xfree(buf);
 	return (opt);
 }
 
@@ -1586,7 +1580,7 @@ spank_option_getopt (spank_t sp, struct spank_option *opt, char **argp)
 
 	if ((sp->phase == SPANK_INIT) ||
 	    (sp->phase == SPANK_INIT_POST_OPT) ||
-	    (sp->phase == STEP_TASK_POST_FORK) ||
+	    (sp->phase == SPANK_STEP_TASK_POST_FORK) ||
 	    (sp->phase == SPANK_SLURMD_EXIT) ||
 	    (sp->phase == SPANK_EXIT))
 		return (ESPANK_NOT_AVAIL);
@@ -1647,13 +1641,32 @@ spank_option_getopt (spank_t sp, struct spank_option *opt, char **argp)
 	return (ESPANK_SUCCESS);
 }
 
+static int _opt_info_find(struct job_option_info *info,
+			  struct spank_plugin_opt *opt)
+{
+	char *buf;
+	char *name = NULL;
+	bool match = true;
+
+	buf = xstrdup(info->option);
+	if ((name = xstrchr(buf, ':')))
+		*(name++) = '\0';
+
+	if (xstrcmp(opt->plugin->name, name) ||
+	    xstrcmp(opt->opt->name, buf))
+		match = false;
+
+	xfree(buf);
+	return match;
+}
+
 static void _spank_stack_get_remote_options_env(struct spank_stack *stack,
-						char **env)
+						char **env, List found_opts)
 {
 	char var [1024];
 	const char *arg;
 	struct spank_plugin_opt *option;
-	ListIterator i;
+	list_itr_t *i;
 	List option_cache = stack->option_cache;
 
 	if (!option_cache)
@@ -1661,6 +1674,10 @@ static void _spank_stack_get_remote_options_env(struct spank_stack *stack,
 
 	i = list_iterator_create (option_cache);
 	while ((option = list_next (i))) {
+		if (list_find_first(found_opts, (ListFindF) _opt_info_find,
+				    option))
+			continue;
+
 		if (!(arg = getenvp (env, _opt_env_name (option, var, sizeof(var)))))
 			continue;
 
@@ -1680,10 +1697,10 @@ static void _spank_stack_get_remote_options_env(struct spank_stack *stack,
 }
 
 static void _spank_stack_get_remote_options(struct spank_stack *stack,
-					    List opts)
+					    List opts, List found_opts)
 {
 	const struct job_option_info *j;
-	ListIterator li;
+	list_itr_t *li;
 
 	if (!opts)
 		return;
@@ -1702,6 +1719,8 @@ static void _spank_stack_get_remote_options(struct spank_stack *stack,
 			error("spank: failed to process option %s=%s",
 			      opt->opt->name, j->optarg);
 		}
+
+		job_options_append(found_opts, j->type, j->option, j->optarg);
 	}
 	list_iterator_destroy(li);
 }
@@ -1711,7 +1730,7 @@ static void _spank_stack_get_remote_options(struct spank_stack *stack,
  *   spank option env vars  have a prefix of SPANK_OPTION_ENV_PREFIX,
  *   or SPANK_ + SPANK_OPTION_ENV_PREFIX
  */
-int spank_clear_remote_options_env (char **env)
+void spank_clear_remote_options_env (char **env)
 {
 	char **ep;
 	int len = strlen (SPANK_OPTION_ENV_PREFIX);
@@ -1731,15 +1750,15 @@ int spank_clear_remote_options_env (char **env)
 			}
 		}
 	}
-	return (0);
+	return;
 }
 
 
 
 static int tasks_execd (spank_t spank)
 {
-	return ( (spank->phase == STEP_TASK_POST_FORK)
-	      || (spank->phase == STEP_TASK_EXIT)
+	return ( (spank->phase == SPANK_STEP_TASK_POST_FORK)
+	      || (spank->phase == SPANK_STEP_TASK_EXIT)
 	      || (spank->phase == SPANK_EXIT) );
 }
 
@@ -2411,6 +2430,35 @@ spank_err_t spank_job_control_unsetenv (spank_t spank, const char *var)
 	return (ESPANK_SUCCESS);
 }
 
+spank_err_t spank_prepend_task_argv(spank_t spank, int argc,
+				    const char *argv[])
+{
+	int new_argc, j = 0;
+	char **new_argv;
+
+	if (!spank || (spank->magic != SPANK_MAGIC) || !argv)
+		return ESPANK_BAD_ARG;
+
+	if (!spank->task || !spank->task->argv ||
+	    ((spank->phase != SPANK_STEP_TASK_INIT_PRIV) &&
+	     (spank->phase != SPANK_STEP_USER_TASK_INIT)))
+		return ESPANK_NOT_TASK;
+
+	new_argc = argc + spank->task->argc;
+	new_argv = xcalloc(new_argc + 1, sizeof(char *));
+
+	for (int i = 0; i < argc && argv[i]; i++)
+		new_argv[j++] = xstrdup(argv[i]);
+	for (int i = 0; i < spank->task->argc && spank->task->argv[i]; i++)
+		new_argv[j++] = spank->task->argv[i];
+	new_argv[j] = NULL;
+
+	spank->task->argc = new_argc;
+	spank->task->argv = new_argv;
+
+	return ESPANK_SUCCESS;
+}
+
 /*
  * spank_get_plugin_names
  * Get names of all spank plugins
@@ -2426,7 +2474,7 @@ spank_err_t spank_job_control_unsetenv (spank_t spank, const char *var)
 size_t spank_get_plugin_names(char ***names)
 {
 	struct spank_plugin *p;
-	ListIterator i;
+	list_itr_t *i;
 	size_t n_names = 0;
 
 	if (!global_spank_stack)
@@ -2464,7 +2512,7 @@ size_t spank_get_plugin_option_names(const char *plugin_name, char ***opts)
 	size_t nopts = 0;
 
 	List options = get_global_option_cache();
-	ListIterator i;
+	list_itr_t *i;
 
 	i = list_iterator_create(options);
 	while ((spopt = list_next(i))) {
@@ -2558,14 +2606,14 @@ extern bool spank_option_get_next_set(char **plugin, char **name,
 				      char **value, void **state)
 {
 	List option_cache = get_global_option_cache();
-	ListIterator *iter = (ListIterator *) *state;
+	list_itr_t **iter = *state;
 	struct spank_plugin_opt *spopt;
 
 	if (option_cache == NULL)
 		return NULL;
 
 	if (!iter) {
-		iter = xmalloc(sizeof(ListIterator));
+		iter = xmalloc(sizeof(*iter));
 		*iter = list_iterator_create(option_cache);
 		*state = iter;
 	}
@@ -2591,12 +2639,27 @@ extern bool spank_option_get_next_set(char **plugin, char **name,
 	return false;
 }
 
-extern bool spank_has_prolog()
+extern bool spank_has_prolog(void)
 {
 	return has_prolog;
 }
 
-extern bool spank_has_epilog()
+extern bool spank_has_epilog(void)
 {
 	return has_epilog;
 }
+
+extern bool spank_has_user_init(void)
+{
+	return has_user_init;
+}
+
+extern bool spank_has_task_post_fork(void)
+{
+	return has_task_post_fork;
+}
+
+extern bool spank_has_task_exit(void)
+{
+	return has_task_exit;
+}
