@@ -247,14 +247,14 @@ static bool _alter_table_after_upgrade(mysql_conn_t *mysql_conn,
 
 	/* check to see if upgrade has happened */
 	if (!have_value) {
-		const char *info;
+		const char *tmp_char;
 		char *query;
 		/*
 		 * confirm MariaDB is being used to avoid any ambiguity with
 		 * MySQL versions
 		 */
-		info = mysql_get_server_info(mysql_conn->db_conn);
-		if (xstrcasestr(info, "mariadb") &&
+		tmp_char = mysql_get_server_info(mysql_conn->db_conn);
+		if (xstrcasestr(tmp_char, "mariadb") &&
 		    (mysql_get_server_version(mysql_conn->db_conn) >= 100201)) {
 			query = "show columns from `qos_table` like 'preempt'";
 			result = mysql_db_query_ret(mysql_conn, query, 0);
@@ -295,7 +295,7 @@ static int _mysql_make_table_current(mysql_conn_t *mysql_conn, char *table_name,
 	MYSQL_ROW row;
 	int i = 0;
 	List columns = NULL;
-	ListIterator itr = NULL;
+	list_itr_t *itr = NULL;
 	char *col = NULL;
 	int adding = 0;
 	int run_update = 0;
