@@ -1,8 +1,7 @@
 /*****************************************************************************\
  *  data_t parser plugin interface
  ******************************************************************************
- *  Copyright (C) 2022 SchedMD LLC.
- *  Written by Nathan Rini <nate@schedmd.com>
+ *  Copyright (C) SchedMD LLC.
  *
  *  This file is part of Slurm, a resource management program.
  *  For details, see <https://slurm.schedmd.com/>.
@@ -79,6 +78,19 @@ typedef struct {
 	void (*free)(void *arg);
 	int (*assign)(void *arg, data_parser_attr_type_t type, void *obj);
 	int (*specify)(void *arg, data_t *dst);
+	openapi_type_t (*resolve_openapi_type)(void *arg,
+					       data_parser_type_t type,
+					       const char *field);
+	const char *(*resolve_type_str)(void *arg, data_parser_type_t type);
+	int (*inc_ref)(void *arg, data_parser_type_t type,
+		       void **references_ptr);
+	int (*populate_schema)(void *arg, data_parser_type_t type,
+		       void **references_ptr, data_t *dst, data_t *schemas);
+	int (*populate_parameters)(void *arg, data_parser_type_t parameter_type,
+				   data_parser_type_t query_type,
+				   void **references_ptr, data_t *dst,
+				   data_t *schemas);
+	void (*release_refs)(void *arg, void **references_ptr);
 } parse_funcs_t;
 
 typedef struct {
@@ -96,6 +108,12 @@ static const char *parse_syms[] = {
 	"data_parser_p_free",
 	"data_parser_p_assign",
 	"data_parser_p_specify",
+	"data_parser_p_resolve_openapi_type",
+	"data_parser_p_resolve_type_string",
+	"data_parser_p_increment_reference",
+	"data_parser_p_populate_schema",
+	"data_parser_p_populate_parameters",
+	"data_parser_p_release_references",
 };
 
 static plugins_t *plugins = NULL;
@@ -247,6 +265,9 @@ static int _load_plugins(plugin_param_t *pparams, plugrack_foreach_t listf,
 
 	slurm_mutex_lock(&init_mutex);
 
+	if ((rc = serializer_g_init(MIME_TYPE_JSON_PLUGIN, NULL)))
+		fatal("JSON plugin loading failed: %s", slurm_strerror(rc));
+
 	xassert(sizeof(parse_funcs_t) ==
 		(sizeof(void *) * ARRAY_SIZE(parse_syms)));
 
@@ -777,6 +798,88 @@ cleanup:
 	return rc;
 }
 
+/*
+ * Dump object using data_parser/v0.0.39.
+ * Dump meta data and errors/warnings with SLURM_DATA_PARSER_VERSION
+ */
+extern int data_parser_dump_cli_stdout_v39(data_parser_type_t type, void *obj,
+					   int obj_bytes, const char *key,
+					   void *acct_db_conn,
+					   const char *mime_type,
+					   data_parser_dump_cli_ctxt_t *ctxt,
+					   openapi_resp_meta_t *meta)
+{
+	data_t *dresp = data_set_dict(data_new());
+	data_t *dout = data_key_set(dresp, key);
+	data_t *dmeta = data_key_set(dresp, "meta");
+	data_t *dwarning = data_key_set(dresp, "warnings");
+	data_t *derror = data_key_set(dresp, "errors");
+	data_parser_t *parser = NULL;
+	data_parser_t *meta_parser = NULL;
+	char *out = NULL;
+	int rc = SLURM_SUCCESS;
+
+	if (!(parser = data_parser_cli_parser(
+			SLURM_DATA_PARSER_VERSION_DEPRECATED, ctxt))) {
+		rc = ESLURM_DATA_INVALID_PARSER;
+		error("%s output not supported by %s",
+		      mime_type, SLURM_DATA_PARSER_VERSION);
+		goto cleanup;
+	}
+
+	if (!(meta_parser = data_parser_cli_parser(SLURM_DATA_PARSER_VERSION,
+						   ctxt))) {
+		rc = ESLURM_DATA_INVALID_PARSER;
+		error("%s output not supported by %s",
+		      mime_type, SLURM_DATA_PARSER_VERSION);
+		goto cleanup;
+	}
+
+	if (acct_db_conn)
+		data_parser_g_assign(parser, DATA_PARSER_ATTR_DBCONN_PTR,
+				     acct_db_conn);
+
+	if (!meta->plugin.data_parser)
+		meta->plugin.data_parser =
+			xstrdup(data_parser_get_plugin(parser));
+
+	if ((!data_parser_g_dump(parser, type, obj, obj_bytes, dout) &&
+	    (data_get_type(dout) != DATA_TYPE_NULL)) &&
+	    (!data_parser_g_dump(meta_parser, DATA_PARSER_OPENAPI_META, meta,
+				 sizeof(*meta), dmeta) &&
+	    (data_get_type(dmeta) != DATA_TYPE_NULL)) &&
+	    (!data_parser_g_dump(meta_parser, DATA_PARSER_OPENAPI_ERRORS,
+				 &ctxt->errors, sizeof(ctxt->errors), derror) &&
+	    (data_get_type(derror) != DATA_TYPE_NULL)) &&
+	    (!data_parser_g_dump(meta_parser, DATA_PARSER_OPENAPI_WARNINGS,
+				 &ctxt->warnings, sizeof(ctxt->warnings),
+				 dwarning) &&
+	    (data_get_type(dwarning) != DATA_TYPE_NULL))) {
+		serialize_g_data_to_string(&out, NULL, dresp, mime_type,
+					   SER_FLAGS_PRETTY);
+	}
+
+	if (out && out[0])
+		printf("%s\n", out);
+	else
+		debug("No output generated");
+
+cleanup:
+	xfree(out);
+	FREE_NULL_DATA(dresp);
+	FREE_NULL_DATA_PARSER(parser);
+	FREE_NULL_DATA_PARSER(meta_parser);
+
+	return rc;
+}
+
+extern bool is_data_parser_deprecated(const char *data_parser)
+{
+	char *full_name = "data_parser/" SLURM_DATA_PARSER_VERSION_DEPRECATED;
+	return !xstrcmp(data_parser, SLURM_DATA_PARSER_VERSION_DEPRECATED) ||
+		!xstrcmp(data_parser, full_name);
+}
+
 extern int data_parser_g_specify(data_parser_t *parser, data_t *dst)
 {
 	int rc;
@@ -807,3 +910,113 @@ extern data_parser_t *data_parser_cli_parser(const char *data_parser, void *arg)
 				 (data_parser ?  data_parser :
 				  SLURM_DATA_PARSER_VERSION), NULL, false);
 }
+
+extern openapi_type_t data_parser_g_resolve_openapi_type(
+	data_parser_t *parser,
+	data_parser_type_t type,
+	const char *field)
+{
+	const parse_funcs_t *funcs;
+
+	if (!parser)
+		return OPENAPI_TYPE_INVALID;
+
+	funcs = plugins->functions[parser->plugin_offset];
+
+	xassert(parser);
+	xassert(plugins);
+	xassert(parser->magic == PARSE_MAGIC);
+	xassert(parser->plugin_offset < plugins->count);
+
+	return funcs->resolve_openapi_type(parser->arg, type, field);
+}
+
+extern const char *data_parser_g_resolve_type_string(data_parser_t *parser,
+						     data_parser_type_t type)
+{
+	const parse_funcs_t *funcs;
+
+	if (!parser)
+		return NULL;
+
+	funcs = plugins->functions[parser->plugin_offset];
+
+	xassert(plugins);
+	xassert(parser->magic == PARSE_MAGIC);
+	xassert(parser->plugin_offset < plugins->count);
+
+	return funcs->resolve_type_str(parser->arg, type);
+}
+
+extern int data_parser_g_increment_reference(data_parser_t *parser,
+					     data_parser_type_t type,
+					     void **references_ptr)
+{
+	const parse_funcs_t *funcs;
+
+	if (!parser)
+		return EINVAL;
+
+	funcs = plugins->functions[parser->plugin_offset];
+
+	xassert(parser->magic == PARSE_MAGIC);
+	xassert(parser->plugin_offset < plugins->count);
+
+	return funcs->inc_ref(parser->arg, type, references_ptr);
+}
+
+extern int data_parser_g_populate_schema(data_parser_t *parser,
+					 data_parser_type_t type,
+					 void **references_ptr, data_t *dst,
+					 data_t *schemas)
+{
+	const parse_funcs_t *funcs;
+
+	if (!parser)
+		return EINVAL;
+
+	funcs = plugins->functions[parser->plugin_offset];
+
+	xassert(parser->magic == PARSE_MAGIC);
+	xassert(parser->plugin_offset < plugins->count);
+
+	return funcs->populate_schema(parser->arg, type, references_ptr, dst,
+				      schemas);
+}
+
+extern int data_parser_g_populate_parameters(data_parser_t *parser,
+					     data_parser_type_t parameter_type,
+					     data_parser_type_t query_type,
+					     void **references_ptr, data_t *dst,
+					     data_t *schemas)
+{
+	const parse_funcs_t *funcs;
+
+	if (!parser)
+		return EINVAL;
+
+	funcs = plugins->functions[parser->plugin_offset];
+
+	xassert(parser->magic == PARSE_MAGIC);
+	xassert(parser->plugin_offset < plugins->count);
+
+	return funcs->populate_parameters(parser->arg, parameter_type,
+					  query_type, references_ptr, dst,
+					  schemas);
+}
+
+extern void data_parser_g_release_references(data_parser_t *parser,
+					     void **references_ptr)
+{
+	const parse_funcs_t *funcs;
+
+	if (!parser)
+		return;
+
+	funcs = plugins->functions[parser->plugin_offset];
+
+	xassert(parser->magic == PARSE_MAGIC);
+	xassert(parser->plugin_offset < plugins->count);
+
+	return funcs->release_refs(parser->arg, references_ptr);
+}
