@@ -3,7 +3,6 @@
  *                               information from the as_mysql jobacct
  *                               storage.
  *****************************************************************************
- *
  *  Copyright (C) 2008-2009 Lawrence Livermore National Security.
  *  Copyright (C) 2004-2007 The Regents of the University of California.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
@@ -45,7 +44,7 @@
 #include "as_mysql_jobacct_process.h"
 
 typedef struct {
-	hostlist_t hl;
+	hostlist_t *hl;
 	time_t start;
 	time_t end;
 	bitstr_t *asked_bitmap;
@@ -59,10 +58,13 @@ char *job_req_inx[] = {
 	"t1.array_max_tasks",
 	"t1.array_task_str",
 	"t1.constraints",
+	"t1.container",
 	"t1.cpus_req",
 	"t1.derived_ec",
 	"t1.derived_es",
 	"t1.exit_code",
+	"t1.extra",
+	"t1.failed_node",
 	"t1.flags",
 	"t1.id_array_job",
 	"t1.id_array_task",
@@ -80,6 +82,7 @@ char *job_req_inx[] = {
 	"t1.job_db_inx",
 	"t1.job_name",
 	"t1.kill_requid",
+	"t1.licenses",
 	"t1.mem_req",
 	"t1.node_inx",
 	"t1.nodelist",
@@ -95,17 +98,21 @@ char *job_req_inx[] = {
 	"t1.time_submit",
 	"t1.time_suspended",
 	"t1.timelimit",
-	"t1.track_steps",
 	"t1.wckey",
-	"t1.gres_alloc",
-	"t1.gres_req",
 	"t1.gres_used",
 	"t1.tres_alloc",
 	"t1.tres_req",
 	"t1.work_dir",
 	"t1.mcs_label",
+	"t4.batch_script",
+	"t1.std_err",
+	"t1.std_in",
+	"t1.std_out",
+	"t1.submit_line",
+	"t4.env_vars",
 	"t2.acct",
 	"t2.lft",
+	"t2.lineage",
 	"t2.user"
 };
 
@@ -115,10 +122,13 @@ enum {
 	JOB_REQ_ARRAY_MAX,
 	JOB_REQ_ARRAY_STR,
 	JOB_REQ_CONSTRAINTS,
+	JOB_REQ_CONTAINER,
 	JOB_REQ_REQ_CPUS,
 	JOB_REQ_DERIVED_EC,
 	JOB_REQ_DERIVED_ES,
 	JOB_REQ_EXIT_CODE,
+	JOB_REQ_EXTRA,
+	JOB_REQ_FAILED_NODE,
 	JOB_REQ_FLAGS,
 	JOB_REQ_ARRAYJOBID,
 	JOB_REQ_ARRAYTASKID,
@@ -136,6 +146,7 @@ enum {
 	JOB_REQ_DB_INX,
 	JOB_REQ_NAME,
 	JOB_REQ_KILL_REQUID,
+	JOB_REQ_LICENSES,
 	JOB_REQ_REQ_MEM,
 	JOB_REQ_NODE_INX,
 	JOB_REQ_NODELIST,
@@ -151,17 +162,21 @@ enum {
 	JOB_REQ_SUBMIT,
 	JOB_REQ_SUSPENDED,
 	JOB_REQ_TIMELIMIT,
-	JOB_REQ_TRACKSTEPS,
 	JOB_REQ_WCKEY,
-	JOB_REQ_GRES_ALLOC,
-	JOB_REQ_GRES_REQ,
 	JOB_REQ_GRES_USED,
 	JOB_REQ_TRESA,
 	JOB_REQ_TRESR,
 	JOB_REQ_WORK_DIR,
 	JOB_REQ_MCS_LABEL,
+	JOB_REQ_SCRIPT,
+	JOB_REQ_STDERR,
+	JOB_REQ_STDIN,
+	JOB_REQ_STDOUT,
+	JOB_REQ_SUBMIT_LINE,
+	JOB_REQ_ENV,
 	JOB_REQ_ACCOUNT,
 	JOB_REQ_LFT,
+	JOB_REQ_LINEAGE,
 	JOB_REQ_USER_NAME,
 	JOB_REQ_COUNT
 };
@@ -170,6 +185,7 @@ enum {
  * enum below also t1 is step_table */
 char *step_req_inx[] = {
 	"t1.id_step",
+	"t1.step_het_comp",
 	"t1.time_start",
 	"t1.time_end",
 	"t1.time_suspended",
@@ -188,9 +204,11 @@ char *step_req_inx[] = {
 	"t1.sys_usec",
 	"t1.act_cpufreq",
 	"t1.consumed_energy",
+	"t1.container",
 	"t1.req_cpufreq_min",
 	"t1.req_cpufreq",
 	"t1.req_cpufreq_gov",
+	"t1.submit_line",
 	"t1.tres_alloc",
 	"t1.tres_usage_in_max",
 	"t1.tres_usage_in_max_taskid",
@@ -212,6 +230,7 @@ char *step_req_inx[] = {
 
 enum {
 	STEP_REQ_STEPID,
+	STEP_REQ_STEP_HET_COMP,
 	STEP_REQ_START,
 	STEP_REQ_END,
 	STEP_REQ_SUSPENDED,
@@ -230,9 +249,11 @@ enum {
 	STEP_REQ_SYS_USEC,
 	STEP_REQ_ACT_CPUFREQ,
 	STEP_REQ_CONSUMED_ENERGY,
+	STEP_REQ_CONTAINER,
 	STEP_REQ_REQ_CPUFREQ_MIN,
 	STEP_REQ_REQ_CPUFREQ_MAX,
 	STEP_REQ_REQ_CPUFREQ_GOV,
+	STEP_REQ_SUBMIT_LINE,
 	STEP_REQ_TRES,
 	STEP_REQ_TRES_USAGE_IN_MAX,
 	STEP_REQ_TRES_USAGE_IN_MAX_TASKID,
@@ -256,8 +277,8 @@ enum {
 static void _setup_job_cond_selected_steps(slurmdb_job_cond_t *job_cond,
 					   char *cluster_name, char **extra)
 {
-	ListIterator itr = NULL;
-	slurmdb_selected_step_t *selected_step = NULL;
+	list_itr_t *itr = NULL;
+	slurm_selected_step_t *selected_step = NULL;
 
 	if (!job_cond || (job_cond->flags & JOBCOND_FLAG_RUNAWAY))
 		return;
@@ -278,7 +299,7 @@ static void _setup_job_cond_selected_steps(slurmdb_job_cond_t *job_cond,
 				if (array_task_ids)
 					xstrcat(array_task_ids, " ,");
 				xstrfmtcat(array_task_ids, "(%u, %u)",
-					   selected_step->jobid,
+					   selected_step->step_id.job_id,
 					   selected_step->array_task_id);
 			} else if (selected_step->het_job_offset != NO_VAL) {
 				if (het_job_ids)
@@ -286,7 +307,7 @@ static void _setup_job_cond_selected_steps(slurmdb_job_cond_t *job_cond,
 				if (het_job_offset)
 					xstrcat(het_job_offset, " ,");
 				xstrfmtcat(het_job_ids, "%u",
-					   selected_step->jobid);
+					   selected_step->step_id.job_id);
 				xstrfmtcat(het_job_offset, "%u",
 					   selected_step->het_job_offset);
 			} else {
@@ -295,9 +316,9 @@ static void _setup_job_cond_selected_steps(slurmdb_job_cond_t *job_cond,
 				if (array_job_ids)
 					xstrcat(array_job_ids, " ,");
 				xstrfmtcat(job_ids, "%u",
-					   selected_step->jobid);
+					   selected_step->step_id.job_id);
 				xstrfmtcat(array_job_ids, "%u",
-					   selected_step->jobid);
+					   selected_step->step_id.job_id);
 			}
 		}
 		list_iterator_destroy(itr);
@@ -305,12 +326,12 @@ static void _setup_job_cond_selected_steps(slurmdb_job_cond_t *job_cond,
 		if (job_ids) {
 			if (job_cond->flags & JOBCOND_FLAG_WHOLE_HETJOB)
 				xstrfmtcat(*extra, "t1.id_job in (%s) || "
-					   "(t1.het_job_offset<>%u && "
 					   "t1.het_job_id in (select "
 					   "t4.het_job_id from \"%s_%s\" as "
-					   "t4 where t4.id_job in (%s)))",
-					   job_ids, NO_VAL, cluster_name,
-					   job_table, job_ids);
+					   "t4 where t4.id_job in (%s) && "
+					   "t4.het_job_id)",
+					   job_ids, cluster_name, job_table,
+					   job_ids);
 			else if (job_cond->flags & JOBCOND_FLAG_NO_WHOLE_HETJOB)
 				xstrfmtcat(*extra, "t1.id_job in (%s)",
 					   job_ids);
@@ -465,84 +486,51 @@ static int _cluster_get_jobs(mysql_conn_t *mysql_conn,
 {
 	char *query = NULL;
 	char *extra = xstrdup(sent_extra);
-	uint16_t private_data = slurm_get_private_data();
-	slurmdb_selected_step_t *selected_step = NULL;
+	slurm_selected_step_t *selected_step = NULL;
 	MYSQL_RES *result = NULL, *step_result = NULL;
 	MYSQL_ROW row, step_row;
 	slurmdb_job_rec_t *job = NULL;
 	slurmdb_step_rec_t *step = NULL;
 	time_t now = time(NULL);
 	List job_list = list_create(slurmdb_destroy_job_rec);
-	ListIterator itr = NULL, itr2 = NULL;
+	list_itr_t *itr = NULL, *itr2 = NULL;
 	List local_cluster_list = NULL;
 	int set = 0;
 	char *prefix="t2";
 	int rc = SLURM_SUCCESS;
 	int last_id = -1, curr_id = -1;
+	int comb_id = 0;
 	local_cluster_t *curr_cluster = NULL;
+	bool jobid_filtered = false;
 
 	/* This is here to make sure we are looking at only this user
 	 * if this flag is set.  We also include any accounts they may be
 	 * coordinator of.
+	 *
+	 * This clause must be kept in sync with the access check in
+	 * as_mysql_jobacct_process_get_jobs().
 	 */
-	if (!is_admin && (private_data & PRIVATE_DATA_JOBS)) {
-		query = xstrdup_printf("select lft from \"%s_%s\" "
-				       "where user='%s'",
-				       cluster_name, assoc_table, user->name);
-		if (user->coord_accts) {
+	if (!is_admin && ((slurm_conf.private_data & PRIVATE_DATA_JOBS) ||
+			  (job_cond->flags & JOBCOND_FLAG_SCRIPT) ||
+			  (job_cond->flags & JOBCOND_FLAG_ENV))) {
+		if (!extra)
+			xstrcat(extra, " where ");
+		else
+			xstrcat(extra, " && ");
+		xstrfmtcat(extra, "((%s.lineage like '%%/0-%s/%%')",
+			   prefix, user->name);
+		if (!(slurmdbd_conf->flags & DBD_CONF_FLAG_DISABLE_COORD_DBD) &&
+		    user->coord_accts && list_count(user->coord_accts)) {
 			slurmdb_coord_rec_t *coord = NULL;
 			itr = list_iterator_create(user->coord_accts);
 			while ((coord = list_next(itr))) {
-				xstrfmtcat(query, " || acct='%s'",
-					   coord->name);
+				xstrfmtcat(extra,
+					   " || (%s.lineage like '%%/%s/%%')",
+					   prefix, coord->name);
 			}
 			list_iterator_destroy(itr);
 		}
-		if (debug_flags & DEBUG_FLAG_DB_JOB)
-			DB_DEBUG(mysql_conn->conn, "query\n%s", query);
-		if (!(result = mysql_db_query_ret(
-			      mysql_conn, query, 0))) {
-			xfree(extra);
-			xfree(query);
-			rc = SLURM_ERROR;
-			goto end_it;
-		}
-		xfree(query);
-		set = 0;
-		while ((row = mysql_fetch_row(result))) {
-			if (set) {
-				xstrfmtcat(extra,
-					   " || (%s between %s.lft and %s.rgt)",
-					   row[0], prefix, prefix);
-			} else {
-				set = 1;
-				if (extra)
-					xstrfmtcat(extra,
-						   " && ((%s between %s.lft "
-						   "and %s.rgt)",
-						   row[0], prefix,
-						   prefix);
-				else
-					xstrfmtcat(extra,
-						   " where ((%s between %s.lft "
-						   "and %s.rgt)",
-						   row[0], prefix,
-						   prefix);
-			}
-		}
-
-		mysql_free_result(result);
-
-		if (set)
-			xstrcat(extra, ")");
-		else {
-			xfree(extra);
-			debug("User %s has no associations, and is not admin, "
-			      "so not returning any jobs.", user->name);
-			/* This user has no valid associations, so
-			 * they will not have any jobs. */
-			goto end_it;
-		}
+		xstrcatchar(extra, ')');
 	}
 
 	setup_job_cluster_cond_limits(mysql_conn, job_cond,
@@ -566,6 +554,17 @@ static int _cluster_get_jobs(mysql_conn_t *mysql_conn,
 			       cluster_name, assoc_table,
 			       cluster_name, resv_table);
 
+	if (job_cond->flags & JOBCOND_FLAG_SCRIPT)
+		xstrfmtcat(query,
+			   " left join \"%s_%s\" as t4 "
+			   "on t1.script_hash_inx=t4.hash_inx",
+			   cluster_name, job_script_table);
+	else if (job_cond->flags & JOBCOND_FLAG_ENV)
+		xstrfmtcat(query,
+			   " left join \"%s_%s\" as t4 "
+			   "on t1.env_hash_inx=t4.hash_inx",
+			   cluster_name, job_env_table);
+
 	if (job_cond->flags & JOBCOND_FLAG_RUNAWAY) {
 		if (extra)
 			xstrcat(extra, " && (t1.time_end=0)");
@@ -577,6 +576,8 @@ static int _cluster_get_jobs(mysql_conn_t *mysql_conn,
 		xstrcat(query, extra);
 		xfree(extra);
 	}
+	if (job_cond->step_list)
+		jobid_filtered = true;
 
 	/* Here we want to order them this way in such a way so it is
 	   easy to look for duplicates, it is also easy to sort the
@@ -584,8 +585,7 @@ static int _cluster_get_jobs(mysql_conn_t *mysql_conn,
 	*/
 	xstrcat(query, " order by id_job, time_submit desc");
 
-	if (debug_flags & DEBUG_FLAG_DB_JOB)
-		DB_DEBUG(mysql_conn->conn, "query\n%s", query);
+	DB_DEBUG(DB_JOB, mysql_conn->conn, "query\n%s", query);
 	if (!(result = mysql_db_query_ret(mysql_conn, query, 0))) {
 		xfree(query);
 		rc = SLURM_ERROR;
@@ -614,13 +614,26 @@ static int _cluster_get_jobs(mysql_conn_t *mysql_conn,
 		char *db_inx_char = row[JOB_REQ_DB_INX];
 		bool job_ended = 0;
 		int start = slurm_atoul(row[JOB_REQ_START]);
+		int arrayjob = slurm_atoul(row[JOB_REQ_ARRAYJOBID]);
+		int hetjob = slurm_atoul(row[JOB_REQ_HET_JOB_ID]);
 
 		curr_id = slurm_atoul(row[JOB_REQ_JOBID]);
-
-		if (job_cond && !(job_cond->flags & JOBCOND_FLAG_DUP)
-		    && (curr_id == last_id)
-		    && (slurm_atoul(row[JOB_REQ_STATE]) != JOB_RESIZING))
-			continue;
+		if (job_cond && !(job_cond->flags & JOBCOND_FLAG_DUP)) {
+			if ((curr_id == last_id) &&
+			    (slurm_atoul(row[JOB_REQ_STATE]) != JOB_RESIZING))
+				continue;
+			/*
+			 * Doing advanced duplication removal when requesting
+			 * specific jobIDs and hetjobs/arrayjobs involved
+			*/
+			if (jobid_filtered) {
+				if ((last_id != hetjob) &&
+				    (last_id != arrayjob)) {
+					comb_id = arrayjob + hetjob;
+				} else if (comb_id != (arrayjob + hetjob))
+					continue;
+			}
+		}
 
 		/* check the bitmap to see if this is one of the jobs
 		   we are looking for */
@@ -644,10 +657,6 @@ static int _cluster_get_jobs(mysql_conn_t *mysql_conn,
 			list_append(job_list, job);
 		last_id = curr_id;
 
-		if (row[JOB_REQ_GRES_ALLOC])
-			job->alloc_gres = xstrdup(row[JOB_REQ_GRES_ALLOC]);
-		else
-			job->alloc_gres = xstrdup("");
 		job->alloc_nodes = slurm_atoul(row[JOB_REQ_ALLOC_NODES]);
 		job->associd = slurm_atoul(row[JOB_REQ_ASSOCID]);
 		job->array_job_id = slurm_atoul(row[JOB_REQ_ARRAYJOBID]);
@@ -662,6 +671,14 @@ static int _cluster_get_jobs(mysql_conn_t *mysql_conn,
 		if (!job->array_job_id && !job->array_task_id)
 			job->array_task_id = NO_VAL;
 
+		/*
+		 * This shouldn't happen with new jobs.
+		 * If older jobs have het_job_id == 0 and het_job_offset == 0,
+		 * then correct het_job_offset to be NO_VAL.
+		 */
+		if (!job->het_job_id && !job->het_job_offset)
+			job->het_job_offset = NO_VAL;
+
 		if (row[JOB_REQ_RESV_NAME] && row[JOB_REQ_RESV_NAME][0])
 			job->resv_name = xstrdup(row[JOB_REQ_RESV_NAME]);
 
@@ -680,12 +697,15 @@ static int _cluster_get_jobs(mysql_conn_t *mysql_conn,
 		if (row[JOB_REQ_USER_NAME])
 			job->user = xstrdup(row[JOB_REQ_USER_NAME]);
 
-		if (!job->user || (job_cond->flags & JOBCOND_FLAG_DBD_UID))
+		if (row[JOB_REQ_UID])
 			job->uid = slurm_atoul(row[JOB_REQ_UID]);
 
 		if (row[JOB_REQ_LFT])
 			job->lft = slurm_atoul(row[JOB_REQ_LFT]);
 
+		if (row[JOB_REQ_LINEAGE])
+			job->lineage = xstrdup(row[JOB_REQ_LINEAGE]);
+
 		if (row[JOB_REQ_ACCOUNT] && row[JOB_REQ_ACCOUNT][0])
 			job->account = xstrdup(row[JOB_REQ_ACCOUNT]);
 		else if (row[JOB_REQ_ACCOUNT1] && row[JOB_REQ_ACCOUNT1][0])
@@ -710,10 +730,20 @@ static int _cluster_get_jobs(mysql_conn_t *mysql_conn,
 		job->end = slurm_atoul(row[JOB_REQ_END]);
 		job->timelimit = slurm_atoul(row[JOB_REQ_TIMELIMIT]);
 
+		job->script = xstrdup(row[JOB_REQ_SCRIPT]);
+
+		job->env = xstrdup(row[JOB_REQ_ENV]);
+
+		job->std_err = xstrdup(row[JOB_REQ_STDERR]);
+		job->std_in = xstrdup(row[JOB_REQ_STDIN]);
+		job->std_out = xstrdup(row[JOB_REQ_STDOUT]);
+
+		job->submit_line = xstrdup(row[JOB_REQ_SUBMIT_LINE]);
+
 		/* since the job->end could be set later end it here */
 		if (job->end) {
 			job_ended = 1;
-			if (!job->start || (job->start > job->end))
+			if (job->start > job->end)
 				job->start = job->end;
 		}
 
@@ -812,12 +842,25 @@ static int _cluster_get_jobs(mysql_conn_t *mysql_conn,
 		job->jobname = xstrdup(row[JOB_REQ_NAME]);
 		job->gid = slurm_atoul(row[JOB_REQ_GID]);
 		job->exitcode = slurm_atoul(row[JOB_REQ_EXIT_CODE]);
+		job->failed_node = xstrdup(row[JOB_REQ_FAILED_NODE]);
 		job->derived_ec = slurm_atoul(row[JOB_REQ_DERIVED_EC]);
 		job->derived_es = xstrdup(row[JOB_REQ_DERIVED_ES]);
 		job->admin_comment = xstrdup(row[JOB_REQ_ADMIN_COMMENT]);
 		job->system_comment = xstrdup(row[JOB_REQ_SYSTEM_COMMENT]);
 		job->constraints = xstrdup(row[JOB_REQ_CONSTRAINTS]);
+		job->container = xstrdup(row[JOB_REQ_CONTAINER]);
+		job->extra = xstrdup(row[JOB_REQ_EXTRA]);
+		job->licenses = xstrdup(row[JOB_REQ_LICENSES]);
 		job->flags = slurm_atoul(row[JOB_REQ_FLAGS]);
+
+		/*
+		 * This tells us we never had a start time so the job was
+		 * canceled before it ran.
+		 */
+		if (!job->start && job->end &&
+		    (job->flags & SLURMDB_JOB_FLAG_START_R))
+			job->start = NO_VAL;
+
 		job->state_reason_prev = slurm_atoul(row[JOB_REQ_STATE_REASON]);
 
 		if (row[JOB_REQ_PARTITION])
@@ -831,15 +874,13 @@ static int _cluster_get_jobs(mysql_conn_t *mysql_conn,
 			job->nodes = xstrdup("(unknown)");
 		}
 
-		job->track_steps = slurm_atoul(row[JOB_REQ_TRACKSTEPS]);
 		job->priority = slurm_atoul(row[JOB_REQ_PRIORITY]);
 		job->req_cpus = slurm_atoul(row[JOB_REQ_REQ_CPUS]);
-		if (row[JOB_REQ_GRES_REQ])
-			job->req_gres = xstrdup(row[JOB_REQ_GRES_REQ]);
-		else
-			job->req_gres = xstrdup("");
 		job->req_mem = slurm_atoull(row[JOB_REQ_REQ_MEM]);
-		job->requid = slurm_atoul(row[JOB_REQ_KILL_REQUID]);
+		if (!row[JOB_REQ_KILL_REQUID])
+			job->requid = INFINITE;
+		else
+			job->requid = slurm_atoul(row[JOB_REQ_KILL_REQUID]);
 		job->qosid = slurm_atoul(row[JOB_REQ_QOS]);
 		job->show_full = 1;
 
@@ -859,9 +900,11 @@ static int _cluster_get_jobs(mysql_conn_t *mysql_conn,
 			set = 0;
 			itr = list_iterator_create(job_cond->step_list);
 			while ((selected_step = list_next(itr))) {
-				if ((selected_step->jobid != job->jobid) &&
-				    (selected_step->jobid != job->het_job_id)&&
-				    (selected_step->jobid !=
+				if ((selected_step->step_id.job_id !=
+				     job->jobid) &&
+				    (selected_step->step_id.job_id !=
+				     job->het_job_id)&&
+				    (selected_step->step_id.job_id !=
 				     job->array_job_id)) {
 					continue;
 				} else if ((selected_step->array_task_id !=
@@ -874,28 +917,25 @@ static int _cluster_get_jobs(mysql_conn_t *mysql_conn,
 					   (selected_step->het_job_offset !=
 					    job->het_job_offset)) {
 					continue;
-				} else if (selected_step->stepid == NO_VAL) {
+				} else if (selected_step->step_id.step_id ==
+					   NO_VAL) {
 					job->show_full = 1;
 					break;
-				} else if (selected_step->stepid == INFINITE)
-					selected_step->stepid =
-						SLURM_BATCH_SCRIPT;
-
+				}
 				if (set)
 					xstrcat(extra, " || ");
 				else
 					xstrcat(extra, " && (");
 
-				/* The stepid could be -2 so use %d not %u */
+				/*
+				 * The stepid could be negative so use
+				 * %d not %u
+				 */
 				xstrfmtcat(extra, "t1.id_step=%d",
-					   selected_step->stepid);
+					   selected_step->step_id.step_id);
+
 				set = 1;
 				job->show_full = 0;
-				/* Set it back just in case we are
-				   looking at a job array.
-				*/
-				if (selected_step->stepid == SLURM_BATCH_SCRIPT)
-					selected_step->stepid = INFINITE;
 			}
 			list_iterator_destroy(itr);
 			if (set)
@@ -916,8 +956,7 @@ static int _cluster_get_jobs(mysql_conn_t *mysql_conn,
 			xfree(extra);
 		}
 
-		if (debug_flags & DEBUG_FLAG_DB_STEP)
-			DB_DEBUG(mysql_conn->conn, "query\n%s", query);
+		DB_DEBUG(DB_STEP, mysql_conn->conn, "query\n%s", query);
 
 		if (!(step_result = mysql_db_query_ret(
 			      mysql_conn, query, 0))) {
@@ -948,9 +987,12 @@ static int _cluster_get_jobs(mysql_conn_t *mysql_conn,
 			if (!job->first_step_ptr)
 				job->first_step_ptr = step;
 			list_append(job->steps, step);
-			step->stepid = slurm_atoul(step_row[STEP_REQ_STEPID]);
-			/* info("got step %u.%u", */
-/* 			     job->header.jobnum, step->stepnum); */
+			step->step_id.job_id = job->jobid;
+			step->step_id.step_id = slurm_atoul(
+				step_row[STEP_REQ_STEPID]);
+			step->step_id.step_het_comp =
+				slurm_atoul(step_row[STEP_REQ_STEP_HET_COMP]);
+			/* info("got %ps", &step->step_id); */
 			step->state = slurm_atoul(step_row[STEP_REQ_STATE]);
 			step->exitcode =
 				slurm_atoul(step_row[STEP_REQ_EXIT_CODE]);
@@ -1016,15 +1058,21 @@ static int _cluster_get_jobs(mysql_conn_t *mysql_conn,
 
 			step->stepname = xstrdup(step_row[STEP_REQ_NAME]);
 			step->nodes = xstrdup(step_row[STEP_REQ_NODELIST]);
-			step->requid =
-				slurm_atoul(step_row[STEP_REQ_KILL_REQUID]);
+			if (!step_row[STEP_REQ_KILL_REQUID])
+				step->requid = INFINITE;
+			else
+				step->requid = slurm_atoul(
+					step_row[STEP_REQ_KILL_REQUID]);
 
-			step->user_cpu_sec = slurm_atoul(
+			step->submit_line =
+				xstrdup(step_row[STEP_REQ_SUBMIT_LINE]);
+
+			step->user_cpu_sec = slurm_atoull(
 				step_row[STEP_REQ_USER_SEC]);
 			step->user_cpu_usec = slurm_atoul(
 				step_row[STEP_REQ_USER_USEC]);
-			step->sys_cpu_sec =
-				slurm_atoul(step_row[STEP_REQ_SYS_SEC]);
+			step->sys_cpu_sec =slurm_atoull(
+				step_row[STEP_REQ_SYS_SEC]);
 			step->sys_cpu_usec = slurm_atoul(
 				step_row[STEP_REQ_SYS_USEC]);
 			step->tot_cpu_sec +=
@@ -1083,37 +1131,13 @@ static int _cluster_get_jobs(mysql_conn_t *mysql_conn,
 				atof(step_row[STEP_REQ_ACT_CPUFREQ]);
 			step->stats.consumed_energy = slurm_atoull(
 				step_row[STEP_REQ_CONSUMED_ENERGY]);
+			step->container = xstrdup(step_row[STEP_REQ_CONTAINER]);
 
 			if (step_row[STEP_REQ_TRES])
 				step->tres_alloc_str =
 					xstrdup(step_row[STEP_REQ_TRES]);
 		}
 		mysql_free_result(step_result);
-
-		if (!job->track_steps) {
-			uint64_t j_cpus, s_cpus;
-			/* If we don't have track_steps we want to see
-			   if we have multiple steps.  If we only have
-			   1 step check the job name against the step
-			   name in most all cases it will be
-			   different.  If it is different print out
-			   the step separate.  It could also be a single
-			   step/allocation where the job was allocated more than
-			   the step requested (eg. CR_Socket).
-			*/
-			if (list_count(job->steps) > 1)
-				job->track_steps = 1;
-			else if (step &&
-				 (xstrcmp(step->stepname, job->jobname) ||
-				  (((j_cpus = slurmdb_find_tres_count_in_string(
-					     job->tres_alloc_str, TRES_CPU))
-				    != INFINITE64) &&
-				   ((s_cpus = slurmdb_find_tres_count_in_string(
-					     step->tres_alloc_str, TRES_CPU))
-				    != INFINITE64) &&
-				  j_cpus != s_cpus)))
-					job->track_steps = 1;
-		}
 	skip_steps:
 		/* need to reset here to make the above test valid */
 		step = NULL;
@@ -1141,8 +1165,8 @@ extern List setup_cluster_list_with_inx(mysql_conn_t *mysql_conn,
 	time_t now = time(NULL);
 	MYSQL_RES *result = NULL;
 	MYSQL_ROW row;
-	hostlist_t temp_hl = NULL;
-	hostlist_iterator_t h_itr = NULL;
+	hostlist_t *temp_hl = NULL;
+	hostlist_iterator_t *h_itr = NULL;
 	char *query = NULL;
 	int dims = 0;
 
@@ -1185,8 +1209,7 @@ extern List setup_cluster_list_with_inx(mysql_conn_t *mysql_conn,
 			   job_cond->usage_end, job_cond->usage_start);
 	}
 
-	if (debug_flags & DEBUG_FLAG_DB_JOB)
-		DB_DEBUG(mysql_conn->conn, "query\n%s", query);
+	DB_DEBUG(DB_JOB, mysql_conn->conn, "query\n%s", query);
 	if (!(result = mysql_db_query_ret(mysql_conn, query, 0))) {
 		xfree(query);
 		goto no_hosts;
@@ -1214,7 +1237,7 @@ extern List setup_cluster_list_with_inx(mysql_conn_t *mysql_conn,
 		if (bit_ffs(local_cluster->asked_bitmap) != -1) {
 			list_append(local_cluster_list, local_cluster);
 			if (local_cluster->end == 0) {
-				local_cluster->end = now;
+				local_cluster->end = now + 1;
 				(*curr_cluster) = local_cluster;
 			} else if (!(*curr_cluster)
 				   || (((local_cluster_t *)(*curr_cluster))->end
@@ -1256,7 +1279,7 @@ extern int good_nodes_from_inx(List local_cluster_list,
 		    || (start >= (*curr_cluster)->end)) {
 			local_cluster_t *local_cluster = NULL;
 
-			ListIterator itr =
+			list_itr_t *itr =
 				list_iterator_create(local_cluster_list);
 			while ((local_cluster = list_next(itr))) {
 				if ((start >= local_cluster->start)
@@ -1285,7 +1308,7 @@ extern int setup_job_cluster_cond_limits(mysql_conn_t *mysql_conn,
 					 char *cluster_name, char **extra)
 {
 	int set = 0;
-	ListIterator itr = NULL;
+	list_itr_t *itr = NULL;
 	char *object = NULL;
 
 	if (!job_cond)
@@ -1366,7 +1389,7 @@ no_resv:
 
 	/* Don't show revoked sibling federated jobs w/out -D */
 	if (!(job_cond->flags & JOBCOND_FLAG_DUP))
-		xstrfmtcat(*extra, " %s (t1.state != %d)",
+		xstrfmtcat(*extra, " %s (t1.state != %"PRIu64")",
 			   *extra ? "&&" : "where",
 			   JOB_REVOKED);
 
@@ -1377,7 +1400,7 @@ extern int setup_job_cond_limits(slurmdb_job_cond_t *job_cond,
 				 char **extra)
 {
 	int set = 0;
-	ListIterator itr = NULL;
+	list_itr_t *itr = NULL;
 	char *object = NULL;
 
 	if (!job_cond || (job_cond->flags & JOBCOND_FLAG_RUNAWAY))
@@ -1571,12 +1594,17 @@ extern int setup_job_cond_limits(slurmdb_job_cond_t *job_cond,
 		else
 			xstrcat(*extra, " where (");
 
+		xstrfmtcat(*extra,
+			   "(CONVERT(SUBSTRING_INDEX(t1.tres_alloc,'%d=',-1),"
+			   "UNSIGNED INTEGER)",
+			   TRES_CPU);
+
 		if (job_cond->cpus_max) {
-			xstrfmtcat(*extra, "(t1.ext_1 between %u and %u))",
+			xstrfmtcat(*extra, " between %u and %u))",
 				   job_cond->cpus_min, job_cond->cpus_max);
 
 		} else {
-			xstrfmtcat(*extra, "(t1.ext_1='%u'))",
+			xstrfmtcat(*extra, "='%u'))",
 				   job_cond->cpus_min);
 
 		}
@@ -1618,6 +1646,7 @@ extern int setup_job_cond_limits(slurmdb_job_cond_t *job_cond,
 		}
 	}
 
+	/* Time window should be exclusive of the end time, ie [start,end) */
 	if (!job_cond->state_list || !list_count(job_cond->state_list)) {
 		/*
 		 * There's an explicit list of jobs, so don't hide
@@ -1633,7 +1662,9 @@ extern int setup_job_cond_limits(slurmdb_job_cond_t *job_cond,
 					xstrcat(*extra, " where (");
 
 				xstrfmtcat(*extra,
-					   "(t1.time_submit <= %ld) && (t1.time_end >= %ld || t1.time_end = 0))",
+					   "(t1.time_submit < %ld) && "
+					   "(t1.time_end >= %ld ||"
+					   " t1.time_end = 0))",
 					   job_cond->usage_end,
 					   job_cond->usage_start);
 			}
@@ -1694,23 +1725,28 @@ extern List as_mysql_jobacct_process_get_jobs(mysql_conn_t *mysql_conn,
 {
 	char *extra = NULL;
 	char *tmp = NULL, *tmp2 = NULL;
-	ListIterator itr = NULL;
+	list_itr_t *itr = NULL;
 	int is_admin=1;
 	int i;
 	List job_list = NULL;
-	uint16_t private_data = 0;
 	slurmdb_user_rec_t user;
 	int only_pending = 0;
-	List use_cluster_list = as_mysql_cluster_list;
+	List use_cluster_list = NULL;
 	char *cluster_name;
+	bool locked = false;
 	assoc_mgr_lock_t locks = { NO_LOCK, NO_LOCK, NO_LOCK, NO_LOCK,
 				   READ_LOCK, NO_LOCK, NO_LOCK };
 
 	memset(&user, 0, sizeof(slurmdb_user_rec_t));
 	user.uid = uid;
 
-	private_data = slurm_get_private_data();
-	if (private_data & PRIVATE_DATA_JOBS) {
+	/*
+	 * This clause must be kept in sync with the access check in
+	 * _cluster_get_jobs().
+	 */
+	if ((slurm_conf.private_data & PRIVATE_DATA_JOBS) ||
+	    (job_cond->flags & JOBCOND_FLAG_SCRIPT) ||
+	    (job_cond->flags & JOBCOND_FLAG_ENV)) {
 		if (!(is_admin = is_user_min_admin_level(
 			      mysql_conn, uid, SLURMDB_ADMIN_OPERATOR))) {
 			/*
@@ -1731,12 +1767,36 @@ extern List as_mysql_jobacct_process_get_jobs(mysql_conn_t *mysql_conn,
 	    && (slurm_atoul(list_peek(job_cond->state_list)) == JOB_PENDING))
 		only_pending = 1;
 
+	if (job_cond &&
+	    (!job_cond->step_list || !list_count(job_cond->step_list))) {
+		char *reason = NULL;
+
+		if (job_cond->flags & JOBCOND_FLAG_SCRIPT)
+			reason = "job scripts";
+
+		if (job_cond->flags & JOBCOND_FLAG_ENV)
+			reason = "job environment";
+
+		if (reason) {
+			error("User %u is requesting %s, but no job requested, this is not allowed",
+			      user.uid, reason);
+			return NULL;
+		}
+	}
+
 	setup_job_cond_limits(job_cond, &extra);
 
 	xfree(tmp);
 	xstrfmtcat(tmp, "%s", job_req_inx[0]);
 	for (i = 1; i < JOB_REQ_COUNT; i++) {
-		xstrfmtcat(tmp, ", %s", job_req_inx[i]);
+		/* Only get the script if requesting it */
+		if (((i == JOB_REQ_SCRIPT) &&
+		     (!job_cond || !(job_cond->flags & JOBCOND_FLAG_SCRIPT))) ||
+		    ((i == JOB_REQ_ENV) &&
+		     (!job_cond || !(job_cond->flags & JOBCOND_FLAG_ENV))))
+			xstrcat(tmp, ", ''");
+		else
+			xstrfmtcat(tmp, ", %s", job_req_inx[i]);
 	}
 
 	xfree(tmp2);
@@ -1748,8 +1808,11 @@ extern List as_mysql_jobacct_process_get_jobs(mysql_conn_t *mysql_conn,
 	if (job_cond
 	    && job_cond->cluster_list && list_count(job_cond->cluster_list))
 		use_cluster_list = job_cond->cluster_list;
-	else
-		slurm_mutex_lock(&as_mysql_cluster_list_lock);
+	else {
+		slurm_rwlock_rdlock(&as_mysql_cluster_list_lock);
+		use_cluster_list = list_shallow_copy(as_mysql_cluster_list);
+		locked = true;
+	}
 
 	assoc_mgr_lock(&locks);
 
@@ -1769,8 +1832,10 @@ extern List as_mysql_jobacct_process_get_jobs(mysql_conn_t *mysql_conn,
 
 	assoc_mgr_unlock(&locks);
 
-	if (use_cluster_list == as_mysql_cluster_list)
-		slurm_mutex_unlock(&as_mysql_cluster_list_lock);
+	if (locked) {
+		FREE_NULL_LIST(use_cluster_list);
+		slurm_rwlock_unlock(&as_mysql_cluster_list_lock);
+	}
 
 	xfree(tmp);
 	xfree(tmp2);
