@@ -1,7 +1,6 @@
 /*****************************************************************************\
  *  as_mysql_qos.c - functions dealing with qos.
  *****************************************************************************
- *
  *  Copyright (C) 2004-2007 The Regents of the University of California.
  *  Copyright (C) 2008-2010 Lawrence Livermore National Security.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
@@ -126,6 +125,85 @@ static int _preemption_loop(mysql_conn_t *mysql_conn, int begin_qosid,
 	return rc;
 }
 
+static int _setup_qos_cond_limits(slurmdb_qos_cond_t *qos_cond, char **extra)
+{
+	int set = 0;
+	list_itr_t *itr = NULL;
+	char *object = NULL;
+
+	xassert(extra);
+	xassert(*extra);
+
+	if (!qos_cond)
+		return 0;
+
+	/*
+	 * Don't handle with_deleted here, we don't want to delete or modify
+	 * deleted things
+	 */
+	/* if (qos_cond->with_deleted) */
+	/* 	xstrcat(extra, "where (deleted=0 || deleted=1)"); */
+	/* else */
+	/* 	xstrcat(extra, "where deleted=0"); */
+
+	if (qos_cond->description_list &&
+	    list_count(qos_cond->description_list)) {
+		set = 0;
+		xstrcat(*extra, " && (");
+		itr = list_iterator_create(qos_cond->description_list);
+		while ((object = list_next(itr))) {
+			if (set)
+				xstrcat(*extra, " || ");
+			xstrfmtcat(*extra, "description='%s'", object);
+			set = 1;
+		}
+		list_iterator_destroy(itr);
+		xstrcat(*extra, ")");
+	}
+
+	if (qos_cond->id_list &&
+	    list_count(qos_cond->id_list)) {
+		set = 0;
+		xstrcat(*extra, " && (");
+		itr = list_iterator_create(qos_cond->id_list);
+		while ((object = list_next(itr))) {
+			if (set)
+				xstrcat(*extra, " || ");
+			xstrfmtcat(*extra, "id='%s'", object);
+			set = 1;
+		}
+		list_iterator_destroy(itr);
+		xstrcat(*extra, ")");
+	}
+
+	if (qos_cond->name_list &&
+	    list_count(qos_cond->name_list)) {
+		set = 0;
+		xstrcat(*extra, " && (");
+		itr = list_iterator_create(qos_cond->name_list);
+		while ((object = list_next(itr))) {
+			if (set)
+				xstrcat(*extra, " || ");
+			xstrfmtcat(*extra, "name='%s'", object);
+			set = 1;
+		}
+		list_iterator_destroy(itr);
+		xstrcat(*extra, ")");
+	}
+
+	if ((qos_cond->preempt_mode != NO_VAL16) && qos_cond->preempt_mode) {
+		set = 1;
+		xstrfmtcat(*extra, " && (preempt_mode&%d",
+			   qos_cond->preempt_mode);
+		if (qos_cond->preempt_mode & PREEMPT_MODE_COND_OFF)
+			xstrcat(*extra, " || preempt_mode=0");
+		xstrcat(*extra, ")");
+	}
+
+
+	return set;
+}
+
 static int _setup_qos_limits(slurmdb_qos_rec_t *qos,
 			     char **cols, char **vals,
 			     char **extra, char **added_preempt,
@@ -182,6 +260,8 @@ static int _setup_qos_limits(slurmdb_qos_rec_t *qos,
 			qos->usage_factor = 1;
 		if (fuzzy_equal(qos->usage_thres, NO_VAL))
 			qos->usage_thres = (double)INFINITE;
+		if (fuzzy_equal(qos->limit_factor, NO_VAL))
+			qos->limit_factor = INFINITE;
 	}
 
 	if (qos->description) {
@@ -367,7 +447,7 @@ static int _setup_qos_limits(slurmdb_qos_rec_t *qos,
 		char *preempt_val = NULL;
 		char *tmp_char = NULL, *last_preempt = NULL;
 		bool adding_straight = 0;
-		ListIterator preempt_itr =
+		list_itr_t *preempt_itr =
 			list_iterator_create(qos->preempt_list);
 
 		xstrcat(*cols, ", preempt");
@@ -432,9 +512,7 @@ static int _setup_qos_limits(slurmdb_qos_rec_t *qos,
 			   qos->preempt_exempt_time);
 	}
 
-	if ((qos->preempt_mode != NO_VAL16)
-	    && ((int16_t)qos->preempt_mode >= 0)) {
-		qos->preempt_mode &= (~PREEMPT_MODE_GANG);
+	if (qos->preempt_mode != NO_VAL16) {
 		xstrcat(*cols, ", preempt_mode");
 		xstrfmtcat(*vals, ", %u", qos->preempt_mode);
 		xstrfmtcat(*extra, ", preempt_mode=%u", qos->preempt_mode);
@@ -473,6 +551,17 @@ static int _setup_qos_limits(slurmdb_qos_rec_t *qos,
 		xstrfmtcat(*extra, ", usage_thres=%f", qos->usage_thres);
 	}
 
+	if (fuzzy_equal(qos->limit_factor, INFINITE)) {
+		xstrcat(*cols, ", limit_factor");
+		xstrcat(*vals, ", NULL");
+		xstrcat(*extra, ", limit_factor=NULL");
+	} else if (!fuzzy_equal(qos->limit_factor, NO_VAL)
+		   && (qos->limit_factor > 0)) {
+		xstrcat(*cols, ", limit_factor");
+		xstrfmtcat(*vals, ", %f", qos->limit_factor);
+		xstrfmtcat(*extra, ", limit_factor=%f", qos->limit_factor);
+	}
+
 	/* When modifying anything below this comment it happens in
 	 * the actual function since we have to wait until we hear
 	 * about the original first.
@@ -627,7 +716,7 @@ end_modify:
 extern int as_mysql_add_qos(mysql_conn_t *mysql_conn, uint32_t uid,
 			    List qos_list)
 {
-	ListIterator itr = NULL;
+	list_itr_t *itr = NULL;
 	int rc = SLURM_SUCCESS;
 	slurmdb_qos_rec_t *object = NULL;
 	char *cols = NULL, *extra = NULL, *vals = NULL, *query = NULL,
@@ -647,6 +736,11 @@ extern int as_mysql_add_qos(mysql_conn_t *mysql_conn, uint32_t uid,
 	if (!is_user_min_admin_level(mysql_conn, uid, SLURMDB_ADMIN_SUPER_USER))
 		return ESLURM_ACCESS_DENIED;
 
+	if (!qos_list || !list_count(qos_list)) {
+		error("%s: Trying to add empty qos list", __func__);
+		return ESLURM_EMPTY_LIST;
+	}
+
 	assoc_mgr_lock(&locks);
 	qos_cnt = g_qos_count;
 	assoc_mgr_unlock(&locks);
@@ -680,8 +774,7 @@ extern int as_mysql_add_qos(mysql_conn_t *mysql_conn, uint32_t uid,
 			   qos_table, cols, vals, extra);
 
 
-		if (debug_flags & DEBUG_FLAG_DB_QOS)
-			DB_DEBUG(mysql_conn->conn, "query\n%s", query);
+		DB_DEBUG(DB_QOS, mysql_conn->conn, "query\n%s", query);
 		object->id = (uint32_t)mysql_db_insert_ret_id(
 			mysql_conn, query);
 		xfree(query);
@@ -747,14 +840,13 @@ extern List as_mysql_modify_qos(mysql_conn_t *mysql_conn, uint32_t uid,
 				slurmdb_qos_cond_t *qos_cond,
 				slurmdb_qos_rec_t *qos)
 {
-	ListIterator itr = NULL;
 	List ret_list = NULL;
 	int rc = SLURM_SUCCESS;
 	char *object = NULL;
 	char *vals = NULL, *extra = NULL, *query = NULL, *name_char = NULL;
 	time_t now = time(NULL);
 	char *user_name = NULL;
-	int set = 0, i;
+	int i;
 	MYSQL_RES *result = NULL;
 	MYSQL_ROW row;
 	char *tmp_char1=NULL, *tmp_char2=NULL;
@@ -780,50 +872,7 @@ extern List as_mysql_modify_qos(mysql_conn_t *mysql_conn, uint32_t uid,
 
 	xstrcat(extra, "where deleted=0");
 
-	if (qos_cond->description_list
-	    && list_count(qos_cond->description_list)) {
-		set = 0;
-		xstrcat(extra, " && (");
-		itr = list_iterator_create(qos_cond->description_list);
-		while ((object = list_next(itr))) {
-			if (set)
-				xstrcat(extra, " || ");
-			xstrfmtcat(extra, "description='%s'", object);
-			set = 1;
-		}
-		list_iterator_destroy(itr);
-		xstrcat(extra, ")");
-	}
-
-	if (qos_cond->id_list
-	    && list_count(qos_cond->id_list)) {
-		set = 0;
-		xstrcat(extra, " && (");
-		itr = list_iterator_create(qos_cond->id_list);
-		while ((object = list_next(itr))) {
-			if (set)
-				xstrcat(extra, " || ");
-			xstrfmtcat(extra, "id='%s'", object);
-			set = 1;
-		}
-		list_iterator_destroy(itr);
-		xstrcat(extra, ")");
-	}
-
-	if (qos_cond->name_list
-	    && list_count(qos_cond->name_list)) {
-		set = 0;
-		xstrcat(extra, " && (");
-		itr = list_iterator_create(qos_cond->name_list);
-		while ((object = list_next(itr))) {
-			if (set)
-				xstrcat(extra, " || ");
-			xstrfmtcat(extra, "name='%s'", object);
-			set = 1;
-		}
-		list_iterator_destroy(itr);
-		xstrcat(extra, ")");
-	}
+	_setup_qos_cond_limits(qos_cond, &extra);
 
 	_setup_qos_limits(qos, &tmp_char1, &tmp_char2,
 			  &vals, &added_preempt, 0);
@@ -918,7 +967,7 @@ extern List as_mysql_modify_qos(mysql_conn_t *mysql_conn, uint32_t uid,
 			     qos->max_tres_mins_pj, row[MQOS_MTMPJ],
 			     NULL, "max_tres_mins_pj", &vals, qos_rec->id, 0);
 		mod_tres_str(&qos_rec->max_tres_run_mins_pa,
-			     qos->max_tres_run_mins_pa, row[MQOS_MTRM],
+			     qos->max_tres_run_mins_pa, row[MQOS_MTRMA],
 			     NULL, "max_tres_run_mins_pa", &vals,
 			     qos_rec->id, 0);
 		mod_tres_str(&qos_rec->max_tres_run_mins_pu,
@@ -943,7 +992,7 @@ extern List as_mysql_modify_qos(mysql_conn_t *mysql_conn, uint32_t uid,
 		qos_rec->priority = qos->priority;
 
 		if (qos->preempt_list) {
-			ListIterator new_preempt_itr =
+			list_itr_t *new_preempt_itr =
 				list_iterator_create(qos->preempt_list);
 			char *new_preempt = NULL;
 			bool cleared = 0;
@@ -964,10 +1013,8 @@ extern List as_mysql_modify_qos(mysql_conn_t *mysql_conn, uint32_t uid,
 				} else {
 					if (!cleared) {
 						cleared = 1;
-						bit_nclear(
-							qos_rec->preempt_bitstr,
-							0,
-							qos_cnt-1);
+						bit_clear_all(
+							qos_rec->preempt_bitstr);
 					}
 
 					bit_set(qos_rec->preempt_bitstr,
@@ -981,6 +1028,7 @@ extern List as_mysql_modify_qos(mysql_conn_t *mysql_conn, uint32_t uid,
 
 		qos_rec->usage_factor = qos->usage_factor;
 		qos_rec->usage_thres = qos->usage_thres;
+		qos_rec->limit_factor = qos->limit_factor;
 
 		if (addto_update_list(mysql_conn->update_list,
 				      SLURMDB_MODIFY_QOS, qos_rec)
@@ -1003,9 +1051,8 @@ extern List as_mysql_modify_qos(mysql_conn_t *mysql_conn, uint32_t uid,
 
 	if (!list_count(ret_list)) {
 		errno = SLURM_NO_CHANGE_IN_DATA;
-		if (debug_flags & DEBUG_FLAG_DB_QOS)
-			DB_DEBUG(mysql_conn->conn,
-				 "didn't effect anything\n%s", query);
+		DB_DEBUG(DB_QOS, mysql_conn->conn,
+		         "didn't affect anything\n%s", query);
 		xfree(vals);
 		xfree(query);
 		return ret_list;
@@ -1031,7 +1078,7 @@ extern List as_mysql_modify_qos(mysql_conn_t *mysql_conn, uint32_t uid,
 extern List as_mysql_remove_qos(mysql_conn_t *mysql_conn, uint32_t uid,
 				slurmdb_qos_cond_t *qos_cond)
 {
-	ListIterator itr = NULL;
+	list_itr_t *itr = NULL;
 	List ret_list = NULL;
 	int rc = SLURM_SUCCESS;
 	char *object = NULL;
@@ -1039,9 +1086,9 @@ extern List as_mysql_remove_qos(mysql_conn_t *mysql_conn, uint32_t uid,
 		*name_char = NULL, *assoc_char = NULL;
 	time_t now = time(NULL);
 	char *user_name = NULL;
-	int set = 0;
 	MYSQL_RES *result = NULL;
 	MYSQL_ROW row;
+	List cluster_list_tmp = NULL;
 
 	if (!qos_cond) {
 		error("we need something to change");
@@ -1058,54 +1105,8 @@ extern List as_mysql_remove_qos(mysql_conn_t *mysql_conn, uint32_t uid,
 	}
 
 	xstrcat(extra, "where deleted=0");
-	if (qos_cond->description_list
-	    && list_count(qos_cond->description_list)) {
-		set = 0;
-		xstrcat(extra, " && (");
-		itr = list_iterator_create(qos_cond->description_list);
-		while ((object = list_next(itr))) {
-			if (set)
-				xstrcat(extra, " || ");
-			xstrfmtcat(extra, "description='%s'", object);
-			set = 1;
-		}
-		list_iterator_destroy(itr);
-		xstrcat(extra, ")");
-	}
 
-	if (qos_cond->id_list
-	    && list_count(qos_cond->id_list)) {
-		set = 0;
-		xstrcat(extra, " && (");
-		itr = list_iterator_create(qos_cond->id_list);
-		while ((object = list_next(itr))) {
-			if (!object[0])
-				continue;
-			if (set)
-				xstrcat(extra, " || ");
-			xstrfmtcat(extra, "id='%s'", object);
-			set = 1;
-		}
-		list_iterator_destroy(itr);
-		xstrcat(extra, ")");
-	}
-
-	if (qos_cond->name_list
-	    && list_count(qos_cond->name_list)) {
-		set = 0;
-		xstrcat(extra, " && (");
-		itr = list_iterator_create(qos_cond->name_list);
-		while ((object = list_next(itr))) {
-			if (!object[0])
-				continue;
-			if (set)
-				xstrcat(extra, " || ");
-			xstrfmtcat(extra, "name='%s'", object);
-			set = 1;
-		}
-		list_iterator_destroy(itr);
-		xstrcat(extra, ")");
-	}
+	_setup_qos_cond_limits(qos_cond, &extra);
 
 	if (!extra) {
 		error("Nothing to remove");
@@ -1135,10 +1136,10 @@ extern List as_mysql_remove_qos(mysql_conn_t *mysql_conn, uint32_t uid,
 		else
 			xstrfmtcat(assoc_char, " || id_qos='%s'", row[0]);
 		xstrfmtcat(extra,
-			   ", qos=replace(qos, ',%s,', '')"
-			   ", delta_qos=replace(delta_qos, ',+%s,', '')"
-			   ", delta_qos=replace(delta_qos, ',-%s,', '')",
-			   row[0], row[0], row[0]);
+			   ", qos=replace(qos, ',%s,', if(qos=',%s,', '', ','))"
+			   ", delta_qos=replace(delta_qos, ',+%s,', if(delta_qos=',+%s,', '', ','))"
+			   ", delta_qos=replace(delta_qos, ',-%s,', if(delta_qos=',-%s,', '', ','))",
+			   row[0], row[0], row[0], row[0], row[0], row[0]);
 
 		qos_rec = xmalloc(sizeof(slurmdb_qos_rec_t));
 		/* we only need id when removing no real need to init */
@@ -1152,8 +1153,8 @@ extern List as_mysql_remove_qos(mysql_conn_t *mysql_conn, uint32_t uid,
 
 	if (!list_count(ret_list)) {
 		errno = SLURM_NO_CHANGE_IN_DATA;
-		if (debug_flags & DEBUG_FLAG_DB_QOS)
-			DB_DEBUG(mysql_conn->conn, "didn't effect anything\n%s", query);
+		DB_DEBUG(DB_QOS, mysql_conn->conn,
+		         "didn't affect anything\n%s", query);
 		xfree(query);
 		return ret_list;
 	}
@@ -1161,9 +1162,10 @@ extern List as_mysql_remove_qos(mysql_conn_t *mysql_conn, uint32_t uid,
 
 	user_name = uid_to_string((uid_t) uid);
 
-	slurm_mutex_lock(&as_mysql_cluster_list_lock);
-	if (list_count(as_mysql_cluster_list)) {
-		itr = list_iterator_create(as_mysql_cluster_list);
+	slurm_rwlock_rdlock(&as_mysql_cluster_list_lock);
+	cluster_list_tmp = list_shallow_copy(as_mysql_cluster_list);
+	if (list_count(cluster_list_tmp)) {
+		itr = list_iterator_create(cluster_list_tmp);
 		while ((object = list_next(itr))) {
 			/*
 			 * remove this qos from all the associations
@@ -1172,8 +1174,7 @@ extern List as_mysql_remove_qos(mysql_conn_t *mysql_conn, uint32_t uid,
 			query = xstrdup_printf("update \"%s_%s\" set mod_time=%ld %s where deleted=0;",
 					       object, assoc_table,
 					       now, extra);
-			if (debug_flags & DEBUG_FLAG_DB_QOS)
-				DB_DEBUG(mysql_conn->conn, "query\n%s", query);
+			DB_DEBUG(DB_QOS, mysql_conn->conn, "query\n%s", query);
 			rc = mysql_db_query(mysql_conn, query);
 			xfree(query);
 
@@ -1184,7 +1185,8 @@ extern List as_mysql_remove_qos(mysql_conn_t *mysql_conn, uint32_t uid,
 
 			if ((rc = remove_common(mysql_conn, DBD_REMOVE_QOS, now,
 						user_name, qos_table, name_char,
-						assoc_char, object, NULL, NULL))
+						assoc_char, object, NULL, NULL,
+						NULL))
 			    != SLURM_SUCCESS)
 				break;
 		}
@@ -1192,9 +1194,10 @@ extern List as_mysql_remove_qos(mysql_conn_t *mysql_conn, uint32_t uid,
 	} else
 		rc = remove_common(mysql_conn, DBD_REMOVE_QOS, now,
 				   user_name, qos_table, name_char,
-				   assoc_char, NULL, NULL, NULL);
+				   assoc_char, NULL, NULL, NULL, NULL);
 
-	slurm_mutex_unlock(&as_mysql_cluster_list_lock);
+	FREE_NULL_LIST(cluster_list_tmp);
+	slurm_rwlock_unlock(&as_mysql_cluster_list_lock);
 
 	xfree(extra);
 	xfree(assoc_char);
@@ -1215,9 +1218,6 @@ extern List as_mysql_get_qos(mysql_conn_t *mysql_conn, uid_t uid,
 	char *extra = NULL;
 	char *tmp = NULL;
 	List qos_list = NULL;
-	ListIterator itr = NULL;
-	char *object = NULL;
-	int set = 0;
 	int i=0;
 	MYSQL_RES *result = NULL;
 	MYSQL_ROW row;
@@ -1261,6 +1261,7 @@ extern List as_mysql_get_qos(mysql_conn_t *mysql_conn, uid_t uid,
 		"usage_factor",
 		"usage_thres",
 		"min_tres_pj",
+		"limit_factor",
 	};
 	enum {
 		QOS_REQ_NAME,
@@ -1297,6 +1298,7 @@ extern List as_mysql_get_qos(mysql_conn_t *mysql_conn, uid_t uid,
 		QOS_REQ_UF,
 		QOS_REQ_UT,
 		QOS_REQ_MITPJ,
+		QOS_REQ_LF,
 		QOS_REQ_COUNT
 	};
 
@@ -1313,51 +1315,7 @@ extern List as_mysql_get_qos(mysql_conn_t *mysql_conn, uid_t uid,
 	else
 		xstrcat(extra, "where deleted=0");
 
-
-	if (qos_cond->description_list
-	    && list_count(qos_cond->description_list)) {
-		set = 0;
-		xstrcat(extra, " && (");
-		itr = list_iterator_create(qos_cond->description_list);
-		while ((object = list_next(itr))) {
-			if (set)
-				xstrcat(extra, " || ");
-			xstrfmtcat(extra, "description='%s'", object);
-			set = 1;
-		}
-		list_iterator_destroy(itr);
-		xstrcat(extra, ")");
-	}
-
-	if (qos_cond->id_list
-	    && list_count(qos_cond->id_list)) {
-		set = 0;
-		xstrcat(extra, " && (");
-		itr = list_iterator_create(qos_cond->id_list);
-		while ((object = list_next(itr))) {
-			if (set)
-				xstrcat(extra, " || ");
-			xstrfmtcat(extra, "id='%s'", object);
-			set = 1;
-		}
-		list_iterator_destroy(itr);
-		xstrcat(extra, ")");
-	}
-
-	if (qos_cond->name_list
-	    && list_count(qos_cond->name_list)) {
-		set = 0;
-		xstrcat(extra, " && (");
-		itr = list_iterator_create(qos_cond->name_list);
-		while ((object = list_next(itr))) {
-			if (set)
-				xstrcat(extra, " || ");
-			xstrfmtcat(extra, "name='%s'", object);
-			set = 1;
-		}
-		list_iterator_destroy(itr);
-		xstrcat(extra, ")");
-	}
+	_setup_qos_cond_limits(qos_cond, &extra);
 
 empty:
 
@@ -1371,8 +1329,7 @@ empty:
 	xfree(tmp);
 	xfree(extra);
 
-	if (debug_flags & DEBUG_FLAG_DB_QOS)
-		DB_DEBUG(mysql_conn->conn, "query\n%s", query);
+	DB_DEBUG(DB_QOS, mysql_conn->conn, "query\n%s", query);
 	if (!(result = mysql_db_query_ret(
 		      mysql_conn, query, 0))) {
 		xfree(query);
@@ -1517,6 +1474,11 @@ empty:
 
 		if (row[QOS_REQ_MITPJ][0])
 			qos->min_tres_pj = xstrdup(row[QOS_REQ_MITPJ]);
+
+		if (row[QOS_REQ_LF])
+			qos->limit_factor = atof(row[QOS_REQ_LF]);
+		else
+			qos->limit_factor = (double)INFINITE;
 	}
 	mysql_free_result(result);
 
