@@ -1,7 +1,6 @@
 /*****************************************************************************\
  *  as_mysql_wckey.c - functions dealing with the wckey.
  *****************************************************************************
- *
  *  Copyright (C) 2004-2007 The Regents of the University of California.
  *  Copyright (C) 2008-2010 Lawrence Livermore National Security.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
@@ -40,12 +39,30 @@
 #include "as_mysql_wckey.h"
 #include "as_mysql_usage.h"
 
+typedef struct {
+	char *cluster_name;
+	char *default_wckey;
+	mysql_conn_t *mysql_conn;
+	time_t now;
+	int rc;
+	char *ret_str;
+	bool ret_str_err;
+	char *ret_str_pos;
+	char *txn_query;
+	char *txn_query_pos;
+	list_t *user_list;
+	char *user_name;
+	list_t *wckey_list;
+	char *wckey_user_name;
+} add_wckey_cond_t;
+
 /* if this changes you will need to edit the corresponding enum */
 char *wckey_req_inx[] = {
 	"id_wckey",
 	"is_def",
 	"wckey_name",
 	"user",
+	"deleted",
 };
 
 enum {
@@ -53,6 +70,7 @@ enum {
 	WCKEY_REQ_DEFAULT,
 	WCKEY_REQ_NAME,
 	WCKEY_REQ_USER,
+	WCKEY_REQ_DELETED,
 	WCKEY_REQ_COUNT
 };
 
@@ -77,8 +95,7 @@ static int _reset_default_wckey(mysql_conn_t *mysql_conn,
 		   wckey->user, wckey->name,
 		   wckey->cluster, wckey_table,
 		   wckey->user, wckey->name);
-	if (debug_flags & DEBUG_FLAG_DB_WCKEY)
-		DB_DEBUG(mysql_conn->conn, "query\n%s", query);
+	DB_DEBUG(DB_WCKEY, mysql_conn->conn, "query\n%s", query);
 	if (!(result = mysql_db_query_ret(mysql_conn, query, 1))) {
 		xfree(query);
 		rc = SLURM_ERROR;
@@ -109,80 +126,87 @@ end_it:
 	return rc;
 }
 
+static int _make_sure_user_has_default_internal(
+	mysql_conn_t *mysql_conn, char *user, char *cluster)
+{
+	MYSQL_RES *result = NULL;
+	MYSQL_ROW row;
+	char *query;
+	int rc = SLURM_SUCCESS;
+
+	/* only look at non * and non deleted ones */
+	query = xstrdup_printf(
+		"select distinct is_def, wckey_name from "
+		"\"%s_%s\" where user='%s' and wckey_name "
+		"not like '*%%' and deleted=0 ORDER BY "
+		"is_def desc, creation_time desc LIMIT 1;",
+		cluster, wckey_table, user);
+	debug4("%d(%s:%d) query\n%s",
+	       mysql_conn->conn, THIS_FILE, __LINE__, query);
+	if (!(result = mysql_db_query_ret(mysql_conn, query, 0))) {
+		xfree(query);
+		error("couldn't query the database");
+		return SLURM_ERROR;
+	}
+	xfree(query);
+	/* Check to see if the user is even added to
+	   the cluster.
+	*/
+	if (!mysql_num_rows(result)) {
+		mysql_free_result(result);
+		return SLURM_SUCCESS;
+	}
+
+	/* check if row is default */
+	row = mysql_fetch_row(result);
+	if (row[0][0] == '1') {
+		/* default found, continue */
+		mysql_free_result(result);
+		return SLURM_SUCCESS;
+	}
+
+	/* if we made it here, there is no default */
+	query = xstrdup_printf(
+		"update \"%s_%s\" set is_def=1 where "
+		"user='%s' and wckey_name='%s';",
+		cluster, wckey_table, user, row[1]);
+	mysql_free_result(result);
+
+	DB_DEBUG(DB_WCKEY, mysql_conn->conn, "query\n%s",
+		 query);
+	rc = mysql_db_query(mysql_conn, query);
+	xfree(query);
+	if (rc != SLURM_SUCCESS) {
+		error("problem with update query");
+		return rc;
+	}
+
+	return rc;
+}
+
 /* This needs to happen to make since 2.1 code doesn't have enough
  * smarts to figure out it isn't adding a default wckey if just
  * adding a new wckey for a user that has never been on the cluster before.
  */
 static int _make_sure_users_have_default(
-	mysql_conn_t *mysql_conn, List user_list)
+	mysql_conn_t *mysql_conn, List user_list, List cluster_list)
 {
-	char *query = NULL, *cluster = NULL, *user = NULL;
-	ListIterator itr = NULL, clus_itr = NULL;
+	char *cluster = NULL, *user = NULL;
+	list_itr_t *itr = NULL, *clus_itr = NULL;
 	int rc = SLURM_SUCCESS;
 
 	if (!user_list)
 		return SLURM_SUCCESS;
 
-	slurm_mutex_lock(&as_mysql_cluster_list_lock);
-
-	clus_itr = list_iterator_create(as_mysql_cluster_list);
+	clus_itr = list_iterator_create(cluster_list);
 	itr = list_iterator_create(user_list);
 
 	while ((user = list_next(itr))) {
 		while ((cluster = list_next(clus_itr))) {
-			MYSQL_RES *result = NULL;
-			MYSQL_ROW row;
-			char *wckey = NULL;
-			/* only look at non * and non deleted ones */
-			query = xstrdup_printf(
-				"select distinct is_def, wckey_name from "
-				"\"%s_%s\" where user='%s' and wckey_name "
-				"not like '*%%' and deleted=0 FOR UPDATE;",
-				cluster, wckey_table, user);
-			debug4("%d(%s:%d) query\n%s",
-			       mysql_conn->conn, THIS_FILE, __LINE__, query);
-			if (!(result = mysql_db_query_ret(
-				      mysql_conn, query, 0))) {
-				xfree(query);
-				error("couldn't query the database");
-				rc = SLURM_ERROR;
+			if ((rc = _make_sure_user_has_default_internal(
+				     mysql_conn, user, cluster)) !=
+			    SLURM_SUCCESS)
 				break;
-			}
-			xfree(query);
-			/* Check to see if the user is even added to
-			   the cluster.
-			*/
-			if (!mysql_num_rows(result)) {
-				mysql_free_result(result);
-				continue;
-			}
-			while ((row = mysql_fetch_row(result))) {
-				if (row[0][0] == '1')
-					break;
-				if (!wckey)
-					wckey = xstrdup(row[1]);
-			}
-			mysql_free_result(result);
-
-			/* we found one so just continue */
-			if (row || !wckey) {
-				xfree(wckey);
-				continue;
-			}
-			query = xstrdup_printf(
-				"update \"%s_%s\" set is_def=1 where "
-				"user='%s' and wckey_name='%s';",
-				cluster, wckey_table, user, wckey);
-			xfree(wckey);
-			if (debug_flags & DEBUG_FLAG_DB_WCKEY)
-				DB_DEBUG(mysql_conn->conn, "query\n%s", query);
-			rc = mysql_db_query(mysql_conn, query);
-			xfree(query);
-			if (rc != SLURM_SUCCESS) {
-				error("problem with update query");
-				rc = SLURM_ERROR;
-				break;
-			}
 		}
 		if (rc != SLURM_SUCCESS)
 			break;
@@ -190,7 +214,6 @@ static int _make_sure_users_have_default(
 	}
 	list_iterator_destroy(itr);
 	list_iterator_destroy(clus_itr);
-	slurm_mutex_unlock(&as_mysql_cluster_list_lock);
 
 	return rc;
 }
@@ -201,7 +224,7 @@ static int _setup_wckey_cond_limits(slurmdb_wckey_cond_t *wckey_cond,
 				    char **extra)
 {
 	int set = 0;
-	ListIterator itr = NULL;
+	list_itr_t *itr = NULL;
 	char *object = NULL;
 	char *prefix = "t1";
 	if (!wckey_cond)
@@ -314,9 +337,8 @@ static int _cluster_remove_wckeys(mysql_conn_t *mysql_conn,
 
 	if (!list_count(ret_list)) {
 		errno = SLURM_NO_CHANGE_IN_DATA;
-		if (debug_flags & DEBUG_FLAG_DB_WCKEY)
-			DB_DEBUG(mysql_conn->conn,
-				 "didn't effect anything\n%s", query);
+		DB_DEBUG(DB_WCKEY, mysql_conn->conn,
+		         "didn't affect anything\n%s", query);
 		xfree(query);
 		xfree(assoc_char);
 		return SLURM_SUCCESS;
@@ -325,7 +347,7 @@ static int _cluster_remove_wckeys(mysql_conn_t *mysql_conn,
 	xfree(query);
 	rc = remove_common(mysql_conn, DBD_REMOVE_WCKEYS, now,
 			   user_name, wckey_table, assoc_char, assoc_char,
-			   cluster_name, NULL, NULL);
+			   cluster_name, NULL, NULL, NULL);
 	xfree(assoc_char);
 
 	if (rc == SLURM_ERROR) {
@@ -402,9 +424,8 @@ static int _cluster_modify_wckeys(mysql_conn_t *mysql_conn,
 
 	if (!list_count(ret_list)) {
 		errno = SLURM_NO_CHANGE_IN_DATA;
-		if (debug_flags & DEBUG_FLAG_DB_WCKEY)
-			DB_DEBUG(mysql_conn->conn,
-				 "didn't effect anything\n%s", query);
+		DB_DEBUG(DB_WCKEY, mysql_conn->conn,
+		         "didn't affect anything\n%s", query);
 		xfree(query);
 		xfree(wckey_char);
 		return SLURM_SUCCESS;
@@ -439,8 +460,7 @@ static int _cluster_get_wckeys(mysql_conn_t *mysql_conn,
 		   "order by wckey_name, user;",
 		   fields, cluster_name, wckey_table, extra);
 
-	if (debug_flags & DEBUG_FLAG_DB_WCKEY)
-		DB_DEBUG(mysql_conn->conn, "query\n%s", query);
+	DB_DEBUG(DB_WCKEY, mysql_conn->conn, "query\n%s", query);
 	if (!(result = mysql_db_query_ret(
 		      mysql_conn, query, 0))) {
 		xfree(query);
@@ -467,6 +487,9 @@ static int _cluster_get_wckeys(mysql_conn_t *mysql_conn,
 		wckey->is_def = slurm_atoul(row[WCKEY_REQ_DEFAULT]);
 		wckey->user = xstrdup(row[WCKEY_REQ_USER]);
 
+		if (slurm_atoul(row[WCKEY_REQ_DELETED]))
+			wckey->flags |= SLURMDB_WCKEY_FLAG_DELETED;
+
 		/* we want a blank wckey if the name is null */
 		if (row[WCKEY_REQ_NAME])
 			wckey->name = xstrdup(row[WCKEY_REQ_NAME]);
@@ -487,12 +510,164 @@ static int _cluster_get_wckeys(mysql_conn_t *mysql_conn,
 	return SLURM_SUCCESS;
 }
 
+static int _add_wckey_cond_wckey(void *x, void *arg)
+{
+	add_wckey_cond_t *add_wckey_cond = arg;
+	char *extra, *tmp_extra;
+	int rc;
+	char *query;
+	slurmdb_wckey_rec_t *object, check_object;
+
+	/* Check to see if it is already in the assoc_mgr */
+	memset(&check_object, 0, sizeof(check_object));
+	check_object.cluster = add_wckey_cond->cluster_name;
+	check_object.name = x;
+	check_object.user = add_wckey_cond->wckey_user_name;
+	check_object.uid = NO_VAL;
+
+	rc = assoc_mgr_fill_in_wckey(add_wckey_cond->mysql_conn,
+				     &check_object,
+				     ACCOUNTING_ENFORCE_WCKEYS, NULL, false);
+	if (rc == SLURM_SUCCESS) {
+		debug("WCKey %s/%s/%s is already here, not adding again.",
+		      check_object.cluster, check_object.name,
+		      check_object.user);
+		return 0;
+	}
+
+	if (!xstrcmp(add_wckey_cond->default_wckey, check_object.name)) {
+		check_object.is_def = true;
+		if ((add_wckey_cond->rc = _reset_default_wckey(
+			     add_wckey_cond->mysql_conn, &check_object) !=
+		     SLURM_SUCCESS)) {
+			add_wckey_cond->ret_str_err = true;
+			xfree(add_wckey_cond->ret_str);
+			add_wckey_cond->ret_str = xstrdup_printf(
+				"Problem resetting old default wckeys for C = %s W = %s U = %s",
+				check_object.cluster, check_object.name,
+				check_object.user);
+			error("%s", add_wckey_cond->ret_str);
+			return -1;
+		}
+	}
+
+	/* Else, add it */
+	object = xmalloc(sizeof(*object));
+	object->cluster = xstrdup(check_object.cluster);
+	object->name = xstrdup(check_object.name);
+	object->user = xstrdup(check_object.user);
+	object->is_def = check_object.is_def;
+
+	query = xstrdup_printf(
+		"insert into \"%s_%s\" (creation_time, mod_time, wckey_name, user, is_def) values (%ld, %ld, '%s', '%s', %d) on duplicate key update deleted=0, id_wckey=LAST_INSERT_ID(id_wckey), is_def=VALUES(is_def), mod_time=VALUES(mod_time);",
+		object->cluster, wckey_table,
+		add_wckey_cond->now, add_wckey_cond->now,
+		object->name, object->user, object->is_def);
+
+	DB_DEBUG(DB_WCKEY, add_wckey_cond->mysql_conn->conn,
+		 "query\n%s", query);
+	object->id = (uint32_t) mysql_db_insert_ret_id(
+		add_wckey_cond->mysql_conn, query);
+	xfree(query);
+	if (!object->id) {
+		add_wckey_cond->rc = SLURM_ERROR;
+		add_wckey_cond->ret_str_err = true;
+		xfree(add_wckey_cond->ret_str);
+		add_wckey_cond->ret_str = xstrdup_printf(
+			"Couldn't add wckey C = %s W = %s U = %s\n",
+			object->cluster, object->name, object->user);
+		slurmdb_destroy_wckey_rec(object);
+		error("%s", add_wckey_cond->ret_str);
+		return -1;
+	}
+
+	extra = xstrdup_printf("mod_time=%ld, wckey_name='%s', user='%s', is_def=%d",
+			       add_wckey_cond->now,
+			       object->name, object->user, object->is_def);
+	tmp_extra = slurm_add_slash_to_quotes(extra);
+
+	if (!add_wckey_cond->txn_query)
+		xstrfmtcatat(add_wckey_cond->txn_query,
+			     &add_wckey_cond->txn_query_pos,
+			     "insert into %s (timestamp, action, name, actor, info, cluster) values ",
+			     txn_table);
+	else
+		xstrcatat(add_wckey_cond->txn_query,
+			  &add_wckey_cond->txn_query_pos,
+			  ", ");
+	xstrfmtcatat(add_wckey_cond->txn_query,
+		     &add_wckey_cond->txn_query_pos,
+		     "(%ld, %u, 'id_wckey=%u', '%s', '%s', '%s')",
+		     add_wckey_cond->now, DBD_ADD_WCKEYS,
+		     object->id, add_wckey_cond->user_name,
+		     tmp_extra, object->cluster);
+	xfree(tmp_extra);
+	xfree(extra);
+
+	if (addto_update_list(add_wckey_cond->mysql_conn->update_list,
+			      SLURMDB_ADD_WCKEY,
+			      object) == SLURM_SUCCESS) {
+		if (!add_wckey_cond->ret_str)
+			xstrcatat(add_wckey_cond->ret_str,
+				  &add_wckey_cond->ret_str_pos,
+				  " Wckey(s)\n");
+
+		xstrfmtcatat(add_wckey_cond->ret_str,
+			     &add_wckey_cond->ret_str_pos,
+			     "  C = %-10.10s W = %-10.10s U = %-9.9s\n",
+			     object->cluster, object->name, object->user);
+		object = NULL;
+	}
+
+	slurmdb_destroy_wckey_rec(object);
+
+	return 0;
+}
+
+static int _add_wckey_cond_user(void *x, void *arg)
+{
+	add_wckey_cond_t *add_wckey_cond = arg;
+	int rc;
+
+	add_wckey_cond->wckey_user_name = x;
+
+	rc = list_for_each_ro(add_wckey_cond->wckey_list,
+			      _add_wckey_cond_wckey,
+			      add_wckey_cond);
+	if (add_wckey_cond->rc == SLURM_SUCCESS) {
+		add_wckey_cond->rc = _make_sure_user_has_default_internal(
+			add_wckey_cond->mysql_conn, x,
+			add_wckey_cond->cluster_name);
+		if (add_wckey_cond->rc != SLURM_SUCCESS)
+			rc = -1;
+	}
+	add_wckey_cond->wckey_user_name = NULL;
+
+	return rc;
+}
+
+static int _add_wckey_cond_cluster(void *x, void *arg)
+{
+	add_wckey_cond_t *add_wckey_cond = arg;
+	int rc;
+
+	add_wckey_cond->cluster_name = x;
+
+	rc = list_for_each_ro(add_wckey_cond->user_list,
+			      _add_wckey_cond_user,
+			      add_wckey_cond);
+
+	add_wckey_cond->cluster_name = NULL;
+
+	return rc;
+}
+
 /* extern functions */
 
 extern int as_mysql_add_wckeys(mysql_conn_t *mysql_conn, uint32_t uid,
 			       List wckey_list)
 {
-	ListIterator itr = NULL;
+	list_itr_t *itr = NULL;
 	int rc = SLURM_SUCCESS;
 	slurmdb_wckey_rec_t *object = NULL;
 	char *cols = NULL, *extra = NULL, *vals = NULL, *query = NULL,
@@ -501,6 +676,7 @@ extern int as_mysql_add_wckeys(mysql_conn_t *mysql_conn, uint32_t uid,
 	char *user_name = NULL;
 	int affect_rows = 0;
 	int added = 0;
+	List local_cluster_list = NULL;
 	List added_user_list = NULL;
 
 	if (check_connection(mysql_conn) != SLURM_SUCCESS)
@@ -509,6 +685,13 @@ extern int as_mysql_add_wckeys(mysql_conn_t *mysql_conn, uint32_t uid,
 	if (!is_user_min_admin_level(mysql_conn, uid, SLURMDB_ADMIN_OPERATOR))
 		return ESLURM_ACCESS_DENIED;
 
+	if (!wckey_list || !list_count(wckey_list)) {
+		error("%s: Trying to add empty wckey list", __func__);
+		return ESLURM_EMPTY_LIST;
+	}
+
+	local_cluster_list = list_create(NULL);
+
 	user_name = uid_to_string((uid_t) uid);
 	itr = list_iterator_create(wckey_list);
 	while ((object = list_next(itr))) {
@@ -524,7 +707,10 @@ extern int as_mysql_add_wckeys(mysql_conn_t *mysql_conn, uint32_t uid,
 
 		if (!added_user_list)
 			added_user_list = list_create(NULL);
-		list_append(added_user_list, object->user);
+		if (!list_find_first(added_user_list,
+				     slurm_find_char_in_list,
+				     object->user))
+			list_append(added_user_list, object->user);
 		xstrcat(cols, "creation_time, mod_time, user");
 		xstrfmtcat(vals, "%ld, %ld, '%s'",
 			   now, now, object->user);
@@ -558,8 +744,7 @@ extern int as_mysql_add_wckeys(mysql_conn_t *mysql_conn, uint32_t uid,
 			   "id_wckey=LAST_INSERT_ID(id_wckey)%s;",
 			   object->cluster, wckey_table, cols, vals, extra);
 
-		if (debug_flags & DEBUG_FLAG_DB_WCKEY)
-			DB_DEBUG(mysql_conn->conn, "query\n%s", query);
+		DB_DEBUG(DB_WCKEY, mysql_conn->conn, "query\n%s", query);
 		object->id = (uint32_t)mysql_db_insert_ret_id(
 			mysql_conn, query);
 		xfree(query);
@@ -582,6 +767,11 @@ extern int as_mysql_add_wckeys(mysql_conn_t *mysql_conn, uint32_t uid,
 			continue;
 		}
 
+		if (!list_find_first(local_cluster_list,
+				     slurm_find_char_in_list,
+				     object->cluster))
+			list_append(local_cluster_list, object->cluster);
+
 		/* we always have a ', ' as the first 2 chars */
 		tmp_extra = slurm_add_slash_to_quotes(extra+2);
 
@@ -632,12 +822,94 @@ extern int as_mysql_add_wckeys(mysql_conn_t *mysql_conn, uint32_t uid,
 	list_iterator_destroy(itr);
 end_it:
 	if (rc == SLURM_SUCCESS)
-		_make_sure_users_have_default(mysql_conn, added_user_list);
+		_make_sure_users_have_default(mysql_conn, added_user_list,
+					      local_cluster_list);
 	FREE_NULL_LIST(added_user_list);
+	FREE_NULL_LIST(local_cluster_list);
 
 	return rc;
 }
 
+extern char *as_mysql_add_wckeys_cond(mysql_conn_t *mysql_conn, uint32_t uid,
+				      slurmdb_add_assoc_cond_t *add_assoc,
+				      slurmdb_user_rec_t *user)
+{
+	add_wckey_cond_t add_wckey_cond;
+	int rc;
+	list_t *use_cluster_list;
+
+	if (!add_assoc->wckey_list || !list_count(add_assoc->wckey_list)) {
+		DB_DEBUG(DB_WCKEY, mysql_conn->conn,
+			 "Trying to add empty wckey list");
+		return NULL;
+	}
+
+	if (check_connection(mysql_conn) != SLURM_SUCCESS) {
+		errno = ESLURM_DB_CONNECTION;
+		return NULL;
+	}
+
+	if (!is_user_min_admin_level(mysql_conn, uid, SLURMDB_ADMIN_OPERATOR)) {
+		errno = ESLURM_ACCESS_DENIED;
+		return NULL;
+	}
+
+	if (add_assoc->cluster_list && list_count(add_assoc->cluster_list))
+		use_cluster_list = add_assoc->cluster_list;
+	else
+		/*
+		 * No need to do a shallow copy here as we are doing a
+		 * list_for_each_ro() which will handle the locks for us.
+		 */
+		use_cluster_list = as_mysql_cluster_list;
+
+	memset(&add_wckey_cond, 0, sizeof(add_wckey_cond));
+	if (user->default_wckey) {
+		add_wckey_cond.default_wckey = user->default_wckey;
+	} else {
+		add_wckey_cond.default_wckey = list_peek(add_assoc->wckey_list);
+		DB_DEBUG(DB_WCKEY, mysql_conn->conn,
+			 "Default wckey not given, using %s",
+			 add_wckey_cond.default_wckey);
+	}
+	add_wckey_cond.mysql_conn = mysql_conn;
+	add_wckey_cond.now = time(NULL);
+	add_wckey_cond.user_list = add_assoc->user_list;
+	add_wckey_cond.user_name = uid_to_string((uid_t) uid);
+	add_wckey_cond.wckey_list = add_assoc->wckey_list;
+
+	(void) list_for_each_ro(use_cluster_list, _add_wckey_cond_cluster,
+				&add_wckey_cond);
+	xfree(add_wckey_cond.user_name);
+
+	if (add_wckey_cond.txn_query) {
+		xstrcatat(add_wckey_cond.txn_query,
+			  &add_wckey_cond.txn_query_pos,
+			  ";");
+		rc = mysql_db_query(mysql_conn, add_wckey_cond.txn_query);
+		xfree(add_wckey_cond.txn_query);
+		if (rc != SLURM_SUCCESS) {
+			error("Couldn't add txn");
+			rc = SLURM_SUCCESS;
+		}
+	}
+
+	if (add_wckey_cond.rc != SLURM_SUCCESS) {
+		reset_mysql_conn(mysql_conn);
+		if (!add_wckey_cond.ret_str_err)
+			xfree(add_wckey_cond.ret_str);
+		errno = add_wckey_cond.rc;
+		return add_wckey_cond.ret_str;
+	} else if (!add_wckey_cond.ret_str) {
+		DB_DEBUG(DB_WCKEY, mysql_conn->conn, "didn't affect anything");
+		errno = SLURM_NO_CHANGE_IN_DATA;
+		return NULL;
+	}
+
+	errno = SLURM_SUCCESS;
+	return add_wckey_cond.ret_str;
+}
+
 extern List as_mysql_modify_wckeys(mysql_conn_t *mysql_conn,
 				   uint32_t uid,
 				   slurmdb_wckey_cond_t *wckey_cond,
@@ -647,8 +919,9 @@ extern List as_mysql_modify_wckeys(mysql_conn_t *mysql_conn,
 	int rc = SLURM_SUCCESS;
 	char *extra = NULL, *object = NULL, *vals = NULL;
 	char *user_name = NULL;
-	List use_cluster_list = as_mysql_cluster_list;
-	ListIterator itr;
+	List use_cluster_list = NULL;
+	list_itr_t *itr;
+	bool locked = false;
 
 	if (!wckey_cond || !wckey) {
 		error("we need something to change");
@@ -692,13 +965,15 @@ is_same_user:
 		return NULL;
 	}
 
-	if (wckey_cond->cluster_list && list_count(wckey_cond->cluster_list))
-		use_cluster_list = wckey_cond->cluster_list;
-
 	user_name = uid_to_string((uid_t) uid);
 
-	if (use_cluster_list == as_mysql_cluster_list)
-		slurm_mutex_lock(&as_mysql_cluster_list_lock);
+	if (wckey_cond->cluster_list && list_count(wckey_cond->cluster_list))
+		use_cluster_list = wckey_cond->cluster_list;
+	else {
+		slurm_rwlock_rdlock(&as_mysql_cluster_list_lock);
+		use_cluster_list = list_shallow_copy(as_mysql_cluster_list);
+		locked = true;
+	}
 
 	ret_list = list_create(xfree_ptr);
 	itr = list_iterator_create(use_cluster_list);
@@ -713,8 +988,10 @@ is_same_user:
 	xfree(extra);
 	xfree(user_name);
 
-	if (use_cluster_list == as_mysql_cluster_list)
-		slurm_mutex_unlock(&as_mysql_cluster_list_lock);
+	if (locked) {
+		FREE_NULL_LIST(use_cluster_list);
+		slurm_rwlock_unlock(&as_mysql_cluster_list_lock);
+	}
 
 	if (rc == SLURM_ERROR) {
 		FREE_NULL_LIST(ret_list);
@@ -732,8 +1009,9 @@ extern List as_mysql_remove_wckeys(mysql_conn_t *mysql_conn,
 	int rc = SLURM_SUCCESS;
 	char *extra = NULL, *object = NULL;
 	char *user_name = NULL;
-	List use_cluster_list = as_mysql_cluster_list;
-	ListIterator itr;
+	List use_cluster_list = NULL;
+	list_itr_t *itr;
+	bool locked = false;
 
 	if (!wckey_cond) {
 		xstrcat(extra, " where deleted=0");
@@ -750,8 +1028,6 @@ extern List as_mysql_remove_wckeys(mysql_conn_t *mysql_conn,
 
 	(void) _setup_wckey_cond_limits(wckey_cond, &extra);
 
-	if (wckey_cond->cluster_list && list_count(wckey_cond->cluster_list))
-		use_cluster_list = wckey_cond->cluster_list;
 empty:
 	if (!extra) {
 		error("Nothing to remove");
@@ -760,8 +1036,14 @@ empty:
 
 	user_name = uid_to_string((uid_t) uid);
 
-	if (use_cluster_list == as_mysql_cluster_list)
-		slurm_mutex_lock(&as_mysql_cluster_list_lock);
+	if (wckey_cond && wckey_cond->cluster_list &&
+	    list_count(wckey_cond->cluster_list)) {
+		use_cluster_list = wckey_cond->cluster_list;
+	} else {
+		slurm_rwlock_rdlock(&as_mysql_cluster_list_lock);
+		use_cluster_list = list_shallow_copy(as_mysql_cluster_list);
+		locked = true;
+	}
 	ret_list = list_create(xfree_ptr);
 	itr = list_iterator_create(use_cluster_list);
 	while ((object = list_next(itr))) {
@@ -774,8 +1056,10 @@ empty:
 	xfree(extra);
 	xfree(user_name);
 
-	if (use_cluster_list == as_mysql_cluster_list)
-		slurm_mutex_unlock(&as_mysql_cluster_list_lock);
+	if (locked) {
+		FREE_NULL_LIST(use_cluster_list);
+		slurm_rwlock_unlock(&as_mysql_cluster_list_lock);
+	}
 
 	if (rc == SLURM_ERROR) {
 		FREE_NULL_LIST(ret_list);
@@ -794,10 +1078,10 @@ extern List as_mysql_get_wckeys(mysql_conn_t *mysql_conn, uid_t uid,
 	char *cluster_name = NULL;
 	List wckey_list = NULL;
 	int i=0, is_admin=1;
-	uint16_t private_data = 0;
 	slurmdb_user_rec_t user;
-	List use_cluster_list = as_mysql_cluster_list;
-	ListIterator itr;
+	List use_cluster_list = NULL;
+	list_itr_t *itr;
+	bool locked = false;
 
 	if (!wckey_cond) {
 		xstrcat(extra, " where deleted=0");
@@ -810,8 +1094,7 @@ extern List as_mysql_get_wckeys(mysql_conn_t *mysql_conn, uid_t uid,
 	memset(&user, 0, sizeof(slurmdb_user_rec_t));
 	user.uid = uid;
 
-	private_data = slurm_get_private_data();
-	if (private_data & PRIVATE_DATA_USERS) {
+	if (slurm_conf.private_data & PRIVATE_DATA_USERS) {
 		if (!(is_admin = is_user_min_admin_level(
 			      mysql_conn, uid, SLURMDB_ADMIN_OPERATOR))) {
 			assoc_mgr_fill_in_user(
@@ -826,8 +1109,6 @@ extern List as_mysql_get_wckeys(mysql_conn_t *mysql_conn, uid_t uid,
 
 	(void) _setup_wckey_cond_limits(wckey_cond, &extra);
 
-	if (wckey_cond->cluster_list && list_count(wckey_cond->cluster_list))
-		use_cluster_list = wckey_cond->cluster_list;
 empty:
 	xfree(tmp);
 	xstrfmtcat(tmp, "t1.%s", wckey_req_inx[i]);
@@ -839,13 +1120,19 @@ empty:
 	 * if this flag is set.  We also include any accounts they may be
 	 * coordinator of.
 	 */
-	if (!is_admin && (private_data & PRIVATE_DATA_USERS))
+	if (!is_admin && (slurm_conf.private_data & PRIVATE_DATA_USERS))
 		xstrfmtcat(extra, " && t1.user='%s'", user.name);
 
 	wckey_list = list_create(slurmdb_destroy_wckey_rec);
 
-	if (use_cluster_list == as_mysql_cluster_list)
-		slurm_mutex_lock(&as_mysql_cluster_list_lock);
+	if (wckey_cond && wckey_cond->cluster_list &&
+	    list_count(wckey_cond->cluster_list)) {
+		use_cluster_list = wckey_cond->cluster_list;
+	} else {
+		slurm_rwlock_rdlock(&as_mysql_cluster_list_lock);
+		use_cluster_list = list_shallow_copy(as_mysql_cluster_list);
+		locked = true;
+	}
 	//START_TIMER;
 	itr = list_iterator_create(use_cluster_list);
 	while ((cluster_name = list_next(itr))) {
@@ -859,8 +1146,10 @@ empty:
 	}
 	list_iterator_destroy(itr);
 
-	if (use_cluster_list == as_mysql_cluster_list)
-		slurm_mutex_unlock(&as_mysql_cluster_list_lock);
+	if (locked) {
+		FREE_NULL_LIST(use_cluster_list);
+		slurm_rwlock_unlock(&as_mysql_cluster_list_lock);
+	}
 
 	xfree(tmp);
 	xfree(extra);
