@@ -2,7 +2,6 @@
  *  cred_common.c
  *****************************************************************************
  *  Copyright (C) SchedMD LLC.
- *  Written by Tim Wickberg <tim@schedmd.com>
  *
  *  This file is part of Slurm, a resource management program.
  *  For details, see <https://slurm.schedmd.com/>.
@@ -79,7 +78,7 @@ static void _packstr_and_switch(char *string, void *switch_step, buf_t *buffer,
 
 	start = get_buf_offset(buffer);
 	packstr(string, buffer);
-	switch_g_pack_jobinfo(switch_step, buffer, protocol_version);
+	switch_g_pack_stepinfo(switch_step, buffer, protocol_version);
 	pack8(0, buffer); /* ensure trailing NUL */
 	end = get_buf_offset(buffer);
 	set_buf_offset(buffer, start);
@@ -110,10 +109,10 @@ static int _unpackstr_and_switch(char **string, void **switch_step,
 		/* Second hidden field */
 		uint32_t end = get_buf_offset(buffer);
 		set_buf_offset(buffer, start + string_len + 4);
-		if (switch_g_unpack_jobinfo(&switch_tmp, buffer,
+		if (switch_g_unpack_stepinfo(&switch_tmp, buffer,
 					     protocol_version)) {
-			error("switch_g_unpack_jobinfo: %m");
-			switch_g_free_jobinfo(switch_tmp);
+			error("switch_g_unpack_stepinfo: %m");
+			switch_g_free_stepinfo(switch_tmp);
 			goto unpack_error;
 		}
 		*switch_step = switch_tmp;
@@ -221,7 +220,7 @@ extern slurm_cred_t *cred_create(slurm_cred_arg_t *cred,
 				     buffer);
 		}
 		packstr(cred->job_selinux_context, buffer);
-	} else if (protocol_version >= SLURM_23_02_PROTOCOL_VERSION) {
+	} else if (protocol_version >= SLURM_MIN_PROTOCOL_VERSION) {
 		pack_step_id(&cred->step_id, buffer, protocol_version);
 		pack_identity(cred->id, buffer, protocol_version);
 
@@ -299,78 +298,6 @@ extern slurm_cred_t *cred_create(slurm_cred_arg_t *cred,
 				     buffer);
 		}
 		packstr(cred->job_selinux_context, buffer);
-	} else if (protocol_version >= SLURM_MIN_PROTOCOL_VERSION) {
-		pack_step_id(&cred->step_id, buffer, protocol_version);
-		pack_identity(cred->id, buffer, protocol_version);
-
-		(void) gres_job_state_pack(cred->job_gres_list, buffer,
-					   cred->step_id.job_id, false,
-					   protocol_version);
-		gres_step_state_pack(cred->step_gres_list, buffer,
-				     &cred->step_id, protocol_version);
-		pack16(cred->job_core_spec, buffer);
-		packstr(cred->job_account, buffer);
-		packstr(cred->job_alias_list, buffer);
-		packstr(cred->job_comment, buffer);
-		packstr(cred->job_constraints, buffer);
-		packstr(cred->job_partition, buffer);
-		packstr(cred->job_reservation, buffer);
-		pack16(cred->job_restart_cnt, buffer);
-		packstr(cred->job_std_err, buffer);
-		packstr(cred->job_std_in, buffer);
-		packstr(cred->job_std_out, buffer);
-		packstr(cred->step_hostlist, buffer);
-		pack16(cred->job_x11, buffer);
-		pack_time(ctime, buffer);
-
-		if (cred->job_core_bitmap)
-			tot_core_cnt = bit_size(cred->job_core_bitmap);
-		pack32(tot_core_cnt, buffer);
-		pack_bit_str_hex(cred->job_core_bitmap, buffer);
-		pack_bit_str_hex(cred->step_core_bitmap, buffer);
-		pack16(cred->core_array_size, buffer);
-		if (cred->core_array_size) {
-			pack16_array(cred->cores_per_socket,
-				     cred->core_array_size,
-				     buffer);
-			pack16_array(cred->sockets_per_node,
-				     cred->core_array_size,
-				     buffer);
-			pack32_array(cred->sock_core_rep_count,
-				     cred->core_array_size,
-				     buffer);
-		}
-		pack32(cred->cpu_array_count, buffer);
-		if (cred->cpu_array_count) {
-			pack16_array(cred->cpu_array,
-				     cred->cpu_array_count,
-				     buffer);
-			pack32_array(cred->cpu_array_reps,
-				     cred->cpu_array_count,
-				     buffer);
-		}
-		pack32(cred->job_nhosts, buffer);
-		pack32(cred->job_ntasks, buffer);
-		packstr(cred->job_hostlist, buffer);
-		pack32(cred->job_mem_alloc_size, buffer);
-		if (cred->job_mem_alloc_size) {
-			pack64_array(cred->job_mem_alloc,
-				     cred->job_mem_alloc_size,
-				     buffer);
-			pack32_array(cred->job_mem_alloc_rep_count,
-				     cred->job_mem_alloc_size,
-				     buffer);
-		}
-		pack32(cred->step_mem_alloc_size, buffer);
-		if (cred->step_mem_alloc_size) {
-			pack64_array(cred->step_mem_alloc,
-				     cred->step_mem_alloc_size,
-				     buffer);
-			pack32_array(cred->step_mem_alloc_rep_count,
-				     cred->step_mem_alloc_size,
-				     buffer);
-		}
-		packstr(cred->job_selinux_context, buffer);
 	}
 
 	return credential;
@@ -489,7 +416,7 @@ extern int cred_unpack(void **out, buf_t *buffer, uint16_t protocol_version)
 		}
 
 		safe_unpackstr(&cred_arg->job_selinux_context, buffer);
-	} else if (protocol_version >= SLURM_23_02_PROTOCOL_VERSION) {
+	} else if (protocol_version >= SLURM_MIN_PROTOCOL_VERSION) {
 		if (unpack_step_id_members(&cred_arg->step_id, buffer,
 					   protocol_version) != SLURM_SUCCESS)
 			goto unpack_error;
@@ -588,99 +515,6 @@ extern int cred_unpack(void **out, buf_t *buffer, uint16_t protocol_version)
 				goto unpack_error;
 		}
 
-		safe_unpackstr(&cred_arg->job_selinux_context, buffer);
-	} else if (protocol_version >= SLURM_MIN_PROTOCOL_VERSION) {
-		if (unpack_step_id_members(&cred_arg->step_id, buffer,
-					   protocol_version) != SLURM_SUCCESS)
-			goto unpack_error;
-
-		if (unpack_identity(&cred_arg->id, buffer, protocol_version))
-			goto unpack_error;
-
-		if (gres_job_state_unpack(&cred_arg->job_gres_list, buffer,
-					  cred_arg->step_id.job_id,
-					  protocol_version)
-		    != SLURM_SUCCESS)
-			goto unpack_error;
-		if (gres_step_state_unpack(&cred_arg->step_gres_list,
-					   buffer, &cred_arg->step_id,
-					   protocol_version)
-		    != SLURM_SUCCESS) {
-			goto unpack_error;
-		}
-		safe_unpack16(&cred_arg->job_core_spec, buffer);
-		safe_unpackstr(&cred_arg->job_account, buffer);
-		safe_unpackstr(&cred_arg->job_alias_list, buffer);
-		safe_unpackstr(&cred_arg->job_comment, buffer);
-		safe_unpackstr(&cred_arg->job_constraints, buffer);
-		safe_unpackstr(&cred_arg->job_partition, buffer);
-		safe_unpackstr(&cred_arg->job_reservation, buffer);
-		safe_unpack16(&cred_arg->job_restart_cnt, buffer);
-		safe_unpackstr(&cred_arg->job_std_err, buffer);
-		safe_unpackstr(&cred_arg->job_std_in, buffer);
-		safe_unpackstr(&cred_arg->job_std_out, buffer);
-		safe_unpackstr(&cred_arg->step_hostlist, buffer);
-		safe_unpack16(&cred_arg->job_x11, buffer);
-		safe_unpack_time(&cred->ctime, buffer);
-		safe_unpack32(&tot_core_cnt, buffer);
-		unpack_bit_str_hex(&cred_arg->job_core_bitmap, buffer);
-		unpack_bit_str_hex(&cred_arg->step_core_bitmap, buffer);
-		safe_unpack16(&cred_arg->core_array_size, buffer);
-		if (cred_arg->core_array_size) {
-			safe_unpack16_array(&cred_arg->cores_per_socket, &len,
-					    buffer);
-			if (len != cred_arg->core_array_size)
-				goto unpack_error;
-			safe_unpack16_array(&cred_arg->sockets_per_node, &len,
-					    buffer);
-			if (len != cred_arg->core_array_size)
-				goto unpack_error;
-			safe_unpack32_array(&cred_arg->sock_core_rep_count,
-					    &len, buffer);
-			if (len != cred_arg->core_array_size)
-				goto unpack_error;
-		}
-		safe_unpack32(&cred_arg->cpu_array_count, buffer);
-		if (cred_arg->cpu_array_count) {
-			safe_unpack16_array(&cred_arg->cpu_array, &len, buffer);
-			if (len != cred_arg->cpu_array_count)
-				goto unpack_error;
-			safe_unpack32_array(&cred_arg->cpu_array_reps, &len,
-					    buffer);
-			if (len != cred_arg->cpu_array_count)
-				goto unpack_error;
-		}
-		safe_unpack32(&cred_arg->job_nhosts, buffer);
-		safe_unpack32(&cred_arg->job_ntasks, buffer);
-		safe_unpackstr(&cred_arg->job_hostlist, buffer);
-
-		safe_unpack32(&cred_arg->job_mem_alloc_size, buffer);
-		if (cred_arg->job_mem_alloc_size) {
-			safe_unpack64_array(&cred_arg->job_mem_alloc, &len,
-					    buffer);
-			if (len != cred_arg->job_mem_alloc_size)
-				goto unpack_error;
-
-			safe_unpack32_array(&cred_arg->job_mem_alloc_rep_count,
-					    &len, buffer);
-			if (len != cred_arg->job_mem_alloc_size)
-				goto unpack_error;
-
-		}
-
-		safe_unpack32(&cred_arg->step_mem_alloc_size, buffer);
-		if (cred_arg->step_mem_alloc_size) {
-			safe_unpack64_array(&cred_arg->step_mem_alloc, &len,
-					    buffer);
-			if (len != cred_arg->step_mem_alloc_size)
-				goto unpack_error;
-
-			safe_unpack32_array(&cred_arg->step_mem_alloc_rep_count,
-					    &len, buffer);
-			if (len != cred_arg->step_mem_alloc_size)
-				goto unpack_error;
-		}
-
 		safe_unpackstr(&cred_arg->job_selinux_context, buffer);
 	} else {
 		error("%s: protocol_version %hu not supported",
@@ -773,55 +607,47 @@ extern sbcast_cred_t *sbcast_cred_unpack(buf_t *buffer, uint32_t *siglen,
 {
 	sbcast_cred_t *sbcast_cred = xmalloc(sizeof(*sbcast_cred));
 	uint32_t cred_start = get_buf_offset(buffer);
-	uid_t uid = SLURM_AUTH_NOBODY;
-	gid_t gid = SLURM_AUTH_NOBODY;
-	char *user_name = NULL;
-	uint32_t ngids = 0, *gids = NULL;
 
 	if (protocol_version >= SLURM_23_11_PROTOCOL_VERSION) {
 		if (unpack_identity(&sbcast_cred->arg.id, buffer,
 				    protocol_version))
 			goto unpack_error;
-		uid = sbcast_cred->arg.id->uid;
-		gid = sbcast_cred->arg.id->gid;
 		safe_unpack_time(&sbcast_cred->ctime, buffer);
 		safe_unpack_time(&sbcast_cred->arg.expiration, buffer);
 		safe_unpack32(&sbcast_cred->arg.job_id, buffer);
 		safe_unpack32(&sbcast_cred->arg.het_job_id, buffer);
 		safe_unpack32(&sbcast_cred->arg.step_id, buffer);
 		safe_unpackstr(&sbcast_cred->arg.nodes, buffer);
+
+		if (!sbcast_cred->arg.id->pw_name) {
+			uid_t uid = sbcast_cred->arg.id->uid;
+			gid_t gid = sbcast_cred->arg.id->gid;
+
+			debug2("%s: need to fetch identity", __func__);
+			FREE_NULL_IDENTITY(sbcast_cred->arg.id);
+			sbcast_cred->arg.id = fetch_identity(uid, gid, false);
+			if (!sbcast_cred->arg.id)
+				goto unpack_error;
+		}
 	} else if (protocol_version >= SLURM_MIN_PROTOCOL_VERSION) {
+		uint32_t uint32_tmp = 0;
+		sbcast_cred->arg.id = xmalloc(sizeof(*sbcast_cred->arg.id));
+
 		safe_unpack_time(&sbcast_cred->ctime, buffer);
 		safe_unpack_time(&sbcast_cred->arg.expiration, buffer);
 		safe_unpack32(&sbcast_cred->arg.job_id, buffer);
 		safe_unpack32(&sbcast_cred->arg.het_job_id, buffer);
 		safe_unpack32(&sbcast_cred->arg.step_id, buffer);
-		safe_unpack32(&uid, buffer);
-		safe_unpack32(&gid, buffer);
-		safe_unpackstr(&user_name, buffer);
-		safe_unpack32_array(&gids, &ngids, buffer);
+		safe_unpack32(&sbcast_cred->arg.id->uid, buffer);
+		safe_unpack32(&sbcast_cred->arg.id->gid, buffer);
+		safe_unpackstr(&sbcast_cred->arg.id->pw_name, buffer);
+		safe_unpack32_array(&sbcast_cred->arg.id->gids, &uint32_tmp,
+				    buffer);
+		sbcast_cred->arg.id->ngids = uint32_tmp;
 		safe_unpackstr(&sbcast_cred->arg.nodes, buffer);
 	} else
 		goto unpack_error;
 
-	if (sbcast_cred->arg.id && !sbcast_cred->arg.id->pw_name) {
-		debug2("%s: need to fetch identity", __func__);
-		FREE_NULL_IDENTITY(sbcast_cred->arg.id);
-	}
-
-	if (!user_name && !sbcast_cred->arg.id) {
-		sbcast_cred->arg.id = fetch_identity(uid, gid, false);
-		if (!sbcast_cred->arg.id)
-			goto unpack_error;
-	} else if (!sbcast_cred->arg.id) {
-		sbcast_cred->arg.id = xmalloc(sizeof(*sbcast_cred->arg.id));
-		sbcast_cred->arg.id->uid = uid;
-		sbcast_cred->arg.id->gid = gid;
-		sbcast_cred->arg.id->pw_name = user_name;
-		sbcast_cred->arg.id->ngids = ngids;
-		sbcast_cred->arg.id->gids = gids;
-	}
-
 	identity_debug2(sbcast_cred->arg.id, __func__);
 
 	*siglen = get_buf_offset(buffer) - cred_start;
