@@ -1,8 +1,7 @@
 /*****************************************************************************\
  *  gres_c_s.c - common functions for shared gres plugins
  *****************************************************************************
- *  Copyright (C) 2021 SchedMD LLC
- *  Written by Danny Auble <da@schedmd.com>
+ *  Copyright (C) SchedMD LLC.
  *
  *  This file is part of Slurm, a resource management program.
  *  For details, see <https://slurm.schedmd.com/>.
@@ -230,7 +229,7 @@ static uint64_t _build_shared_dev_info(List gres_conf_list)
 	uint64_t shared_count = 0;
 	gres_slurmd_conf_t *gres_slurmd_conf;
 	shared_dev_info_t *shared_conf;
-	ListIterator iter;
+	list_itr_t *iter;
 
 	FREE_NULL_LIST(shared_info);
 	shared_info = list_create(xfree_ptr);
@@ -269,10 +268,10 @@ static int _remove_shared_recs(void *x, void *arg)
  */
 static List _build_sharing_list(List gres_list, char *sharing_name)
 {
-	ListIterator itr;
+	list_itr_t *itr;
 	gres_slurmd_conf_t *gres_slurmd_conf, *sharing_record;
 	List sharing_list;
-	hostlist_t hl;
+	hostlist_t *hl;
 	char *f_name;
 	bool log_fname = true;
 
@@ -348,10 +347,10 @@ static List _build_sharing_list(List gres_list, char *sharing_name)
  */
 static List _build_shared_list(List gres_list, char *shared_name)
 {
-	ListIterator itr;
+	list_itr_t *itr;
 	gres_slurmd_conf_t *gres_slurmd_conf, *shared_record;
 	List shared_list;
-	hostlist_t hl;
+	hostlist_t *hl;
 	char *f_name;
 	uint64_t count_per_file;
 	int shared_no_file_recs = 0, shared_file_recs = 0;
@@ -524,7 +523,7 @@ extern void gres_c_s_send_stepd(buf_t *buffer)
 {
 	uint32_t shared_cnt;
 	shared_dev_info_t *shared_ptr;
-	ListIterator itr;
+	list_itr_t *itr;
 
 	if (!shared_info) {
 		shared_cnt = 0;
