@@ -47,7 +47,6 @@
 #include <sched.h>
 #include <fcntl.h>
 #include <sys/mount.h>
-#include <linux/limits.h>
 #include <semaphore.h>
 
 #include "src/common/slurm_xlator.h"
@@ -58,13 +57,16 @@
 #include "src/common/read_config.h"
 #include "src/common/run_command.h"
 #include "src/common/stepd_api.h"
+#include "src/common/uid.h"
 #include "src/common/xmalloc.h"
 #include "src/common/xstring.h"
+#include "src/interfaces/switch.h"
+#include "src/slurmd/slurmstepd/slurmstepd_job.h"
 
 #include "read_jcconf.h"
 
-static int _create_ns(uint32_t job_id, uid_t uid, bool remount);
-static int _delete_ns(uint32_t job_id, bool is_slurmd);
+static int _create_ns(uint32_t job_id, stepd_step_rec_t *step);
+static int _delete_ns(uint32_t job_id);
 
 #if defined (__APPLE__)
 extern slurmd_conf_t *conf __attribute__((weak_import));
@@ -78,54 +80,20 @@ const uint32_t plugin_version   = SLURM_VERSION_NUMBER;
 
 static slurm_jc_conf_t *jc_conf = NULL;
 static int step_ns_fd = -1;
-List legacy_jobs;
+static bool plugin_disabled = false;
 
-typedef struct {
-	uint32_t job_id;
-	uint16_t protocol_version;
-} legacy_job_info_t;
-
-static int _create_paths(uint32_t job_id,
-			 char *job_mount,
-			 char *ns_holder,
-			 char *src_bind)
+static void _create_paths(uint32_t job_id, char **job_mount, char **ns_holder,
+			  char **src_bind)
 {
-	jc_conf = get_slurm_jc_conf();
-
-	if (!jc_conf) {
-		error("%s: Configuration not read correctly: did %s not exist?",
-		      __func__, tmpfs_conf_file);
-		return SLURM_ERROR;
-	}
-
 	xassert(job_mount);
 
-	if (snprintf(job_mount, PATH_MAX, "%s/%u", jc_conf->basepath, job_id)
-	    >= PATH_MAX) {
-		error("%s: Unable to build job %u mount path: %m",
-			__func__, job_id);
-		return SLURM_ERROR;
-	}
+	xstrfmtcat(*job_mount, "%s/%u", jc_conf->basepath, job_id);
 
-	if (ns_holder) {
-		if (snprintf(ns_holder, PATH_MAX, "%s/.ns", job_mount)
-		    >= PATH_MAX) {
-			error("%s: Unable to build job %u ns_holder path: %m",
-			      __func__, job_id);
-			return SLURM_ERROR;
-		}
-	}
+	if (ns_holder)
+		xstrfmtcat(*ns_holder, "%s/.ns", *job_mount);
 
-	if (src_bind) {
-		if (snprintf(src_bind, PATH_MAX, "%s/.%u", job_mount, job_id)
-		    >= PATH_MAX) {
-			error("%s: Unable to build job %u src_bind path: %m",
-			__func__, job_id);
-			return SLURM_ERROR;
-		}
-	}
-
-	return SLURM_SUCCESS;
+	if (src_bind)
+		xstrfmtcat(*src_bind, "%s/.%u", *job_mount, job_id);
 }
 
 static int _find_step_in_list(step_loc_t *stepd, uint32_t *job_id)
@@ -133,74 +101,65 @@ static int _find_step_in_list(step_loc_t *stepd, uint32_t *job_id)
 	return (stepd->step_id.job_id == *job_id);
 }
 
-static int _find_legacy_job_in_list(legacy_job_info_t *job, uint32_t *job_id)
+static bool _is_plugin_disabled(char *basepath)
 {
-	return (job->job_id == *job_id);
+	return ((!basepath) || (!xstrncasecmp(basepath, "none", 4)));
 }
 
 static int _restore_ns(List steps, const char *d_name)
 {
+	char *endptr;
 	int fd;
-	uint32_t job_id;
+	unsigned long job_id;
 	step_loc_t *stepd;
 
-	if (!(job_id = slurm_atoul(d_name))) {
+	errno = 0;
+	job_id = strtoul(d_name, &endptr, 10);
+	if ((errno != 0) || (job_id >= NO_VAL) || (*endptr != '\0')) {
 		debug3("ignoring %s, could not convert to jobid.", d_name);
 		return SLURM_SUCCESS;
 	}
 
 	/* here we think this is a job container */
-	debug3("determine if job %u is still running", job_id);
+	log_flag(JOB_CONT, "determine if job %lu is still running", job_id);
 	stepd = list_find_first(steps, (ListFindF)_find_step_in_list, &job_id);
 	if (!stepd) {
-		debug("%s: Job %u not found, deleting the namespace",
+		debug("%s: Job %lu not found, deleting the namespace",
 		      __func__, job_id);
-		return _delete_ns(job_id, false);
+		return _delete_ns(job_id);
 	}
 
 	fd = stepd_connect(stepd->directory, stepd->nodename,
 			   &stepd->step_id, &stepd->protocol_version);
 	if (fd == -1) {
-		error("%s: failed to connect to stepd for %u.",
+		error("%s: failed to connect to stepd for %lu.",
 		      __func__, job_id);
-		return _delete_ns(job_id, false);
+		return _delete_ns(job_id);
 	}
 
 	close(fd);
 
-	if (stepd->protocol_version == SLURM_20_11_PROTOCOL_VERSION) {
-		legacy_job_info_t *job = xmalloc(sizeof(*job));
-		if (!legacy_jobs)
-			legacy_jobs = list_create(NULL);
-		job->job_id = job_id;
-		job->protocol_version = stepd->protocol_version;
-		list_append(legacy_jobs, job);
-		return _create_ns(job_id, 0, true);
-	}
-
 	return SLURM_SUCCESS;
 }
 
-extern void container_p_reconfig(void)
-{
-	return;
-}
-
 /*
  * init() is called when the plugin is loaded, before any other functions
  *	are called.  Put global initialization here.
  */
 extern int init(void)
 {
-#if defined(__APPLE__) || defined(__FreeBSD__)
-	fatal("%s is not available on this system. (mount bind limitation)",
-	      plugin_name);
-#endif
-
-	if (!get_slurm_jc_conf()) {
-		error("%s: Configuration not read correctly: Does '%s' not exist?",
-		      plugin_type, tmpfs_conf_file);
-		return SLURM_ERROR;
+	if (running_in_slurmd()) {
+		/*
+		 * Only init the config here for the slurmd. It will be sent by
+		 * the slurmd to the slurmstepd at launch time.
+		 */
+		if (!(jc_conf = init_slurm_jc_conf())) {
+			error("%s: Configuration not read correctly: Does '%s' not exist?",
+			      plugin_type, tmpfs_conf_file);
+			return SLURM_ERROR;
+		}
+		plugin_disabled = _is_plugin_disabled(jc_conf->basepath);
+		debug("job_container.conf read successfully");
 	}
 
 	debug("%s loaded", plugin_name);
@@ -208,20 +167,6 @@ extern int init(void)
 	return SLURM_SUCCESS;
 }
 
-static int _legacy_fini(void *x, void *arg) {
-	char job_mount[PATH_MAX];
-	legacy_job_info_t *job = (legacy_job_info_t *)x;
-
-	if (_create_paths(job->job_id, job_mount, NULL, NULL)
-	    != SLURM_SUCCESS)
-		return SLURM_ERROR;
-
-	if (umount2(job_mount, MNT_DETACH))
-		debug2("umount2: %s failed: %s", job_mount, strerror(errno));
-
-	return SLURM_SUCCESS;
-}
-
 /*
  * fini() is called when the plugin is removed. Clear any allocated
  *	storage here.
@@ -231,22 +176,14 @@ extern int fini(void)
 	int rc = SLURM_SUCCESS;
 	debug("%s unloaded", plugin_name);
 
-#ifdef HAVE_NATIVE_CRAY
-	return SLURM_SUCCESS;
-#endif
-
 	if (step_ns_fd != -1) {
 		close(step_ns_fd);
 		step_ns_fd = -1;
 	}
 
-	if (!legacy_jobs)
-		return SLURM_SUCCESS;
-
-	rc = list_for_each(legacy_jobs, _legacy_fini, NULL);
-
-	FREE_NULL_LIST(legacy_jobs);
-
+#ifdef MEMORY_LEAK_DEBUG
+	free_jc_conf();
+#endif
 	return rc;
 }
 
@@ -257,59 +194,29 @@ extern int container_p_restore(char *dir_name, bool recover)
 	List steps;
 	int rc = SLURM_SUCCESS;
 
-#ifdef HAVE_NATIVE_CRAY
-	return SLURM_SUCCESS;
-#endif
-
-	jc_conf = get_slurm_jc_conf();
-	if (!jc_conf) {
-		error("%s: Configuration not loaded", __func__);
-		return SLURM_ERROR;
-	}
-
-	debug("job_container.conf read successfully");
+	if (plugin_disabled)
+		return SLURM_SUCCESS;
 
 	if (jc_conf->auto_basepath) {
 		int fstatus;
-		char *mnt_point, *p;
-		mode_t omask;
+		mode_t omask = umask(S_IWGRP | S_IWOTH);
 
-		omask = umask(S_IWGRP | S_IWOTH);
-
-		fstatus = mkdir(jc_conf->basepath, 0755);
-		if (fstatus && errno != EEXIST) {
-			if (jc_conf->basepath[0] != '/') {
-				debug("unable to create ns directory '%s' : does not start with '/'",
-				      jc_conf->basepath);
-				umask(omask);
-				return SLURM_ERROR;
-			}
-			mnt_point = xstrdup(jc_conf->basepath);
-			p = mnt_point;
-			while ((p = xstrchr(p+1, '/')) != NULL) {
-				*p = '\0';
-				fstatus = mkdir(mnt_point, 0755);
-				if (fstatus && errno != EEXIST) {
-					debug("unable to create ns required directory '%s'",
-					      mnt_point);
-					xfree(mnt_point);
-					umask(omask);
-					return SLURM_ERROR;
-				}
-				*p='/';
-			}
-			xfree(mnt_point);
-			fstatus = mkdir(jc_conf->basepath, 0755);
-		}
-
-		if (fstatus && errno != EEXIST) {
-			debug("unable to create ns directory '%s' : %m",
-			      jc_conf->basepath);
+		if (jc_conf->basepath[0] != '/') {
+			debug("%s: unable to create ns directory '%s' : does not start with '/'",
+			      __func__, jc_conf->basepath);
 			umask(omask);
 			return SLURM_ERROR;
 		}
-		umask(omask);
 
+		if ((fstatus = mkdirpath(jc_conf->basepath, 0755, true))) {
+			debug("%s: unable to create ns directory '%s' : %s",
+			      __func__, jc_conf->basepath,
+			      slurm_strerror(fstatus));
+			umask(omask);
+			return SLURM_ERROR;
+		}
+
+		umask(omask);
 	}
 
 	steps = stepd_available(conf->spooldir, conf->node_name);
@@ -326,8 +233,11 @@ extern int container_p_restore(char *dir_name, bool recover)
 	}
 
 	while ((ep = readdir(dp))) {
-		if (_restore_ns(steps, ep->d_name))
-			rc = SLURM_ERROR;
+		/* If possible, only check directories */
+		if ((ep->d_type == DT_DIR) || (ep->d_type == DT_UNKNOWN)) {
+			if (_restore_ns(steps, ep->d_name))
+				rc = SLURM_ERROR;
+		}
 	}
 	closedir(dp);
 	FREE_NULL_LIST(steps);
@@ -338,54 +248,120 @@ extern int container_p_restore(char *dir_name, bool recover)
 	return rc;
 }
 
-static int _mount_private_tmp(char *path)
+static int _mount_private_dirs(char *path, uid_t uid)
 {
+	char *buffer = NULL, *mount_path = NULL, *save_ptr = NULL, *token;
+	int rc = 0;
+
 	if (!path) {
-		error("%s: cannot mount /tmp", __func__);
+		error("%s: no path to private directories specified.",
+		      __func__);
 		return -1;
 	}
-#if !defined(__APPLE__) && !defined(__FreeBSD__)
-	if (mount(NULL, "/", NULL, MS_PRIVATE|MS_REC, NULL)) {
-		error("%s: making root private: failed: %s",
-		      __func__, strerror(errno));
-		return -1;
+	buffer = xstrdup(jc_conf->dirs);
+	token = strtok_r(buffer, ",", &save_ptr);
+	while (token) {
+		/* skip /dev/shm, this is handled elsewhere */
+		if (!xstrcmp(token, "/dev/shm")) {
+			token = strtok_r(NULL, ",", &save_ptr);
+			continue;
+		}
+		xstrfmtcat(mount_path, "%s/%s", path, token);
+		for (char *t = mount_path + strlen(path) + 1; *t; t++) {
+			if (*t == '/')
+				*t = '_';
+		}
+		rc = mkdir(mount_path, 0700);
+		if (rc && errno != EEXIST) {
+			error("%s: Failed to create %s, %m",
+			      __func__, mount_path);
+			goto private_mounts_exit;
+		}
+		rc = lchown(mount_path, uid, -1);
+		if (rc) {
+			error("%s: lchown failed for %s: %m",
+			      __func__, mount_path);
+			goto private_mounts_exit;
+		}
+		if (mount(mount_path, token, NULL, MS_BIND, NULL)) {
+			error("%s: %s mount failed, %m", __func__, token);
+			rc = -1;
+			goto private_mounts_exit;
+		}
+		token = strtok_r(NULL, ",", &save_ptr);
+		xfree(mount_path);
 	}
-	if (mount(path, "/tmp", NULL, MS_BIND|MS_REC, NULL)) {
-		error("%s: /tmp mount failed, %s",
-		      __func__, strerror(errno));
-		return -1;
-	}
-#endif
-	return 0;
+
+private_mounts_exit:
+	xfree(buffer);
+	xfree(mount_path);
+	return rc;
 }
 
 static int _mount_private_shm(void)
 {
+	char *loc = NULL;
 	int rc = 0;
 
-	rc = umount("/dev/shm");
-	if (rc && errno != EINVAL) {
-		error("%s: umount /dev/shm failed: %s\n",
-		      __func__, strerror(errno));
+	/* return early if "/dev/shm" is not in the mount list */
+	if (!(loc = xstrcasestr(jc_conf->dirs, "/dev/shm")))
 		return rc;
+	if (!((loc[8] == ',') || (loc[8] == 0)))
+		return rc;
+
+	/* handle mounting a new /dev/shm */
+	if (!jc_conf->shared) {
+		/*
+		 * only unmount old /dev/shm if private, otherwise this can
+		 * impact the root namespace
+		 */
+		rc = umount("/dev/shm");
+		if (rc && errno != EINVAL) {
+			error("%s: umount /dev/shm failed: %m", __func__);
+			return rc;
+		}
 	}
-#if !defined(__APPLE__) && !defined(__FreeBSD__)
 	rc = mount("tmpfs", "/dev/shm", "tmpfs", 0, NULL);
 	if (rc) {
-		error("%s: mounting private /dev/shm failed: %s\n",
-		      __func__, strerror(errno));
+		error("%s: /dev/shm mount failed: %m", __func__);
 		return -1;
 	}
-#endif
 	return rc;
 }
 
-
-static int _create_ns(uint32_t job_id, uid_t uid, bool remount)
+static int _clean_job_basepath(uint32_t job_id)
 {
-	char job_mount[PATH_MAX];
-	char ns_holder[PATH_MAX];
-	char src_bind[PATH_MAX];
+	DIR *dp;
+	struct dirent *ep;
+	char *path = NULL;
+
+	if (!(dp = opendir(jc_conf->basepath))) {
+		error("%s: Unable to open %s", __func__, jc_conf->basepath);
+		return SLURM_ERROR;
+	}
+
+	while ((ep = readdir(dp))) {
+		if (!xstrcmp(ep->d_name, ".") || !xstrcmp(ep->d_name, ".."))
+			continue;
+		/* If possible, only attempt with directories */
+		if ((ep->d_type == DT_DIR) || (ep->d_type == DT_UNKNOWN)) {
+			xstrfmtcat(path, "%s/%s",
+				   jc_conf->basepath, ep->d_name);
+			/* it is not important if this fails */
+			if (umount2(path, MNT_DETACH))
+				log_flag(JOB_CONT, "failed to unmount %s for job %u",
+					 path, job_id);
+			xfree(path);
+		}
+	}
+	closedir(dp);
+
+	return SLURM_SUCCESS;
+}
+
+static int _create_ns(uint32_t job_id, stepd_step_rec_t *step)
+{
+	char *job_mount = NULL, *ns_holder = NULL, *src_bind = NULL;
 	char *result = NULL;
 	int fd;
 	int rc = 0;
@@ -393,30 +369,14 @@ static int _create_ns(uint32_t job_id, uid_t uid, bool remount)
 	sem_t *sem2 = NULL;
 	pid_t cpid;
 
-#ifdef HAVE_NATIVE_CRAY
-	return 0;
-#endif
+	_create_paths(job_id, &job_mount, &ns_holder, &src_bind);
 
-	if (_create_paths(job_id, job_mount, ns_holder, src_bind)
-	    != SLURM_SUCCESS) {
-		return -1;
+	if (mkdir(job_mount, 0700)) {
+		error("%s: mkdir %s failed: %m", __func__, job_mount);
+		rc = SLURM_ERROR;
+		goto end_it;
 	}
 
-	rc = mkdir(job_mount, 0700);
-	if (rc && errno != EEXIST) {
-		error("%s: mkdir %s failed: %s",
-		      __func__, job_mount, strerror(errno));
-		return -1;
-	} else if (!remount && rc && errno == EEXIST) {
-		/*
-		 * This is coming from sbcast likely,
-		 * exit as success
-		 */
-		rc = 0;
-		goto exit2;
-	}
-
-#if !defined(__APPLE__) && !defined(__FreeBSD__)
 	/*
 	 * MS_BIND mountflag would make mount() ignore all other mountflags
 	 * except MS_REC. We need MS_PRIVATE mountflag as well to make the
@@ -424,27 +384,21 @@ static int _create_ns(uint32_t job_id, uid_t uid, bool remount)
 	 * done by calling mount() a second time with MS_PRIVATE and MS_REC
 	 * flags.
 	 */
-	if (mount(job_mount, job_mount, "xfs", MS_BIND, NULL)) {
-		error("%s: Initial base mount failed, %s",
-		      __func__, strerror(errno));
-		return SLURM_ERROR;
+	if (mount(job_mount, job_mount, NULL, MS_BIND, NULL)) {
+		error("%s: Initial base mount failed: %m", __func__);
+		rc = SLURM_ERROR;
+		goto end_it;
 	}
-	if (mount(job_mount, job_mount, "xfs", MS_PRIVATE | MS_REC, NULL)) {
-		error("%s: Initial base mount failed, %s",
-		      __func__, strerror(errno));
-		return SLURM_ERROR;
+	if (mount(job_mount, job_mount, NULL, MS_PRIVATE | MS_REC, NULL)) {
+		error("%s: Initial base mount failed: %m", __func__);
+		rc = SLURM_ERROR;
+		goto end_it;
 	}
-#endif
 
 	fd = open(ns_holder, O_CREAT|O_RDWR, S_IRWXU);
 	if (fd == -1) {
-		if (!remount) {
-			error("%s: open failed %s: %s",
-			      __func__, ns_holder, strerror(errno));
-			rc = -1;
-		} else
-			debug3("ignoring wrong ns_holder path %s: %m",
-			       ns_holder);
+		error("%s: open failed %s: %m", __func__, ns_holder);
+		rc = -1;
 		goto exit2;
 	}
 	close(fd);
@@ -458,11 +412,29 @@ static int _create_ns(uint32_t job_id, uid_t uid, bool remount)
 			.status = &rc,
 		};
 		run_command_args.env = env_array_create();
+		if (step->het_job_id && (step->het_job_id != NO_VAL))
+			env_array_overwrite_fmt(&run_command_args.env,
+						"SLURM_HET_JOB_ID", "%u",
+						step->het_job_id);
+		env_array_overwrite_fmt(&run_command_args.env,
+					"SLURM_JOB_GID", "%u",
+					step->gid);
 		env_array_overwrite_fmt(&run_command_args.env,
 					"SLURM_JOB_ID", "%u", job_id);
+
 		env_array_overwrite_fmt(&run_command_args.env,
 					"SLURM_JOB_MOUNTPOINT_SRC", "%s",
 					src_bind);
+		env_array_overwrite_fmt(&run_command_args.env,
+					"SLURM_JOB_UID", "%u",
+					step->uid);
+		env_array_overwrite_fmt(&run_command_args.env,
+					"SLURM_JOB_USER", "%s",
+					step->user_name);
+		if (step->cwd)
+			env_array_overwrite_fmt(&run_command_args.env,
+						"SLURM_JOB_WORK_DIR", "%s",
+						step->cwd);
 		env_array_overwrite_fmt(&run_command_args.env,
 					"SLURM_CONF", "%s",
 					slurm_conf.slurm_conf);
@@ -477,22 +449,21 @@ static int _create_ns(uint32_t job_id, uid_t uid, bool remount)
 			xfree(result);
 			goto exit2;
 		} else {
-			debug3("initscript stdout: %s", result);
+			log_flag(JOB_CONT, "initscript stdout: %s", result);
 		}
 		xfree(result);
 	}
 
 	rc = mkdir(src_bind, 0700);
-	if (rc && (!remount || errno != EEXIST)) {
-		error("%s: mkdir failed %s, %s",
-		      __func__, src_bind, strerror(errno));
+	if (rc && (errno != EEXIST)) {
+		error("%s: mkdir failed %s, %m", __func__, src_bind);
 		goto exit2;
 	}
 
 	sem1 = mmap(NULL, sizeof(*sem1), PROT_READ|PROT_WRITE,
 		    MAP_SHARED|MAP_ANONYMOUS, -1, 0);
 	if (sem1 == MAP_FAILED) {
-		error("%s: mmap failed: %s", __func__, strerror(errno));
+		error("%s: mmap failed: %m", __func__);
 		rc = -1;
 		goto exit2;
 	}
@@ -500,7 +471,7 @@ static int _create_ns(uint32_t job_id, uid_t uid, bool remount)
 	sem2 = mmap(NULL, sizeof(*sem2), PROT_READ|PROT_WRITE,
 		    MAP_SHARED|MAP_ANONYMOUS, -1, 0);
 	if (sem2 == MAP_FAILED) {
-		error("%s: mmap failed: %s", __func__, strerror(errno));
+		error("%s: mmap failed: %m", __func__);
 		sem_destroy(sem1);
 		munmap(sem1, sizeof(*sem1));
 		rc = -1;
@@ -509,19 +480,19 @@ static int _create_ns(uint32_t job_id, uid_t uid, bool remount)
 
 	rc = sem_init(sem1, 1, 0);
 	if (rc) {
-		error("%s: sem_init: %s", __func__, strerror(errno));
+		error("%s: sem_init: %m", __func__);
 		goto exit1;
 	}
 	rc = sem_init(sem2, 1, 0);
 	if (rc) {
-		error("%s: sem_init: %s", __func__, strerror(errno));
+		error("%s: sem_init: %m", __func__);
 		goto exit1;
 	}
 
 	cpid = fork();
 
 	if (cpid == -1) {
-		error("%s: fork Failed: %s\n", __func__, strerror(errno));
+		error("%s: fork Failed: %m", __func__);
 		rc = -1;
 		goto exit1;
 	}
@@ -529,27 +500,49 @@ static int _create_ns(uint32_t job_id, uid_t uid, bool remount)
 	if (cpid == 0) {
 		rc = unshare(CLONE_NEWNS);
 		if (rc) {
-			error("%s: %s", __func__, strerror(errno));
+			error("%s: %m", __func__);
 			goto child_exit;
 		}
 		if (sem_post(sem1) < 0) {
-			error("%s: sem_post failed: %s",
-			      __func__, strerror(errno));
+			error("%s: sem_post failed: %m", __func__);
 			rc = -1;
 			goto child_exit;
 		}
 		if (sem_wait(sem2) < 0) {
-			error("%s: sem_wait failed %s",
-			      __func__, strerror(errno));
+			error("%s: sem_wait failed %m", __func__);
 			rc = -1;
 			goto child_exit;
 		}
+		if (!jc_conf->shared) {
+			/* Set root filesystem to private */
+			if (mount(NULL, "/", NULL, MS_PRIVATE|MS_REC, NULL)) {
+				error("%s: Failed to make root private: %m",
+				      __func__);
+				rc = -1;
+				goto child_exit;
+			}
+		} else {
+			/* Set root filesystem to shared */
+			if (mount(NULL, "/", NULL, MS_SHARED | MS_REC, NULL)) {
+				error("%s: Failed to make root shared: %m",
+				      __func__);
+				rc = -1;
+				goto child_exit;
+			}
+			/* Set root filesystem to slave */
+			if (mount(NULL, "/", NULL, MS_SLAVE | MS_REC, NULL)) {
+				error("%s: Failed to make root slave: %m",
+				      __func__);
+				rc = -1;
+				goto child_exit;
+			}
+		}
 
 		/*
 		 * Now we have a persistent mount namespace.
-		 * Mount private /tmp inside the namespace.
+		 * Mount private directories inside the namespace.
 		 */
-		if (_mount_private_tmp(src_bind) == -1) {
+		if (_mount_private_dirs(src_bind, step->uid) == -1) {
 			rc = -1;
 			goto child_exit;
 		}
@@ -558,14 +551,22 @@ static int _create_ns(uint32_t job_id, uid_t uid, bool remount)
 		 * this happens when restarting the slurmd, the ownership should
 		 * already be correct here.
 		 */
-		if (!remount) {
-			rc = lchown(src_bind, uid, -1);
-			if (rc) {
-				error("%s: lchown failed for %s: %s",
-				      __func__, src_bind, strerror(errno));
-				rc = -1;
-				goto child_exit;
-			}
+		rc = chown(src_bind, step->uid, -1);
+		if (rc) {
+			error("%s: chown failed for %s: %m",
+			      __func__, src_bind);
+			rc = -1;
+			goto child_exit;
+		}
+
+		/*
+		 * switch/nvidia_imex needs to create an ephemeral device
+		 * node under /dev in this new namespace.
+		 */
+		if ((rc = switch_g_fs_init(step))) {
+			error("%s: switch_g_fs_init failed", __func__);
+			rc = -1;
+			goto child_exit;
 		}
 
 		/*
@@ -574,10 +575,12 @@ static int _create_ns(uint32_t job_id, uid_t uid, bool remount)
 		 * mounts inside the job, they will only see their job mount
 		 * but not the basepath mount.
 		 */
-		rc = umount2(job_mount, MNT_DETACH);
+		if (jc_conf->shared)
+			rc = _clean_job_basepath(job_id);
+		else
+			rc = umount2(job_mount, MNT_DETACH);
 		if (rc) {
-			error("%s: umount2 failed: %s",
-			      __func__, strerror(errno));
+			error("%s: failed to clean job mount(s): %m", __func__);
 			goto child_exit;
 		}
 	child_exit:
@@ -595,41 +598,31 @@ static int _create_ns(uint32_t job_id, uid_t uid, bool remount)
 		exit(rc);
 	} else {
 		int wstatus;
-		char proc_path[PATH_MAX];
+		char *proc_path = NULL;
 
 		if (sem_wait(sem1) < 0) {
-			error("%s: sem_Wait failed: %s",
-			      __func__, strerror(errno));
+			error("%s: sem_Wait failed: %m", __func__);
 			rc = -1;
 			goto exit1;
 		}
 
-		if (snprintf(proc_path, PATH_MAX, "/proc/%u/ns/mnt", cpid)
-		    >= PATH_MAX) {
-			error("%s: Unable to build job %u /proc path: %m",
-			      __func__, job_id);
-			rc = -1;
-			goto exit1;
-		}
+		xstrfmtcat(proc_path, "/proc/%u/ns/mnt", cpid);
 
 		/*
 		 * Bind mount /proc/pid/ns/mnt to hold namespace active
 		 * without a process attached to it
 		 */
-#if !defined(__APPLE__) && !defined(__FreeBSD__)
 		rc = mount(proc_path, ns_holder, NULL, MS_BIND, NULL);
+		xfree(proc_path);
 		if (rc) {
-			error("%s: ns base mount failed: %s",
-			      __func__, strerror(errno));
+			error("%s: ns base mount failed: %m", __func__);
 			if (sem_post(sem2) < 0)
-				error("%s: Could not release semaphore: %s",
-				      __func__, strerror(errno));
+				error("%s: Could not release semaphore: %m",
+				      __func__);
 			goto exit1;
 		}
-#endif
 		if (sem_post(sem2) < 0) {
-			error("%s: sem_post failed: %s",
-			      __func__, strerror(errno));
+			error("%s: sem_post failed: %m", __func__);
 			goto exit1;
 		}
 
@@ -655,53 +648,53 @@ exit2:
 		if ((failures = rmdir_recursive(job_mount, false))) {
 			error("%s: failed to remove %d files from %s",
 			      __func__, failures, job_mount);
-			return SLURM_ERROR;
+			rc = SLURM_ERROR;
+			goto end_it;
 		}
-		umount2(job_mount, MNT_DETACH);
+		if (umount2(job_mount, MNT_DETACH))
+			error("%s: umount2 %s failed: %m",
+			      __func__, job_mount);
+		if (rmdir(job_mount))
+			error("rmdir %s failed: %m", job_mount);
 	}
 
+end_it:
+	xfree(job_mount);
+	xfree(src_bind);
+	xfree(ns_holder);
+
 	return rc;
 }
 
-extern int container_p_create(uint32_t job_id, uid_t uid)
-{
-	return SLURM_SUCCESS;
-}
-
 extern int container_p_join_external(uint32_t job_id)
 {
-	char job_mount[PATH_MAX];
-	char ns_holder[PATH_MAX];
+	char *job_mount = NULL, *ns_holder = NULL;
 
-	if (_create_paths(job_id, job_mount, ns_holder, NULL)
-	    != SLURM_SUCCESS) {
-		return -1;
-	}
+	if (plugin_disabled)
+		return SLURM_SUCCESS;
+
+	_create_paths(job_id, &job_mount, &ns_holder, NULL);
 
 	if (step_ns_fd == -1) {
 		step_ns_fd = open(ns_holder, O_RDONLY);
 		if (step_ns_fd == -1)
-			error("%s: %s", __func__, strerror(errno));
+			error("%s: %m", __func__);
 	}
 
+	xfree(job_mount);
+	xfree(ns_holder);
+
 	return step_ns_fd;
 }
 
-extern int container_p_add_cont(uint32_t job_id, uint64_t cont_id)
-{
-	return SLURM_SUCCESS;
-}
-
 extern int container_p_join(uint32_t job_id, uid_t uid)
 {
-	char job_mount[PATH_MAX];
-	char ns_holder[PATH_MAX];
+	char *job_mount = NULL, *ns_holder = NULL;
 	int fd;
-	int rc = 0;
+	int rc = SLURM_SUCCESS;
 
-#ifdef HAVE_NATIVE_CRAY
-	return SLURM_SUCCESS;
-#endif
+	if (plugin_disabled)
+		return SLURM_SUCCESS;
 
 	/*
 	 * Jobid 0 means we are not a real job, but a script running instead we
@@ -710,74 +703,58 @@ extern int container_p_join(uint32_t job_id, uid_t uid)
 	if (job_id == 0)
 		return SLURM_SUCCESS;
 
-	if (_create_paths(job_id, job_mount, ns_holder, NULL)
-	    != SLURM_SUCCESS) {
-		return SLURM_ERROR;
-	}
+	_create_paths(job_id, &job_mount, &ns_holder, NULL);
 
 	/* This is called on the slurmd so we can't use ns_fd. */
 	fd = open(ns_holder, O_RDONLY);
 	if (fd == -1) {
-		error("%s: open failed for %s: %s",
-		      __func__, ns_holder, strerror(errno));
+		error("%s: open failed for %s: %m", __func__, ns_holder);
+		xfree(job_mount);
+		xfree(ns_holder);
 		return SLURM_ERROR;
 	}
 
 	rc = setns(fd, CLONE_NEWNS);
 	if (rc) {
-		error("%s: setns failed for %s: %s",
-		      __func__, ns_holder, strerror(errno));
-		/* closed after strerror(errno) */
+		error("%s: setns failed for %s: %m", __func__, ns_holder);
+		/* closed after error() */
 		close(fd);
+		xfree(job_mount);
+		xfree(ns_holder);
 		return SLURM_ERROR;
 	} else {
-		debug3("job entered namespace");
+		log_flag(JOB_CONT, "job %u entered namespace", job_id);
 	}
 
 	close(fd);
+	xfree(job_mount);
+	xfree(ns_holder);
 
 	return SLURM_SUCCESS;
 }
 
-static int _delete_ns(uint32_t job_id, bool is_slurmd)
+static int _delete_ns(uint32_t job_id)
 {
-	char job_mount[PATH_MAX];
-	char ns_holder[PATH_MAX];
+	char *job_mount = NULL, *ns_holder = NULL;
 	int rc = 0, failures = 0;
 
-#ifdef HAVE_NATIVE_CRAY
-	return SLURM_SUCCESS;
-#endif
-
-	if (_create_paths(job_id, job_mount, ns_holder, NULL)
-	    != SLURM_SUCCESS) {
-		return SLURM_ERROR;
-	}
+	_create_paths(job_id, &job_mount, &ns_holder, NULL);
 
 	errno = 0;
 
 	/*
-	 * FIXME: only used for SLURM_20_11_PROTOCOL_VERSION
-	 * This is only here to handle upgrades from 20.11, and should be
-	 * removed once upgrading from 20.11 is no longer supported.
+	 * Close the step_ns_fd if it was opened.  If close fails here, it
+	 * should be safe to continue since ns_holder is lazy unmounted later
+	 * and will get cleaned up when the slurmstepd process ends.
 	 */
-	if (is_slurmd && legacy_jobs) {
-		legacy_job_info_t *job;
-		job = list_find_first(legacy_jobs,
-				      (ListFindF)_find_legacy_job_in_list,
-				      &job_id);
-		/* if we didn't find the job, it doesn't need legacy handling */
-		if (!job)
-			return SLURM_SUCCESS;
-		/* cleanup the list */
-		list_delete_first(legacy_jobs,
-				  (ListFindF)_find_legacy_job_in_list,
-				  &job_id);
-		xfree(job);
-		if (list_count(legacy_jobs) == 0)
-			FREE_NULL_LIST(legacy_jobs);
-	} else if (is_slurmd)
-		return SLURM_SUCCESS;
+	if (step_ns_fd != -1) {
+		if (close(step_ns_fd))
+			log_flag(JOB_CONT, "job %u close step_ns_fd(%d) failed: %m",
+				 job_id, step_ns_fd);
+
+		else
+			step_ns_fd = -1;
+	}
 
 	/*
 	 * umount2() sets errno to EINVAL if the target is not a mount point
@@ -786,12 +763,14 @@ static int _delete_ns(uint32_t job_id, bool is_slurmd)
 	 */
 	rc = umount2(ns_holder, MNT_DETACH);
 	if (rc) {
-		if (errno == EINVAL) {
-			debug2("%s: umount2 %s failed: %s",
-			       __func__, ns_holder, strerror(errno));
+		if ((errno == EINVAL) || (errno == ENOENT)) {
+			log_flag(JOB_CONT, "%s: umount2 %s failed: %m",
+				 __func__, ns_holder);
 		} else {
-			error("%s: umount2 %s failed: %s",
-			      __func__, ns_holder, strerror(errno));
+			error("%s: umount2 %s failed: %m",
+			      __func__, ns_holder);
+			xfree(job_mount);
+			xfree(ns_holder);
 			return SLURM_ERROR;
 		}
 	}
@@ -800,24 +779,69 @@ static int _delete_ns(uint32_t job_id, bool is_slurmd)
 		error("%s: failed to remove %d files from %s",
 		      __func__, failures, job_mount);
 	if (umount2(job_mount, MNT_DETACH))
-		debug2("umount2: %s failed: %s", job_mount, strerror(errno));
-	rmdir(job_mount);
+		log_flag(JOB_CONT, "umount2: %s failed: %m", job_mount);
+	if (rmdir(job_mount))
+		error("rmdir %s failed: %m", job_mount);
+
+	xfree(job_mount);
+	xfree(ns_holder);
 
 	return SLURM_SUCCESS;
 }
 
-extern int container_p_delete(uint32_t job_id)
+extern int container_p_stepd_create(uint32_t job_id, stepd_step_rec_t *step)
 {
-	/* change to return SLURM_SUCCESS when 20.11 is not supported) */
-	return _delete_ns(job_id, true);
-}
+	if (plugin_disabled)
+		return SLURM_SUCCESS;
 
-extern int container_p_stepd_create(uint32_t job_id, uid_t uid)
-{
-	return _create_ns(job_id, uid, false);
+	return _create_ns(job_id, step);
 }
 
 extern int container_p_stepd_delete(uint32_t job_id)
 {
-	return _delete_ns(job_id, false);
+	if (plugin_disabled)
+		return SLURM_SUCCESS;
+
+	return _delete_ns(job_id);
+}
+
+extern int container_p_send_stepd(int fd)
+{
+	int len;
+	buf_t *buf;
+
+	buf = get_slurm_jc_conf_buf();
+
+	/* The config should have been inited by now */
+	xassert(buf);
+
+	len = get_buf_offset(buf);
+	safe_write(fd, &len, sizeof(len));
+	safe_write(fd, get_buf_data(buf), len);
+
+	return SLURM_SUCCESS;
+rwfail:
+	error("%s: failed", __func__);
+	return SLURM_ERROR;
+}
+
+extern int container_p_recv_stepd(int fd)
+{
+	int len;
+	buf_t *buf;
+
+	safe_read(fd, &len, sizeof(len));
+
+	buf = init_buf(len);
+	safe_read(fd, buf->head, len);
+
+	if (!(jc_conf = set_slurm_jc_conf(buf)))
+		goto rwfail;
+
+	plugin_disabled = _is_plugin_disabled(jc_conf->basepath);
+
+	return SLURM_SUCCESS;
+rwfail:
+	error("%s: failed", __func__);
+	return SLURM_ERROR;
 }
