@@ -43,6 +43,7 @@
  *  DEPENDENCY		Original list of jobids dependencies
  *  DERIVED_EC		Derived exit code and after : the signal number (if any)
  *  END			Time of job termination, UTS
+ *  ELIGIBLE		Eligble time of job, UTS
  *  EXITCODE		Job's exit code and after : the signal number (if any)
  *  GID			Group ID of job owner
  *  GROUPNAME		Group name of job owner
@@ -92,13 +93,15 @@
 #include "slurm/slurm.h"
 #include "slurm/slurm_errno.h"
 
+#include "src/common/slurm_xlator.h"
+#include "src/common/fd.h"
+#include "src/common/id_util.h"
 #include "src/common/list.h"
 #include "src/common/macros.h"
-#include "src/common/node_select.h"
 #include "src/common/parse_time.h"
-#include "src/common/slurm_jobcomp.h"
+#include "src/interfaces/select.h"
+#include "src/interfaces/jobcomp.h"
 #include "src/common/slurm_protocol_defs.h"
-#include "src/common/uid.h"
 #include "src/common/xmalloc.h"
 #include "src/common/xstring.h"
 #include "src/slurmctld/slurmctld.h"
@@ -132,7 +135,7 @@ const char plugin_name[]       	= "Job completion logging script plugin";
 const char plugin_type[]       	= "jobcomp/script";
 const uint32_t plugin_version	= SLURM_VERSION_NUMBER;
 
-static char * script = NULL;
+static char *jobcomp_script = NULL;
 static List comp_list = NULL;
 
 static pthread_t script_thread = 0;
@@ -162,6 +165,7 @@ struct jobcomp_info {
 	uint16_t batch_flag;
 	time_t submit;
 	time_t start;
+	time_t eligible;
 	time_t end;
 	char *cluster;
 	char *constraints;
@@ -195,9 +199,9 @@ static struct jobcomp_info *_jobcomp_info_create(job_record_t *job)
 	j->state_reason_prev = job->state_reason_prev_db;
 	j->derived_ec = job->derived_ec;
 	j->uid = job->user_id;
-	j->user_name = xstrdup(uid_to_string_cached((uid_t)job->user_id));
+	j->user_name = user_from_job(job);
 	j->gid = job->group_id;
-	j->group_name = gid_to_string((gid_t)job->group_id);
+	j->group_name = group_from_job(job);
 	j->name = xstrdup (job->name);
 	if (job->assoc_ptr && job->assoc_ptr->cluster &&
 	    job->assoc_ptr->cluster[0])
@@ -248,6 +252,10 @@ static struct jobcomp_info *_jobcomp_info_create(job_record_t *job)
 		j->limit = job->part_ptr->max_time;
 	else
 		j->limit = job->time_limit;
+
+	if (job->details && job->details->begin_time)
+		j->eligible = job->details->begin_time;
+
 	j->submit = job->details ? job->details->submit_time:job->start_time;
 	j->batch_flag = job->batch_flag;
 	j->nodes = xstrdup (job->nodes);
@@ -415,6 +423,7 @@ static char ** _create_environment (struct jobcomp_info *job)
 	_env_append_fmt (&env, "START", "%ld", (long)job->start);
 	_env_append_fmt (&env, "END",   "%ld", (long)job->end);
 	_env_append_fmt (&env, "SUBMIT","%ld", (long)job->submit);
+	_env_append_fmt(&env, "ELIGIBLE","%ld", (long)job->eligible);
 	_env_append_fmt (&env, "PROCS", "%u",  job->nprocs);
 	_env_append_fmt (&env, "NODECNT", "%u", job->nnodes);
 
@@ -437,7 +446,7 @@ static char ** _create_environment (struct jobcomp_info *job)
 	_env_append (&env, "USERNAME", job->user_name);
 	_env_append (&env, "GROUPNAME", job->group_name);
 	_env_append (&env, "STATEREASONPREV",
-		     job_reason_string(job->state_reason_prev));
+		     job_state_reason_string(job->state_reason_prev));
 	if (job->std_in)
 		_env_append (&env, "STDIN",     job->std_in);
 	if (job->std_out)
@@ -463,13 +472,20 @@ static int _redirect_stdio (void)
 	int devnull;
 	if ((devnull = open ("/dev/null", O_RDWR)) < 0)
 		return error ("jobcomp/script: Failed to open /dev/null: %m");
-	if (dup2 (devnull, STDIN_FILENO) < 0)
+	if (dup2(devnull, STDIN_FILENO) < 0) {
+		close(devnull);
 		return error ("jobcomp/script: Failed to redirect stdin: %m");
-	if (dup2 (devnull, STDOUT_FILENO) < 0)
+	}
+	if (dup2(devnull, STDOUT_FILENO) < 0) {
+		close(devnull);
 		return error ("jobcomp/script: Failed to redirect stdout: %m");
-	if (dup2 (devnull, STDERR_FILENO) < 0)
+	}
+	if (dup2(devnull, STDERR_FILENO) < 0) {
+		close(devnull);
 		return error ("jobcomp/script: Failed to redirect stderr: %m");
-	close (devnull);
+	}
+	(void) close(devnull);
+	closeall(3);
 	return (0);
 }
 
@@ -491,16 +507,16 @@ static void _jobcomp_child (char * script, struct jobcomp_info *job)
 	log_reinit ();
 
 	if (_redirect_stdio () < 0)
-		exit (1);
+		_exit (1);
 
 	if (chdir (tmpdir) != 0) {
 		error ("jobcomp/script: chdir (%s): %m", tmpdir);
-		exit(1);
+		_exit(1);
 	}
 
 	if (!(env = _create_environment (job))) {
 		error ("jobcomp/script: Failed to create env!");
-		exit (1);
+		_exit (1);
 	}
 
 	execve(script, args, env);
@@ -509,7 +525,7 @@ static void _jobcomp_child (char * script, struct jobcomp_info *job)
 	 * Failure of execve implies error
 	 */
 	error ("jobcomp/script: execve(%s): %m", script);
-	exit (1);
+	_exit (1);
 }
 
 static int _jobcomp_exec_child (char *script, struct jobcomp_info *job)
@@ -563,7 +579,7 @@ static void * _script_agent (void *args)
 		slurm_mutex_unlock(&comp_list_mutex);
 
 		if ((job = list_pop(comp_list))) {
-			_jobcomp_exec_child (script, job);
+			_jobcomp_exec_child(jobcomp_script, job);
 			_jobcomp_info_destroy (job);
 		}
 
@@ -602,8 +618,9 @@ extern int init(void)
 }
 
 /* Set the location of the script to run*/
-extern int slurm_jobcomp_set_location (char * location)
+extern int jobcomp_p_set_location(void)
 {
+	char *location = slurm_conf.job_comp_loc;
 	if (location == NULL) {
 		return error("jobcomp/script JobCompLoc needs to be set");
 	}
@@ -611,13 +628,13 @@ extern int slurm_jobcomp_set_location (char * location)
 	if (_check_script_permissions(location) != SLURM_SUCCESS)
 		return SLURM_ERROR;
 
-	xfree(script);
-	script = xstrdup(location);
+	xfree(jobcomp_script);
+	jobcomp_script = xstrdup(location);
 
 	return SLURM_SUCCESS;
 }
 
-int slurm_jobcomp_log_record(job_record_t *record)
+extern int jobcomp_p_log_record(job_record_t *record)
 {
 	struct jobcomp_info * job;
 
@@ -644,12 +661,11 @@ extern int fini ( void )
 		slurm_mutex_lock(&comp_list_mutex);
 		slurm_cond_broadcast(&comp_list_cond);
 		slurm_mutex_unlock(&comp_list_mutex);
-		pthread_join(script_thread, NULL);
-		script_thread = 0;
+		slurm_thread_join(script_thread);
 	}
 	slurm_mutex_unlock(&thread_flag_mutex);
 
-	xfree(script);
+	xfree(jobcomp_script);
 	slurm_mutex_lock(&comp_list_mutex);
 	FREE_NULL_LIST(comp_list);
 	slurm_mutex_unlock(&comp_list_mutex);
@@ -662,18 +678,9 @@ extern int fini ( void )
  * in/out job_list List of job_rec_t *
  * note List needs to be freed when called
  */
-extern List slurm_jobcomp_get_jobs(slurmdb_job_cond_t *job_cond)
+extern List jobcomp_p_get_jobs(slurmdb_job_cond_t *job_cond)
 {
 
 	info("This function is not implemented.");
 	return NULL;
 }
-
-/*
- * expire old info from the storage
- */
-extern int slurm_jobcomp_archive(slurmdb_archive_cond_t *archive_cond)
-{
-	info("This function is not implemented.");
-	return SLURM_SUCCESS;
-}
