@@ -10,7 +10,7 @@
  * This file is part of Slurm, a resource management program.
  * For details, see <https://slurm.schedmd.com/>.
  * Please also read the included file: DISCLAIMER.
- * 
+ *
  * LLNL Preamble Notice
  *
  * A. This notice is required to be provided under our contract with
@@ -105,12 +105,13 @@
  *   right - right value for rightmost app process in subtree
  */
 
+#include <signal.h>
 #include <stdlib.h>
 #include <unistd.h>
 
 #include "src/common/slurm_xlator.h"
-#include "src/common/slurm_protocol_interface.h"
 #include "src/common/slurm_protocol_api.h"
+#include "src/common/slurm_protocol_socket.h"
 #include "src/common/xmalloc.h"
 
 #include "pmi.h"
@@ -158,7 +159,7 @@ static int pmix_stepd_width = 16;
 static int pmix_stepd_children = 0;
 
 /* we allocate a hostlist in init and destroy it in finalize */
-static hostlist_t pmix_stepd_hostlist = NULL;
+static hostlist_t *pmix_stepd_hostlist = NULL;
 
 /* return rank of our parent in stepd tree,
  * returns -1 if we're the root */
@@ -229,8 +230,9 @@ static int pmix_stepd_send(const char* buf, uint32_t size, int rank)
 		retries++;
 		if (retries >= MAX_RETRIES) {
 			/* cancel the step to avoid tasks hang */
-			slurm_kill_job_step(job_info.jobid, job_info.stepid,
-					    SIGKILL);
+			slurm_kill_job_step(job_info.step_id.job_id,
+					    job_info.step_id.step_id,
+					    SIGKILL, 0);
 		}
 
 		/* didn't succeeded, but we'll retry again,
@@ -243,7 +245,7 @@ static int pmix_stepd_send(const char* buf, uint32_t size, int rank)
 	free(host); /* strdup-ed */
 
 	return rc;
-} 
+}
 
 /* allocate resources to track PMIX_Ring state */
 int pmix_ring_init(const pmi2_job_info_t* job, char*** env)
@@ -337,9 +339,7 @@ int pmix_ring_finalize()
 	}
 
 	/* free host list */
-	if (pmix_stepd_hostlist != NULL) {
-		hostlist_destroy(pmix_stepd_hostlist);
-        }
+	FREE_NULL_HOSTLIST(pmix_stepd_hostlist);
 
 	return rc;
 }
@@ -417,7 +417,7 @@ int pmix_ring_out(int count, char* left, char* right)
 		/* TODO: do we need hton translation? */
 
 		/* construct message */
-		Buf buf = init_buf(1024);
+		buf_t *buf = init_buf(1024);
 		pack16(TREE_CMD_RING_RESP,    buf); /* specify message type (RING_OUT) */
 		pack32((uint32_t) msg->count, buf); /* send count value */
 		packstr(msg->left,            buf); /* send left value */
@@ -435,7 +435,7 @@ int pmix_ring_out(int count, char* left, char* right)
 		/* TODO: use tmp_rc here to catch any failure */
 
 		/* free message */
-		free_buf(buf);
+		FREE_NULL_BUFFER(buf);
 	}
 
 	/* now send messages to children app procs,
@@ -547,7 +547,7 @@ int pmix_ring_in(int ring_id, int count, char* left, char* right)
 			/* TODO: do we need hton translation? */
 
 			/* construct message */
-			Buf buf = init_buf(1024);
+			buf_t *buf = init_buf(1024);
 			pack16(TREE_CMD_RING, buf); /* specify message type (RING_IN) */
 			pack32(my_rank,       buf); /* send our rank */
 			pack32(sum,           buf); /* send count value */
@@ -566,7 +566,7 @@ int pmix_ring_in(int ring_id, int count, char* left, char* right)
 			/* TODO: use tmp_rc here to catch any failure */
 
 			/* free message */
-			free_buf(buf);
+			FREE_NULL_BUFFER(buf);
 		} else {
 			/* we're the root of the tree, send values back down */
 
