@@ -1,7 +1,7 @@
 /*****************************************************************************\
  *  setup_nic.c - Library for managing HPE Slingshot networks
  *****************************************************************************
- *  Copyright 2021 Hewlett Packard Enterprise Development LP
+ *  Copyright 2021-2023 Hewlett Packard Enterprise Development LP
  *  Written by Jim Nordby <james.nordby@hpe.com>
  *
  *  This file is part of Slurm, a resource management program.
@@ -52,6 +52,7 @@ static void *cxi_handle = NULL;
 static bool cxi_avail = false;
 static struct cxil_dev **cxi_devs = NULL;
 static int cxi_ndevs = 0;
+static bool rdzv_get_en_default = true;
 
 /* Define struct not defined in earlier versions of libcxi */
 #ifndef HAVE_STRUCT_CXI_RSRC_USE
@@ -171,7 +172,8 @@ static bool _adjust_dev_limits(int dev, struct cxil_devinfo *devinfo)
 		}
 		num_system_svc++;
 #if HAVE_STRUCT_CXI_RSRC_USE
-		if ((rc = cxil_get_svc_rsrc_use_p(cxi_devs[dev],
+		if (cxil_get_svc_rsrc_use_p &&
+		    (rc = cxil_get_svc_rsrc_use_p(cxi_devs[dev],
 						  list->descs[svc].svc_id,
 						  &usage))) {
 			error("Could not get resource usage for CXI dev_id=%d (%s) svc_id=%d: %s",
@@ -208,7 +210,7 @@ static bool _adjust_dev_limits(int dev, struct cxil_devinfo *devinfo)
 /*
  * Set up basic access to the CXI devices in the daemon
  */
-static bool _create_cxi_devs(slingshot_jobinfo_t *job)
+static bool _create_cxi_devs(slingshot_stepinfo_t *job)
 {
 	struct cxil_device_list *list;
 	int dev, rc;
@@ -285,7 +287,7 @@ static struct cxi_limits _set_desc_limits(int rsrc,
  */
 static void _create_cxi_descriptor(struct cxi_svc_desc *desc,
 				   const struct cxil_devinfo *devinfo,
-				   const slingshot_jobinfo_t *job,
+				   const slingshot_stepinfo_t *job,
 				   uint32_t uid, uint16_t step_cpus)
 {
 	int cpus;
@@ -359,7 +361,7 @@ static void _create_cxi_descriptor(struct cxi_svc_desc *desc,
  * Open the Slingshot CXI library; set up functions and set cxi_avail
  * if successful (default is 'false')
  */
-extern bool slingshot_open_cxi_lib(slingshot_jobinfo_t *job)
+extern bool slingshot_open_cxi_lib(slingshot_stepinfo_t *job)
 {
 	if (!(cxi_handle = dlopen(HPE_SLINGSHOT_LIB,
 				  RTLD_LAZY | RTLD_GLOBAL))) {
@@ -397,253 +399,6 @@ static struct cxil_dev *_device_name_to_dev(const char *devname)
 	return NULL;
 }
 
-/*
- * Open, write-lock, and read 'fname'; convert the (hex string) contents
- * of the file to a bitstr_t and return in *retbitsp;
- * also fill in the file size in *fsizep.
- * Returns the file descriptor on success, or -1 on failure
- */
-static int _read_bitstr_file(const char *fname, bitstr_t **retbitsp,
-			     size_t *fsizep)
-{
-	int fd, rc;
-	size_t fsize;
-	struct flock lock;
-	struct stat statbuf;
-
-	fd = open(fname, (O_RDWR | O_CREAT), 0644);
-	if (fd == -1) {
-		error("open(%s) failed: %m", fname);
-		return -1;
-	}
-
-	/*
-	 * Exclusive lock on the first byte of the file
-	 * Automatically released when the file descriptor is closed
-	 */
-	lock.l_type = F_WRLCK;
-	lock.l_whence = SEEK_SET;
-	lock.l_start = 0;
-	lock.l_len = 1;
-	rc = fcntl(fd, F_SETLKW, &lock);
-	if (rc == -1) {
-		error("fcntl(F_SETLKW, %s) failed: %m", fname);
-		goto errout;
-	}
-
-	/* Read in the whole file, convert to bitstr_t */
-	if (fstat(fd, &statbuf) == -1) {
-		error("fstat(%s) failed: %m", fname);
-		goto errout;
-	}
-	fsize = statbuf.st_size;
-	if (fsize > SLINGSHOT_VNI_PIDS_BUFSIZ) {
-		error("%s too large: %zu vs max %d",
-			fname, fsize, SLINGSHOT_VNI_PIDS_BUFSIZ);
-		goto errout;
-	}
-	*retbitsp = bit_alloc(SLINGSHOT_VNI_PIDS);
-	/* Non-empty file; read it in, convert to bitstr_t */
-	if (fsize > 0) {
-		char buf[SLINGSHOT_VNI_PIDS_BUFSIZ];
-		ssize_t sret = read(fd, buf, fsize);
-		if (sret == -1) {
-			error("read(%s, %zu) failed: %m", fname, fsize);
-			goto errout;
-		} else if (sret != fsize) {
-			error("read(%s, %zu) ret %zu: %m", fname, fsize, sret);
-			goto errout;
-		}
-		log_flag(SWITCH, "read(%s, %zu) contents='%s'",
-			fname, fsize, buf);
-		/* Convert from hex bitmask to bitstr_t */
-		if (bit_unfmt_hexmask(*retbitsp, buf) == -1) {
-			error("%s: invalid VNI pids bitmask '%s'", fname, buf);
-			goto errout;
-		}
-	}
-	*fsizep = fsize;
-	return fd;
-errout:
-	close(fd);
-	return -1;
-}
-
-/*
- * Convert the bitstr_t into a hex string and replace the contents of the file
- * with name 'fname', file descriptor 'fd', and size 'oldsize';
- * if the bitstring has no bits set, unlink the file
- */
-static bool _write_bitstr_file(const char *fname, int fd, size_t oldsize,
-			       bitstr_t *bitstr)
-{
-	char *mask = NULL;
-	bool retval = false;
-	/*
-	 * If bitstr has any bits set, convert it to a hex string,
-	 * then overwrite the file; if no bits left, remove file
-	 */
-	if (bit_ffs(bitstr) >= 0) {
-		size_t fsize;
-		ssize_t sret;
-		mask = bit_fmt_hexmask_trim(bitstr);
-		 fsize = strlen(mask) + 1;	/* write NULL on end */
-		xassert(fsize <= SLINGSHOT_VNI_PIDS_BUFSIZ);
-		if (fsize < oldsize) {
-			if (ftruncate(fd, 0) == -1) {
-				error("ftruncate(%s) failed: %m", fname);
-				goto out;
-			}
-		}
-		sret = pwrite(fd, mask, fsize, 0);
-		log_flag(SWITCH, "pwrite(%s, '%s', %zu)", fname, mask, fsize);
-		if (sret == -1) {
-			error("pwrite(%d, %zu) failed: %m", fd, fsize);
-			goto out;
-		} else if (sret != fsize) {
-			error("pwrite(%d, %zu) ret %zu: %m", fd, fsize, sret);
-			goto out;
-		}
-	} else {
-		log_flag(SWITCH, "unlink(%s)", fname);
-		if (unlink(fname) == -1) {
-			error("unlink(%s) failed: %m", fname);
-			goto out;
-		}
-	}
-	retval = true;
-out:
-	xfree(mask);
-	return retval;
-}
-
-/*
- * If this job step has a job VNI, allocate a set of Slingshot "PIDs"
- * (a set of 'step_cpus' numbers 0-255 that are disjoint from the sets
- * used by any other job step in the job on this node).
- * Write a hex representation of this set into a job-specific file.
- * Returns true/false on success/failure.
- */
-static bool _alloc_vni_pids(slingshot_jobinfo_t *step, int step_cpus,
-			    uint32_t job_id)
-{
-	int bit, fd = -1, num_bits = 0;
-	char *fname = NULL;
-	size_t fsize;
-	bitstr_t *job_vni_pids = NULL;
-	bool retval = false;	/* assume failure */
-
-	/* Return if VNI PIDs not configured, or no job VNI */
-	if (!(step->flags & SLINGSHOT_FLAGS_VNI_PIDS) || step->num_vnis < 2)
-		return true;
-
-	/* Create filename for this job's allocated VNI PIDs */
-	xstrfmtcat(fname, SLINGSHOT_VNI_PIDS_FMT,
-		   slurm_conf.slurmd_spooldir, job_id);
-	log_flag(SWITCH, "job_id=%u cpus=%d fname=%s",
-		 job_id, step_cpus, fname);
-	fd = _read_bitstr_file(fname, &job_vni_pids, &fsize);
-	if (fd == -1)
-		goto out;
-
-	/* Allocate step_cpus bits from job_vni_pids, add to step->vni_pids */
-	xassert(step->vni_pids == NULL);
-	step->vni_pids = bit_alloc(SLINGSHOT_VNI_PIDS);
-	if (fsize == 0) {
-		/*
-		 * File just created, must be the first step;
-		 * allocate step_cpus bits from bitmask
-		 */
-		bit_nset(job_vni_pids, 0, step_cpus - 1);
-		bit_nset(step->vni_pids, 0, step_cpus - 1);
-		log_flag(SWITCH, "cpus=%d startbit=0", step_cpus);
-	} else if ((bit = bit_nffc(job_vni_pids, step_cpus)) >= 0) {
-		/*
-		 * Subsequent job steps:
-		 * allocate step_cpus free bits from per-job bitstring
-		 * First try for configuous bits
-		 */
-		bit_nset(job_vni_pids, bit, bit + step_cpus - 1);
-		bit_nset(step->vni_pids, bit, bit + step_cpus - 1);
-		log_flag(SWITCH, "cpus=%d startbit=%d", step_cpus, bit);
-	} else {
-		/* Not enough contiguous bits: alloc a bit at a time */
-		for (num_bits = 0; num_bits < step_cpus; num_bits++) {
-			if ((bit = bit_ffc(job_vni_pids)) == -1)
-				break;
-			bit_set(job_vni_pids, bit);
-			bit_set(step->vni_pids, bit);
-		}
-		log_flag(SWITCH, "got num_bits %d/%d", num_bits, step_cpus);
-		if (num_bits < step_cpus)
-			goto out;
-	}
-
-	/* Write the new bitstring out to the file */
-	if (!_write_bitstr_file(fname, fd, fsize, job_vni_pids))
-		goto out;
-
-	retval = true;
-out:
-	if (job_vni_pids)
-		bit_free(job_vni_pids);
-	if (fd >= 0)
-		close(fd);
-	xfree(fname);
-	return retval;
-}
-
-/*
- * If this job step has a set of VNI "PIDs", remove them from
- * the set of allocated "PIDs" for this job.
- * Returns true/false on success/failure.
- */
-static bool _free_vni_pids(slingshot_jobinfo_t *step, uint32_t job_id)
-{
-	bool retval = false;	/* assume failure */
-	bitstr_t *job_vni_pids = NULL;
-	int fd = -1;
-	char *fname = NULL;
-	size_t fsize;
-
-	/* Return if no VNI PIDs */
-	if (!step->vni_pids)
-		return true;
-
-	/* Create filename for this job's allocated VNI PIDs */
-	xstrfmtcat(fname, SLINGSHOT_VNI_PIDS_FMT,
-		   slurm_conf.slurmd_spooldir, job_id);
-	log_flag(SWITCH, "job_id=%u fname=%s", job_id, fname);
-	fd = _read_bitstr_file(fname, &job_vni_pids, &fsize);
-	if (fd == -1)
-		goto out;
-
-	/* Remove job step's VNI PIDs from job's VNI PIDS */
-	xassert(bit_super_set(step->vni_pids, job_vni_pids) == 1);
-	bit_and_not(job_vni_pids, step->vni_pids);
-	if (slurm_conf.debug_flags & DEBUG_FLAG_SWITCH) {
-		char *jpids = bit_fmt_full(job_vni_pids);
-		char *spids = bit_fmt_full(step->vni_pids);
-		log_flag(SWITCH, "free: step %s remaining %s",
-			spids, jpids);
-		xfree(spids);
-		xfree(jpids);
-	}
-
-	/* Write the new bitstring out to the file */
-	if (!_write_bitstr_file(fname, fd, fsize, job_vni_pids))
-		goto out;
-
-	retval = true;
-out:
-	if (job_vni_pids)
-		bit_free(job_vni_pids);
-	if (fd >= 0)
-		close(fd);
-	xfree(fname);
-	return retval;
-}
-
 /*
  * Attempt to destroy a CXI service; retry a few times on EBUSY
  */
@@ -667,11 +422,91 @@ static bool _destroy_cxi_service(struct cxil_dev *dev, const char *devname,
 	return false;
 }
 
+/*
+ * Determine whether "rdzv_get_en" is enabled or disabled on this node
+ * Returns true on failure, since this parameter is enabled by default
+ */
+static bool _get_rdzv_get_en_default(void)
+{
+	bool enabled = true;
+	FILE *fp = NULL;
+	int param;
+
+	/* Open the file */
+	if (!(fp = fopen(SLINGSHOT_RDZV_GET_EN_DEFAULT_FILE, "r"))) {
+		error("Couldn't open %s for reading: %m",
+		      SLINGSHOT_RDZV_GET_EN_DEFAULT_FILE);
+		return enabled;
+	}
+
+	/* The file will contain a single character, Y/y/1/N/n/0 */
+	param = fgetc(fp);
+	switch (param) {
+	case 'Y':
+	case 'y':
+	case '1':
+		enabled = true;
+		break;
+	case 'N':
+	case 'n':
+	case '0':
+		enabled = false;
+		break;
+	case EOF:
+		error("Couldn't read from %s: %m",
+		      SLINGSHOT_RDZV_GET_EN_DEFAULT_FILE);
+		break;
+	default:
+		error("Unexpected char '%c' from %s",
+		      param, SLINGSHOT_RDZV_GET_EN_DEFAULT_FILE);
+		break;
+	}
+
+	log_flag(SWITCH, "Rendezvous gets are %s by default",
+		 enabled ? "enabled" : "disabled");
+	fclose(fp);
+	return enabled;
+}
+
+/*
+ * Configure rendezvous gets for the given device.
+ * Set to 1 to enable, or 0 to disable.
+ * Returns the value written, or -1 on failure.
+ */
+static int _set_rdzv_get_en(int device, int val)
+{
+	char *fname = NULL;
+	FILE *fp = NULL;
+
+	/* Get the file name to write to */
+	xstrfmtcat(fname, SLINGSHOT_RDZV_GET_EN_FMT, device);
+
+	/* Open the file */
+	if (!(fp = fopen(fname, "w"))) {
+		error("Couldn't open %s for writing: %m", fname);
+		xfree(fname);
+		return -1;
+	}
+
+	/* Write to the file */
+	log_flag(SWITCH, "Writing %d to %s", val, fname);
+	if (fprintf(fp, "%d", val) < 0) {
+		error("Couldn't write %d to %s: %m", val, fname);
+		fclose(fp);
+		xfree(fname);
+		return -1;
+	}
+
+	fclose(fp);
+	xfree(fname);
+	return val;
+}
+
 /*
  * In the daemon, when the shepherd for an App terminates, free any CXI
  * Services we have allocated for it
  */
-extern bool slingshot_destroy_services(slingshot_jobinfo_t *job,
+extern bool slingshot_destroy_services(slingshot_stepinfo_t *job,
 				       uint32_t job_id)
 {
 	bool retval = true;
@@ -700,20 +535,17 @@ extern bool slingshot_destroy_services(slingshot_jobinfo_t *job,
 		/* Try to destroy service (with retries) */
 		if (!_destroy_cxi_service(dev, devname, svc_id))
 			retval = false;
+
+		/* Reset rendezvous gets to default */
+		if (rdzv_get_en_default &&
+		    (job->flags & SLINGSHOT_FLAGS_DISABLE_RDZV_GET))
+			_set_rdzv_get_en(dev->info.dev_id, 1);
 	}
 
 	xfree(job->profiles);
 	job->profiles = NULL;
 	job->num_profiles = 0;
 
-	/* De-allocate this step's VNI "PIDs" from job's set */
-	if (job->vni_pids) {
-		if (!_free_vni_pids(job, job_id))
-			retval = false;
-		bit_free(job->vni_pids);
-		job->vni_pids = NULL;
-	}
-
 	return retval;
 }
 
@@ -772,7 +604,7 @@ static void _alloc_fail_info(struct cxil_dev *dev,
 /*
  * Set up CXI services for each of the CXI NICs on this host
  */
-extern bool slingshot_create_services(slingshot_jobinfo_t *job, uint32_t uid,
+extern bool slingshot_create_services(slingshot_stepinfo_t *job, uint32_t uid,
 				      uint16_t step_cpus, uint32_t job_id)
 {
 	int prof, devn;
@@ -793,6 +625,9 @@ extern bool slingshot_create_services(slingshot_jobinfo_t *job, uint32_t uid,
 		return true;
 	}
 
+	/* Determine whether rendezvous gets are enabled */
+	rdzv_get_en_default = _get_rdzv_get_en_default();
+
 	/* Figure out number of working NICs = services to create */
 	job->num_profiles = 0;
 	for (int i = 0; i < cxi_ndevs; i++) {
@@ -819,6 +654,11 @@ extern bool slingshot_create_services(slingshot_jobinfo_t *job, uint32_t uid,
 			goto error;
 		}
 
+		/* Disable rendezvous gets if requested */
+		if (rdzv_get_en_default &&
+		    (job->flags & SLINGSHOT_FLAGS_DISABLE_RDZV_GET))
+			_set_rdzv_get_en(dev->info.dev_id, 0);
+
 		profile = &job->profiles[prof];
 		profile->svc_id = svc_id;
 		for (int v = 0; v < job->num_vnis; v++)
@@ -836,13 +676,6 @@ extern bool slingshot_create_services(slingshot_jobinfo_t *job, uint32_t uid,
 		prof++;
 	}
 
-	/*
-	 * If this job step has a job VNI, allocate a set of "PIDs" (0-255)
-	 * so the MPI library can distinguish between job steps within the job
-	 */
-	if (!_alloc_vni_pids(job, step_cpus, job_id))
-		goto error;
-
 	return true;
 
 error:
