@@ -40,6 +40,7 @@
 
 #include "src/sacctmgr/sacctmgr.h"
 #include "src/common/assoc_mgr.h"
+#include "src/interfaces/data_parser.h"
 
 static int _set_cond(int *start, int argc, char **argv,
 		     slurmdb_account_cond_t *acct_cond,
@@ -83,15 +84,15 @@ static int _set_cond(int *start, int argc, char **argv,
 		} else if (!end &&
 			   !xstrncasecmp(argv[i], "WithAssoc",
 					 MAX(command_len, 5))) {
-			acct_cond->with_assocs = 1;
+			acct_cond->flags |= SLURMDB_ACCT_FLAG_WASSOC;
 		} else if (!end &&
 			   !xstrncasecmp(argv[i], "WithCoordinators",
 					 MAX(command_len, 5))) {
-			acct_cond->with_coords = 1;
+			acct_cond->flags |= SLURMDB_ACCT_FLAG_WCOORD;
 		} else if (!end &&
 			   !xstrncasecmp(argv[i], "WithDeleted",
 					 MAX(command_len, 5))) {
-			acct_cond->with_deleted = 1;
+			acct_cond->flags |= SLURMDB_ACCT_FLAG_DELETED;
 			assoc_cond->with_deleted = 1;
 		} else if (!end &&
 			   !xstrncasecmp(argv[i], "WithRawQOSLevel",
@@ -126,8 +127,13 @@ static int _set_cond(int *start, int argc, char **argv,
 			if (slurm_addto_char_list(acct_cond->description_list,
 						 argv[i]+end))
 				cond_set |= SA_SET_USER;
+		} else if (!xstrncasecmp(argv[i], "Flags",
+					 MAX(command_len, 2))) {
+			acct_cond->flags |= str_2_slurmdb_acct_flags(
+				argv[i]+end);
+			cond_set |= SA_SET_USER;
 		} else if (!xstrncasecmp(argv[i], "Format",
-					 MAX(command_len, 1))) {
+					 MAX(command_len, 2))) {
 			if (format_list)
 				slurm_addto_char_list(format_list, argv[i]+end);
 		} else if (!xstrncasecmp(argv[i], "Organizations",
@@ -220,6 +226,10 @@ static int _set_rec(int *start, int argc, char **argv,
 			acct->description =  strip_quotes(argv[i]+end, NULL, 1);
 			rec_set |= SA_SET_USER;
 
+		} else if (!xstrncasecmp(argv[i], "Flags",
+					 MAX(command_len, 2))) {
+			acct->flags = str_2_slurmdb_acct_flags(argv[i]+end);
+			rec_set |= SA_SET_USER;
 		} else if (!xstrncasecmp(argv[i], "Organization",
 					 MAX(command_len, 1))) {
 			acct->organization = strip_quotes(argv[i]+end, NULL, 1);
@@ -252,416 +262,83 @@ static int _set_rec(int *start, int argc, char **argv,
 	return rec_set;
 }
 
-static int _isdefault_old(List acct_list)
-{
-	int rc = 0;
-	slurmdb_user_cond_t user_cond;
-	List ret_list = NULL;
-
-	if (!acct_list || !list_count(acct_list))
-		return rc;
-
-	memset(&user_cond, 0, sizeof(slurmdb_user_cond_t));
-	user_cond.def_acct_list = acct_list;
-
-	ret_list = slurmdb_users_get(db_conn, &user_cond);
-	if (ret_list && list_count(ret_list)) {
-		ListIterator itr = list_iterator_create(ret_list);
-		slurmdb_user_rec_t *user = NULL;
-		fprintf(stderr," Users listed below have these "
-			"as their Default Accounts.\n");
-		while((user = list_next(itr))) {
-			fprintf(stderr, " User - %-10.10s Account - %s\n",
-				user->name, user->default_acct);
-		}
-		list_iterator_destroy(itr);
-		rc = 1;
-	}
-
-	FREE_NULL_LIST(ret_list);
-
-	return rc;
-}
-
-static int _isdefault(int cond_set, List acct_list, List assoc_list)
-{
-	int rc = 0;
-	ListIterator itr = NULL;
-	ListIterator itr2 = NULL;
-	char *acct;
-	char *output = NULL;
-	slurmdb_assoc_rec_t *assoc = NULL;
-
-	if (!acct_list || !list_count(acct_list)
-	    || !assoc_list || !list_count(assoc_list))
-		return rc;
-
-	/* Since not all plugins have been converted to the new style
-	   of default accounts we have to handle those that aren't.
-	   If the plugin have been converted all the associations here
-	   will have is_def set.
-	*/
-	assoc = list_peek(assoc_list);
-	if (!assoc->is_def)
-		return _isdefault_old(acct_list);
-
-	itr = list_iterator_create(acct_list);
-	itr2 = list_iterator_create(assoc_list);
-	while ((acct = list_next(itr))) {
-		while ((assoc = list_next(itr2))) {
-			char tmp[1000];
-			/* The pgsql plugin doesn't have the idea of
-			   only_defs, so thre query could return all
-			   the associations, even without defaults. */
-			if (cond_set == SA_SET_USER) {
-				if (xstrcasecmp(acct, assoc->acct))
-					continue;
-			} else {
-				snprintf(tmp, 1000, " A = %s ", assoc->acct);
-				if (!strstr(acct, tmp))
-					continue;
-			}
-			snprintf(tmp, 1000, "C = %-10s A = %-20s U = %-9s\n",
-				 assoc->cluster, assoc->acct, assoc->user);
-			if (output && strstr(output, tmp))
-				continue;
-
-			xstrcat(output, tmp);
-			rc = 1;
-		}
-		list_iterator_reset(itr2);
-	}
-	list_iterator_destroy(itr);
-	list_iterator_destroy(itr2);
-	if (output) {
-		fprintf(stderr," Users listed below have these "
-			"as their Default Accounts.\n%s", output);
-		xfree(output);
-	}
-	return rc;
-}
-
 extern int sacctmgr_add_account(int argc, char **argv)
 {
 	int rc = SLURM_SUCCESS;
 	int i=0;
-	ListIterator itr = NULL, itr_c = NULL;
-	slurmdb_account_rec_t *acct = NULL;
-	slurmdb_assoc_rec_t *assoc = NULL;
-	slurmdb_assoc_cond_t assoc_cond;
-	List name_list = list_create(xfree_ptr);
-	List cluster_list = list_create(xfree_ptr);
-	char *cluster = NULL;
-	char *name = NULL;
-	List acct_list = NULL;
-	List assoc_list = NULL;
-	List local_assoc_list = NULL;
-	List local_account_list = NULL;
-	char *acct_str = NULL;
-	char *assoc_str = NULL;
+	char *ret_str = NULL;
 	int limit_set = 0;
-	slurmdb_account_rec_t *start_acct =
-		xmalloc(sizeof(slurmdb_account_rec_t));
-	slurmdb_assoc_rec_t *start_assoc =
-		xmalloc(sizeof(slurmdb_assoc_rec_t));
+	slurmdb_add_assoc_cond_t add_assoc;
+	slurmdb_account_rec_t *start_acct = xmalloc(sizeof(*start_acct));
+	slurmdb_assoc_rec_t *start_assoc;
 
-	slurmdb_init_assoc_rec(start_assoc, 0);
+	slurmdb_init_add_assoc_cond(&add_assoc, 0);
+	start_assoc = &add_assoc.assoc;
+	add_assoc.acct_list = list_create(xfree_ptr);
+	add_assoc.cluster_list = list_create(xfree_ptr);
 
 	for (i = 0; i < argc; i++) {
 		int command_len = strlen(argv[i]);
 		if (!xstrncasecmp(argv[i], "Where", MAX(command_len, 5))
 		    || !xstrncasecmp(argv[i], "Set", MAX(command_len, 3)))
 			i++;
-		limit_set += _set_rec(&i, argc, argv, name_list, cluster_list,
+		limit_set += _set_rec(&i, argc, argv,
+				      add_assoc.acct_list,
+				      add_assoc.cluster_list,
 				      start_acct, start_assoc);
 	}
 	if (exit_code) {
-		FREE_NULL_LIST(name_list);
-		FREE_NULL_LIST(cluster_list);
-		slurmdb_destroy_assoc_rec(start_assoc);
 		slurmdb_destroy_account_rec(start_acct);
 		return SLURM_ERROR;
 	}
 
-	if (!name_list || !list_count(name_list)) {
-		FREE_NULL_LIST(name_list);
-		FREE_NULL_LIST(cluster_list);
-		slurmdb_destroy_assoc_rec(start_assoc);
+	if (!list_count(add_assoc.acct_list)) {
+		FREE_NULL_LIST(add_assoc.acct_list);
+		FREE_NULL_LIST(add_assoc.cluster_list);
 		slurmdb_destroy_account_rec(start_acct);
 		exit_code = 1;
 		fprintf(stderr, " Need name of account to add.\n");
 		return SLURM_SUCCESS;
-	} else {
-		slurmdb_account_cond_t account_cond;
-		memset(&account_cond, 0, sizeof(slurmdb_account_cond_t));
-		memset(&assoc_cond, 0, sizeof(slurmdb_assoc_cond_t));
-
-		assoc_cond.acct_list = name_list;
-		account_cond.assoc_cond = &assoc_cond;
-
-		local_account_list = slurmdb_accounts_get(
-			db_conn, &account_cond);
-	}
-
-	if (!local_account_list) {
-		exit_code = 1;
-		fprintf(stderr, " Problem getting accounts from database.  "
-			"Contact your admin.\n");
-		FREE_NULL_LIST(name_list);
-		FREE_NULL_LIST(cluster_list);
-		slurmdb_destroy_assoc_rec(start_assoc);
-		slurmdb_destroy_account_rec(start_acct);
-		return SLURM_ERROR;
-	}
-
-	if (!start_assoc->parent_acct)
-		start_assoc->parent_acct = xstrdup("root");
-
-	if (!cluster_list || !list_count(cluster_list)) {
-		slurmdb_cluster_rec_t *cluster_rec = NULL;
-		List tmp_list =
-			slurmdb_clusters_get(db_conn, NULL);
-		if (!tmp_list) {
-			exit_code=1;
-			fprintf(stderr,
-				" Problem getting clusters from database.  "
-				"Contact your admin.\n");
-			FREE_NULL_LIST(name_list);
-			FREE_NULL_LIST(cluster_list);
-			slurmdb_destroy_assoc_rec(start_assoc);
-			slurmdb_destroy_account_rec(start_acct);
-			FREE_NULL_LIST(local_account_list);
-			return SLURM_ERROR;
-		}
-
-		if (!list_count(tmp_list)) {
-			exit_code=1;
-			fprintf(stderr,
-				"  Can't add accounts, no cluster "
-				"defined yet.\n"
-				" Please contact your administrator.\n");
-			FREE_NULL_LIST(name_list);
-			FREE_NULL_LIST(cluster_list);
-			slurmdb_destroy_assoc_rec(start_assoc);
-			slurmdb_destroy_account_rec(start_acct);
-			FREE_NULL_LIST(local_account_list);
-			return SLURM_ERROR;
-		}
-		if (!cluster_list)
-			list_create(xfree_ptr);
-		else
-			list_flush(cluster_list);
-
-		itr_c = list_iterator_create(tmp_list);
-		while((cluster_rec = list_next(itr_c))) {
-			if (cluster_rec->flags & CLUSTER_FLAG_EXT)
-				continue;
-			list_append(cluster_list, xstrdup(cluster_rec->name));
-		}
-		list_iterator_destroy(itr_c);
-		FREE_NULL_LIST(tmp_list);
-	} else if (sacctmgr_validate_cluster_list(cluster_list)
-		   != SLURM_SUCCESS) {
-		slurmdb_destroy_assoc_rec(start_assoc);
-		slurmdb_destroy_account_rec(start_acct);
-		FREE_NULL_LIST(local_account_list);
-
-		return SLURM_ERROR;
-	}
-
-
-	acct_list = list_create(slurmdb_destroy_account_rec);
-	assoc_list = list_create(slurmdb_destroy_assoc_rec);
-
-	memset(&assoc_cond, 0, sizeof(slurmdb_assoc_cond_t));
-
-	assoc_cond.acct_list = list_create(NULL);
-	itr = list_iterator_create(name_list);
-	while((name = list_next(itr)))
-		list_append(assoc_cond.acct_list, name);
-	list_iterator_destroy(itr);
-	list_append(assoc_cond.acct_list, start_assoc->parent_acct);
-
-	assoc_cond.cluster_list = cluster_list;
-	local_assoc_list = slurmdb_associations_get(
-		db_conn, &assoc_cond);
-	FREE_NULL_LIST(assoc_cond.acct_list);
-	if (!local_assoc_list) {
-		exit_code=1;
-		fprintf(stderr, " Problem getting associations from database.  "
-			"Contact your admin.\n");
-		FREE_NULL_LIST(name_list);
-		FREE_NULL_LIST(cluster_list);
-		slurmdb_destroy_assoc_rec(start_assoc);
-		slurmdb_destroy_account_rec(start_acct);
-		FREE_NULL_LIST(local_account_list);
-		return SLURM_ERROR;
-	}
-
-	itr = list_iterator_create(name_list);
-	while((name = list_next(itr))) {
-		if (!name[0]) {
-			exit_code=1;
-			fprintf(stderr, " No blank names are "
-				"allowed when adding.\n");
-			rc = SLURM_ERROR;
-			continue;
-		}
-
-		acct = NULL;
-		if (!sacctmgr_find_account_from_list(local_account_list, name)) {
-			acct = xmalloc(sizeof(slurmdb_account_rec_t));
-			acct->assoc_list =
-				list_create(slurmdb_destroy_assoc_rec);
-			acct->name = xstrdup(name);
-			if (start_acct->description)
-				acct->description =
-					xstrdup(start_acct->description);
-			else
-				acct->description = xstrdup(name);
-
-			if (start_acct->organization)
-				acct->organization =
-					xstrdup(start_acct->organization);
-			else if (xstrcmp(start_assoc->parent_acct, "root"))
-				acct->organization =
-					xstrdup(start_assoc->parent_acct);
-			else
-				acct->organization = xstrdup(name);
-
-			xstrfmtcat(acct_str, "  %s\n", name);
-			list_append(acct_list, acct);
-		}
-
-		itr_c = list_iterator_create(cluster_list);
-		while((cluster = list_next(itr_c))) {
-			if (sacctmgr_find_account_base_assoc_from_list(
-				   local_assoc_list, name, cluster)) {
-				//printf(" already have this assoc\n");
-				continue;
-			}
-			if (!sacctmgr_find_account_base_assoc_from_list(
-				   local_assoc_list, start_assoc->parent_acct,
-				   cluster)) {
-				exit_code=1;
-				fprintf(stderr, " Parent account '%s' "
-					"doesn't exist on "
-					"cluster %s\n"
-					"        Contact your admin "
-					"to add this account.\n",
-					start_assoc->parent_acct, cluster);
-				continue;
-			}
-
-			assoc = xmalloc(sizeof(slurmdb_assoc_rec_t));
-			slurmdb_init_assoc_rec(assoc, 0);
-			assoc->acct = xstrdup(name);
-			assoc->cluster = xstrdup(cluster);
-			assoc->def_qos_id = start_assoc->def_qos_id;
-
-			assoc->parent_acct = xstrdup(start_assoc->parent_acct);
-			assoc->shares_raw = start_assoc->shares_raw;
-
-			slurmdb_copy_assoc_rec_limits(assoc, start_assoc);
-
-			if (acct)
-				list_append(acct->assoc_list, assoc);
-			else
-				list_append(assoc_list, assoc);
-			xstrfmtcat(assoc_str,
-				   "  A = %-10.10s"
-				   " C = %-10.10s\n",
-				   assoc->acct,
-				   assoc->cluster);
-
-		}
-		list_iterator_destroy(itr_c);
-	}
-	list_iterator_destroy(itr);
-	FREE_NULL_LIST(local_account_list);
-	FREE_NULL_LIST(local_assoc_list);
-
-
-	if (!list_count(acct_list) && !list_count(assoc_list)) {
-		printf(" Nothing new added.\n");
-		rc = SLURM_ERROR;
-		goto end_it;
-	} else if (!assoc_str) {
-		exit_code=1;
-		fprintf(stderr, " No associations created.\n");
-		goto end_it;
-	}
-
-	if (acct_str) {
-		printf(" Adding Account(s)\n%s", acct_str);
-		printf(" Settings\n");
-		if (start_acct->description)
-			printf("  Description     = %s\n",
-			       start_acct->description);
-		else
-			printf("  Description     = %s\n", "Account Name");
-
-		if (start_acct->organization)
-			printf("  Organization    = %s\n",
-			       start_acct->organization);
-		else
-			printf("  Organization    = %s\n",
-			       "Parent/Account Name");
-
-		xfree(acct_str);
-	}
-
-	if (assoc_str) {
-		printf(" Associations\n%s", assoc_str);
-		xfree(assoc_str);
-	}
-
-	if (limit_set) {
-		printf(" Settings\n");
-		sacctmgr_print_assoc_limits(start_assoc);
 	}
 
 	notice_thread_init();
-	if (list_count(acct_list))
-		rc = slurmdb_accounts_add(db_conn, acct_list);
-
-
-	if (rc == SLURM_SUCCESS) {
-		if (list_count(assoc_list))
-			rc = slurmdb_associations_add(db_conn, assoc_list);
-	} else {
-		exit_code=1;
-		fprintf(stderr, " Problem adding accounts: %s\n",
-			slurm_strerror(rc));
-		rc = SLURM_ERROR;
-		notice_thread_fini();
-		goto end_it;
-	}
+	ret_str = slurmdb_accounts_add_cond(db_conn, &add_assoc, start_acct);
+	rc = errno;
 	notice_thread_fini();
 
 	if (rc == SLURM_SUCCESS) {
+		if (ret_str) {
+			printf("%s", ret_str);
+			xfree(ret_str);
+			if (limit_set) {
+				printf(" Settings\n");
+				sacctmgr_print_assoc_limits(start_assoc);
+			}
+		}
 		if (commit_check("Would you like to commit changes?")) {
 			slurmdb_connection_commit(db_conn, 1);
 		} else {
 			printf(" Changes Discarded\n");
 			slurmdb_connection_commit(db_conn, 0);
 		}
+	} else if (rc == SLURM_NO_CHANGE_IN_DATA) {
+		printf(" %s", ret_str ? ret_str : slurm_strerror(rc));
 	} else {
 		exit_code=1;
-		fprintf(stderr,
-			" error: Problem adding account associations: %s\n",
-			slurm_strerror(rc));
+		if (ret_str)
+			fprintf(stderr, " error: %s\n", ret_str);
+		else
+			fprintf(stderr,
+				" error: Problem adding account associations: %s\n",
+				slurm_strerror(rc));
 		rc = SLURM_ERROR;
 	}
 
-end_it:
-	FREE_NULL_LIST(name_list);
-	FREE_NULL_LIST(cluster_list);
-	FREE_NULL_LIST(acct_list);
-	FREE_NULL_LIST(assoc_list);
-
-	slurmdb_destroy_assoc_rec(start_assoc);
+	xfree(ret_str);
+	FREE_NULL_LIST(add_assoc.acct_list);
+	FREE_NULL_LIST(add_assoc.cluster_list);
 	slurmdb_destroy_account_rec(start_acct);
+
 	return rc;
 }
 
@@ -672,10 +349,12 @@ extern int sacctmgr_list_account(int argc, char **argv)
 		xmalloc(sizeof(slurmdb_account_cond_t));
  	List acct_list;
 	int i=0, cond_set=0, prev_set=0;
-	ListIterator itr = NULL;
-	ListIterator itr2 = NULL;
+	list_itr_t *itr = NULL;
+	list_itr_t *itr2 = NULL;
 	slurmdb_account_rec_t *acct = NULL;
 	slurmdb_assoc_rec_t *assoc = NULL;
+	char *tmp_char = NULL;
+	uint32_t tmp_uint32;
 
 	int field_count = 0;
 
@@ -684,7 +363,8 @@ extern int sacctmgr_list_account(int argc, char **argv)
 	List format_list = list_create(xfree_ptr);
 	List print_fields_list; /* types are of print_field_t */
 
-	acct_cond->with_assocs = with_assoc_flag;
+	if (with_assoc_flag)
+		acct_cond->flags |= SLURMDB_ACCT_FLAG_WASSOC;
 
 	for (i=0; i<argc; i++) {
 		int command_len = strlen(argv[i]);
@@ -701,7 +381,7 @@ extern int sacctmgr_list_account(int argc, char **argv)
 		return SLURM_ERROR;
 	} else if (!list_count(format_list)) {
 		slurm_addto_char_list(format_list, "Acc,Des,O");
-		if (acct_cond->with_assocs)
+		if (acct_cond->flags & SLURMDB_ACCT_FLAG_WASSOC)
 			slurm_addto_char_list(format_list,
 					      "Cl,ParentN,U,Share,Priority,"
 					      "GrpJ,GrpN,"
@@ -709,12 +389,13 @@ extern int sacctmgr_list_account(int argc, char **argv)
 					      "MaxJ,MaxN,MaxCPUs,MaxS,MaxW,"
 					      "MaxCPUMins,QOS,DefaultQOS");
 
-		if (acct_cond->with_coords)
+		if (acct_cond->flags & SLURMDB_ACCT_FLAG_WCOORD)
 			slurm_addto_char_list(format_list, "Coord");
 
 	}
 
-	if (!acct_cond->with_assocs && (cond_set & SA_SET_ASSOC)) {
+	if (!(acct_cond->flags & SLURMDB_ACCT_FLAG_WASSOC) &&
+	    (cond_set & SA_SET_ASSOC)) {
 		if (!commit_check("You requested options that are only valid "
 				 "when querying with the withassoc option.\n"
 				 "Are you sure you want to continue?")) {
@@ -737,6 +418,20 @@ extern int sacctmgr_list_account(int argc, char **argv)
 	acct_list = slurmdb_accounts_get(db_conn, acct_cond);
 	slurmdb_destroy_account_cond(acct_cond);
 
+	if (mime_type) {
+		if (is_data_parser_deprecated(data_parser))
+			DATA_DUMP_CLI_DEPRECATED(ACCOUNT_LIST, acct_list,
+						 "accounts", argc, argv,
+						 db_conn, mime_type, rc);
+		else
+			DATA_DUMP_CLI_SINGLE(OPENAPI_ACCOUNTS_RESP, acct_list,
+					     argc, argv, db_conn, mime_type,
+					     data_parser, rc);
+		FREE_NULL_LIST(print_fields_list);
+		FREE_NULL_LIST(acct_list);
+		return rc;
+	}
+
 	if (!acct_list) {
 		exit_code=1;
 		fprintf(stderr, " Problem with query.\n");
@@ -752,7 +447,7 @@ extern int sacctmgr_list_account(int argc, char **argv)
 
 	while((acct = list_next(itr))) {
 		if (acct->assoc_list) {
-			ListIterator itr3 =
+			list_itr_t *itr3 =
 				list_iterator_create(acct->assoc_list);
 			while((assoc = list_next(itr3))) {
 				int curr_inx = 1;
@@ -767,7 +462,7 @@ extern int sacctmgr_list_account(int argc, char **argv)
 					case PRINT_COORDS:
 						field->print_routine(
 							field,
-							acct->coordinators,
+							&acct->coordinators,
 							(curr_inx ==
 							 field_count));
 						break;
@@ -803,10 +498,13 @@ extern int sacctmgr_list_account(int argc, char **argv)
 			while((field = list_next(itr2))) {
 				switch(field->type) {
 				case PRINT_QOS:
+					tmp_char = get_qos_complete_str(NULL,
+									NULL);
 					field->print_routine(
-						field, NULL,
-						NULL,
+						field,
+						tmp_char,
 						(curr_inx == field_count));
+					xfree(tmp_char);
 					break;
 				case PRINT_ACCT:
 					field->print_routine(
@@ -817,7 +515,7 @@ extern int sacctmgr_list_account(int argc, char **argv)
 				case PRINT_COORDS:
 					field->print_routine(
 						field,
-						acct->coordinators,
+						&acct->coordinators,
 						(curr_inx ==
 						 field_count));
 					break;
@@ -827,6 +525,14 @@ extern int sacctmgr_list_account(int argc, char **argv)
 						(curr_inx ==
 						 field_count));
 					break;
+				case PRINT_FLAGS:
+					tmp_char = slurmdb_acct_flags_2_str(
+						acct->flags);
+					field->print_routine(
+						field, tmp_char,
+						(curr_inx == field_count));
+					xfree(tmp_char);
+					break;
 				case PRINT_ORG:
 					field->print_routine(
 						field, acct->organization,
@@ -834,9 +540,10 @@ extern int sacctmgr_list_account(int argc, char **argv)
 						 field_count));
 					break;
 				case PRINT_PRIO:
+					tmp_uint32 = INFINITE;
 					field->print_routine(
 						field,
-						INFINITE,
+						&tmp_uint32,
 						(curr_inx == field_count));
 					break;
 				default:
@@ -943,7 +650,7 @@ extern int sacctmgr_modify_account(int argc, char **argv)
 			db_conn, acct_cond, acct);
 		if (ret_list && list_count(ret_list)) {
 			char *object = NULL;
-			ListIterator itr = list_iterator_create(ret_list);
+			list_itr_t *itr = list_iterator_create(ret_list);
 			printf(" Modified accounts...\n");
 			while((object = list_next(itr))) {
 				printf("  %s\n", object);
@@ -976,7 +683,23 @@ assoc_start:
 			goto assoc_end;
 		}
 
-		if (assoc->parent_acct) {
+		ret_list = slurmdb_associations_modify(
+			db_conn, acct_cond->assoc_cond, assoc);
+
+		if (ret_list && list_count(ret_list)) {
+			char *object = NULL;
+			list_itr_t *itr = list_iterator_create(ret_list);
+			printf(" Modified account associations...\n");
+			while ((object = list_next(itr))) {
+				printf("  %s\n", object);
+			}
+			list_iterator_destroy(itr);
+			set = 1;
+		} else if (ret_list) {
+			printf(" Nothing modified\n");
+			rc = SLURM_ERROR;
+		} else if ((errno == ESLURM_INVALID_PARENT_ACCOUNT) &&
+			   assoc->parent_acct) {
 			slurmdb_account_rec_t *acct_rec =
 				sacctmgr_find_account(assoc->parent_acct);
 			if (!acct_rec) {
@@ -987,35 +710,6 @@ assoc_start:
 				rc = SLURM_ERROR;
 				goto assoc_end;
 			}
-		}
-
-		ret_list = slurmdb_associations_modify(
-			db_conn, acct_cond->assoc_cond, assoc);
-
-		if (ret_list && list_count(ret_list)) {
-			set = 1;
-			if (assoc->def_qos_id != NO_VAL)
-				set = sacctmgr_check_default_qos(
-					     assoc->def_qos_id,
-					     acct_cond->assoc_cond);
-			else if (assoc->qos_list)
-				set = sacctmgr_check_default_qos(
-					     -1, acct_cond->assoc_cond);
-
-			if (set) {
-				char *object = NULL;
-				ListIterator itr = list_iterator_create(
-					ret_list);
-				printf(" Modified account associations...\n");
-				while((object = list_next(itr))) {
-					printf("  %s\n", object);
-				}
-				list_iterator_destroy(itr);
-				set = 1;
-			}
-		} else if (ret_list) {
-			printf(" Nothing modified\n");
-			rc = SLURM_ERROR;
 		} else {
 			exit_code=1;
 			fprintf(stderr, " Error with request: %s\n",
@@ -1051,8 +745,8 @@ extern int sacctmgr_delete_account(int argc, char **argv)
 	slurmdb_account_cond_t *acct_cond =
 		xmalloc(sizeof(slurmdb_account_cond_t));
 	int i = 0;
-	List ret_list = NULL, local_assoc_list = NULL;
-	ListIterator itr = NULL;
+	List ret_list = NULL;
+	list_itr_t *itr = NULL;
 	int cond_set = 0, prev_set = 0;
 
 	for (i = 0; i < argc; i++) {
@@ -1106,9 +800,6 @@ extern int sacctmgr_delete_account(int argc, char **argv)
 		}
 	}
 
-	acct_cond->assoc_cond->only_defs = 1;
-	local_assoc_list = slurmdb_associations_get(
-		db_conn, acct_cond->assoc_cond);
 	acct_cond->assoc_cond->only_defs = 0;
 
 	notice_thread_init();
@@ -1125,25 +816,30 @@ extern int sacctmgr_delete_account(int argc, char **argv)
 
 	if (ret_list && list_count(ret_list)) {
 		char *object = NULL;
-		ListIterator itr = NULL;
+		list_itr_t *itr = NULL;
+		itr = list_iterator_create(ret_list);
 
 		/* Check to see if person is trying to remove a default
 		 * account of a user.  _isdefault only works with the
 		 * output from slurmdb_accounts_remove, and
 		 * with a previously got assoc_list.
 		 */
-		if (_isdefault(cond_set, ret_list, local_assoc_list)) {
-			exit_code=1;
+		if (rc == ESLURM_NO_REMOVE_DEFAULT_ACCOUNT){
+			fprintf(stderr, " Error with request: %s\n",
+				slurm_strerror(rc));
+			while((object = list_next(itr))) {
+				fprintf(stderr,"  %s\n", object);
+			}
 			fprintf(stderr, " Please either remove the "
 				"accounts listed "
 				"above from list and resubmit,\n"
-				" or change these users default account to "
+				" or change these users' default accounts to "
 				"remove the account(s).\n"
 				" Changes Discarded\n");
 			slurmdb_connection_commit(db_conn, 0);
 			goto end_it;
 		}
-		itr = list_iterator_create(ret_list);
+
 		/* If there were jobs running with an association to
 		   be deleted, don't.
 		*/
@@ -1186,7 +882,6 @@ extern int sacctmgr_delete_account(int argc, char **argv)
 end_it:
 
 	FREE_NULL_LIST(ret_list);
-	FREE_NULL_LIST(local_assoc_list);
 
 	return rc;
 }
