@@ -41,12 +41,121 @@
 #include "src/common/assoc_mgr.h"
 #include "src/common/uid.h"
 #include "src/common/xstring.h"
+#include "src/interfaces/data_parser.h"
 
 typedef struct {
 	char *cluster;
 	char *user;
 } regret_t;
 
+static int _set_add_cond(int *start, int argc, char **argv,
+			 slurmdb_add_assoc_cond_t *add_assoc,
+			 slurmdb_user_rec_t *user)
+{
+	int i, end, command_len, option = 0, set = 0;
+
+	xassert(add_assoc);
+	xassert(user);
+
+	for (i = (*start); i < argc; i++) {
+		end = parse_option_end(argv[i]);
+		if (!end)
+			command_len = strlen(argv[i]);
+		else {
+			command_len = end - 1;
+			if (argv[i][end] == '=') {
+				option = (int)argv[i][end - 1];
+				end++;
+			}
+		}
+
+		if (!end ||
+		    !xstrncasecmp(argv[i], "Names", MAX(command_len, 1)) ||
+		    !xstrncasecmp(argv[i], "Users", MAX(command_len, 1))) {
+			if (!add_assoc->user_list)
+				add_assoc->user_list = list_create(xfree_ptr);
+			if (!slurm_addto_char_list_with_case(
+				    add_assoc->user_list,
+				    argv[i] + end,
+				    user_case_norm))
+				exit_code=1;
+		} else if (!xstrncasecmp(argv[i], "Accounts",
+					   MAX(command_len, 3)) ||
+			   !xstrncasecmp(argv[i], "Acct",
+					 MAX(command_len, 4))) {
+			if (!add_assoc->acct_list)
+				add_assoc->acct_list = list_create(xfree_ptr);
+			slurm_addto_char_list(add_assoc->acct_list,
+					      argv[i] + end);
+		} else if (!xstrncasecmp(argv[i], "AdminLevel",
+					 MAX(command_len, 2))) {
+			user->admin_level =
+				str_2_slurmdb_admin_level(argv[i] + end);
+		} else if (!xstrncasecmp(argv[i], "Clusters",
+					 MAX(command_len, 1))) {
+			if (!add_assoc->cluster_list)
+				add_assoc->cluster_list =
+					list_create(xfree_ptr);
+			slurm_addto_char_list(add_assoc->cluster_list,
+					      argv[i] + end);
+		} else if (!xstrncasecmp(argv[i], "DefaultAccount",
+					 MAX(command_len, 8))) {
+			if (user->default_acct) {
+				fprintf(stderr,
+					" Already listed DefaultAccount %s\n",
+					user->default_acct);
+				exit_code = 1;
+				continue;
+			}
+			user->default_acct =
+				strip_quotes(argv[i]+end, NULL, 1);
+			if (!add_assoc->acct_list)
+				add_assoc->acct_list = list_create(xfree_ptr);
+			slurm_addto_char_list(add_assoc->acct_list,
+					      user->default_acct);
+		} else if (!xstrncasecmp(argv[i], "DefaultWCKey",
+					 MAX(command_len, 8))) {
+			if (user->default_wckey) {
+				fprintf(stderr,
+					" Already listed DefaultWCKey %s\n",
+					user->default_wckey);
+				exit_code = 1;
+				continue;
+			}
+			user->default_wckey =
+				strip_quotes(argv[i]+end, NULL, 1);
+			if (!add_assoc->wckey_list)
+				add_assoc->wckey_list = list_create(xfree_ptr);
+			slurm_addto_char_list(add_assoc->wckey_list,
+					      user->default_wckey);
+		} else if (!xstrncasecmp(argv[i], "Partitions",
+					 MAX(command_len, 1))) {
+			if (!add_assoc->partition_list)
+				add_assoc->partition_list =
+					list_create(xfree_ptr);
+			slurm_addto_char_list(add_assoc->partition_list,
+					      argv[i] + end);
+		} else if (!xstrncasecmp(argv[i], "WCKeys",
+					 MAX(command_len, 1))) {
+			if (!add_assoc->wckey_list)
+				add_assoc->wckey_list = list_create(xfree_ptr);
+			slurm_addto_char_list(add_assoc->wckey_list, argv[i] + end);
+		} else {
+			set = sacctmgr_set_assoc_rec(&add_assoc->assoc,
+						     argv[i], argv[i] + end,
+						     command_len, option);
+			if (!set) {
+				exit_code=1;
+				fprintf(stderr, " Unknown option: %s\n",
+					argv[i]);
+			}
+		}
+	}
+
+	(*start) = i;
+	return set;
+}
+
 static int _set_cond(int *start, int argc, char **argv,
 		     slurmdb_user_cond_t *user_cond,
 		     slurmdb_wckey_cond_t *wckey_cond,
@@ -308,7 +417,7 @@ static int _check_and_set_cluster_list(List cluster_list)
 {
 	int rc = SLURM_SUCCESS;
 	List tmp_list = NULL;
-	ListIterator itr_c;
+	list_itr_t *itr_c;
 	slurmdb_cluster_rec_t *cluster_rec = NULL;
 
 	xassert(cluster_list);
@@ -352,8 +461,8 @@ static int _check_default_assocs(char *def_acct,
 	char *user = NULL, *cluster = NULL;
 	List regret_list = NULL;
 	List local_assoc_list = NULL;
-	ListIterator itr = NULL;
-	ListIterator itr_c = NULL;
+	list_itr_t *itr = NULL;
+	list_itr_t *itr_c = NULL;
 	regret_t *regret = NULL;
 	slurmdb_assoc_cond_t assoc_cond;
 	int rc = SLURM_SUCCESS;
@@ -437,8 +546,8 @@ static int _check_default_wckeys(char *def_wckey,
 	char *user = NULL, *cluster = NULL;
 	List regret_list = NULL;
 	List local_wckey_list = NULL;
-	ListIterator itr = NULL;
-	ListIterator itr_c = NULL;
+	list_itr_t *itr = NULL;
+	list_itr_t *itr_c = NULL;
 	regret_t *regret = NULL;
 	slurmdb_wckey_cond_t wckey_cond;
 	int rc = SLURM_SUCCESS;
@@ -519,11 +628,11 @@ static int _check_default_wckeys(char *def_wckey,
 /*
  * IN: user_cond - used for the assoc_cond pointing to the user and
  *     account list
- * IN: check - whether or not to check if the existance of the above lists
+ * IN: check - whether or not to check if the existence of the above lists
  */
 static int _check_coord_request(slurmdb_user_cond_t *user_cond, bool check)
 {
-	ListIterator itr = NULL, itr2 = NULL;
+	list_itr_t *itr = NULL, *itr2 = NULL;
 	char *name = NULL;
 	slurmdb_user_rec_t *user_rec = NULL;
 	slurmdb_account_rec_t *acct_rec = NULL;
@@ -628,695 +737,113 @@ static int _check_coord_request(slurmdb_user_cond_t *user_cond, bool check)
 	return rc;
 }
 
-static bool _check_user_has_default_assoc(char *user_name, List assoc_list)
+static int _check_uid(void *x, void *arg)
 {
-	ListIterator itr = list_iterator_create(assoc_list);
-	slurmdb_assoc_rec_t *assoc;
-	bool def_found = 0, any_def_found = 1;
-	char *last_cluster = NULL;
+	char *name = x;
+	uid_t pw_uid;
 
-	while ((assoc = list_next(itr))) {
-		if (last_cluster && xstrcmp(last_cluster, assoc->cluster)) {
-			if (!def_found) {
-				exit_code = 1;
-				fprintf(stderr,
-					" User %s on cluster %s no "
-					"longer has a default account.\n",
-					user_name, last_cluster);
-				any_def_found = 0;
-			}
-			def_found = 0;
+	if (uid_from_string(name, &pw_uid) < 0) {
+		char *warning = xstrdup_printf(
+			"There is no uid for user '%s'\nAre you sure you want to continue?",
+			name);
+
+		if (!commit_check(warning)) {
+			xfree(warning);
+			exit_code = 1;
+			return -1;
 		}
-
-		last_cluster = assoc->cluster;
-
-		if (assoc->is_def)
-			def_found = 1;
+		xfree(warning);
 	}
-	list_iterator_destroy(itr);
 
-	if (!def_found) {
-		exit_code = 1;
-		fprintf(stderr,
-			" User %s on cluster %s no "
-			"longer has a default account.\n",
-			user_name, last_cluster);
-		any_def_found = 0;
-	}
-	return any_def_found;
+	return 0;
 }
 
-
 extern int sacctmgr_add_user(int argc, char **argv)
 {
 	int rc = SLURM_SUCCESS;
-	int i=0;
-	ListIterator itr = NULL;
-	ListIterator itr_a = NULL;
-	ListIterator itr_c = NULL;
-	ListIterator itr_p = NULL;
-	ListIterator itr_w = NULL;
-	slurmdb_user_rec_t *user = NULL;
-	slurmdb_assoc_rec_t *assoc = NULL;
-	slurmdb_assoc_rec_t start_assoc;
-	char *default_acct = NULL;
-	char *default_wckey = NULL;
-	slurmdb_assoc_cond_t *assoc_cond = NULL;
-	slurmdb_wckey_rec_t *wckey = NULL;
-	slurmdb_wckey_cond_t *wckey_cond = NULL;
-	slurmdb_admin_level_t admin_level = SLURMDB_ADMIN_NOTSET;
-	char *name = NULL, *account = NULL, *cluster = NULL, *partition = NULL;
-	int partition_set = 0;
-	List user_list = NULL;
-	List assoc_list = NULL;
-	List wckey_list = NULL;
-	List local_assoc_list = NULL;
-	List local_acct_list = NULL;
-	List local_user_list = NULL;
-	List local_wckey_list = NULL;
-	char *user_str = NULL;
-	char *assoc_str = NULL;
-	char *wckey_str = NULL;
+	char *ret_str = NULL;
 	int limit_set = 0;
-	int first = 1;
-	int acct_first = 1;
-	int command_len = 0;
-	int option = 0;
-	uint16_t track_wckey = slurm_get_track_wckey();
+	slurmdb_add_assoc_cond_t add_assoc;
+	slurmdb_user_rec_t user;
+	slurmdb_assoc_rec_t *start_assoc;
 
-/* 	if (!list_count(sacctmgr_cluster_list)) { */
-/* 		printf(" Can't add users, no cluster defined yet.\n" */
-/* 		       " Please contact your administrator.\n"); */
-/* 		return SLURM_ERROR; */
-/* 	} */
-	slurmdb_init_assoc_rec(&start_assoc, 0);
+	slurmdb_init_add_assoc_cond(&add_assoc, 0);
+	start_assoc = &add_assoc.assoc;
 
-	assoc_cond = xmalloc(sizeof(slurmdb_assoc_cond_t));
+	memset(&user, 0, sizeof(user));
+	user.admin_level = SLURMDB_ADMIN_NOTSET;
 
-	assoc_cond->user_list = list_create(xfree_ptr);
-	assoc_cond->acct_list = list_create(xfree_ptr);
-	assoc_cond->cluster_list = list_create(xfree_ptr);
-	assoc_cond->partition_list = list_create(xfree_ptr);
-
-	wckey_cond = xmalloc(sizeof(slurmdb_wckey_cond_t));
-
-	wckey_cond->name_list = list_create(xfree_ptr);
-
-	for (i = 0; i < argc; i++) {
-		int end = parse_option_end(argv[i]);
-		if (!end)
-			command_len=strlen(argv[i]);
-		else {
-			command_len=end-1;
-			if (argv[i][end] == '=') {
-				option = (int)argv[i][end-1];
-				end++;
-			}
-		}
-
-		if (!end
-		   || !xstrncasecmp(argv[i], "Names", MAX(command_len, 1))
-		   || !xstrncasecmp(argv[i], "Users", MAX(command_len, 1))) {
-			if (!slurm_addto_char_list_with_case(assoc_cond->user_list,
-							     argv[i]+end,
-							     user_case_norm))
-				exit_code=1;
-		} else if (!xstrncasecmp(argv[i], "AdminLevel",
-					 MAX(command_len, 2))) {
-			admin_level = str_2_slurmdb_admin_level(argv[i]+end);
-		} else if (!xstrncasecmp(argv[i], "DefaultAccount",
-					 MAX(command_len, 8))) {
-			if (default_acct) {
-				fprintf(stderr,
-					" Already listed DefaultAccount %s\n",
-					default_acct);
-				exit_code = 1;
-				continue;
-			}
-			default_acct = strip_quotes(argv[i]+end, NULL, 1);
-			slurm_addto_char_list(assoc_cond->acct_list,
-					      default_acct);
-		} else if (!xstrncasecmp(argv[i], "DefaultWCKey",
-					 MAX(command_len, 8))) {
-			if (default_wckey) {
-				fprintf(stderr,
-					" Already listed DefaultWCKey %s\n",
-					default_wckey);
-				exit_code = 1;
-				continue;
-			}
-			default_wckey = strip_quotes(argv[i]+end, NULL, 1);
-			slurm_addto_char_list(wckey_cond->name_list,
-					      default_wckey);
-		} else if (!xstrncasecmp(argv[i], "WCKeys",
-					 MAX(command_len, 1))) {
-			slurm_addto_char_list(wckey_cond->name_list,
-					      argv[i]+end);
-		} else if (!(limit_set = sacctmgr_set_assoc_rec(
-				    &start_assoc, argv[i], argv[i]+end,
-				    command_len, option))
-			  && !(limit_set = sacctmgr_set_assoc_cond(
-				       assoc_cond, argv[i], argv[i]+end,
-				       command_len, option))) {
-			exit_code=1;
-			fprintf(stderr, " Unknown option: %s\n", argv[i]);
-		}
-	}
-
-	if (exit_code) {
-		xfree(default_acct);
-		xfree(default_wckey);
-		slurmdb_destroy_wckey_cond(wckey_cond);
-		slurmdb_destroy_assoc_cond(assoc_cond);
-		return SLURM_ERROR;
-	} else if (!list_count(assoc_cond->user_list)) {
-		xfree(default_acct);
-		xfree(default_wckey);
-		slurmdb_destroy_wckey_cond(wckey_cond);
-		slurmdb_destroy_assoc_cond(assoc_cond);
-		exit_code = 1;
-		fprintf(stderr, " Need name of user to add.\n");
-		return SLURM_ERROR;
-	} else {
- 		slurmdb_user_cond_t user_cond;
-		slurmdb_assoc_cond_t temp_assoc_cond;
-
-		memset(&user_cond, 0, sizeof(slurmdb_user_cond_t));
-		memset(&temp_assoc_cond, 0, sizeof(slurmdb_assoc_cond_t));
-		user_cond.with_wckeys = 1;
-		user_cond.with_assocs = 1;
-
-		temp_assoc_cond.only_defs = 1;
-		temp_assoc_cond.user_list = assoc_cond->user_list;
-		user_cond.assoc_cond = &temp_assoc_cond;
-
-		local_user_list = slurmdb_users_get(db_conn, &user_cond);
-	}
-
-	if (!local_user_list) {
-		exit_code = 1;
-		fprintf(stderr, " Problem getting users from database.  "
-			"Contact your admin.\n");
-		xfree(default_acct);
-		xfree(default_wckey);
-		slurmdb_destroy_wckey_cond(wckey_cond);
-		slurmdb_destroy_assoc_cond(assoc_cond);
-		return SLURM_ERROR;
-	}
-
-
-	if (!list_count(assoc_cond->cluster_list)) {
-		if (_check_and_set_cluster_list(assoc_cond->cluster_list)
-		    != SLURM_SUCCESS) {
-			xfree(default_acct);
-			xfree(default_wckey);
-			slurmdb_destroy_wckey_cond(wckey_cond);
-			slurmdb_destroy_assoc_cond(assoc_cond);
-			FREE_NULL_LIST(local_user_list);
-			FREE_NULL_LIST(local_acct_list);
-			return SLURM_ERROR;
-		}
-	} else if (sacctmgr_validate_cluster_list(assoc_cond->cluster_list)
-		   != SLURM_SUCCESS) {
-		xfree(default_acct);
-		xfree(default_wckey);
-		slurmdb_destroy_wckey_cond(wckey_cond);
-		slurmdb_destroy_assoc_cond(assoc_cond);
-		FREE_NULL_LIST(local_user_list);
-		FREE_NULL_LIST(local_acct_list);
-		return SLURM_ERROR;
-	}
-
-	if (!list_count(assoc_cond->acct_list)) {
-		if (!list_count(wckey_cond->name_list)) {
-			xfree(default_acct);
-			xfree(default_wckey);
-			slurmdb_destroy_wckey_cond(wckey_cond);
-			slurmdb_destroy_assoc_cond(assoc_cond);
-			exit_code = 1;
-			fprintf(stderr, " Need name of account to "
-				"add user to.\n");
-			return SLURM_ERROR;
-		}
-	} else {
- 		slurmdb_account_cond_t account_cond;
-		slurmdb_assoc_cond_t query_assoc_cond;
-
-		memset(&account_cond, 0, sizeof(slurmdb_account_cond_t));
-		account_cond.assoc_cond = assoc_cond;
-
-		local_acct_list = slurmdb_accounts_get(
-			db_conn, &account_cond);
-
-		if (!local_acct_list) {
-			exit_code = 1;
-			fprintf(stderr, " Problem getting accounts "
-				"from database.  Contact your admin.\n");
-			FREE_NULL_LIST(local_user_list);
-			xfree(default_acct);
-			xfree(default_wckey);
-			slurmdb_destroy_wckey_cond(wckey_cond);
-			slurmdb_destroy_assoc_cond(assoc_cond);
-			return SLURM_ERROR;
-		}
-
-		memset(&query_assoc_cond, 0,
-		       sizeof(slurmdb_assoc_cond_t));
-		query_assoc_cond.acct_list = assoc_cond->acct_list;
-		query_assoc_cond.cluster_list = assoc_cond->cluster_list;
-		local_assoc_list = slurmdb_associations_get(
-			db_conn, &query_assoc_cond);
-
-		if (!local_assoc_list) {
-			xfree(default_acct);
-			xfree(default_wckey);
-			exit_code = 1;
-			fprintf(stderr, " Problem getting assocs "
-				"from database.  Contact your admin.\n");
-			FREE_NULL_LIST(local_user_list);
-			FREE_NULL_LIST(local_acct_list);
-			slurmdb_destroy_wckey_cond(wckey_cond);
-			slurmdb_destroy_assoc_cond(assoc_cond);
-			return SLURM_ERROR;
-		}
-	}
-
-	/*
-	 * If we aren't tracking WCKeys but the user is adding them, make sure
-	 * we do.
-	 */
-	if (!track_wckey)
-		track_wckey = !list_is_empty(wckey_cond->name_list);
-
-	if (track_wckey || default_wckey) {
-		wckey_cond->cluster_list = assoc_cond->cluster_list;
-		wckey_cond->user_list = assoc_cond->user_list;
-		if (!(local_wckey_list = slurmdb_wckeys_get(
-			     db_conn, wckey_cond)))
-			info("If you are a coordinator ignore "
-			     "the previous error");
-
-		wckey_cond->cluster_list = NULL;
-		wckey_cond->user_list = NULL;
-
-	}
-
-	/* we are adding these lists to the global lists and will be
-	   freed when they are */
-	user_list = list_create(slurmdb_destroy_user_rec);
-	assoc_list = list_create(slurmdb_destroy_assoc_rec);
-	wckey_list = list_create(slurmdb_destroy_wckey_rec);
-
-	itr = list_iterator_create(assoc_cond->user_list);
-	while((name = list_next(itr))) {
-		slurmdb_user_rec_t *user_rec = NULL;
-		char *local_def_acct = NULL;
-		char *local_def_wckey = NULL;
-
-		if (!name[0]) {
-			exit_code=1;
-			fprintf(stderr, " No blank names are "
-				"allowed when adding.\n");
+	for (int i = 0; i < argc; i++) {
+		int command_len = strlen(argv[i]);
+		if (!xstrncasecmp(argv[i], "Where", MAX(command_len, 5)) ||
+		    !xstrncasecmp(argv[i], "Set", MAX(command_len, 3)))
+			i++;
+		limit_set = _set_add_cond(&i, argc, argv, &add_assoc, &user);
+		if (exit_code) {
 			rc = SLURM_ERROR;
-			continue;
+			goto end_it;
 		}
-
-		local_def_acct = xstrdup(default_acct);
-		local_def_wckey = xstrdup(default_wckey);
-
-		user = NULL;
-		if (!(user_rec = sacctmgr_find_user_from_list(
-			     local_user_list, name))) {
-			uid_t pw_uid;
-
-			if (!local_def_acct
-			    && assoc_cond->acct_list
-			    && list_count(assoc_cond->acct_list))
-				local_def_acct = xstrdup(
-					list_peek(assoc_cond->acct_list));
-
-			if (!local_def_wckey
-			    && wckey_cond->name_list
-			    && list_count(wckey_cond->name_list))
-				local_def_wckey = xstrdup(
-					list_peek(wckey_cond->name_list));
-
-			if (!local_def_acct || !local_def_acct[0]) {
-				exit_code=1;
-				fprintf(stderr, " Need a default account for "
-					"these users to add.\n");
-				rc = SLURM_ERROR;
-				xfree(local_def_acct);
-				xfree(local_def_wckey);
-				goto no_default;
-			}
-			if (first) {
-				if (!sacctmgr_find_account_from_list(
-					   local_acct_list, local_def_acct)) {
-					exit_code=1;
-					fprintf(stderr, " This account '%s' "
-						"doesn't exist.\n"
-						"        Contact your admin "
-						"to add this account.\n",
-						local_def_acct);
-					xfree(local_def_acct);
-					xfree(local_def_wckey);
-					continue;
-				}
-				first = 0;
-			}
-			if (uid_from_string (name, &pw_uid) < 0) {
-				char *warning = xstrdup_printf(
-					"There is no uid for user '%s'"
-					"\nAre you sure you want to continue?",
-					name);
-
-				if (!commit_check(warning)) {
-					xfree(warning);
-					rc = SLURM_ERROR;
-					list_flush(user_list);
-					xfree(local_def_acct);
-					xfree(local_def_wckey);
-					goto end_it;
-				}
-				xfree(warning);
-			}
-
-			user = xmalloc(sizeof(slurmdb_user_rec_t));
-			user->assoc_list =
-				list_create(slurmdb_destroy_assoc_rec);
-			user->wckey_list =
-				list_create(slurmdb_destroy_wckey_rec);
-			user->name = xstrdup(name);
-			user->default_acct = xstrdup(local_def_acct);
-			user->default_wckey = xstrdup(local_def_wckey);
-
-			user->admin_level = admin_level;
-
-			xstrfmtcat(user_str, "  %s\n", name);
-
-			list_append(user_list, user);
-		}
-
-		itr_a = list_iterator_create(assoc_cond->acct_list);
-		while ((account = list_next(itr_a))) {
-			if (acct_first) {
-				if (!sacctmgr_find_account_from_list(
-					   local_acct_list, account)) {
-					exit_code=1;
-					fprintf(stderr, " This account '%s' "
-						"doesn't exist.\n"
-						"        Contact your admin "
-						"to add this account.\n",
-						account);
-					continue;
-				}
-			}
-			itr_c = list_iterator_create(assoc_cond->cluster_list);
-			while ((cluster = list_next(itr_c))) {
-				/* We need to check this every time
-				   for a cluster to make sure there
-				   isn't one already set for that
-				   cluster.
-				*/
-				if (!sacctmgr_find_account_base_assoc_from_list(
-					   local_assoc_list, account,
-					   cluster)) {
-					if (acct_first) {
-						exit_code=1;
-						fprintf(stderr, " This "
-							"account '%s' "
-							"doesn't exist on "
-							"cluster %s\n"
-							"        Contact your "
-							"admin to add "
-							"this account.\n",
-							account, cluster);
-					}
-					continue;
-				} else if (!local_def_acct) {
-					slurmdb_assoc_rec_t *assoc_rec;
-					if (user_rec
-					    && (assoc_rec =
-						sacctmgr_find_assoc_from_list(
-						     user_rec->assoc_list,
-						     name, NULL,
-						     cluster, "*")))
-						local_def_acct = xstrdup(
-							assoc_rec->acct);
-					else if (assoc_cond
-						   && assoc_cond->acct_list
-						   && list_count(assoc_cond->
-								 acct_list))
-						local_def_acct = xstrdup(
-							list_peek(assoc_cond->
-								  acct_list));
-				}
-
-				itr_p = list_iterator_create(
-					assoc_cond->partition_list);
-				while((partition = list_next(itr_p))) {
-					partition_set = 1;
-					if (sacctmgr_find_assoc_from_list(
-						   local_assoc_list,
-						   name, account,
-						   cluster, partition))
-						continue;
-					assoc = xmalloc(
-						sizeof(slurmdb_assoc_rec_t));
-					slurmdb_init_assoc_rec(assoc, 0);
-					assoc->user = xstrdup(name);
-					assoc->acct = xstrdup(account);
-					assoc->cluster = xstrdup(cluster);
-					assoc->partition = xstrdup(partition);
-					if (local_def_acct &&
-					    !xstrcmp(local_def_acct, account))
-						assoc->is_def = 1;
-
-					assoc->def_qos_id =
-						start_assoc.def_qos_id;
-
-					assoc->shares_raw =
-						start_assoc.shares_raw;
-
-					slurmdb_copy_assoc_rec_limits(
-						assoc, &start_assoc);
-
-					if (user)
-						list_append(user->assoc_list,
-							    assoc);
-					else
-						list_append(assoc_list, assoc);
-					xstrfmtcat(assoc_str,
-						   "  U = %-9.9s"
-						   " A = %-10.10s"
-						   " C = %-10.10s"
-						   " P = %-10.10s\n",
-						   assoc->user, assoc->acct,
-						   assoc->cluster,
-						   assoc->partition);
-				}
-				list_iterator_destroy(itr_p);
-				if (partition_set) {
-					if (!default_acct && local_def_acct)
-						xfree(local_def_acct);
-					continue;
-				}
-
-				if (sacctmgr_find_assoc_from_list(
-					   local_assoc_list,
-					   name, account, cluster, NULL)) {
-					if (!default_acct && local_def_acct)
-						xfree(local_def_acct);
-					continue;
-				}
-
-				assoc = xmalloc(
-					sizeof(slurmdb_assoc_rec_t));
-				slurmdb_init_assoc_rec(assoc, 0);
-				assoc->user = xstrdup(name);
-				if (local_def_acct
-				   && !xstrcmp(local_def_acct, account))
-					assoc->is_def = 1;
-				assoc->acct = xstrdup(account);
-				assoc->cluster = xstrdup(cluster);
-
-				assoc->def_qos_id = start_assoc.def_qos_id;
-
-				assoc->shares_raw = start_assoc.shares_raw;
-
-				slurmdb_copy_assoc_rec_limits(
-					assoc, &start_assoc);
-
-				if (user)
-					list_append(user->assoc_list, assoc);
-				else
-					list_append(assoc_list, assoc);
-				xstrfmtcat(assoc_str,
-					   "  U = %-9.9s"
-					   " A = %-10.10s"
-					   " C = %-10.10s\n",
-					   assoc->user, assoc->acct,
-					   assoc->cluster);
-				if (!default_acct && local_def_acct)
-					xfree(local_def_acct);
-			}
-			list_iterator_destroy(itr_c);
-		}
-		list_iterator_destroy(itr_a);
-		acct_first = 0;
-
-		xfree(local_def_acct);
-		/* continue here if not doing wckeys */
-		if (!track_wckey && !local_def_wckey)
-			continue;
-
-		itr_w = list_iterator_create(wckey_cond->name_list);
-		while((account = list_next(itr_w))) {
-			itr_c = list_iterator_create(assoc_cond->cluster_list);
-			while((cluster = list_next(itr_c))) {
-				if (sacctmgr_find_wckey_from_list(
-					   local_wckey_list, name, account,
-					   cluster)) {
-					continue;
-				} else if (user_rec && !local_def_wckey) {
-					slurmdb_wckey_rec_t *wckey_rec;
-					if ((wckey_rec =
-					     sacctmgr_find_wckey_from_list(
-						     user_rec->wckey_list,
-						     name, NULL, cluster)))
-						local_def_wckey = xstrdup(
-							wckey_rec->name);
-					else if (wckey_cond
-						 && wckey_cond->name_list
-						 && list_count(
-							 wckey_cond->name_list))
-						local_def_wckey = xstrdup(
-							list_peek(wckey_cond->
-								  name_list));
-				}
-
-				wckey = xmalloc(sizeof(slurmdb_wckey_rec_t));
-				wckey->user = xstrdup(name);
-				wckey->name = xstrdup(account);
-				wckey->cluster = xstrdup(cluster);
-				if (local_def_wckey
-				   && !xstrcmp(local_def_wckey, account))
-					wckey->is_def = 1;
-				if (user)
-					list_append(user->wckey_list, wckey);
-				else
-					list_append(wckey_list, wckey);
-				xstrfmtcat(wckey_str,
-					   "  U = %-9.9s"
-					   " W = %-10.10s"
-					   " C = %-10.10s\n",
-					   wckey->user, wckey->name,
-					   wckey->cluster);
-				if (!default_wckey && local_def_wckey)
-					xfree(local_def_wckey);
-			}
-			list_iterator_destroy(itr_c);
-		}
-		list_iterator_destroy(itr_w);
-		xfree(local_def_wckey);
 	}
-no_default:
-	list_iterator_destroy(itr);
-	FREE_NULL_LIST(local_user_list);
-	FREE_NULL_LIST(local_acct_list);
-	FREE_NULL_LIST(local_assoc_list);
-	FREE_NULL_LIST(local_wckey_list);
-	slurmdb_destroy_wckey_cond(wckey_cond);
-	slurmdb_destroy_assoc_cond(assoc_cond);
 
-	if (!list_count(user_list) && !list_count(assoc_list)
-	   && !list_count(wckey_list)) {
-		printf(" Nothing new added.\n");
+	if (!list_count(add_assoc.user_list)) {
+		fprintf(stderr, " Need name of user to add.\n");
+		exit_code = 1;
 		rc = SLURM_ERROR;
 		goto end_it;
-	} else if (!assoc_str && !wckey_str) {
+	}
+
+	if (!list_count(add_assoc.acct_list) &&
+	    !list_count(add_assoc.wckey_list)) {
+		fprintf(stderr, " Need name of account or wckey to add.\n");
 		exit_code = 1;
-		fprintf(stderr, " No associations or wckeys created.\n");
+		rc = SLURM_ERROR;
 		goto end_it;
 	}
 
-	if (user_str) {
-		printf(" Adding User(s)\n%s", user_str);
-		if (default_acct || default_wckey ||
-		    (admin_level != SLURMDB_ADMIN_NOTSET))
-			printf(" Settings =\n");
-		if (default_acct)
-			printf("  Default Account = %s\n", default_acct);
-		if (default_wckey)
-			printf("  Default WCKey   = %s\n", default_wckey);
+	list_for_each_ro(add_assoc.user_list, _check_uid, NULL);
 
-		if (admin_level != SLURMDB_ADMIN_NOTSET)
-			printf("  Admin Level     = %s\n",
-			       slurmdb_admin_level_str(admin_level));
-		xfree(user_str);
-	}
-
-	if (assoc_str) {
-		printf(" Associations =\n%s", assoc_str);
-		xfree(assoc_str);
-	}
-
-	if (wckey_str) {
-		printf(" WCKeys =\n%s", wckey_str);
-		xfree(wckey_str);
-	}
-
-	if (limit_set) {
-		printf(" Non Default Settings\n");
-		sacctmgr_print_assoc_limits(&start_assoc);
-		FREE_NULL_LIST(start_assoc.qos_list);
+	if (exit_code) {
+		rc = SLURM_ERROR;
+		goto end_it;
 	}
 
 	notice_thread_init();
-	if (list_count(user_list)) {
-		rc = slurmdb_users_add(db_conn, user_list);
-	}
-
-	if (rc == SLURM_SUCCESS) {
-		if (list_count(assoc_list))
-			rc = slurmdb_associations_add(db_conn, assoc_list);
-	}
-
-	if (rc == SLURM_SUCCESS) {
-		if (list_count(wckey_list))
-			rc = slurmdb_wckeys_add(db_conn, wckey_list);
-	} else {
-		exit_code = 1;
-		fprintf(stderr, " Problem adding users: %s\n",
-			slurm_strerror(rc));
-		rc = SLURM_ERROR;
-		notice_thread_fini();
-		goto end_it;
-	}
-
+	ret_str = slurmdb_users_add_cond(db_conn, &add_assoc, &user);
+	rc = errno;
 	notice_thread_fini();
 
 	if (rc == SLURM_SUCCESS) {
+		if (ret_str) {
+			printf("%s", ret_str);
+			xfree(ret_str);
+			if (limit_set) {
+				printf(" Non Default Settings\n");
+				sacctmgr_print_assoc_limits(start_assoc);
+			}
+		}
 		if (commit_check("Would you like to commit changes?")) {
 			slurmdb_connection_commit(db_conn, 1);
 		} else {
 			printf(" Changes Discarded\n");
 			slurmdb_connection_commit(db_conn, 0);
 		}
+	} else if (rc == SLURM_NO_CHANGE_IN_DATA) {
+		if (ret_str) {
+			printf("%sNothing added.\nMake sure the accounts requested here have been added to the system/cluster. (sacctmgr add account...)\n", ret_str);
+		} else
+			printf("%s\n", slurm_strerror(rc));
 	} else {
-		exit_code = 1;
-		fprintf(stderr, " Problem adding user associations: %s\n",
+		exit_code=1;
+		fprintf(stderr,
+			" error: Problem adding user associations: %s\n",
 			slurm_strerror(rc));
 		rc = SLURM_ERROR;
 	}
 
 end_it:
-	FREE_NULL_LIST(user_list);
-	FREE_NULL_LIST(assoc_list);
-	FREE_NULL_LIST(wckey_list);
-	xfree(default_acct);
-	xfree(default_wckey);
+	xfree(ret_str);
+	slurmdb_free_add_assoc_cond_members(&add_assoc);
+	slurmdb_free_user_rec_members(&user);
 
 	return rc;
 }
@@ -1330,7 +857,7 @@ extern int sacctmgr_add_coord(int argc, char **argv)
 	char *name = NULL;
 	char *user_str = NULL;
 	char *acct_str = NULL;
-	ListIterator itr = NULL;
+	list_itr_t *itr = NULL;
 
 	for (i=0; i<argc; i++) {
 		int command_len = strlen(argv[i]);
@@ -1403,8 +930,8 @@ extern int sacctmgr_list_user(int argc, char **argv)
 	slurmdb_user_cond_t *user_cond = xmalloc(sizeof(slurmdb_user_cond_t));
 	List user_list;
 	int i=0, cond_set=0, prev_set=0;
-	ListIterator itr = NULL;
-	ListIterator itr2 = NULL;
+	list_itr_t *itr = NULL;
+	list_itr_t *itr2 = NULL;
 	slurmdb_user_rec_t *user = NULL;
 	slurmdb_assoc_rec_t *assoc = NULL;
 
@@ -1478,6 +1005,20 @@ extern int sacctmgr_list_user(int argc, char **argv)
 	user_list = slurmdb_users_get(db_conn, user_cond);
 	slurmdb_destroy_user_cond(user_cond);
 
+	if (mime_type) {
+		if (is_data_parser_deprecated(data_parser))
+			DATA_DUMP_CLI_DEPRECATED(USER_LIST, user_list, "users",
+						 argc, argv, db_conn, mime_type,
+						 rc);
+		else
+			DATA_DUMP_CLI_SINGLE(OPENAPI_USERS_RESP, user_list,
+					     argc, argv, db_conn, mime_type,
+					     data_parser, rc);
+		FREE_NULL_LIST(print_fields_list);
+		FREE_NULL_LIST(user_list);
+		return rc;
+	}
+
 	if (!user_list) {
 		exit_code=1;
 		fprintf(stderr, " Problem with query.\n");
@@ -1492,13 +1033,15 @@ extern int sacctmgr_list_user(int argc, char **argv)
 	field_count = list_count(print_fields_list);
 
 	while((user = list_next(itr))) {
+		char *tmp_char = NULL;
+		uint32_t tmp_uint32;
 		if (user->assoc_list) {
 			char *curr_cluster = NULL;
-			ListIterator itr3 =
+			list_itr_t *itr3 =
 				list_iterator_create(user->assoc_list);
-			ListIterator itr4 =
+			list_itr_t *itr4 =
 				list_iterator_create(user->assoc_list);
-			ListIterator wckey_itr = NULL;
+			list_itr_t *wckey_itr = NULL;
 			if (user->wckey_list)
 				wckey_itr =
 					list_iterator_create(user->wckey_list);
@@ -1564,18 +1107,20 @@ extern int sacctmgr_list_user(int argc, char **argv)
 				while((field = list_next(itr2))) {
 					switch(field->type) {
 					case PRINT_ADMIN:
-						field->print_routine(
-							field,
+						tmp_char =
 							slurmdb_admin_level_str(
 								user->
-								admin_level),
+								admin_level);
+						field->print_routine(
+							field,
+							tmp_char,
 							(curr_inx ==
 							 field_count));
 						break;
 					case PRINT_COORDS:
 						field->print_routine(
 							field,
-							user->coord_accts,
+							&user->coord_accts,
 							(curr_inx ==
 							 field_count));
 						break;
@@ -1583,8 +1128,6 @@ extern int sacctmgr_list_user(int argc, char **argv)
 						field->print_routine(
 							field,
 							user->default_acct,
-							(curr_inx ==
-							 field_count),
 							(curr_inx ==
 							 field_count));
 						break;
@@ -1592,8 +1135,6 @@ extern int sacctmgr_list_user(int argc, char **argv)
 						field->print_routine(
 							field,
 							user->default_wckey,
-							(curr_inx ==
-							 field_count),
 							(curr_inx ==
 							 field_count));
 						break;
@@ -1615,22 +1156,26 @@ extern int sacctmgr_list_user(int argc, char **argv)
 			while((field = list_next(itr2))) {
 				switch(field->type) {
 				case PRINT_QOS:
-					field->print_routine(
-						field, NULL,
-						NULL,
-						(curr_inx == field_count));
-					break;
-				case PRINT_ADMIN:
+					tmp_char = get_qos_complete_str(NULL,
+									NULL);
 					field->print_routine(
 						field,
-						slurmdb_admin_level_str(
-							user->admin_level),
+						tmp_char,
+						(curr_inx == field_count));
+					xfree(tmp_char);
+					break;
+				case PRINT_ADMIN:
+					tmp_char = slurmdb_admin_level_str(
+							user->admin_level);
+					field->print_routine(
+						field,
+						tmp_char,
 						(curr_inx == field_count));
 					break;
 				case PRINT_COORDS:
 					field->print_routine(
 						field,
-						user->coord_accts,
+						&user->coord_accts,
 						(curr_inx == field_count));
 					break;
 				case PRINT_DACCT:
@@ -1652,9 +1197,10 @@ extern int sacctmgr_list_user(int argc, char **argv)
 						(curr_inx == field_count));
 					break;
 				case PRINT_PRIO:
+					tmp_uint32 = INFINITE;
 					field->print_routine(
 						field,
-						INFINITE,
+						&tmp_uint32,
 						(curr_inx == field_count));
 					break;
 				default:
@@ -1795,7 +1341,7 @@ extern int sacctmgr_modify_user(int argc, char **argv)
 			db_conn, user_cond, user);
 		if (ret_list && list_count(ret_list)) {
 			set = 1;
-			if (user->default_acct
+			if (user->default_acct && user->default_acct[0]
 			    && _check_default_assocs(
 				    user->default_acct, ret_list,
 				    user_cond->assoc_cond->cluster_list)
@@ -1813,7 +1359,7 @@ extern int sacctmgr_modify_user(int argc, char **argv)
 
 			if (set) {
 				char *object;
-				ListIterator itr =
+				list_itr_t *itr =
 					list_iterator_create(ret_list);
 				printf(" Modified users...\n");
 				while ((object = list_next(itr))) {
@@ -1855,25 +1401,13 @@ assoc_start:
 
 		if (ret_list && list_count(ret_list)) {
 			char *object = NULL;
-			ListIterator itr;
-			set = 1;
-
-			if (assoc->def_qos_id != NO_VAL)
-				set = sacctmgr_check_default_qos(
-					     assoc->def_qos_id,
-					     user_cond->assoc_cond);
-			else if (assoc->qos_list)
-				set = sacctmgr_check_default_qos(
-					     -1, user_cond->assoc_cond);
-
-			if (set) {
-				itr = list_iterator_create(ret_list);
-				printf(" Modified user associations...\n");
-				while((object = list_next(itr))) {
-					printf("  %s\n", object);
-				}
-				list_iterator_destroy(itr);
+			list_itr_t *itr = list_iterator_create(ret_list);
+			printf(" Modified user associations...\n");
+			while((object = list_next(itr))) {
+				printf("  %s\n", object);
 			}
+			list_iterator_destroy(itr);
+			set = 1;
 		} else if (ret_list) {
 			printf(" Nothing modified\n");
 			rc = SLURM_ERROR;
@@ -1928,17 +1462,38 @@ extern int sacctmgr_delete_user(int argc, char **argv)
 	/* Since the association flag isn't set we need to change
 	   things to handle things correctly.
 	*/
-	if (user_cond->assoc_cond && user_cond->assoc_cond->cluster_list
-	    && list_count(user_cond->assoc_cond->cluster_list)) {
+	if (user_cond->assoc_cond) {
 		if (cond_set & SA_SET_WCKEY) {
-			wckey_cond->cluster_list =
-				user_cond->assoc_cond->cluster_list;
-			user_cond->assoc_cond->cluster_list = NULL;
-			wckey_cond->user_list =
-				user_cond->assoc_cond->user_list;
-			user_cond->assoc_cond->user_list = NULL;
-		} else
+			/*
+			 * You can no delete associations and wckeys at the same
+			 * time, so if SA_SET_WCKEY is set we need to grab some
+			 * lists that are only set up in the assoc_cond and use
+			 * them in the wckey_cond.
+			 */
+			if (user_cond->assoc_cond->cluster_list &&
+			    list_count(user_cond->assoc_cond->cluster_list)) {
+				wckey_cond->cluster_list =
+					user_cond->assoc_cond->cluster_list;
+				user_cond->assoc_cond->cluster_list = NULL;
+			}
+			if (user_cond->assoc_cond->user_list &&
+			    list_count(user_cond->assoc_cond->user_list)) {
+				wckey_cond->user_list =
+					user_cond->assoc_cond->user_list;
+				user_cond->assoc_cond->user_list = NULL;
+			}
+		} else if (user_cond->assoc_cond->cluster_list &&
+			   list_count(user_cond->assoc_cond->cluster_list)) {
+			/*
+			 * If not deleting wckeys specifically we need to check
+			 * if we have a cluster list.  If we do we are only
+			 * deleting a user from a set of clusters and not really
+			 * from the whole system. If this is the case then we
+			 * need to set SA_SET_ASSOC so we don't remove the user
+			 * from the whole system.
+			 */
 			cond_set |= SA_SET_ASSOC;
+		}
 	}
 
 	if (!cond_set) {
@@ -1976,7 +1531,7 @@ extern int sacctmgr_delete_user(int argc, char **argv)
 	if (ret_list && list_count(ret_list)) {
 		char *object = NULL;
 		List del_user_list = NULL;
-		ListIterator itr = list_iterator_create(ret_list);
+		list_itr_t *itr = list_iterator_create(ret_list);
 		/* If there were jobs running with an association to
 		   be deleted, don't.
 		*/
@@ -1991,6 +1546,27 @@ extern int sacctmgr_delete_user(int argc, char **argv)
 			slurmdb_connection_commit(db_conn, 0);
 			return rc;
 		}
+
+		if (rc == ESLURM_NO_REMOVE_DEFAULT_ACCOUNT) {
+			fprintf(stderr, " Error with request: %s\n",
+				slurm_strerror(rc));
+
+			while((object = list_next(itr))) {
+				fprintf(stderr,"  %s\n", object);
+			}
+			fprintf(stderr,
+				" You must change the default "
+				"account of these users or "
+				"remove the users completely "
+				"from the affected clusters "
+				"to allow these changes.\n"
+				" Changes Discarded\n");
+			list_iterator_destroy(itr);
+			FREE_NULL_LIST(ret_list);
+			slurmdb_connection_commit(db_conn, 0);
+			return rc;
+		}
+
 		if (cond_set == SA_SET_USER) {
 			printf(" Deleting users...\n");
 		} else if (cond_set & SA_SET_ASSOC) {
@@ -2002,7 +1578,7 @@ extern int sacctmgr_delete_user(int argc, char **argv)
 			printf("  %s\n", object);
 			if (cond_set & SA_SET_ASSOC) {
 				static const char *needle = "U = ";
-				int i = 4; /* strlen(needle) */
+				int i = 0;
 				char *tmp;
 
 				if (!(tmp = strstr(object, needle))) {
@@ -2013,6 +1589,7 @@ extern int sacctmgr_delete_user(int argc, char **argv)
 					rc = SLURM_ERROR;
 					goto end_it;
 				}
+				tmp += 4; /* strlen(needle) */
 
 				/* If the association has a partition on
 				 * it we need to get only the name portion, so
@@ -2028,7 +1605,6 @@ extern int sacctmgr_delete_user(int argc, char **argv)
 						break;
 					}
 					i++;
-					continue;
 				}
 
 				if (!del_user_list)
@@ -2046,7 +1622,6 @@ extern int sacctmgr_delete_user(int argc, char **argv)
 			slurmdb_user_cond_t del_user_cond;
 			slurmdb_assoc_cond_t del_user_assoc_cond;
 			slurmdb_user_rec_t *user = NULL;
-			bool default_deleted = false;
 			/* Use a fresh cond here so we check all
 			   clusters and such to make sure there are no
 			   associations.
@@ -2070,11 +1645,6 @@ extern int sacctmgr_delete_user(int argc, char **argv)
 				itr = list_iterator_create(user_list);
 				while ((user = list_next(itr))) {
 					if (user->assoc_list) {
-						if (!_check_user_has_default_assoc(
-							    user->name,
-							    user->assoc_list))
-							default_deleted = true;
-
 						continue;
 					}
 					if (!del_user_list) {
@@ -2092,19 +1662,7 @@ extern int sacctmgr_delete_user(int argc, char **argv)
 				list_iterator_destroy(itr);
 				FREE_NULL_LIST(user_list);
 
-				if (default_deleted) {
-					rc = exit_code = 1;
-					fprintf(stderr,
-						" You must change the default "
-						"account of these users or "
-						"remove the users completely "
-						"from the affected clusters "
-						"to allow these changes.\n"
-						" Changes Discarded\n");
-					slurmdb_connection_commit(db_conn, 0);
-					goto end_it;
 
-				}
 			}
 
 			if (del_user_list) {
@@ -2165,7 +1723,7 @@ extern int sacctmgr_delete_coord(int argc, char **argv)
 	char *name = NULL;
 	char *user_str = NULL;
 	char *acct_str = NULL;
-	ListIterator itr = NULL;
+	list_itr_t *itr = NULL;
 	List ret_list = NULL;
 
 	for (i=0; i<argc; i++) {
@@ -2240,7 +1798,7 @@ extern int sacctmgr_delete_coord(int argc, char **argv)
 
 	if (ret_list && list_count(ret_list)) {
 		char *object = NULL;
-		ListIterator itr = list_iterator_create(ret_list);
+		list_itr_t *itr = list_iterator_create(ret_list);
 		printf(" Removed Coordinators (sub accounts not listed)...\n");
 		while((object = list_next(itr))) {
 			printf("  %s\n", object);
