@@ -75,6 +75,7 @@
 
 extern char *command_name;
 extern List clusters;
+extern char *cluster_names;
 extern int all_flag;	/* display even hidden partitions */
 extern int detail_flag;	/* display additional details */
 extern int future_flag;	/* display future nodes */
@@ -89,6 +90,7 @@ extern int sibling_flag; /* show sibling jobs (if any fed job). */
 extern uint32_t cluster_flags; /* what type of cluster are we talking to */
 extern uint32_t euid; /* send request to the slurmctld in behave of this user */
 extern const char *mime_type; /* user requested JSON or YAML */
+extern const char *data_parser; /* data_parser args */
 
 extern front_end_info_msg_t *old_front_end_info_ptr;
 extern job_info_msg_t *old_job_info_ptr;
@@ -103,6 +105,7 @@ extern int	scontrol_callerid(int argc, char **argv);
 extern int	scontrol_create_part(int argc, char **argv);
 extern int	scontrol_create_res(int argc, char **argv);
 extern int	scontrol_encode_hostlist(char *hostlist, bool sorted);
+extern void	scontrol_getaddrs(char *node_list);
 extern void	scontrol_gethost(const char *stepd_node, const char *node_name);
 extern int	scontrol_hold(char *op, char *job_id_str);
 extern int	scontrol_job_notify(int argc, char **argv);
@@ -155,6 +158,10 @@ extern int	scontrol_update_part (int argc, char **argv);
 extern int	scontrol_update_res (int argc, char **argv);
 extern int	scontrol_update_step (int argc, char **argv);
 
+/* power_node.c */
+extern int scontrol_power_nodes(char *node_list, bool power_up, bool asap,
+				bool force);
+
 /* reboot_node.c */
 extern int      scontrol_cancel_reboot(char *nodes);
 extern int      scontrol_reboot_nodes(char *node_list, bool asap,
