@@ -3,7 +3,7 @@
  *****************************************************************************
  *  Copyright (C) 2002-2007 The Regents of the University of California.
  *  Copyright (C) 2008-2010 Lawrence Livermore National Security.
- *  Portions Copyright (C) 2010-2017 SchedMD <https://www.schedmd.com>.
+ *  Copyright (C) SchedMD LLC.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
  *  Written by Joey Ekstrom <ekstrom1@llnl.gov> and
  *  Morris Jette <jette1@llnl.gov>
@@ -50,6 +50,7 @@
 #include "src/common/list.h"
 #include "src/common/parse_time.h"
 #include "src/common/read_config.h"
+#include "src/common/uid.h"
 #include "src/common/xmalloc.h"
 #include "src/common/xstring.h"
 
@@ -83,7 +84,7 @@ static char *_str_tolower(char *upper_str);
  *****************************************************************************/
 int print_sinfo_list(List sinfo_list)
 {
-	ListIterator i = list_iterator_create(sinfo_list);
+	list_itr_t *i = list_iterator_create(sinfo_list);
 	sinfo_data_t *current;
 
 	if (params.node_field_flag)
@@ -103,7 +104,7 @@ int print_sinfo_list(List sinfo_list)
 
 int print_sinfo_entry(sinfo_data_t *sinfo_data)
 {
-	ListIterator i = list_iterator_create(params.format_list);
+	list_itr_t *i = list_iterator_create(params.format_list);
 	sinfo_format_t *current;
 
 	while ((current = list_next(i))) {
@@ -376,7 +377,7 @@ format_prepend_function(List list, int width, bool right, char *suffix,
 static void _set_node_field_size(List sinfo_list)
 {
 	char *tmp = NULL;
-	ListIterator i = list_iterator_create(sinfo_list);
+	list_itr_t *i = list_iterator_create(sinfo_list);
 	sinfo_data_t *current;
 	int max_width = MIN_NODE_FIELD_SIZE, this_width = 0;
 
@@ -392,7 +393,7 @@ static void _set_node_field_size(List sinfo_list)
 
 static void _set_part_field_size(List sinfo_list)
 {
-	ListIterator i = list_iterator_create(sinfo_list);
+	list_itr_t *i = list_iterator_create(sinfo_list);
 	sinfo_data_t *current;
 	int max_width = MIN_PART_FIELD_SIZE, this_width = 0;
 
@@ -1096,7 +1097,8 @@ int _print_state_complete(sinfo_data_t * sinfo_data, int width,
 		uint32_t my_state;
 
 		my_state = sinfo_data->node_state;
-		state = xstrtolower(node_state_string_complete(my_state));
+		state = node_state_string_complete(my_state);
+		xstrtolower(state);
 		_print_str(state, width, right_justify, true);
 		xfree(state);
 	} else if (sinfo_data)
@@ -1173,15 +1175,13 @@ int _print_user(sinfo_data_t * sinfo_data, int width,
 			bool right_justify, char *suffix)
 {
 	if (sinfo_data && (sinfo_data->reason_uid != NO_VAL)) {
-		char user[FORMAT_STRING_SIZE];
-		struct passwd *pw = NULL;
+		char *user = uid_to_string_or_null(sinfo_data->reason_uid);
 
-		if ((pw=getpwuid(sinfo_data->reason_uid)))
-			snprintf(user, sizeof(user), "%s", pw->pw_name);
-		else
-			snprintf(user, sizeof(user), "Unk(%u)",
-				 sinfo_data->reason_uid);
+		if (!user)
+			xstrfmtcat(user, "Unk(%u)", sinfo_data->reason_uid);
 		_print_str(user, width, right_justify, true);
+
+		xfree(user);
 	} else if (sinfo_data)
 		_print_str("Unknown", width, right_justify, true);
 	else
@@ -1196,16 +1196,14 @@ int _print_user_long(sinfo_data_t * sinfo_data, int width,
 			bool right_justify, char *suffix)
 {
 	if (sinfo_data && (sinfo_data->reason_uid != NO_VAL)) {
-		char user[FORMAT_STRING_SIZE];
-		struct passwd *pw = NULL;
+		char *user = uid_to_string_or_null(sinfo_data->reason_uid);
 
-		if ((pw=getpwuid(sinfo_data->reason_uid)))
-			snprintf(user, sizeof(user), "%s(%u)", pw->pw_name,
-				 sinfo_data->reason_uid);
-		else
-			snprintf(user, sizeof(user), "Unk(%u)",
-				 sinfo_data->reason_uid);
+		if (!user)
+			user = xstrdup("Unk");
+		xstrfmtcat(user, "(%u)", sinfo_data->reason_uid);
 		_print_str(user, width, right_justify, true);
+
+		xfree(user);
 	} else if (sinfo_data)
 		_print_str("Unknown", width, right_justify, true);
 	else
