@@ -3,7 +3,7 @@
  *****************************************************************************
  *  Copyright (C) 2002-2007 The Regents of the University of California.
  *  Copyright (C) 2008-2010 Lawrence Livermore National Security.
- *  Portions Copyright (C) 2010-2017 SchedMD <https://www.schedmd.com>.
+ *  Copyright (C) SchedMD LLC.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
  *  Written by Joey Ekstrom <ekstrom1@llnl.gov>, Morris Jette <jette1@llnl.gov>
  *  CODE-OCEC-09-009. All rights reserved.
@@ -41,10 +41,11 @@
 #include "slurm/slurmdb.h"
 #include "src/common/xstring.h"
 #include "src/common/macros.h"
-#include "src/common/node_select.h"
+#include "src/interfaces/select.h"
 #include "src/common/slurm_time.h"
 #include "src/sinfo/sinfo.h"
 #include "src/sinfo/print.h"
+#include "src/interfaces/data_parser.h"
 
 /********************
  * Global Variables *
@@ -75,6 +76,8 @@ static pthread_mutex_t sinfo_list_mutex = PTHREAD_MUTEX_INITIALIZER;
 /*************
  * Functions *
  *************/
+static void _free_sinfo_format(void *object);
+static void _free_params(void);
 void *      _build_part_info(void *args);
 static int  _build_sinfo_data(List sinfo_list,
 			      partition_info_msg_t *partition_msg,
@@ -83,8 +86,8 @@ static sinfo_data_t *_create_sinfo(partition_info_t* part_ptr,
 				   uint16_t part_inx, node_info_t *node_ptr);
 static int  _find_part_list(void *x, void *key);
 static bool _filter_out(node_info_t *node_ptr);
-static int  _get_info(bool clear_old, slurmdb_federation_rec_t *fed,
-		      char *cluster_name);
+static int _get_info(bool clear_old, slurmdb_federation_rec_t *fed,
+		     char *cluster_name, int argc, char **argv);
 static int  _insert_node_ptr(List sinfo_list, uint16_t part_num,
 			     partition_info_t *part_ptr,
 			     node_info_t *node_ptr);
@@ -92,7 +95,7 @@ static int  _load_resv(reserve_info_msg_t ** reserv_pptr, bool clear_old);
 static bool _match_node_data(sinfo_data_t *sinfo_ptr, node_info_t *node_ptr);
 static bool _match_part_data(sinfo_data_t *sinfo_ptr,
 			     partition_info_t* part_ptr);
-static int  _multi_cluster(List clusters);
+static int _multi_cluster(List clusters, int argc, char **argv);
 static void _node_list_delete(void *data);
 static void _part_list_delete(void *data);
 static List _query_fed_servers(slurmdb_federation_rec_t *fed,
@@ -110,8 +113,10 @@ int main(int argc, char **argv)
 	log_options_t opts = LOG_OPTS_STDERR_ONLY;
 	int rc = 0;
 
-	slurm_conf_init(NULL);
+	slurm_init(NULL);
 	log_init(xbasename(argv[0]), opts, SYSLOG_FACILITY_USER, NULL);
+	memset(&params, 0, sizeof(params));
+	params.format_list = list_create(_free_sinfo_format);
 	parse_command_line(argc, argv);
 	if (params.verbose) {
 		opts.stderr_level += params.verbose;
@@ -119,14 +124,14 @@ int main(int argc, char **argv)
 	}
 
 	while (1) {
-		if ((!params.no_header) &&
+		if (!params.no_header && !params.mimetype &&
 		    (params.iterate || params.verbose || params.long_output))
 			print_date();
 
 		if (!params.clusters) {
-			if (_get_info(false, params.fed, NULL))
+			if (_get_info(false, params.fed, NULL, argc, argv))
 				rc = 1;
-		} else if (_multi_cluster(params.clusters) != 0)
+		} else if (_multi_cluster(params.clusters, argc, argv))
 			rc = 1;
 		if (params.iterate) {
 			printf("\n");
@@ -135,9 +140,35 @@ int main(int argc, char **argv)
 			break;
 	}
 
+	_free_params();
+
 	exit(rc);
 }
 
+static void _free_sinfo_format(void *object)
+{
+	sinfo_format_t *x = (sinfo_format_t *)object;
+
+	if (!x)
+		return;
+	xfree(x->suffix);
+	xfree(x);
+}
+
+static void _free_params(void)
+{
+	FREE_NULL_LIST(params.clusters);
+	xfree(params.format);
+	xfree(params.nodes);
+	xfree(params.partition);
+	xfree(params.sort);
+	xfree(params.states);
+	FREE_NULL_LIST(params.part_list);
+	FREE_NULL_LIST(params.format_list);
+	FREE_NULL_LIST(params.state_list);
+	slurmdb_destroy_federation_rec(params.fed);
+}
+
 static int _list_find_func(void *x, void *key)
 {
 	sinfo_format_t *sinfo_format = (sinfo_format_t *) x;
@@ -151,12 +182,12 @@ static void prepend_cluster_name(void)
 	if (list_find_first(params.format_list, _list_find_func,
 			    _print_cluster_name))
 		return;
-	format_prepend_cluster_name(params.format_list, 8, false, "");
+	format_prepend_cluster_name(params.format_list, 8, false, NULL);
 }
 
-static int _multi_cluster(List clusters)
+static int _multi_cluster(List clusters, int argc, char **argv)
 {
-	ListIterator itr;
+	list_itr_t *itr;
 	bool first = true;
 	int rc = 0, rc2;
 
@@ -172,7 +203,8 @@ static int _multi_cluster(List clusters)
 				printf("\n");
 			printf("CLUSTER: %s\n", working_cluster_rec->name);
 		}
-		rc2 = _get_info(true, NULL, working_cluster_rec->name);
+		rc2 = _get_info(true, NULL, working_cluster_rec->name, argc,
+				argv);
 		if (rc2)
 			rc = 1;
 	}
@@ -194,7 +226,7 @@ static int _set_cluster_name(void *x, void *arg)
  * fed IN - information about other clusters in this federation
  */
 static int _get_info(bool clear_old, slurmdb_federation_rec_t *fed,
-		     char *cluster_name)
+		     char *cluster_name, int argc, char **argv)
 {
 	List node_info_msg_list = NULL, part_info_msg_list = NULL;
 	reserve_info_msg_t *reserv_msg = NULL;
@@ -226,12 +258,22 @@ static int _get_info(bool clear_old, slurmdb_federation_rec_t *fed,
 	}
 
 	sort_sinfo_list(sinfo_list);
-	print_sinfo_list(sinfo_list);
+	if (params.mimetype)
+		if (is_data_parser_deprecated(params.data_parser))
+			DATA_DUMP_CLI_DEPRECATED(SINFO_DATA_LIST, sinfo_list,
+						 "sinfo", argc, argv, NULL,
+						 params.mimetype, rc);
+		else
+			DATA_DUMP_CLI_SINGLE(OPENAPI_SINFO_RESP, sinfo_list,
+					     argc, argv, NULL, params.mimetype,
+					     params.data_parser, rc);
+	else
+		rc = print_sinfo_list(sinfo_list);
 
 	FREE_NULL_LIST(node_info_msg_list);
 	FREE_NULL_LIST(part_info_msg_list);
 	FREE_NULL_LIST(sinfo_list);
-	return SLURM_SUCCESS;
+	return rc;
 }
 
 /*
@@ -299,13 +341,13 @@ static List _query_server(bool clear_old)
 	static partition_info_msg_t *old_part_ptr = NULL, *new_part_ptr;
 	static node_info_msg_t *old_node_ptr = NULL, *new_node_ptr;
 	int error_code;
-	uint16_t show_flags = 0;
-	int cc;
-	node_info_t *node_ptr;
+	uint16_t show_flags = SHOW_MIXED;
 	List sinfo_list = NULL;
 
 	if (params.all_flag)
 		show_flags |= SHOW_ALL;
+	if (params.future_flag)
+		show_flags |= SHOW_FUTURE;
 
 	if (old_part_ptr) {
 		if (clear_old)
@@ -329,7 +371,7 @@ static List _query_server(bool clear_old)
 	old_part_ptr = new_part_ptr;
 
 	/* GRES used is only populated on nodes with detail flag */
-	if (params.match_flags.gres_used_flag)
+	if (params.match_flags & MATCH_FLAG_GRES_USED)
 		show_flags |= SHOW_DETAIL;
 
 	if (old_node_ptr) {
@@ -362,28 +404,6 @@ static List _query_server(bool clear_old)
 	}
 	old_node_ptr = new_node_ptr;
 
-	/* Set the node state as NODE_STATE_MIXED. */
-	for (cc = 0; cc < new_node_ptr->record_count; cc++) {
-		node_ptr = &(new_node_ptr->node_array[cc]);
-		if (IS_NODE_DRAIN(node_ptr)) {
-			/* don't worry about mixed since the
-			 * whole node is being drained. */
-		} else {
-			uint16_t alloc_cpus = 0, idle_cpus;
-
-			select_g_select_nodeinfo_get(node_ptr->select_nodeinfo,
-						     SELECT_NODEDATA_SUBCNT,
-						     NODE_STATE_ALLOCATED,
-						     &alloc_cpus);
-			idle_cpus = node_ptr->cpus - alloc_cpus;
-
-			if (idle_cpus && (idle_cpus != node_ptr->cpus)) {
-				node_ptr->node_state &= NODE_STATE_FLAGS;
-				node_ptr->node_state |= NODE_STATE_MIXED;
-			}
-		}
-	}
-
 	sinfo_list = list_create(_sinfo_list_delete);
 	_build_sinfo_data(sinfo_list, new_part_ptr, new_node_ptr);
 
@@ -393,27 +413,27 @@ static List _query_server(bool clear_old)
 static void *_load_job_prio_thread(void *args)
 {
 	load_info_struct_t *load_args = (load_info_struct_t *) args;
-	uint16_t show_flags = 0;
+	uint16_t show_flags = SHOW_MIXED;
 	char *node_name = NULL;
 	slurmdb_cluster_rec_t *cluster = load_args->cluster;
 	int error_code;
 	partition_info_msg_t *new_part_ptr;
 	node_info_msg_t *new_node_ptr;
-	int cc;
-	node_info_t *node_ptr;
 	List sinfo_list = NULL;
 
 	if (params.node_name_single)
 		node_name = params.nodes;
 	if (params.all_flag)
 		show_flags |= SHOW_ALL;
+	if (params.future_flag)
+		show_flags |= SHOW_FUTURE;
 
 	error_code = slurm_load_partitions2((time_t) NULL, &new_part_ptr,
 					    show_flags, cluster);
 	if (error_code) {
 		slurm_perror("slurm_load_partitions");
 		xfree(args);
-		return (void *) NULL;
+		return NULL;
 	}
 	list_append(load_args->part_info_msg_list, new_part_ptr);
 
@@ -427,47 +447,25 @@ static void *_load_job_prio_thread(void *args)
 	if (error_code) {
 		slurm_perror("slurm_load_node");
 		xfree(args);
-		return (void *) NULL;
+		return NULL;
 	}
 	list_append(load_args->node_info_msg_list, new_node_ptr);
 
-	/* Set the node state as NODE_STATE_MIXED. */
-	for (cc = 0; cc < new_node_ptr->record_count; cc++) {
-		node_ptr = &(new_node_ptr->node_array[cc]);
-		if (IS_NODE_DRAIN(node_ptr)) {
-			/* don't worry about mixed since the
-			 * whole node is being drained. */
-		} else {
-			uint16_t alloc_cpus = 0, idle_cpus;
-
-			select_g_select_nodeinfo_get(node_ptr->select_nodeinfo,
-						     SELECT_NODEDATA_SUBCNT,
-						     NODE_STATE_ALLOCATED,
-						     &alloc_cpus);
-			idle_cpus = node_ptr->cpus - alloc_cpus;
-
-			if (idle_cpus && (idle_cpus != node_ptr->cpus)) {
-				node_ptr->node_state &= NODE_STATE_FLAGS;
-				node_ptr->node_state |= NODE_STATE_MIXED;
-			}
-		}
-	}
-
 	sinfo_list = list_create(_sinfo_list_delete);
 	_build_sinfo_data(sinfo_list, new_part_ptr, new_node_ptr);
 	if (sinfo_list) {
 		sinfo_data_t *sinfo_ptr;
-		ListIterator iter;
+		list_itr_t *iter;
 		iter = list_iterator_create(sinfo_list);
 		while ((sinfo_ptr = (sinfo_data_t *) list_next(iter)))
 			sinfo_ptr->cluster_name = cluster->name;
 		list_iterator_destroy(iter);
 		list_transfer(load_args->resp_msg_list, sinfo_list);
-		list_destroy(sinfo_list);
+		FREE_NULL_LIST(sinfo_list);
 	}
 
 	xfree(args);
-	return (void *) NULL;
+	return NULL;
 }
 
 /*
@@ -483,7 +481,7 @@ static List _query_fed_servers(slurmdb_federation_rec_t *fed,
 	List resp_msg_list;
 	int pthread_count = 0;
 	pthread_t *load_thread = 0;
-	ListIterator iter;
+	list_itr_t *iter;
 	slurmdb_cluster_rec_t *cluster;
 	load_info_struct_t *load_args;
 	int i;
@@ -511,7 +509,7 @@ static List _query_fed_servers(slurmdb_federation_rec_t *fed,
 
 	/* Wait for all pthreads to complete */
 	for (i = 0; i < pthread_count; i++)
-		pthread_join(load_thread[i], NULL);
+		slurm_thread_join(load_thread[i]);
 	xfree(load_thread);
 
 	return resp_msg_list;
@@ -587,7 +585,7 @@ static int _build_sinfo_data(List sinfo_list,
 	int j;
 
 	/* by default every partition is shown, even if no nodes */
-	if ((!params.node_flag) && params.match_flags.partition_flag) {
+	if ((!params.node_flag) && (params.match_flags & MATCH_FLAG_PARTITION)){
 		part_ptr = partition_msg->partition_array;
 		for (j = 0; j < partition_msg->record_count; j++, part_ptr++) {
 			if ((!params.part_list) ||
@@ -620,7 +618,7 @@ static int _build_sinfo_data(List sinfo_list,
 
 		if (node_msg->record_count == 1) { /* node_name_single */
 			int pos = -1;
-			hostlist_t hl;
+			hostlist_t *hl;
 
 			node_ptr = &(node_msg->node_array[0]);
 			if ((node_ptr->name == NULL) ||
@@ -647,7 +645,7 @@ static int _build_sinfo_data(List sinfo_list,
 		sinfo_cnt++;
 		slurm_mutex_unlock(&sinfo_cnt_mutex);
 
-		slurm_thread_create_detached(NULL, _build_part_info,
+		slurm_thread_create_detached(_build_part_info,
 					     build_struct_ptr);
 	}
 
@@ -669,7 +667,7 @@ static int _build_sinfo_data(List sinfo_list,
  */
 static bool _filter_out(node_info_t *node_ptr)
 {
-	static hostlist_t host_list = NULL;
+	static hostlist_t *host_list = NULL;
 
 	if (params.nodes) {
 		if (host_list == NULL)
@@ -688,38 +686,35 @@ static bool _filter_out(node_info_t *node_ptr)
 		int *node_state;
 		bool match = false;
 		uint32_t base_state;
-		ListIterator iterator;
+		list_itr_t *iterator;
 		uint16_t cpus = 0;
 		node_info_t tmp_node, *tmp_node_ptr = &tmp_node;
 
 		iterator = list_iterator_create(params.state_list);
 		while ((node_state = list_next(iterator))) {
+			match = false;
 			tmp_node_ptr->node_state = *node_state;
 			if (*node_state == NODE_STATE_DRAIN) {
 				/* We search for anything that has the
 				 * drain flag set */
 				if (IS_NODE_DRAIN(node_ptr)) {
 					match = true;
-					break;
 				}
 			} else if (IS_NODE_DRAINING(tmp_node_ptr)) {
 				/* We search for anything that gets mapped to
 				 * DRAINING in node_state_string */
 				if (IS_NODE_DRAINING(node_ptr)) {
 					match = true;
-					break;
 				}
 			} else if (IS_NODE_DRAINED(tmp_node_ptr)) {
 				/* We search for anything that gets mapped to
 				 * DRAINED in node_state_string */
 				if (IS_NODE_DRAINED(node_ptr)) {
 					match = true;
-					break;
 				}
 			} else if (*node_state & NODE_STATE_FLAGS) {
 				if (*node_state & node_ptr->node_state) {
 					match = true;
-					break;
 				}
 			} else if (*node_state == NODE_STATE_ALLOCATED) {
 				slurm_get_select_nodeinfo(
@@ -729,23 +724,19 @@ static bool _filter_out(node_info_t *node_ptr)
 					&cpus);
 				if (cpus) {
 					match = true;
-					break;
-				}
-			} else if (*node_state == NODE_STATE_IDLE) {
-				base_state = node_ptr->node_state &
-					(~NODE_STATE_NO_RESPOND);
-				if (base_state == NODE_STATE_IDLE) {
-					match = true;
-					break;
 				}
 			} else {
 				base_state =
 					node_ptr->node_state & NODE_STATE_BASE;
 				if (base_state == *node_state) {
 					match = true;
-					break;
 				}
 			}
+
+			if (!params.state_list_and && match)
+				break;
+			if (params.state_list_and && !match)
+				break;
 		}
 		list_iterator_destroy(iterator);
 		if (!match)
@@ -757,7 +748,7 @@ static bool _filter_out(node_info_t *node_ptr)
 
 static void _sort_hostlist(List sinfo_list)
 {
-	ListIterator i;
+	list_itr_t *i;
 	sinfo_data_t *sinfo_ptr;
 
 	i = list_iterator_create(sinfo_list);
@@ -775,52 +766,67 @@ static bool _match_node_data(sinfo_data_t *sinfo_ptr, node_info_t *node_ptr)
 	if (params.node_flag)
 		return false;
 
-	if (params.match_flags.hostnames_flag &&
+	if ((params.match_flags & MATCH_FLAG_HOSTNAMES) &&
 	    (hostlist_find(sinfo_ptr->hostnames,
 			   node_ptr->node_hostname) == -1))
 		return false;
 
-	if (params.match_flags.node_addr_flag &&
+	if ((params.match_flags & MATCH_FLAG_NODE_ADDR) &&
 	    (hostlist_find(sinfo_ptr->node_addr, node_ptr->node_addr) == -1))
 		return false;
 
 	if (sinfo_ptr->nodes &&
-	    params.match_flags.features_flag &&
+	    (params.match_flags & MATCH_FLAG_EXTRA) &&
+	    (xstrcmp(node_ptr->extra, sinfo_ptr->extra)))
+		return false;
+
+	if (sinfo_ptr->nodes &&
+	    (params.match_flags & MATCH_FLAG_FEATURES) &&
 	    (xstrcmp(node_ptr->features, sinfo_ptr->features)))
 		return false;
 
 	if (sinfo_ptr->nodes &&
-	    params.match_flags.features_act_flag &&
+	    (params.match_flags & MATCH_FLAG_FEATURES_ACT) &&
 	    (xstrcmp(node_ptr->features_act, sinfo_ptr->features_act)))
 		return false;
 
 	if (sinfo_ptr->nodes &&
-	    params.match_flags.gres_flag &&
+	    (params.match_flags & MATCH_FLAG_GRES) &&
 	    (xstrcmp(node_ptr->gres, sinfo_ptr->gres)))
 		return false;
 
 	if (sinfo_ptr->nodes &&
-	    params.match_flags.gres_used_flag &&
+	    (params.match_flags & MATCH_FLAG_GRES_USED) &&
 	    (xstrcmp(node_ptr->gres_used, sinfo_ptr->gres_used)))
 		return false;
 
 	if (sinfo_ptr->nodes &&
-	    params.match_flags.reason_flag &&
+	    (params.match_flags & MATCH_FLAG_COMMENT) &&
+	    (xstrcmp(node_ptr->comment, sinfo_ptr->comment)))
+		return false;
+
+	if (sinfo_ptr->nodes &&
+	    (params.match_flags & MATCH_FLAG_REASON) &&
 	    (xstrcmp(node_ptr->reason, sinfo_ptr->reason)))
 		return false;
 
 	if (sinfo_ptr->nodes &&
-	    params.match_flags.reason_timestamp_flag &&
+	    (params.match_flags & MATCH_FLAG_REASON_TIMESTAMP) &&
 	    (node_ptr->reason_time != sinfo_ptr->reason_time))
 		return false;
 
 	if (sinfo_ptr->nodes &&
-	    params.match_flags.reason_user_flag &&
+	    (params.match_flags & MATCH_FLAG_REASON_USER) &&
 	    node_ptr->reason_uid != sinfo_ptr->reason_uid) {
 		return false;
 	}
 
-	if (params.match_flags.state_flag) {
+	if (sinfo_ptr->nodes &&
+	    (params.match_flags & MATCH_FLAG_RESV_NAME) &&
+	    xstrcmp(node_ptr->resv_name, sinfo_ptr->resv_name))
+		return false;
+
+	if ((params.match_flags & MATCH_FLAG_STATE)) {
 		char *state1, *state2;
 		state1 = node_state_string(node_ptr->node_state);
 		state2 = node_state_string(sinfo_ptr->node_state);
@@ -828,11 +834,24 @@ static bool _match_node_data(sinfo_data_t *sinfo_ptr, node_info_t *node_ptr)
 			return false;
 	}
 
+	if ((params.match_flags & MATCH_FLAG_STATE_COMPLETE)) {
+		char *state1, *state2;
+		int rc = true;
+		state1 = node_state_string_complete(node_ptr->node_state);
+		state2 = node_state_string_complete(sinfo_ptr->node_state);
+		rc = xstrcmp(state1, state2);
+		xfree(state1);
+		xfree(state2);
+
+		if (rc)
+			return false;
+	}
+
 	select_g_select_nodeinfo_get(node_ptr->select_nodeinfo,
 				     SELECT_NODEDATA_MEM_ALLOC,
 				     NODE_STATE_ALLOCATED,
 				     &tmp);
-	if (params.match_flags.alloc_mem_flag &&
+	if ((params.match_flags & MATCH_FLAG_ALLOC_MEM) &&
 	    (tmp != sinfo_ptr->alloc_memory))
 		return false;
 
@@ -841,43 +860,43 @@ static bool _match_node_data(sinfo_data_t *sinfo_ptr, node_info_t *node_ptr)
 	if (!params.exact_match)
 		return true;
 
-	if (params.match_flags.cpus_flag &&
+	if ((params.match_flags & MATCH_FLAG_CPUS) &&
 	    (node_ptr->cpus        != sinfo_ptr->min_cpus))
 		return false;
 
-	if (params.match_flags.sockets_flag &&
+	if ((params.match_flags & MATCH_FLAG_SOCKETS) &&
 	    (node_ptr->sockets     != sinfo_ptr->min_sockets))
 		return false;
-	if (params.match_flags.cores_flag &&
+	if ((params.match_flags & MATCH_FLAG_CORES) &&
 	    (node_ptr->cores       != sinfo_ptr->min_cores))
 		return false;
-	if (params.match_flags.threads_flag &&
+	if ((params.match_flags & MATCH_FLAG_THREADS) &&
 	    (node_ptr->threads     != sinfo_ptr->min_threads))
 		return false;
-	if (params.match_flags.sct_flag &&
+	if ((params.match_flags & MATCH_FLAG_SCT) &&
 	    ((node_ptr->sockets     != sinfo_ptr->min_sockets) ||
 	     (node_ptr->cores       != sinfo_ptr->min_cores) ||
 	     (node_ptr->threads     != sinfo_ptr->min_threads)))
 		return false;
-	if (params.match_flags.disk_flag &&
+	if ((params.match_flags & MATCH_FLAG_DISK) &&
 	    (node_ptr->tmp_disk    != sinfo_ptr->min_disk))
 		return false;
-	if (params.match_flags.memory_flag &&
+	if ((params.match_flags & MATCH_FLAG_MEMORY) &&
 	    (node_ptr->real_memory != sinfo_ptr->min_mem))
 		return false;
-	if (params.match_flags.weight_flag &&
+	if ((params.match_flags & MATCH_FLAG_WEIGHT) &&
 	    (node_ptr->weight      != sinfo_ptr->min_weight))
 		return false;
-	if (params.match_flags.cpu_load_flag &&
+	if ((params.match_flags & MATCH_FLAG_CPU_LOAD) &&
 	    (node_ptr->cpu_load        != sinfo_ptr->min_cpu_load))
 		return false;
-	if (params.match_flags.free_mem_flag &&
+	if ((params.match_flags & MATCH_FLAG_FREE_MEM) &&
 	    (node_ptr->free_mem        != sinfo_ptr->min_free_mem))
 		return false;
-	if (params.match_flags.port_flag &&
+	if ((params.match_flags & MATCH_FLAG_PORT) &&
 	    (node_ptr->port != sinfo_ptr->port))
 		return false;
-	if (params.match_flags.version_flag &&
+	if ((params.match_flags & MATCH_FLAG_VERSION) &&
 	    (node_ptr->version     != sinfo_ptr->version))
 		return false;
 
@@ -891,7 +910,8 @@ static bool _serial_part_data(void)
 {
 	if (params.list_reasons)	/* Don't care about partition */
 		return true;
-	if (params.match_flags.partition_flag)	/* Match partition name */
+	/* Match partition name */
+	if ((params.match_flags & MATCH_FLAG_PARTITION))
 		return false;
 	return true;
 }
@@ -906,58 +926,58 @@ static bool _match_part_data(sinfo_data_t *sinfo_ptr,
 	if ((part_ptr == NULL) || (sinfo_ptr->part_info == NULL))
 		return false;
 
-	if (params.match_flags.partition_flag
+	if ((params.match_flags & MATCH_FLAG_PARTITION)
 	    && (xstrcmp(part_ptr->name, sinfo_ptr->part_info->name)))
 		return false;
 
-	if (params.match_flags.avail_flag &&
+	if ((params.match_flags & MATCH_FLAG_AVAIL) &&
 	    (part_ptr->state_up != sinfo_ptr->part_info->state_up))
 		return false;
 
-	if (params.match_flags.groups_flag &&
+	if ((params.match_flags & MATCH_FLAG_GROUPS) &&
 	    (xstrcmp(part_ptr->allow_groups,
 		     sinfo_ptr->part_info->allow_groups)))
 		return false;
 
-	if (params.match_flags.job_size_flag &&
+	if ((params.match_flags & MATCH_FLAG_JOB_SIZE) &&
 	    (part_ptr->min_nodes != sinfo_ptr->part_info->min_nodes))
 		return false;
 
-	if (params.match_flags.job_size_flag &&
+	if ((params.match_flags & MATCH_FLAG_JOB_SIZE) &&
 	    (part_ptr->max_nodes != sinfo_ptr->part_info->max_nodes))
 		return false;
 
-	if (params.match_flags.default_time_flag &&
+	if ((params.match_flags & MATCH_FLAG_DEFAULT_TIME) &&
 	    (part_ptr->default_time != sinfo_ptr->part_info->default_time))
 		return false;
 
-	if (params.match_flags.max_time_flag &&
+	if ((params.match_flags & MATCH_FLAG_MAX_TIME) &&
 	    (part_ptr->max_time != sinfo_ptr->part_info->max_time))
 		return false;
 
-	if (params.match_flags.root_flag &&
+	if ((params.match_flags & MATCH_FLAG_ROOT) &&
 	    ((part_ptr->flags & PART_FLAG_ROOT_ONLY) !=
 	     (sinfo_ptr->part_info->flags & PART_FLAG_ROOT_ONLY)))
 		return false;
 
-	if (params.match_flags.oversubscribe_flag &&
+	if ((params.match_flags & MATCH_FLAG_OVERSUBSCRIBE) &&
 	    (part_ptr->max_share != sinfo_ptr->part_info->max_share))
 		return false;
 
-	if (params.match_flags.preempt_mode_flag &&
+	if ((params.match_flags & MATCH_FLAG_PREEMPT_MODE) &&
 	    (part_ptr->preempt_mode != sinfo_ptr->part_info->preempt_mode))
 		return false;
 
-	if (params.match_flags.priority_tier_flag &&
+	if ((params.match_flags & MATCH_FLAG_PRIORITY_TIER) &&
 	    (part_ptr->priority_tier != sinfo_ptr->part_info->priority_tier))
 		return false;
 
-	if (params.match_flags.priority_job_factor_flag &&
+	if ((params.match_flags & MATCH_FLAG_PRIORITY_JOB_FACTOR) &&
 	    (part_ptr->priority_job_factor !=
 	     sinfo_ptr->part_info->priority_job_factor))
 		return false;
 
-	if (params.match_flags.max_cpus_per_node_flag &&
+	if ((params.match_flags & MATCH_FLAG_MAX_CPUS_PER_NODE) &&
 	    (part_ptr->max_cpus_per_node !=
 	     sinfo_ptr->part_info->max_cpus_per_node))
 		return false;
@@ -980,9 +1000,12 @@ static void _update_sinfo(sinfo_data_t *sinfo_ptr, node_info_t *node_ptr)
 		sinfo_ptr->features_act = node_ptr->features_act;
 		sinfo_ptr->gres       = node_ptr->gres;
 		sinfo_ptr->gres_used  = node_ptr->gres_used;
+		sinfo_ptr->comment    = node_ptr->comment;
+		sinfo_ptr->extra      = node_ptr->extra;
 		sinfo_ptr->reason     = node_ptr->reason;
 		sinfo_ptr->reason_time= node_ptr->reason_time;
 		sinfo_ptr->reason_uid = node_ptr->reason_uid;
+		sinfo_ptr->resv_name  = node_ptr->resv_name;
 		sinfo_ptr->min_cpus    = node_ptr->cpus;
 		sinfo_ptr->max_cpus    = node_ptr->cpus;
 		sinfo_ptr->min_sockets = node_ptr->sockets;
@@ -1058,10 +1081,10 @@ static void _update_sinfo(sinfo_data_t *sinfo_ptr, node_info_t *node_ptr)
 
 	if (hostlist_find(sinfo_ptr->nodes, node_ptr->name) == -1)
 		hostlist_push_host(sinfo_ptr->nodes, node_ptr->name);
-	if (params.match_flags.node_addr_flag &&
+	if ((params.match_flags & MATCH_FLAG_NODE_ADDR) &&
 	    (hostlist_find(sinfo_ptr->node_addr, node_ptr->node_addr) == -1))
 		hostlist_push_host(sinfo_ptr->node_addr, node_ptr->node_addr);
-	if (params.match_flags.hostnames_flag &&
+	if ((params.match_flags & MATCH_FLAG_HOSTNAMES) &&
 	    (hostlist_find(sinfo_ptr->hostnames, node_ptr->node_hostname) == -1))
 		hostlist_push_host(sinfo_ptr->hostnames, node_ptr->node_hostname);
 
@@ -1105,7 +1128,7 @@ static int _insert_node_ptr(List sinfo_list, uint16_t part_num,
 {
 	int rc = SLURM_SUCCESS;
 	sinfo_data_t *sinfo_ptr = NULL;
-	ListIterator itr = NULL;
+	list_itr_t *itr = NULL;
 
 	itr = list_iterator_create(sinfo_list);
 	while ((sinfo_ptr = list_next(itr))) {
