@@ -2,7 +2,7 @@
  *  gang.c - Gang scheduler functions.
  *****************************************************************************
  *  Copyright (C) 2008 Hewlett-Packard Development Company, L.P.
- *  Portions Copyright (C) 2010 SchedMD <https://www.schedmd.com>.
+ *  Copyright (C) SchedMD LLC.
  *  Written by Chris Holmes
  *  CODE-OCEC-09-009. All rights reserved.
  *
@@ -49,11 +49,13 @@
 #include "src/common/bitstring.h"
 #include "src/common/list.h"
 #include "src/common/macros.h"
-#include "src/common/node_select.h"
 #include "src/common/slurm_protocol_defs.h"
 #include "src/common/xstring.h"
+
+#include "src/interfaces/preempt.h"
+#include "src/interfaces/select.h"
+
 #include "src/slurmctld/locks.h"
-#include "src/slurmctld/preempt.h"
 #include "src/slurmctld/slurmctld.h"
 
 /* global timeslicer thread variables */
@@ -150,7 +152,6 @@ static List gs_part_list = NULL;
 static uint32_t default_job_list_size = 64;
 static pthread_mutex_t data_mutex = PTHREAD_MUTEX_INITIALIZER;
 
-static uint16_t *gs_bits_per_node = NULL;
 static uint32_t num_sorted_part = 0;
 
 /* function declarations */
@@ -183,7 +184,7 @@ static void _print_jobs(struct gs_part *p_ptr)
 {
 	int i;
 
-	if (slurmctld_conf.debug_flags & DEBUG_FLAG_GANG) {
+	if (slurm_conf.debug_flags & DEBUG_FLAG_GANG) {
 		info("gang:  part %s has %u jobs, %u shadows:",
 		     p_ptr->part_name, p_ptr->num_jobs, p_ptr->num_shadows);
 		for (i = 0; i < p_ptr->num_shadows; i++) {
@@ -209,14 +210,14 @@ static void _print_jobs(struct gs_part *p_ptr)
 
 static uint16_t _get_gr_type(void)
 {
-	if (slurmctld_conf.select_type_param & CR_CORE)
+	if (slurm_conf.select_type_param & CR_CORE)
 		return GS_CORE;
-	if (slurmctld_conf.select_type_param & CR_CPU) {
-		if (!xstrcmp(slurmctld_conf.task_plugin, "task/none"))
+	if (slurm_conf.select_type_param & CR_CPU) {
+		if (!xstrcmp(slurm_conf.task_plugin, "task/none"))
 			return GS_CPU;
 		return GS_CPU2;
 	}
-	if (slurmctld_conf.select_type_param & CR_SOCKET)
+	if (slurm_conf.select_type_param & CR_SOCKET)
 		return GS_SOCKET;
 
 	/* note that CR_MEMORY is node-level scheduling with
@@ -230,7 +231,7 @@ static uint16_t _get_part_gr_type(part_record_t *part_ptr)
 		if (part_ptr->cr_type & CR_CORE)
 			return GS_CORE;
 		if (part_ptr->cr_type & CR_CPU) {
-			if (!xstrcmp(slurmctld_conf.task_plugin, "task/none"))
+			if (!xstrcmp(slurm_conf.task_plugin, "task/none"))
 				return GS_CPU;
 			return GS_CPU2;
 		}
@@ -242,58 +243,20 @@ static uint16_t _get_part_gr_type(part_record_t *part_ptr)
 	return gr_type;
 }
 
-/* For GS_CPU and GS_CPU2 gs_bits_per_node is the total number of CPUs per node.
- * For GS_CORE and GS_SOCKET gs_bits_per_node is the total number of
- *	cores per per node.
- */
-static void _load_phys_res_cnt(void)
-{
-	uint16_t bit = 0, sock = 0;
-	uint32_t i, bit_index = 0;
-	node_record_t *node_ptr;
-
-	xfree(gs_bits_per_node);
-
-	if (gr_type == GS_NODE)
-		return;
-
-	gs_bits_per_node = xmalloc(node_record_count * sizeof(uint16_t));
-
-	for (i = 0, node_ptr = node_record_table_ptr; i < node_record_count;
-	     i++, node_ptr++) {
-		if (gr_type == GS_CPU) {
-			bit = node_ptr->config_ptr->cpus;
-		} else {
-			sock = node_ptr->config_ptr->sockets;
-			bit  = node_ptr->config_ptr->cores * sock;
-		}
-
-		gs_bits_per_node[bit_index++] = bit;
-	}
-
-	if (slurmctld_conf.debug_flags & DEBUG_FLAG_GANG) {
-		for (i = 0; i < bit_index; i++) {
-			info("gang: _load_phys_res_cnt: bits_per_node[%d]=%u",
-			     i, gs_bits_per_node[i]);
-		}
-	}
-}
-
 static uint16_t _get_phys_bit_cnt(int node_index)
 {
-	node_record_t *node_ptr = node_record_table_ptr + node_index;
+	node_record_t *node_ptr = node_record_table_ptr[node_index];
 
 	if (gr_type == GS_CPU)
-		return node_ptr->config_ptr->cpus;
-	return node_ptr->config_ptr->cores *
-		node_ptr->config_ptr->sockets;
+		return node_ptr->cpus;
+	return node_ptr->tot_cores;
 }
 
 static uint16_t _get_socket_cnt(int node_index)
 {
-	node_record_t *node_ptr = node_record_table_ptr + node_index;
+	node_record_t *node_ptr = node_record_table_ptr[node_index];
 
-	return node_ptr->config_ptr->sockets;
+	return node_ptr->config_ptr->tot_sockets;
 }
 
 static void _destroy_parts(void *x)
@@ -315,7 +278,7 @@ static void _destroy_parts(void *x)
  * once a job is added. */
 static void _build_parts(void)
 {
-	ListIterator part_iterator;
+	list_itr_t *part_iterator;
 	part_record_t *p_ptr;
 	struct gs_part *gs_part_ptr;
 	int num_parts;
@@ -366,26 +329,23 @@ static int _find_job_index(struct gs_part *p_ptr, uint32_t job_id)
 /* Return 1 if job "cpu count" fits in this row, else return 0 */
 static int _can_cpus_fit(job_record_t *job_ptr, struct gs_part *p_ptr)
 {
-	int i, j, size;
 	uint16_t *p_cpus, *j_cpus;
 	job_resources_t *job_res = job_ptr->job_resrcs;
 
 	if (gr_type != GS_CPU)
 		return 0;
 
-	size = bit_size(job_res->node_bitmap);
 	p_cpus = p_ptr->active_cpus;
 	j_cpus = job_res->cpus;
 
 	if (!p_cpus || !j_cpus)
 		return 0;
 
-	for (j = 0, i = 0; i < size; i++) {
-		if (bit_test(job_res->node_bitmap, i)) {
-			if (p_cpus[i]+j_cpus[j] > _get_phys_bit_cnt(i))
-				return 0;
-			j++;
-		}
+	for (int j = 0, i = 0; next_node_bitmap(job_res->node_bitmap, &i);
+	     i++) {
+		if (p_cpus[i] + j_cpus[j] > _get_phys_bit_cnt(i))
+			return 0;
+		j++;
 	}
 	return 1;
 }
@@ -406,8 +366,7 @@ static int _job_fits_in_active_row(job_record_t *job_ptr,
 	job_gr_type = _get_part_gr_type(job_ptr->part_ptr);
 	if ((job_gr_type == GS_CPU2) || (job_gr_type == GS_CORE) ||
 	    (job_gr_type == GS_SOCKET)) {
-		return job_fits_into_cores(job_res, p_ptr->active_resmap,
-					   gs_bits_per_node);
+		return job_fits_into_cores(job_res, p_ptr->active_resmap);
 	}
 
 	/* job_gr_type == GS_NODE || job_gr_type == GS_CPU */
@@ -415,8 +374,7 @@ static int _job_fits_in_active_row(job_record_t *job_ptr,
 	bit_and(job_map, p_ptr->active_resmap);
 	/* any set bits indicate contention for the same resource */
 	count = bit_set_count(job_map);
-	if (slurmctld_conf.debug_flags & DEBUG_FLAG_GANG)
-		info("gang: _job_fits_in_active_row: %d bits conflict", count);
+	log_flag(GANG, "gang: %s: %d bits conflict", __func__, count);
 	FREE_NULL_BITMAP(job_map);
 	if (count == 0)
 		return 1;
@@ -482,33 +440,24 @@ static void _add_job_to_active(job_record_t *job_ptr, struct gs_part *p_ptr)
 	job_gr_type = _get_part_gr_type(job_ptr->part_ptr);
 	if ((job_gr_type == GS_CPU2) || (job_gr_type == GS_CORE) ||
 	    (job_gr_type == GS_SOCKET)) {
-		if (p_ptr->jobs_active == 0 && p_ptr->active_resmap) {
-			uint32_t size = bit_size(p_ptr->active_resmap);
-			bit_nclear(p_ptr->active_resmap, 0, size-1);
-		}
-		add_job_to_cores(job_res, &(p_ptr->active_resmap),
-				 gs_bits_per_node);
+		if (p_ptr->jobs_active == 0 && p_ptr->active_resmap)
+			bit_clear_all(p_ptr->active_resmap);
+		add_job_to_cores(job_res, &(p_ptr->active_resmap));
 		if (job_gr_type == GS_SOCKET)
 			_fill_sockets(job_res->node_bitmap, p_ptr);
 	} else { /* GS_NODE or GS_CPU */
 		if (!p_ptr->active_resmap) {
-			if (slurmctld_conf.debug_flags & DEBUG_FLAG_GANG) {
-				info("gang: %s: %pJ first",
-				     __func__, job_ptr);
-			}
+			log_flag(GANG, "gang: %s: %pJ first",
+				 __func__, job_ptr);
 			p_ptr->active_resmap = bit_copy(job_res->node_bitmap);
 		} else if (p_ptr->jobs_active == 0) {
-			if (slurmctld_conf.debug_flags & DEBUG_FLAG_GANG) {
-				info("gang: %s: %pJ copied",
-				     __func__, job_ptr);
-			}
+			log_flag(GANG, "gang: %s: %pJ copied",
+				 __func__, job_ptr);
 			bit_copybits(p_ptr->active_resmap,
 				     job_res->node_bitmap);
 		} else {
-			if (slurmctld_conf.debug_flags & DEBUG_FLAG_GANG) {
-				info("gang: %s: adding %pJ",
-				     __func__, job_ptr);
-			}
+			log_flag(GANG, "gang: %s: adding %pJ",
+				 __func__, job_ptr);
 			bit_or(p_ptr->active_resmap, job_res->node_bitmap);
 		}
 	}
@@ -560,7 +509,7 @@ static int _suspend_job(job_record_t *job_ptr)
 	rc = job_suspend(&msg, 0, -1, false, NO_VAL16);
 	/* job_suspend() returns ESLURM_DISABLED if job is already suspended */
 	if (rc == SLURM_SUCCESS) {
-		if (slurmctld_conf.debug_flags & DEBUG_FLAG_GANG)
+		if (slurm_conf.debug_flags & DEBUG_FLAG_GANG)
 			info("gang: suspending %pJ", job_ptr);
 		else
 			debug("gang: suspending %pJ", job_ptr);
@@ -581,7 +530,7 @@ static void _resume_job(job_record_t *job_ptr)
 	msg.op = RESUME_JOB;
 	rc = job_suspend(&msg, 0, -1, false, NO_VAL16);
 	if (rc == SLURM_SUCCESS) {
-		if (slurmctld_conf.debug_flags & DEBUG_FLAG_GANG)
+		if (slurm_conf.debug_flags & DEBUG_FLAG_GANG)
 			info("gang: resuming %pJ", job_ptr);
 		else
 			debug("gang: resuming %pJ", job_ptr);
@@ -651,8 +600,6 @@ static void _preempt_job_dequeue(void)
 			}
 		}
 	}
-
-	return;
 }
 
 /* This is the reverse order defined by list.h so to generated a list in
@@ -661,23 +608,18 @@ static int _sort_partitions(void *part1, void *part2)
 {
 	struct gs_part *g1;
 	struct gs_part *g2;
-	int prio1;
-	int prio2;
 
 	g1 = *(struct gs_part **)part1;
 	g2 = *(struct gs_part **)part2;
 
-	prio1 = g1->priority;
-	prio2 = g2->priority;
-
-	return prio2 - prio1;
+	return slurm_sort_uint_list_desc(&g1->priority, &g2->priority);
 }
 
 /* Scan the partition list. Add the given job as a "shadow" to every
  * partition with a lower priority than the given partition */
 static void _cast_shadow(struct gs_job *j_ptr, uint16_t priority)
 {
-	ListIterator part_iterator;
+	list_itr_t *part_iterator;
 	struct gs_part *p_ptr;
 	int i;
 
@@ -717,7 +659,7 @@ static void _cast_shadow(struct gs_job *j_ptr, uint16_t priority)
 /* Remove the given job as a "shadow" from all partitions */
 static void _clear_shadow(struct gs_job *j_ptr)
 {
-	ListIterator part_iterator;
+	list_itr_t *part_iterator;
 	struct gs_part *p_ptr;
 	int i;
 
@@ -756,10 +698,8 @@ static void _update_active_row(struct gs_part *p_ptr, int add_new_jobs)
 	struct gs_job *j_ptr;
 	uint16_t preempt_mode;
 
-	if (slurmctld_conf.debug_flags & DEBUG_FLAG_GANG) {
-		info("gang: update_active_row: rebuilding part %s...",
-		     p_ptr->part_name);
-	}
+	log_flag(GANG, "gang: update_active_row: rebuilding part %s...",
+		 p_ptr->part_name);
 	/* rebuild the active row, starting with any shadows */
 	p_ptr->jobs_active = 0;
 	for (i = 0; p_ptr->shadow && p_ptr->shadow[i]; i++) {
@@ -851,7 +791,7 @@ static void _update_active_row(struct gs_part *p_ptr, int add_new_jobs)
  */
 static void _update_all_active_rows(void)
 {
-	ListIterator part_iterator;
+	list_itr_t *part_iterator;
  	struct gs_part *p_ptr;
 
 	/* Sort the partitions. This way the shadows of any high-priority
@@ -887,10 +827,8 @@ static void _remove_job_from_part(uint32_t job_id, struct gs_part *p_ptr,
 
 	j_ptr = p_ptr->job_list[i];
 
-	if (slurmctld_conf.debug_flags & DEBUG_FLAG_GANG) {
-		info("gang: %s: removing %pJ from %s",
-		     __func__, j_ptr->job_ptr, p_ptr->part_name);
-	}
+	log_flag(GANG, "gang: %s: removing %pJ from %s",
+		 __func__, j_ptr->job_ptr, p_ptr->part_name);
 
 	/* remove any shadow first */
 	_clear_shadow(j_ptr);
@@ -905,16 +843,12 @@ static void _remove_job_from_part(uint32_t job_id, struct gs_part *p_ptr,
 	/* make sure the job is not suspended by gang, and then delete it */
 	if (!fini && (j_ptr->sig_state == GS_SUSPEND) &&
 	    j_ptr->job_ptr->priority) {
-		if (slurmctld_conf.debug_flags & DEBUG_FLAG_GANG) {
-			info("gang: %s: resuming suspended %pJ",
-			     __func__, j_ptr->job_ptr);
-		}
+		log_flag(GANG, "gang: %s: resuming suspended %pJ",
+			 __func__, j_ptr->job_ptr);
 		_resume_job(j_ptr->job_ptr);
 	}
 	j_ptr->job_ptr = NULL;
 	xfree(j_ptr);
-
-	return;
 }
 
 /* Add the given job to the given partition, and if it remains running
@@ -933,10 +867,8 @@ static uint16_t _add_job_to_part(struct gs_part *p_ptr, job_record_t *job_ptr)
 	xassert(job_ptr->job_resrcs->node_bitmap);
 	xassert(job_ptr->job_resrcs->core_bitmap);
 
-	if (slurmctld_conf.debug_flags & DEBUG_FLAG_GANG) {
-		info("gang: %s: adding %pJ to %s",
-		     __func__, job_ptr, p_ptr->part_name);
-	}
+	log_flag(GANG, "gang: %s: adding %pJ to %s",
+		 __func__, job_ptr, p_ptr->part_name);
 
 	/* take care of any memory needs */
 	if (!p_ptr->job_list) {
@@ -953,10 +885,8 @@ static uint16_t _add_job_to_part(struct gs_part *p_ptr, job_record_t *job_ptr)
 		 * may have changed. In any case, remove the existing
 		 * job before adding this new one.
 		 */
-		if (slurmctld_conf.debug_flags & DEBUG_FLAG_GANG) {
-			info("gang: %s: duplicate %pJ detected",
-			     __func__, job_ptr);
-		}
+		log_flag(GANG, "gang: %s: duplicate %pJ detected",
+			 __func__, job_ptr);
 		_remove_job_from_part(job_ptr->job_id, p_ptr, false);
 		_update_active_row(p_ptr, 0);
 	}
@@ -982,10 +912,8 @@ static uint16_t _add_job_to_part(struct gs_part *p_ptr, job_record_t *job_ptr)
 	/* determine the immediate fate of this job (run or suspend) */
 	if (!IS_JOB_SUSPENDED(job_ptr) &&
 	    _job_fits_in_active_row(job_ptr, p_ptr)) {
-		if (slurmctld_conf.debug_flags & DEBUG_FLAG_GANG) {
-			info("gang: %s: %pJ remains running",
-			     __func__, job_ptr);
-		}
+		log_flag(GANG, "gang: %s: %pJ remains running",
+			 __func__, job_ptr);
 		_add_job_to_active(job_ptr, p_ptr);
 		/* note that this job is a "filler" for this row */
 		j_ptr->row_state = GS_FILLER;
@@ -997,10 +925,8 @@ static uint16_t _add_job_to_part(struct gs_part *p_ptr, job_record_t *job_ptr)
 		_cast_shadow(j_ptr, p_ptr->priority);
 
 	} else {
-		if (slurmctld_conf.debug_flags & DEBUG_FLAG_GANG) {
-			info("gang: %s: suspending %pJ",
-			     __func__, job_ptr);
-		}
+		log_flag(GANG, "gang: %s: suspending %pJ",
+			 __func__, job_ptr);
 		preempt_mode = slurm_job_preempt_mode(job_ptr);
 		if (p_ptr->num_shadows &&
 		    (preempt_mode != PREEMPT_MODE_OFF) &&
@@ -1025,22 +951,23 @@ static void _scan_slurm_job_list(void)
 	job_record_t *job_ptr;
 	struct gs_part *p_ptr;
 	int i;
-	ListIterator job_iterator;
+	list_itr_t *job_iterator;
 	char *part_name;
 
 	if (!job_list) {	/* no jobs */
-		if (slurmctld_conf.debug_flags & DEBUG_FLAG_GANG)
-			info("gang: _scan_slurm_job_list: job_list NULL");
+		log_flag(GANG, "gang: %s: job_list NULL", __func__);
 		return;
 	}
-	if (slurmctld_conf.debug_flags & DEBUG_FLAG_GANG)
-		info("gang: _scan_slurm_job_list: job_list exists...");
+	log_flag(GANG, "gang: %s: job_list exists...", __func__);
 	job_iterator = list_iterator_create(job_list);
 	while ((job_ptr = list_next(job_iterator))) {
-		if (slurmctld_conf.debug_flags & DEBUG_FLAG_GANG) {
-			info("gang: %s: checking %pJ",
-			     __func__, job_ptr);
-		}
+		log_flag(GANG, "gang: %s: checking %pJ",
+			 __func__, job_ptr);
+
+		/* Exclude HetJobs from gang operation. */
+		if (job_ptr->het_job_id)
+			continue;
+
 		if (IS_JOB_PENDING(job_ptr))
 			continue;
 		if (IS_JOB_SUSPENDED(job_ptr) && (job_ptr->priority == 0))
@@ -1081,8 +1008,6 @@ static void _scan_slurm_job_list(void)
 	/* now that all of the old jobs have been flushed out,
 	 * update the active row of all partitions */
 	_update_all_active_rows();
-
-	return;
 }
 
 
@@ -1121,22 +1046,18 @@ static void _spawn_timeslicer_thread(void)
 /* Initialize data structures and start the gang scheduling thread */
 extern void gs_init(void)
 {
-	if (!(slurmctld_conf.preempt_mode & PREEMPT_MODE_GANG))
+	if (!(slurm_conf.preempt_mode & PREEMPT_MODE_GANG))
 		return;
 
 	if (timeslicer_thread_id)
 		return;
 
 	/* initialize global variables */
-	if (slurmctld_conf.debug_flags & DEBUG_FLAG_GANG)
-		info("gang: entering gs_init");
-	timeslicer_seconds = slurmctld_conf.sched_time_slice;
+	log_flag(GANG, "gang: entering gs_init");
+	timeslicer_seconds = slurm_conf.sched_time_slice;
 	gr_type = _get_gr_type();
 	preempt_job_list = list_create(xfree_ptr);
 
-	/* load the physical resource count data */
-	_load_phys_res_cnt();
-
 	slurm_mutex_lock(&data_mutex);
 	_build_parts();
 	/* load any currently running jobs */
@@ -1145,19 +1066,14 @@ extern void gs_init(void)
 
 	/* spawn the timeslicer thread */
 	_spawn_timeslicer_thread();
-	if (slurmctld_conf.debug_flags & DEBUG_FLAG_GANG)
-		info("gang: leaving gs_init");
+	log_flag(GANG, "gang: leaving gs_init");
 }
 
 /* Terminate the gang scheduling thread and free its data structures */
 extern void gs_fini(void)
 {
-	if (!(slurmctld_conf.preempt_mode & PREEMPT_MODE_GANG))
-		return;
-
 	/* terminate the timeslicer thread */
-	if (slurmctld_conf.debug_flags & DEBUG_FLAG_GANG)
-		info("gang: entering gs_fini");
+	log_flag(GANG, "gang: entering gs_fini");
 	slurm_mutex_lock(&thread_flag_mutex);
 	if (thread_running) {
 		slurm_mutex_lock(&term_lock);
@@ -1168,6 +1084,14 @@ extern void gs_fini(void)
 		usleep(120000);
 		if (timeslicer_thread_id)
 			error("gang: timeslicer pthread still running");
+		else {
+			slurm_mutex_lock(&thread_flag_mutex);
+			thread_running = false;
+			slurm_mutex_unlock(&thread_flag_mutex);
+			slurm_mutex_lock(&term_lock);
+			thread_shutdown = false;
+			slurm_mutex_unlock(&term_lock);
+		}
 	} else {
 		slurm_mutex_unlock(&thread_flag_mutex);
 	}
@@ -1177,10 +1101,8 @@ extern void gs_fini(void)
 	slurm_mutex_lock(&data_mutex);
 	FREE_NULL_LIST(gs_part_list);
 	gs_part_list = NULL;
-	xfree(gs_bits_per_node);
 	slurm_mutex_unlock(&data_mutex);
-	if (slurmctld_conf.debug_flags & DEBUG_FLAG_GANG)
-		info("gang: leaving gs_fini");
+	log_flag(GANG, "gang: leaving gs_fini");
 }
 
 /* Notify the gang scheduler that a job has been resumed or started.
@@ -1191,11 +1113,14 @@ extern void gs_job_start(job_record_t *job_ptr)
 	uint16_t job_sig_state;
 	char *part_name;
 
-	if (!(slurmctld_conf.preempt_mode & PREEMPT_MODE_GANG))
+	if (!(slurm_conf.preempt_mode & PREEMPT_MODE_GANG))
 		return;
 
-	if (slurmctld_conf.debug_flags & DEBUG_FLAG_GANG)
-		info("gang: entering %s for %pJ", __func__, job_ptr);
+	/* Exclude HetJobs from gang operation. */
+	if (job_ptr->het_job_id)
+		return;
+
+	log_flag(GANG, "gang: entering %s for %pJ", __func__, job_ptr);
 	/* add job to partition */
 	if (job_ptr->part_ptr && job_ptr->part_ptr->name)
 		part_name = job_ptr->part_ptr->name;
@@ -1221,8 +1146,7 @@ extern void gs_job_start(job_record_t *job_ptr)
 	}
 
 	_preempt_job_dequeue();	/* MUST BE OUTSIDE OF data_mutex lock */
-	if (slurmctld_conf.debug_flags & DEBUG_FLAG_GANG)
-		info("gang: leaving gs_job_start");
+	log_flag(GANG, "gang: leaving gs_job_start");
 }
 
 /* Gang scheduling has been disabled by change in configuration,
@@ -1230,16 +1154,17 @@ extern void gs_job_start(job_record_t *job_ptr)
 extern void gs_wake_jobs(void)
 {
 	job_record_t *job_ptr;
-	ListIterator job_iterator;
-
-	if (!(slurmctld_conf.preempt_mode & PREEMPT_MODE_GANG))
-		return;
+	list_itr_t *job_iterator;
 
 	if (!job_list)	/* no jobs */
 		return;
 
 	job_iterator = list_iterator_create(job_list);
 	while ((job_ptr = list_next(job_iterator))) {
+		/* Exclude HetJobs from gang operation. */
+		if (job_ptr->het_job_id)
+			continue;
+
 		if (IS_JOB_SUSPENDED(job_ptr) && (job_ptr->priority != 0)) {
 			info("gang waking preempted %pJ", job_ptr);
 			_resume_job(job_ptr);
@@ -1255,11 +1180,14 @@ extern void gs_job_fini(job_record_t *job_ptr)
 	struct gs_part *p_ptr;
 	char *part_name;
 
-	if (!(slurmctld_conf.preempt_mode & PREEMPT_MODE_GANG))
+	if (!(slurm_conf.preempt_mode & PREEMPT_MODE_GANG))
 		return;
 
-	if (slurmctld_conf.debug_flags & DEBUG_FLAG_GANG)
-		info("gang: entering %s for %pJ", __func__, job_ptr);
+	/* Exclude HetJobs from gang operation. */
+	if (job_ptr->het_job_id)
+		return;
+
+	log_flag(GANG, "gang: entering %s for %pJ", __func__, job_ptr);
 	if (job_ptr->part_ptr && job_ptr->part_ptr->name)
 		part_name = job_ptr->part_ptr->name;
 	else
@@ -1268,8 +1196,7 @@ extern void gs_job_fini(job_record_t *job_ptr)
 	p_ptr = list_find_first(gs_part_list, _find_gs_part, part_name);
 	if (!p_ptr) {
 		slurm_mutex_unlock(&data_mutex);
-		if (slurmctld_conf.debug_flags & DEBUG_FLAG_GANG)
-			info("gang: leaving gs_job_fini");
+		log_flag(GANG, "gang: leaving gs_job_fini");
 		return;
 	}
 
@@ -1279,8 +1206,7 @@ extern void gs_job_fini(job_record_t *job_ptr)
 	 * check by updating all active rows */
 	_update_all_active_rows();
 	slurm_mutex_unlock(&data_mutex);
-	if (slurmctld_conf.debug_flags & DEBUG_FLAG_GANG)
-		info("gang: leaving gs_job_fini");
+	log_flag(GANG, "gang: leaving gs_job_fini");
 }
 
 /* rebuild data structures from scratch
@@ -1307,13 +1233,13 @@ extern void gs_job_fini(job_record_t *job_ptr)
 extern void gs_reconfig(void)
 {
 	int i;
-	ListIterator part_iterator;
+	list_itr_t *part_iterator;
 	struct gs_part *p_ptr, *newp_ptr;
 	List old_part_list;
 	job_record_t *job_ptr;
 	struct gs_job *j_ptr;
 
-	if (!(slurmctld_conf.preempt_mode & PREEMPT_MODE_GANG))
+	if (!(slurm_conf.preempt_mode & PREEMPT_MODE_GANG))
 		return;
 
 	if (!timeslicer_thread_id) {
@@ -1322,8 +1248,7 @@ extern void gs_reconfig(void)
 		return;
 	}
 
-	if (slurmctld_conf.debug_flags & DEBUG_FLAG_GANG)
-		info("gang: entering gs_reconfig");
+	log_flag(GANG, "gang: entering gs_reconfig");
 	slurm_mutex_lock(&data_mutex);
 
 	old_part_list = gs_part_list;
@@ -1331,7 +1256,6 @@ extern void gs_reconfig(void)
 
 	/* reset global data */
 	gr_type = _get_gr_type();
-	_load_phys_res_cnt();
 	_build_parts();
 
 	/* scan the old part list and add existing jobs to the new list */
@@ -1394,8 +1318,7 @@ extern void gs_reconfig(void)
 	slurm_mutex_unlock(&data_mutex);
 
 	_preempt_job_dequeue();	/* MUST BE OUTSIDE OF data_mutex lock */
-	if (slurmctld_conf.debug_flags & DEBUG_FLAG_GANG)
-		info("gang: leaving gs_reconfig");
+	log_flag(GANG, "gang: leaving gs_reconfig");
 }
 
 /************************************
@@ -1409,8 +1332,7 @@ static void _build_active_row(struct gs_part *p_ptr)
 	int i;
 	struct gs_job *j_ptr;
 
-	if (slurmctld_conf.debug_flags & DEBUG_FLAG_GANG)
-		info("gang: entering _build_active_row");
+	log_flag(GANG, "gang: entering %s", __func__);
 	p_ptr->jobs_active = 0;
 	if (p_ptr->num_jobs == 0)
 		return;
@@ -1430,8 +1352,7 @@ static void _build_active_row(struct gs_part *p_ptr)
 			j_ptr->row_state = GS_ACTIVE;
 		}
 	}
-	if (slurmctld_conf.debug_flags & DEBUG_FLAG_GANG)
-		info("gang: leaving _build_active_row");
+	log_flag(GANG, "gang: leaving %s", __func__);
 }
 
 /* _cycle_job_list
@@ -1454,8 +1375,7 @@ static void _cycle_job_list(struct gs_part *p_ptr)
 	struct gs_job *j_ptr;
 	uint16_t preempt_mode;
 
-	if (slurmctld_conf.debug_flags & DEBUG_FLAG_GANG)
-		info("gang: entering %s", __func__);
+	log_flag(GANG, "gang: entering %s", __func__);
 	/* re-prioritize the job_list and set all row_states to GS_NO_ACTIVE */
 	for (i = 0; i < p_ptr->num_jobs; i++) {
 		while (p_ptr->job_list[i]->row_state == GS_ACTIVE) {
@@ -1471,12 +1391,10 @@ static void _cycle_job_list(struct gs_part *p_ptr)
 			p_ptr->job_list[i]->row_state = GS_NO_ACTIVE;
 
 	}
-	if (slurmctld_conf.debug_flags & DEBUG_FLAG_GANG)
-		info("gang: %s reordered job list:", __func__);
+	log_flag(GANG, "gang: %s reordered job list:", __func__);
 	/* Rebuild the active row. */
 	_build_active_row(p_ptr);
-	if (slurmctld_conf.debug_flags & DEBUG_FLAG_GANG)
-		info("gang: %s new active job list:", __func__);
+	log_flag(GANG, "gang: %s new active job list:", __func__);
 	_print_jobs(p_ptr);
 
 	/* Suspend running jobs that are GS_NO_ACTIVE */
@@ -1484,10 +1402,8 @@ static void _cycle_job_list(struct gs_part *p_ptr)
 		j_ptr = p_ptr->job_list[i];
 		if ((j_ptr->row_state == GS_NO_ACTIVE) &&
 		    (j_ptr->sig_state == GS_RESUME)) {
-			if (slurmctld_conf.debug_flags & DEBUG_FLAG_GANG) {
-				info("gang: %s: suspending %pJ",
-				     __func__, j_ptr->job_ptr);
-			}
+			log_flag(GANG, "gang: %s: suspending %pJ",
+				 __func__, j_ptr->job_ptr);
 			preempt_mode = slurm_job_preempt_mode(j_ptr->job_ptr);
 			if (p_ptr->num_shadows &&
 			    (preempt_mode != PREEMPT_MODE_OFF) &&
@@ -1506,17 +1422,14 @@ static void _cycle_job_list(struct gs_part *p_ptr)
 		if ((j_ptr->row_state == GS_ACTIVE) &&
 		    (j_ptr->sig_state == GS_SUSPEND) &&
 		    (j_ptr->job_ptr->priority != 0)) {	/* Redundant check */
-			if (slurmctld_conf.debug_flags & DEBUG_FLAG_GANG) {
-				info("gang: %s: resuming %pJ",
-				     __func__, j_ptr->job_ptr);
-			}
+			log_flag(GANG, "gang: %s: resuming %pJ",
+				 __func__, j_ptr->job_ptr);
 			_resume_job(j_ptr->job_ptr);
 			j_ptr->sig_state = GS_RESUME;
 			_cast_shadow(j_ptr, p_ptr->priority);
 		}
 	}
-	if (slurmctld_conf.debug_flags & DEBUG_FLAG_GANG)
-		info("gang: leaving %s", __func__);
+	log_flag(GANG, "gang: leaving %s", __func__);
 }
 
 static void _slice_sleep(void)
@@ -1539,11 +1452,10 @@ static void *_timeslicer_thread(void *arg)
 	/* Write locks on job and read lock on nodes */
 	slurmctld_lock_t job_write_lock = {
 		NO_LOCK, WRITE_LOCK, READ_LOCK, NO_LOCK, READ_LOCK };
-	ListIterator part_iterator;
+	list_itr_t *part_iterator;
 	struct gs_part *p_ptr;
 
-	if (slurmctld_conf.debug_flags & DEBUG_FLAG_GANG)
-		info("gang: starting timeslicer loop");
+	log_flag(GANG, "gang: starting timeslicer loop");
 	while (!thread_shutdown) {
 		_slice_sleep();
 		if (thread_shutdown)
@@ -1554,15 +1466,12 @@ static void *_timeslicer_thread(void *arg)
 		list_sort(gs_part_list, _sort_partitions);
 
 		/* scan each partition... */
-		if (slurmctld_conf.debug_flags & DEBUG_FLAG_GANG)
-			info("gang: _timeslicer_thread: scanning partitions");
+		log_flag(GANG, "gang: %s: scanning partitions", __func__);
 		part_iterator = list_iterator_create(gs_part_list);
 		while ((p_ptr = list_next(part_iterator))) {
-			if (slurmctld_conf.debug_flags & DEBUG_FLAG_GANG) {
-				info("gang: _timeslicer_thread: part %s: "
-				     "run %u total %u", p_ptr->part_name,
-				     p_ptr->jobs_active, p_ptr->num_jobs);
-			}
+			log_flag(GANG, "gang: %s: part %s: run %u total %u",
+				 __func__, p_ptr->part_name,
+				 p_ptr->jobs_active, p_ptr->num_jobs);
 			if (p_ptr->jobs_active <
 			    (p_ptr->num_jobs + p_ptr->num_shadows)) {
 				_cycle_job_list(p_ptr);
@@ -1577,6 +1486,5 @@ static void *_timeslicer_thread(void *arg)
 	}
 
 	timeslicer_thread_id = (pthread_t) 0;
-	pthread_exit((void *) 0);
 	return NULL;
 }
