@@ -152,7 +152,8 @@ static list_t *_build_license_list(char *licenses, bool *valid)
 				break;
 			}
 
-			if (token[i] == ':') {
+			if ((token[i] == ':') ||
+			    (token[i] == '=')) {
 				token[i++] = '\0';
 				num = (int32_t)strtol(&token[i], &end_num, 10);
 				if (*end_num != '\0')
@@ -870,11 +871,7 @@ extern bool license_list_overlap(list_t *list_1, List list_2)
  *
  * Return license counters to the library.
  */
-extern void
-get_all_license_info(char **buffer_ptr,
-                     int *buffer_size,
-                     uid_t uid,
-                     uint16_t protocol_version)
+extern buf_t *get_all_license_info(uint16_t protocol_version)
 {
 	list_itr_t *iter;
 	licenses_t *lic_entry;
@@ -885,9 +882,6 @@ get_all_license_info(char **buffer_ptr,
 
 	debug2("%s: calling for all licenses", __func__);
 
-	buffer_ptr[0] = NULL;
-	*buffer_size = 0;
-
 	buffer = init_buf(BUF_SIZE);
 
 	/* write header: version and time
@@ -919,8 +913,7 @@ get_all_license_info(char **buffer_ptr,
 	pack32(lics_packed, buffer);
 	set_buf_offset(buffer, tmp_offset);
 
-	*buffer_size = get_buf_offset(buffer);
-	buffer_ptr[0] = xfer_buf_data(buffer);
+	return buffer;
 }
 
 extern uint32_t get_total_license_cnt(char *name)
@@ -1020,8 +1013,6 @@ extern void license_set_job_tres_cnt(list_t *license_list,
 
 	if (!locked)
 		assoc_mgr_unlock(&locks);
-
-	return;
 }
 
 /*
@@ -1031,7 +1022,7 @@ extern void license_set_job_tres_cnt(list_t *license_list,
 static void _pack_license(licenses_t *lic, buf_t *buffer,
 			  uint16_t protocol_version)
 {
-	if (protocol_version >= SLURM_23_02_PROTOCOL_VERSION) {
+	if (protocol_version >= SLURM_MIN_PROTOCOL_VERSION) {
 		packstr(lic->name, buffer);
 		pack32(lic->total, buffer);
 		pack32(lic->used, buffer);
@@ -1040,12 +1031,6 @@ static void _pack_license(licenses_t *lic, buf_t *buffer,
 		pack32(lic->last_consumed, buffer);
 		pack32(lic->last_deficit, buffer);
 		pack_time(lic->last_update, buffer);
-	} else if (protocol_version >= SLURM_MIN_PROTOCOL_VERSION) {
-		packstr(lic->name, buffer);
-		pack32(lic->total, buffer);
-		pack32(lic->used, buffer);
-		pack32(lic->reserved, buffer);
-		pack8(lic->remote, buffer);
 	} else {
 		error("%s: protocol_version %hu not supported",
 		      __func__, protocol_version);
@@ -1215,7 +1200,10 @@ extern void slurm_bf_licenses_deduct(bf_licenses_t *licenses,
 		bf_entry = list_find_first(licenses, _bf_licenses_find_rec,
 					   job_entry->name);
 
-		if (bf_entry->remaining < needed) {
+		if (!bf_entry) {
+			error("%s: missing license %s",
+			      __func__, job_entry->name);
+		} else if (bf_entry->remaining < needed) {
 			error("%s: underflow on %s", __func__, bf_entry->name);
 			bf_entry->remaining = 0;
 		} else {
@@ -1244,12 +1232,16 @@ extern void slurm_bf_licenses_transfer(bf_licenses_t *licenses,
 	iter = list_iterator_create(job_ptr->license_list);
 	while ((resv_entry = list_next(iter))) {
 		bf_license_t *bf_entry, *new_entry;
-		int needed = resv_entry->total, reservable;
+		int needed = resv_entry->total;
+		int reservable = resv_entry->total;
 
 		bf_entry = list_find_first(licenses, _bf_licenses_find_rec,
 					   resv_entry->name);
 
-		if (bf_entry->remaining < needed) {
+		if (!bf_entry) {
+			error("%s: missing license %s",
+			      __func__, resv_entry->name);
+		} else if (bf_entry->remaining < needed) {
 			error("%s: underflow on %s", __func__, bf_entry->name);
 			reservable = bf_entry->remaining;
 			bf_entry->remaining = 0;
@@ -1307,7 +1299,7 @@ extern bool slurm_bf_licenses_avail(bf_licenses_t *licenses,
 		bf_entry = list_find_first(licenses, _bf_licenses_find_rec,
 					   need->name);
 
-		if (bf_entry->remaining < needed) {
+		if (!bf_entry || (bf_entry->remaining < needed)) {
 			avail = false;
 			break;
 		}
@@ -1328,7 +1320,7 @@ extern bool slurm_bf_licenses_equal(bf_licenses_t *a, bf_licenses_t *b)
 		entry_b = list_find_first(b, _bf_licenses_find_rec,
 					  entry_a->name);
 
-		if ((entry_a->remaining != entry_b->remaining) ||
+		if (!entry_b || (entry_a->remaining != entry_b->remaining) ||
 		    (entry_a->resv_ptr != entry_b->resv_ptr)) {
 			equivalent = false;
 			break;
