@@ -3,7 +3,7 @@
  *****************************************************************************
  *  Copyright (C) 2002-2007 The Regents of the University of California.
  *  Copyright (C) 2008-2010 Lawrence Livermore National Security.
- *  Copyright (C) 2010-2013 SchedMD LLC.
+ *  Copyright (C) SchedMD LLC.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
  *  Written by Joey Ekstrom <ekstrom1@llnl.gov>, et. al.
  *  CODE-OCEC-09-009. All rights reserved.
@@ -49,18 +49,19 @@
 #include "src/common/hostlist.h"
 #include "src/common/list.h"
 #include "src/common/macros.h"
-#include "src/common/node_select.h"
 #include "src/common/parse_time.h"
-#include "src/common/slurm_acct_gather_profile.h"
+#include "src/interfaces/select.h"
+#include "src/interfaces/acct_gather_profile.h"
 #include "src/common/uid.h"
 #include "src/common/xmalloc.h"
 #include "src/common/xstring.h"
 
 #include "src/squeue/print.h"
+#include "src/common/print_fields.h"
 #include "src/squeue/squeue.h"
 
 static void	_combine_pending_array_tasks(List l);
-static int	_filter_job(job_info_t * job);
+static bool _filter_job(job_info_t *job);
 static int	_filter_job_part(char *part_name);
 static int	_filter_step(job_step_info_t * step);
 static void	_job_list_del(void *x);
@@ -77,44 +78,98 @@ static partition_info_msg_t *part_info_msg = NULL;
 /*****************************************************************************
  * Global Print Functions
  *****************************************************************************/
+typedef struct {
+	int *count;
+	list_t *req_list;
+	job_info_t *job_ptr;
+} foreach_prio_job_req_arg_t;
 
-int print_jobs_array(job_info_t * jobs, int size, List format)
+static int _foreach_create_prio_job_req(void *x, void *arg)
+{
+	char *part_name = x;
+	foreach_prio_job_req_arg_t *req_arg = arg;
+	job_info_t *job_ptr = req_arg->job_ptr;
+	squeue_job_rec_t *job_rec_ptr;
+
+	(*req_arg->count)++;
+
+	if (_filter_job_part(part_name))
+		return SLURM_SUCCESS;
+
+	job_rec_ptr = xmalloc(sizeof(squeue_job_rec_t));
+	job_rec_ptr->job_ptr = req_arg->job_ptr;
+	job_rec_ptr->part_name = xstrdup(part_name);
+	job_rec_ptr->part_prio = _part_get_prio_tier(part_name);
+
+	if (IS_JOB_PENDING(job_ptr) && job_ptr->priority_array) {
+		job_rec_ptr->job_prio =
+			job_ptr->priority_array[(*req_arg->count) - 1];
+	} else {
+		job_rec_ptr->job_prio = job_ptr->priority;
+	}
+	list_append(req_arg->req_list, job_rec_ptr);
+
+	return SLURM_SUCCESS;
+}
+
+static void _create_priority_list(list_t *l,
+				  job_info_t *job_ptr)
+{
+	char *tmp;
+	int j = 0;
+	foreach_prio_job_req_arg_t arg = {0};
+	list_t *part_names = list_create(xfree_ptr);
+
+	/*
+	 * If the job requests multiple partitions then priority_array_parts
+	 * will exist. When a job is running the controller only sends the
+	 * partition that the job is running in in job_ptr->partition so we can
+	 * just use job_ptr->partition and job_ptr->priority.
+	 */
+	if (IS_JOB_PENDING(job_ptr) && job_ptr->priority_array_parts)
+		tmp = job_ptr->priority_array_parts;
+	else
+		tmp = job_ptr->partition;
+	slurm_addto_char_list(part_names, tmp);
+
+	arg.count = &j;
+	arg.job_ptr = job_ptr;
+	arg.req_list = l;
+	list_for_each(part_names, _foreach_create_prio_job_req, &arg);
+}
+
+extern void print_jobs_array(job_info_t *jobs, int size, list_t *format)
 {
 	squeue_job_rec_t *job_rec_ptr;
-	char *tmp, *tok, *save_ptr = NULL;
 	int i;
 	List l;
 
 	l = list_create(_job_list_del);
 	if (!params.no_header)
 		_print_job_from_format(NULL, format);
-	_part_state_load();
+	if (!params.only_state)
+		_part_state_load();
 
 	/* Filter out the jobs of interest */
 	for (i = 0; i < size; i++) {
 		if (_filter_job(&jobs[i]))
 			continue;
 		if (params.priority_flag) {
-			tmp = xstrdup(jobs[i].partition);
-			tok = strtok_r(tmp, ",", &save_ptr);
-			while (tok) {
-				if (_filter_job_part(tok) == 0) {
-					job_rec_ptr = xmalloc(
-						      sizeof(squeue_job_rec_t));
-					job_rec_ptr->job_ptr = jobs + i;
-					job_rec_ptr->part_name = xstrdup(tok);
-					job_rec_ptr->part_prio =
-						_part_get_prio_tier(tok);
-					list_append(l, (void *) job_rec_ptr);
-				}
-				tok = strtok_r(NULL, ",", &save_ptr);
-			}
-			xfree(tmp);
+			_create_priority_list(l, &jobs[i]);
 		} else {
 			if (_filter_job_part(jobs[i].partition))
 				continue;
 			job_rec_ptr = xmalloc(sizeof(squeue_job_rec_t));
 			job_rec_ptr->job_ptr = jobs + i;
+			/*
+			 * Note: for multi-partition definitions the following
+			 * is ill-defined, and you should really use the -P
+			 * option for a consistent view.
+			 * This will sort such jobs alphabetically, but that
+			 * may be confusing as to why "foo,bar" and "bar,foo"
+			 * submissions end up in different places.
+			 */
+			job_rec_ptr->part_name = xstrdup(jobs[i].partition);
 			list_append(l, (void *) job_rec_ptr);
 		}
 	}
@@ -127,11 +182,9 @@ int print_jobs_array(job_info_t * jobs, int size, List format)
 	/* Print the jobs of interest */
 	list_for_each(l, _print_job_from_format, format);
 	FREE_NULL_LIST(l);
-
-	return SLURM_SUCCESS;
 }
 
-int print_steps_array(job_step_info_t * steps, int size, List format)
+extern void print_steps_array(job_step_info_t *steps, int size, list_t *format)
 {
 	if (!params.no_header)
 		_print_step_from_format(NULL, format);
@@ -155,24 +208,44 @@ int print_steps_array(job_step_info_t * steps, int size, List format)
 		list_for_each(step_list, _print_step_from_format, format);
 		FREE_NULL_LIST(step_list);
 	}
+}
 
-	return SLURM_SUCCESS;
+extern void squeue_filter_jobs_for_json(job_info_msg_t *job_info)
+{
+	int new_array_size = 0;
+	job_info_t *tmp_jobs = xcalloc(job_info->record_count,
+				       sizeof(job_info_t));
+
+	for (int i = 0; i < job_info->record_count; i++) {
+		if (!(_filter_job(&job_info->job_array[i])) &&
+		    !(_filter_job_part(job_info->job_array[i].partition))) {
+			tmp_jobs[new_array_size] = job_info->job_array[i];
+			new_array_size++;
+		} else {
+			slurm_free_job_info_members(&job_info->job_array[i]);
+		}
+	}
+
+	xrecalloc(tmp_jobs, new_array_size, sizeof(job_info_t));
+	xfree(job_info->job_array);
+	job_info->job_array = tmp_jobs;
+	job_info->record_count = new_array_size;
 }
 
 /* Combine a job array's task "reason" into the master job array record
  * reason as needed */
 static void _merge_job_reason(job_info_t *job_ptr, job_info_t *task_ptr)
 {
-	char *task_desc;
+	const char *task_desc;
 
 	if (job_ptr->state_reason == task_ptr->state_reason)
 		return;
 
 	if (!job_ptr->state_desc) {
 		job_ptr->state_desc =
-			xstrdup(job_reason_string(job_ptr->state_reason));
+			xstrdup(job_state_reason_string(job_ptr->state_reason));
 	}
-	task_desc = job_reason_string(task_ptr->state_reason);
+	task_desc = job_state_reason_string(task_ptr->state_reason);
 	if (strstr(job_ptr->state_desc, task_desc))
 		return;
 	xstrfmtcat(job_ptr->state_desc, ",%s", task_desc);
@@ -184,7 +257,7 @@ static void _merge_job_reason(job_info_t *job_ptr, job_info_t *task_ptr)
 static void _combine_pending_array_tasks(List job_list)
 {
 	squeue_job_rec_t *job_rec_ptr, *task_rec_ptr;
-	ListIterator job_iterator, task_iterator;
+	list_itr_t *job_iterator, *task_iterator;
 	bitstr_t *task_bitmap;
 	int bitmap_size, update_cnt;
 
@@ -198,7 +271,7 @@ static void _combine_pending_array_tasks(List job_list)
 		    !job_rec_ptr->job_ptr->array_bitmap)
 			continue;
 		update_cnt = 0;
-		task_bitmap = (bitstr_t *) job_rec_ptr->job_ptr->array_bitmap;
+		task_bitmap = job_rec_ptr->job_ptr->array_bitmap;
 		bitmap_size = bit_size(task_bitmap);
 		task_iterator = list_iterator_create(job_list);
 		while ((task_rec_ptr = list_next(task_iterator))) {
@@ -216,9 +289,6 @@ static void _combine_pending_array_tasks(List job_list)
 			if (xstrcmp(task_rec_ptr->job_ptr->partition,
 				    job_rec_ptr->job_ptr->partition))
 				continue;	/* Different partition */
-			/* Want to see each reason separately */
-			if (params.array_unique_flag)
-				continue;
 			/* Combine this task into master job array record */
 			update_cnt++;
 			_merge_job_reason(job_rec_ptr->job_ptr,
@@ -332,13 +402,11 @@ static int _print_str(char *str, int width, bool right, bool cut_output)
 
 int _print_nodes(char *nodes, int width, bool right, bool cut)
 {
-	hostlist_t hl = hostlist_create(nodes);
 	char *buf = NULL;
 	int retval;
-	buf = hostlist_ranged_string_xmalloc(hl);
+	buf = slurm_sort_node_list_str(nodes);
 	retval = _print_str(buf, width, right, false);
 	xfree(buf);
-	hostlist_destroy(hl);
 	return retval;
 }
 
@@ -384,7 +452,7 @@ int _print_secs(long time, int width, bool right, bool cut_output)
 int _print_time(time_t t, int level, int width, bool right)
 {
 	if (t) {
-		char time_str[32];
+		char time_str[256];
 		slurm_make_time_str(&t, time_str, sizeof(time_str));
 		_print_str(time_str, width, right, true);
 	} else
@@ -398,9 +466,8 @@ int _print_time(time_t t, int level, int width, bool right)
  *****************************************************************************/
 static int _print_one_job_from_format(job_info_t * job, List list)
 {
-	ListIterator iter = list_iterator_create(list);
+	list_itr_t *iter = list_iterator_create(list);
 	job_format_t *current;
-	int total_width = 0;
 
 	while ((current = list_next(iter))) {
 		if (current->
@@ -408,10 +475,6 @@ static int _print_one_job_from_format(job_info_t * job, List list)
 			     current->suffix)
 		    != SLURM_SUCCESS)
 			return SLURM_ERROR;
-		if (current->width)
-			total_width += (current->width + 1);
-		else
-			total_width += 10;
 	}
 	list_iterator_destroy(iter);
 
@@ -421,7 +484,6 @@ static int _print_one_job_from_format(job_info_t * job, List list)
 
 static int _print_job_from_format(void *x, void *arg)
 {
-	static int32_t max_array_size = -1;
 	int i, i_first, i_last;
 	bitstr_t *bitmap;
 	squeue_job_rec_t *job_rec_ptr = (squeue_job_rec_t *) x;
@@ -436,16 +498,17 @@ static int _print_job_from_format(void *x, void *arg)
 		xfree(job_rec_ptr->job_ptr->partition);
 		job_rec_ptr->job_ptr->partition = xstrdup(job_rec_ptr->
 							  part_name);
-
 	}
+
+	if (job_rec_ptr->job_prio)
+		job_rec_ptr->job_ptr->priority = job_rec_ptr->job_prio;
+
 	if (job_rec_ptr->job_ptr->array_task_str && params.array_flag) {
 		char *p;
 
-		if (max_array_size == -1)
-			max_array_size = slurm_get_max_array_size();
 		if ((p = strchr(job_rec_ptr->job_ptr->array_task_str, '%')))
 			*p = 0;
-		bitmap = bit_alloc(max_array_size);
+		bitmap = bit_alloc(slurm_conf.max_array_sz);
 		bit_unfmt(bitmap, job_rec_ptr->job_ptr->array_task_str);
 		xfree(job_rec_ptr->job_ptr->array_task_str);
 		i_first = bit_ffs(bitmap);
@@ -571,6 +634,31 @@ int _print_job_cluster_name(job_info_t * job, int width, bool right,
 	return SLURM_SUCCESS;
 }
 
+int _print_job_container(job_info_t *job, int width, bool right, char *suffix)
+{
+	if (!job)		/* Print the Header instead */
+		_print_str("CONTAINER", width, right, true);
+	else
+		_print_str(job->container, width, right, true);
+
+	if (suffix)
+		printf("%s", suffix);
+	return SLURM_SUCCESS;
+}
+
+int _print_job_container_id(job_info_t *job, int width, bool right,
+			    char *suffix)
+{
+	if (!job) /* Print the Header instead */
+		_print_str("CONTAINERID", width, right, true);
+	else
+		_print_str(job->container_id, width, right, true);
+
+	if (suffix)
+		printf("%s", suffix);
+	return SLURM_SUCCESS;
+}
+
 int _print_job_core_spec(job_info_t * job, int width, bool right, char* suffix)
 {
 	char spec[FORMAT_STRING_SIZE];
@@ -678,12 +766,12 @@ int _print_job_reason(job_info_t * job, int width, bool right, char* suffix)
 	if (job == NULL)        /* Print the Header instead */
 		_print_str("REASON", width, right, true);
 	else {
-		char *reason;
+		const char *reason;
 		if (job->state_desc)
 			reason = job->state_desc;
 		else
-			reason = job_reason_string(job->state_reason);
-		_print_str(reason, width, right, true);
+			reason = job_state_reason_string(job->state_reason);
+		_print_str((char *)reason, width, right, true);
 	}
 	if (suffix)
 		printf("%s", suffix);
@@ -763,16 +851,12 @@ int _print_job_group_id(job_info_t * job, int width, bool right, char* suffix)
 
 int _print_job_group_name(job_info_t * job, int width, bool right, char* suffix)
 {
-	struct group *group_info = NULL;
-
 	if (job == NULL)	/* Print the Header instead */
 		_print_str("GROUP", width, right, true);
 	else {
-		group_info = getgrgid((gid_t) job->group_id);
-		if (group_info && group_info->gr_name[0])
-			_print_str(group_info->gr_name, width, right, true);
-		else
-			_print_int(job->group_id, width, right, true);
+		char *group = gid_to_string(job->group_id);
+		_print_str(group, width, right, true);
+		xfree(group);
 	}
 	if (suffix)
 		printf("%s", suffix);
@@ -944,6 +1028,28 @@ int _print_job_time_submit(job_info_t * job, int width, bool right,
 	return SLURM_SUCCESS;
 }
 
+int _print_job_time_pending(job_info_t *job, int width, bool right,
+			    char *suffix)
+{
+	time_t now = time(NULL);
+
+	/*
+	 * If the job has started, defined as (start - submit).
+	 * Else, defined as (now - submit).
+	 */
+
+	if (!job)	/* Print the Header instead */
+		_print_str("PENDING_TIME", width, right, true);
+	else if (job->start_time && (job->start_time < now))
+		_print_int((job->start_time - job->submit_time), width, right,
+			   true);
+	else
+		_print_int((now - job->submit_time), width, right, true);
+	if (suffix)
+		printf("%s", suffix);
+	return SLURM_SUCCESS;
+}
+
 int _print_job_time_start(job_info_t * job, int width, bool right,
 			  char* suffix)
 {
@@ -1046,11 +1152,12 @@ int _print_job_reason_list(job_info_t * job, int width, bool right,
 		       || IS_JOB_OOM(job)
 		       || IS_JOB_DEADLINE(job)
 		       || IS_JOB_FAILED(job))) {
-		char *reason_fmt = NULL, *reason = NULL;
+		char *reason_fmt = NULL;
+		const char *reason = NULL;
 		if (job->state_desc)
 			reason = job->state_desc;
 		else
-			reason = job_reason_string(job->state_reason);
+			reason = job_state_reason_string(job->state_reason);
 		xstrfmtcat(reason_fmt, "(%s)", reason);
 		_print_str(reason_fmt, width, right, true);
 		xfree(reason_fmt);
@@ -1408,6 +1515,18 @@ int _print_job_cluster_features(job_info_t * job, int width, bool right_justify,
 	return SLURM_SUCCESS;
 }
 
+int _print_job_prefer(job_info_t * job, int width, bool right_justify,
+		      char* suffix)
+{
+	if (job == NULL)	/* Print the Header instead */
+		_print_str("PREFER", width, right_justify, true);
+	else
+		_print_str(job->prefer, width, right_justify, true);
+	if (suffix)
+		printf("%s", suffix);
+	return SLURM_SUCCESS;
+}
+
 int _print_job_account(job_info_t * job, int width, bool right_justify,
 			char* suffix)
 {
@@ -1480,24 +1599,6 @@ int _print_job_qos(job_info_t * job, int width, bool right_justify,
 	return SLURM_SUCCESS;
 }
 
-int _print_job_select_jobinfo(job_info_t * job, int width, bool right_justify,
-			char* suffix)
-{
-	char select_buf[100];
-
-	if (job == NULL)	/* Print the Header instead */
-		select_g_select_jobinfo_sprint(NULL,
-			select_buf, sizeof(select_buf), SELECT_PRINT_HEAD);
-	else
-		select_g_select_jobinfo_sprint(job->select_jobinfo,
-			select_buf, sizeof(select_buf), SELECT_PRINT_DATA);
-	_print_str(select_buf, width, right_justify, true);
-
-	if (suffix)
-		printf("%s", suffix);
-	return SLURM_SUCCESS;
-}
-
 int _print_job_reservation(job_info_t * job, int width, bool right_justify,
 			char* suffix)
 {
@@ -1647,12 +1748,23 @@ int _print_job_cpus_per_task(job_info_t * job, int width, bool right_justify,
 }
 
 int _print_job_derived_ec(job_info_t * job, int width, bool right_justify,
-		    char* suffix)
+			  char* suffix)
 {
-	if (job == NULL)
+	uint16_t exit_status = 0, term_sig = 0;
+	char *out = NULL;
+
+	if (!job)
 		_print_str("DERIVED_EC", width, right_justify, true);
-	else
-		_print_int(job->derived_ec, width, right_justify, true);
+	else if (job->derived_ec != NO_VAL) {
+		if (WIFSIGNALED(job->derived_ec))
+			term_sig = WTERMSIG(job->derived_ec);
+		else if (WIFEXITED(job->derived_ec))
+			exit_status = WEXITSTATUS(job->derived_ec);
+
+		xstrfmtcat(out, "%u:%u", exit_status, term_sig);
+		_print_str(out, width, right_justify, true);
+		xfree(out);
+	}
 
 	if (suffix)
 		printf("%s",suffix);
@@ -1675,12 +1787,23 @@ int _print_job_eligible_time(job_info_t * job, int width, bool right_justify,
 }
 
 int _print_job_exit_code(job_info_t * job, int width, bool right_justify,
-		    char* suffix)
+			 char* suffix)
 {
-	if (job == NULL)
+	uint16_t exit_status = 0, term_sig = 0;
+	char *out = NULL;
+
+	if (!job)
 		_print_str("EXIT_CODE", width, right_justify, true);
-	else
-		_print_int(job->exit_code, width, right_justify, true);
+	else if (job->exit_code != NO_VAL) {
+		if (WIFSIGNALED(job->exit_code))
+			term_sig = WTERMSIG(job->exit_code);
+		else if (WIFEXITED(job->exit_code))
+			exit_status = WEXITSTATUS(job->exit_code);
+
+		xstrfmtcat(out, "%u:%u", exit_status, term_sig);
+		_print_str(out, width, right_justify, true);
+		xfree(out);
+	}
 
 	if (suffix)
 		printf("%s",suffix);
@@ -2034,25 +2157,32 @@ int _print_job_sockets_per_board(job_info_t * job, int width,
 
 }
 
+static char *_expand_std_patterns(char *path, job_info_t *job)
+{
+	job_std_pattern_t job_stp;
+
+	job_stp.array_task_id = job->array_task_id;
+	job_stp.first_step_name = "batch";
+	job_stp.first_step_node = job->batch_host;
+	job_stp.jobid = job->job_id;
+	job_stp.jobname = job->name;
+	job_stp.user = job->user_name;
+	job_stp.work_dir = job->work_dir;
+
+	return expand_stdio_fields(path, &job_stp);
+}
+
 int _print_job_std_err(job_info_t * job, int width,
 		       bool right_justify, char* suffix)
 {
-	char tmp_line[1024];
-
 	if (job == NULL)
 		_print_str("STDERR", width, right_justify, true);
-	else if (!job->batch_flag)
-		_print_str("N/A", width, right_justify, true);
-	else if (job->std_err)
+	else if (params.expand_patterns) {
+		char *tmp_str = _expand_std_patterns(job->std_err, job);
+		_print_str(tmp_str, width, right_justify, true);
+		xfree(tmp_str);
+	} else
 		_print_str(job->std_err, width, right_justify, true);
-	else if (job->std_out)
-		_print_str(job->std_out, width, right_justify, true);
-	else {
-		snprintf(tmp_line,sizeof(tmp_line), "%s/slurm-%u.out",
-			 job->work_dir, job->job_id);
-
-		_print_str(tmp_line, width, right_justify, true);
-	}
 
 	if (suffix)
 		printf("%s", suffix);
@@ -2064,7 +2194,11 @@ int _print_job_std_in(job_info_t * job, int width,
 {
 	if (job == NULL)
 		_print_str("STDIN", width, right_justify, true);
-	else
+	else if (params.expand_patterns) {
+		char *tmp_str = _expand_std_patterns(job->std_in, job);
+		_print_str(tmp_str, width, right_justify, true);
+		xfree(tmp_str);
+	} else
 		_print_str(job->std_in, width, right_justify, true);
 
 	if (suffix)
@@ -2076,18 +2210,26 @@ int _print_job_std_in(job_info_t * job, int width,
 int _print_job_std_out(job_info_t * job, int width,
 		       bool right_justify, char* suffix)
 {
-	char tmp_line[1024];
+	/*
+	 * Populate the default patterns in std_out for batch jobs.
+	 */
+	if (job && !job->std_out && job->batch_flag) {
+		if (job->array_job_id)
+			xstrfmtcat(job->std_out, "%s/slurm-%%A_%%a.out",
+				   job->work_dir);
+                else
+			xstrfmtcat(job->std_out, "%s/slurm-%%j.out",
+				   job->work_dir);
+	}
 
 	if (job == NULL)
 		_print_str("STDOUT", width, right_justify, true);
-	else if (job->std_out)
+	else if (params.expand_patterns) {
+		char *tmp_str = _expand_std_patterns(job->std_out, job);
+		_print_str(tmp_str, width, right_justify, true);
+		xfree(tmp_str);
+	} else
 		_print_str(job->std_out, width, right_justify, true);
-	else {
-		snprintf(tmp_line,sizeof(tmp_line), "%s/slurm-%u.out",
-			 job->work_dir, job->job_id);
-
-		_print_str(tmp_line, width, right_justify, true);
-	}
 
 	if (suffix)
 		printf("%s", suffix);
@@ -2315,9 +2457,8 @@ static int _print_step_from_format(void *x, void *arg)
 {
 	job_step_info_t *job_step = (job_step_info_t *) x;
 	List list = (List) arg;
-	ListIterator i = list_iterator_create(list);
+	list_itr_t *i = list_iterator_create(list);
 	step_format_t *current;
-	int total_width = 0;
 
 	while ((current = list_next(i))) {
 		if (current->
@@ -2325,10 +2466,6 @@ static int _print_step_from_format(void *x, void *arg)
 			     current->right_justify, current->suffix)
 		    != SLURM_SUCCESS)
 			return SLURM_ERROR;
-		if (current->width)
-			total_width += current->width;
-		else
-			total_width += 10;
 	}
 	list_iterator_destroy(i);
 	printf("\n");
@@ -2365,42 +2502,52 @@ int _print_step_cluster_name(job_step_info_t *step, int width, bool right,
 	return SLURM_SUCCESS;
 }
 
+int _print_step_container(job_step_info_t *step, int width, bool right,
+			  char *suffix)
+{
+	if (!step)		/* Print the Header instead */
+		_print_str("CONTAINER", width, right, true);
+	else
+		_print_str(step->container, width, right, true);
+
+	if (suffix)
+		printf("%s", suffix);
+	return SLURM_SUCCESS;
+}
+
+int _print_step_container_id(job_step_info_t *step, int width, bool right,
+			     char *suffix)
+{
+	if (!step) /* Print the Header instead */
+		_print_str("CONTAINERID", width, right, true);
+	else
+		_print_str(step->container_id, width, right, true);
+
+	if (suffix)
+		printf("%s", suffix);
+	return SLURM_SUCCESS;
+}
+
 int _print_step_id(job_step_info_t * step, int width, bool right, char* suffix)
 {
 	char id[FORMAT_STRING_SIZE];
 
 	if (step == NULL) {	/* Print the Header instead */
 		_print_str("STEPID", width, right, true);
-	} else if (step->array_job_id) {
-		if (step->step_id == SLURM_PENDING_STEP) {	/* Pending */
-			snprintf(id, FORMAT_STRING_SIZE, "%u_%u.TBD",
-				 step->array_job_id, step->array_task_id);
-		} else if (step->step_id == SLURM_EXTERN_CONT) {
-			snprintf(id, FORMAT_STRING_SIZE, "%u_%u.Extern",
-				 step->array_job_id, step->array_task_id);
-		} else if (step->step_id == SLURM_BATCH_SCRIPT) {
-			snprintf(id, FORMAT_STRING_SIZE, "%u_%u.Batch",
-				 step->array_job_id, step->array_task_id);
-		} else {
-			snprintf(id, FORMAT_STRING_SIZE, "%u_%u.%u",
-				 step->array_job_id, step->array_task_id,
-				 step->step_id);
-		}
-		_print_str(id, width, right, true);
 	} else {
-		if (step->step_id == SLURM_PENDING_STEP) {	/* Pending */
-			snprintf(id, FORMAT_STRING_SIZE, "%u.TBD",
-				 step->job_id);
-		} else if (step->step_id == SLURM_EXTERN_CONT) {
-			snprintf(id, FORMAT_STRING_SIZE, "%u.Extern",
-				 step->job_id);
-		} else if (step->step_id == SLURM_BATCH_SCRIPT) {
-			snprintf(id, FORMAT_STRING_SIZE, "%u.Batch",
-				 step->job_id);
-		} else {
-			snprintf(id, FORMAT_STRING_SIZE, "%u.%u",
-				 step->job_id, step->step_id);
+		uint16_t flags = STEP_ID_FLAG_NO_PREFIX;
+		int len = FORMAT_STRING_SIZE;
+		int pos = 0;
+		if (step->array_job_id) {
+			pos = snprintf(id, len, "%u_%u.",
+				       step->array_job_id, step->array_task_id);
+			flags |= STEP_ID_FLAG_NO_JOB;
+			len -= pos;
 		}
+
+		log_build_step_id_str(&step->step_id,
+				      id+pos, len, flags);
+
 		_print_str(id, width, right, true);
 	}
 	if (suffix)
@@ -2540,7 +2687,7 @@ int _print_step_array_job_id(job_step_info_t * step, int width, bool right,
 	else if (step->array_job_id != NO_VAL)
 		_print_int(step->array_job_id, width, right, true);
 	else
-		_print_int(step->job_id, width, right, true);
+		_print_int(step->step_id.job_id, width, right, true);
 
 	if (suffix)
 		printf("%s", suffix);
@@ -2568,7 +2715,7 @@ int _print_step_job_id(job_step_info_t * step, int width, bool right,
 	if (step == NULL)
 		_print_str("JOB_ID", width, right, true);
 	else
-		_print_int(step->job_id, width, right, true);
+		_print_int(step->step_id.job_id, width, right, true);
 
 	if (suffix)
 		printf("%s", suffix);
@@ -2777,14 +2924,13 @@ int _print_step_tres_per_task(job_step_info_t * step, int width, bool right,
 }
 
 /*
- * Filter job records per input specifications,
- * Returns >0 if job should be filter out (not printed)
- * Returns 0 if job record should be printed
+ * Filter job records per input specifications.
+ * Returns true if the job should be filtered out (not printed).
  */
-static int _filter_job(job_info_t * job)
+static bool _filter_job(job_info_t *job)
 {
-	int i, filter;
-	ListIterator iterator;
+	int i;
+	list_itr_t *iterator;
 	uint32_t *user;
 	uint32_t *state_id;
 	char *account, *license, *qos, *name;
@@ -2792,44 +2938,48 @@ static int _filter_job(job_info_t * job)
 	bool partial_array = false;
 
 	if (job->job_id == 0)
-		return 1;
+		return true;
 
 	if (params.job_list) {
-		filter = 1;
+		bool filter = true;
 		iterator = list_iterator_create(params.job_list);
 		while ((job_step_id = list_next(iterator))) {
 			if (((job_step_id->array_id == NO_VAL)             &&
-			     ((job_step_id->job_id  == job->array_job_id)  ||
-			      (job_step_id->job_id  == job->job_id)))      ||
+			     ((job_step_id->step_id.job_id ==
+			       job->array_job_id) ||
+			      (job_step_id->step_id.job_id ==
+			       job->job_id))) ||
 			    ((job_step_id->array_id == job->array_task_id) &&
-			     (job_step_id->job_id   == job->array_job_id))) {
-				filter = 0;
+			     (job_step_id->step_id.job_id ==
+			      job->array_job_id))) {
+				filter = false;
 				break;
 			}
 			if ((job_step_id->array_id != NO_VAL)             &&
-			    (job_step_id->job_id   == job->array_job_id)  &&
+			    (job_step_id->step_id.job_id ==
+			     job->array_job_id) &&
 			    (job->array_bitmap &&
-			     bit_test((bitstr_t *)job->array_bitmap,
+			     bit_test(job->array_bitmap,
 				      job_step_id->array_id))) {
-				filter = 0;
+				filter = false;
 				partial_array = true;
 				break;
 			}
-			if (job_step_id->job_id == job->het_job_id) {
-				filter = 0;
+			if (job_step_id->step_id.job_id == job->het_job_id) {
+				filter = false;
 				break;
 			}
 		}
 		list_iterator_destroy(iterator);
-		if (filter == 1)
-			return 1;
+		if (filter)
+			return true;
 	}
 
 	if (params.licenses_list) {
 		char *token = NULL, *last = NULL, *tmp_name = NULL;
 		char *tmp_token;
+		bool filter = true;
 
-		filter = 1;
 		if (job->licenses) {
 			tmp_name = xstrdup(job->licenses);
 			token = strtok_r(tmp_name, ",", &last);
@@ -2847,7 +2997,7 @@ static int _filter_job(job_info_t * job)
 			iterator = list_iterator_create(params.licenses_list);
 			while ((license = list_next(iterator))) {
 				if (xstrcmp(token, license) == 0) {
-					filter = 0;
+					filter = false;
 					break;
 				}
 			}
@@ -2855,42 +3005,43 @@ static int _filter_job(job_info_t * job)
 			token = strtok_r(NULL, ",", &last);
 		}
 		xfree(tmp_name);
-		if (filter == 1)
-			return 2;
+		if (filter)
+			return true;
 	}
 
 	if (params.account_list) {
-		filter = 1;
+		bool filter = true;
 		iterator = list_iterator_create(params.account_list);
 		while ((account = list_next(iterator))) {
 			 if ((job->account != NULL) &&
 			     (xstrcasecmp(account, job->account) == 0)) {
-				filter = 0;
+				filter = false;
 				break;
 			}
 		}
 		list_iterator_destroy(iterator);
-		if (filter == 1)
-			return 2;
+		if (filter)
+			return true;
 	}
 
 	if (params.qos_list) {
-		filter = 1;
+		bool filter = true;
 		iterator = list_iterator_create(params.qos_list);
 		while ((qos = list_next(iterator))) {
 			 if ((job->qos != NULL) &&
 			     (xstrcasecmp(qos, job->qos) == 0)) {
-				filter = 0;
+				filter = false;
 				break;
 			}
 		}
 		list_iterator_destroy(iterator);
-		if (filter == 1)
-			return 2;
+		if (filter)
+			return true;
 	}
 
-	if (params.state_list) {
-		filter = 1;
+	if (params.all_states) {
+	} else if (params.state_list) {
+		bool filter = true;
 		iterator = list_iterator_create(params.state_list);
 		while ((state_id = list_next(iterator))) {
 			bool match = false;
@@ -2901,100 +3052,103 @@ static int _filter_job(job_info_t * job)
 			} else if (*state_id == job->job_state)
 				match = true;
 			if (match) {
-				filter = 0;
+				filter = false;
 				break;
 			}
 		}
 		list_iterator_destroy(iterator);
-		if (filter == 1)
-			return 3;
+		if (filter)
+			return true;
 	} else {
 		if (!IS_JOB_PENDING(job) &&
 		    !IS_JOB_RUNNING(job) &&
 		    !IS_JOB_STAGE_OUT(job) &&
 		    !IS_JOB_SUSPENDED(job) &&
 		    !IS_JOB_COMPLETING(job))
-			return 4;
+			return true;
 	}
 
 	if ((params.nodes)
 	    && ((job->nodes == NULL)
 		|| (!hostset_intersects(params.nodes, job->nodes))))
-		return 5;
+		return true;
+
+	if (params.notme_flag && (getuid() == job->user_id))
+		return true;
 
 	if (params.user_list) {
-		filter = 1;
+		bool filter = true;
 		iterator = list_iterator_create(params.user_list);
 		while ((user = list_next(iterator))) {
 			if (*user == job->user_id) {
-				filter = 0;
+				filter = false;
 				break;
 			}
 		}
 		list_iterator_destroy(iterator);
-		if (filter == 1)
-			return 6;
+		if (filter)
+			return true;
 	}
 
 	if (params.reservation) {
 		if ((job->resv_name == NULL) ||
 		    (xstrcmp(job->resv_name, params.reservation))) {
-			return 7;
+			return true;
 		}
 	}
 
 	if (params.name_list) {
-		filter = 1;
+		bool filter = true;
 		iterator = list_iterator_create(params.name_list);
 		while ((name = list_next(iterator))) {
 			if ((job->name != NULL) &&
 			     (xstrcasecmp(name, job->name) == 0)) {
-				filter = 0;
+				filter = false;
 				break;
 			}
 		}
 		list_iterator_destroy(iterator);
-		if (filter == 1)
-			return 8;
+		if (filter)
+			return true;
 	}
 
 	if (partial_array) {
 		/* Print this record, but perhaps only some job array records */
 		bitstr_t *new_array_bitmap;
-		int array_len = bit_size((bitstr_t *)job->array_bitmap);
+		int array_len = bit_size(job->array_bitmap);
 		new_array_bitmap = bit_alloc(array_len);
 		iterator = list_iterator_create(params.job_list);
 		while ((job_step_id = list_next(iterator))) {
-			if ((job_step_id->job_id == job->array_job_id) &&
+			if ((job_step_id->step_id.job_id ==
+			     job->array_job_id) &&
 			    (job_step_id->array_id < array_len)) {
 				bit_set(new_array_bitmap,job_step_id->array_id);
 			}
 		}
 		list_iterator_destroy(iterator);
-		bit_and((bitstr_t *)job->array_bitmap, new_array_bitmap);
-		bit_free(new_array_bitmap);
+		bit_and(job->array_bitmap, new_array_bitmap);
+		FREE_NULL_BITMAP(new_array_bitmap);
 		xfree(job->array_task_str);
-		i = bit_set_count((bitstr_t *)job->array_bitmap);
+		i = bit_set_count(job->array_bitmap);
 		if (i == 1) {
-			job->array_task_id =
-				bit_ffs((bitstr_t *)job->array_bitmap);
-			bit_free((bitstr_t *)job->array_bitmap);
+			job->array_task_id = bit_ffs(job->array_bitmap);
+			FREE_NULL_BITMAP(job->array_bitmap);
 		} else {
 			i = i * 16 + 10;
 			job->array_task_str = xmalloc(i);
 			(void) bit_fmt(job->array_task_str, i,
-				       (bitstr_t *)job->array_bitmap);
+				       job->array_bitmap);
 		}
 	}
 
-	return 0;
+	return false;
 }
 
 /* Return 0 if supplied partition name is to be printed, otherwise return 2 */
 static int _filter_job_part(char *part_name)
 {
 	char *token = NULL, *last = NULL, *tmp_name = NULL, *part;
-	ListIterator iterator;
+	list_itr_t *iterator;
 	int rc = 2;
 
 	if (!params.part_list)
@@ -3025,7 +3179,7 @@ static int _filter_job_part(char *part_name)
 static int _filter_step(job_step_info_t * step)
 {
 	int filter;
-	ListIterator iterator;
+	list_itr_t *iterator;
 	uint32_t *user;
 	char *part;
 	squeue_job_step_t *job_step_id;
@@ -3038,10 +3192,13 @@ static int _filter_step(job_step_info_t * step)
 		iterator = list_iterator_create(params.job_list);
 		while ((job_step_id = list_next(iterator))) {
 			if (((job_step_id->array_id == NO_VAL)   &&
-			     ((job_step_id->job_id  == step->array_job_id)  ||
-			      (job_step_id->job_id  == step->job_id)))      ||
+			     ((job_step_id->step_id.job_id ==
+			       step->array_job_id) ||
+			      (job_step_id->step_id.job_id ==
+			       step->step_id.job_id))) ||
 			    ((job_step_id->array_id == step->array_task_id) &&
-			     (job_step_id->job_id   == step->array_job_id))) {
+			     (job_step_id->step_id.job_id ==
+			      step->array_job_id))) {
 				filter = 0;
 				break;
 			}
@@ -3069,13 +3226,17 @@ static int _filter_step(job_step_info_t * step)
 		filter = 1;
 		iterator = list_iterator_create(params.step_list);
 		while ((job_step_id = list_next(iterator))) {
-			if (job_step_id->step_id != step->step_id)
+			if (job_step_id->step_id.step_id !=
+			    step->step_id.step_id)
 				continue;
-			if (((job_step_id->array_id == NO_VAL)  &&
-			     ((job_step_id->job_id  == step->array_job_id) ||
-			      (job_step_id->job_id  == step->job_id)))      ||
+			if (((job_step_id->array_id == NO_VAL) &&
+			     ((job_step_id->step_id.job_id ==
+			       step->array_job_id) ||
+			      (job_step_id->step_id.job_id ==
+			       step->step_id.job_id))) ||
 			    ((job_step_id->array_id == step->array_task_id) &&
-			     (job_step_id->job_id   == step->array_job_id))) {
+			     (job_step_id->step_id.job_id ==
+			      step->array_job_id))) {
 				filter = 0;
 				break;
 			}
