@@ -2,7 +2,7 @@
  *  job_reports.c - functions for generating job reports
  *                  from accounting infrastructure.
  *****************************************************************************
- *  Copyright (C) 2010-2017 SchedMD LLC.
+ *  Copyright (C) SchedMD LLC.
  *  Copyright (C) 2008 Lawrence Livermore National Security.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
  *  Written by Danny Auble <da@llnl.gov>
@@ -38,7 +38,6 @@
  *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA.
 \*****************************************************************************/
 
-#include "src/common/uid.h"
 #include "job_reports.h"
 
 enum {
@@ -147,100 +146,6 @@ static int _sort_acct_grouping_dec(void *v1, void *v2)
 	return 0;
 }
 
-
-static char *_string_to_uid( char *name )
-{
-	uid_t uid;
-	if ( uid_from_string( name, &uid ) != 0 ) {
-		fprintf(stderr, "Invalid user id: %s\n", name);
-		exit(1);
-	}
-	xfree(name);
-	return xstrdup_printf( "%d", (int) uid );
-}
-
-
-/* returns number of objects added to list */
-static int _addto_uid_char_list(List char_list, char *names)
-{
-	int i = 0, start = 0;
-	char *name = NULL, *tmp_char = NULL;
-	ListIterator itr = NULL;
-	char quote_c = '\0';
-	int quote = 0;
-	int count = 0;
-
-	if (!char_list) {
-		error("No list was given to fill in");
-		return 0;
-	}
-
-	itr = list_iterator_create(char_list);
-	if (names) {
-		if (names[i] == '\"' || names[i] == '\'') {
-			quote_c = names[i];
-			quote = 1;
-			i++;
-		}
-		start = i;
-		while (names[i]) {
-			//info("got %d - %d = %d", i, start, i-start);
-			if (quote && names[i] == quote_c)
-				break;
-			else if (names[i] == '\"' || names[i] == '\'')
-				names[i] = '`';
-			else if (names[i] == ',') {
-				if ((i-start) > 0) {
-					name = xmalloc((i-start+1));
-					memcpy(name, names+start, (i-start));
-					//info("got %s %d", name, i-start);
-					name = _string_to_uid( name );
-
-					while ((tmp_char = list_next(itr))) {
-						if (!xstrcasecmp(tmp_char,
-								 name))
-							break;
-					}
-
-					if (!tmp_char) {
-						list_append(char_list, name);
-						count++;
-					} else
-						xfree(name);
-					list_iterator_reset(itr);
-				}
-				i++;
-				start = i;
-				if (!names[i]) {
-					info("There is a problem with "
-					     "your request.  It appears you "
-					     "have spaces inside your list.");
-					break;
-				}
-			}
-			i++;
-		}
-		if ((i-start) > 0) {
-			name = xmalloc((i-start)+1);
-			memcpy(name, names+start, (i-start));
-			name = _string_to_uid( name );
-
-			while ((tmp_char = list_next(itr))) {
-				if (!xstrcasecmp(tmp_char, name))
-					break;
-			}
-
-			if (!tmp_char) {
-				list_append(char_list, name);
-				count++;
-			} else
-				xfree(name);
-		}
-	}
-	list_iterator_destroy(itr);
-	return count;
-}
-
 static int _set_cond(int *start, int argc, char **argv,
 		     slurmdb_job_cond_t *job_cond,
 		     List format_list, List grouping_list)
@@ -326,7 +231,7 @@ static int _set_cond(int *start, int argc, char **argv,
 		} else if (!xstrncasecmp(argv[i], "Jobs",
 					 MAX(command_len, 1))) {
 			char *end_char = NULL, *start_char = argv[i]+end;
-			slurmdb_selected_step_t *selected_step = NULL;
+			slurm_selected_step_t *selected_step = NULL;
 			char *dot = NULL;
 			if (!job_cond->step_list)
 				job_cond->step_list = list_create(xfree_ptr);
@@ -338,18 +243,21 @@ static int _set_cond(int *start, int argc, char **argv,
 				if (start_char[0] == '\0')
 					continue;
 				selected_step = xmalloc(
-					sizeof(slurmdb_selected_step_t));
+					sizeof(slurm_selected_step_t));
 				list_append(job_cond->step_list, selected_step);
 
 				dot = strstr(start_char, ".");
 				if (dot == NULL) {
 					debug2("No jobstep requested");
-					selected_step->stepid = NO_VAL;
+					selected_step->step_id.step_id = NO_VAL;
 				} else {
 					*dot++ = 0;
-					selected_step->stepid = atoi(dot);
+					selected_step->step_id.step_id =
+						atoi(dot);
 				}
-				selected_step->jobid = atoi(start_char);
+				selected_step->step_id.job_id =
+					atoi(start_char);
+				selected_step->step_id.step_het_comp= NO_VAL;
 				selected_step->array_task_id = NO_VAL;
 				selected_step->het_job_offset = NO_VAL;
 				start_char = end_char + 1;
@@ -383,9 +291,11 @@ static int _set_cond(int *start, int argc, char **argv,
 					 MAX(command_len, 1))) {
 			if (!job_cond->userid_list)
 				job_cond->userid_list = list_create(xfree_ptr);
-			_addto_uid_char_list(job_cond->userid_list,
-					     argv[i]+end);
-			set = 1;
+			if (slurm_addto_id_char_list(job_cond->userid_list,
+			                             argv[i]+end, false))
+				set = 1;
+			else
+				exit_code = 1;
 		} else if (!xstrncasecmp(argv[i], "Wckeys",
 					 MAX(command_len, 2))) {
 			if (!job_cond->wckey_list)
@@ -403,9 +313,8 @@ static int _set_cond(int *start, int argc, char **argv,
 
 	if (!local_cluster_flag && !list_count(job_cond->cluster_list)) {
 		/* Get the default Cluster since no cluster is specified */
-		char *temp = slurm_get_cluster_name();
-		if (temp)
-			list_append(job_cond->cluster_list, temp);
+		list_append(job_cond->cluster_list,
+			    xstrdup(slurm_conf.cluster_name));
 	}
 
 	/* This needs to be done on some systems to make sure
@@ -424,7 +333,7 @@ static int _set_cond(int *start, int argc, char **argv,
 
 static int _setup_print_fields_list(List format_list)
 {
-	ListIterator itr = NULL;
+	list_itr_t *itr = NULL;
 	print_field_t *field = NULL;
 	char *object = NULL;
 
@@ -523,7 +432,7 @@ static int _setup_print_fields_list(List format_list)
 
 static int _setup_grouping_print_fields_list(List grouping_list)
 {
-	ListIterator itr = NULL;
+	list_itr_t *itr = NULL;
 	print_field_t *field = NULL;
 	char *object = NULL;
 	char *last_object = NULL;
@@ -569,7 +478,7 @@ static int _setup_grouping_print_fields_list(List grouping_list)
 		if (print_job_count)
 			field->print_routine = print_fields_uint;
 		else
-			field->print_routine = slurmdb_report_print_time;
+			field->print_routine = print_fields_str;
 		last_size = size;
 		last_object = object;
 		if ((tmp_char = strstr(object, "\%"))) {
@@ -598,7 +507,7 @@ static int _setup_grouping_print_fields_list(List grouping_list)
 		if (print_job_count)
 			field->print_routine = print_fields_uint;
 		else
-			field->print_routine = slurmdb_report_print_time;
+			field->print_routine = print_fields_str;
 		if ((tmp_char = strstr(last_object, "\%"))) {
 			int newlen = atoi(tmp_char+1);
 			if (newlen)
@@ -641,7 +550,7 @@ static void _combine_job_groups(List first_job_list, List new_job_list)
 {
 	slurmdb_report_job_grouping_t *orig_job_group = NULL;
 	slurmdb_report_job_grouping_t *dup_job_group = NULL;
-	ListIterator iter = NULL;
+	list_itr_t *iter = NULL;
 
 	if (!first_job_list || !new_job_list)
 		return;
@@ -694,7 +603,7 @@ static void _combine_acct_groups(List first_acct_list, List new_acct_list)
 {
 	slurmdb_report_acct_grouping_t *orig_report_acct = NULL;
 	slurmdb_report_acct_grouping_t *dup_report_acct = NULL;
-	ListIterator iter = NULL;
+	list_itr_t *iter = NULL;
 
 	if (!first_acct_list || !new_acct_list)
 		return;
@@ -722,7 +631,7 @@ static void _combine_acct_groups(List first_acct_list, List new_acct_list)
 static void _merge_cluster_groups(List slurmdb_report_cluster_grouping_list)
 {
 	slurmdb_report_cluster_grouping_t *group = NULL, *first_group = NULL;
-	ListIterator iter = NULL;
+	list_itr_t *iter = NULL;
 
 	if (list_count(slurmdb_report_cluster_grouping_list) < 2)
 		return;
@@ -760,11 +669,11 @@ static int _run_report(int type, int argc, char **argv)
 	int i = 0, tres_cnt = 0;
 	slurmdb_tres_rec_t *tres;
 	uint64_t count1, count2;
-	ListIterator itr = NULL;
-	ListIterator itr2 = NULL;
-	ListIterator cluster_itr = NULL;
-	ListIterator local_itr = NULL;
-	ListIterator acct_itr = NULL;
+	list_itr_t *itr = NULL;
+	list_itr_t *itr2 = NULL;
+	list_itr_t *cluster_itr = NULL;
+	list_itr_t *local_itr = NULL;
+	list_itr_t *acct_itr = NULL;
 	slurmdb_report_cluster_grouping_t *cluster_group = NULL;
 	slurmdb_report_acct_grouping_t *acct_group = NULL;
 	slurmdb_report_job_grouping_t *job_group = NULL;
@@ -776,7 +685,7 @@ static int _run_report(int type, int argc, char **argv)
 	List format_list = list_create(xfree_ptr);
 	List grouping_list = list_create(xfree_ptr);
 	List header_list = NULL;
-	char *object_str = "";
+	char *object_str = "", *tmp_char;
 
 	/* init memory before chance of going to end_it before being init'ed. */
 	memset(&total_field, 0, sizeof(print_field_t));
@@ -852,8 +761,8 @@ static int _run_report(int type, int argc, char **argv)
 		goto end_it;
 
 	if (print_fields_have_header) {
-		char start_char[20];
-		char end_char[20];
+		char start_char[256];
+		char end_char[256];
 		time_t my_start = job_cond->usage_start;
 		time_t my_end = job_cond->usage_end - 1;
 
@@ -881,7 +790,7 @@ static int _run_report(int type, int argc, char **argv)
 	total_field.type = PRINT_JOB_SIZE;
 	total_field.name = xstrdup("% of cluster");
 	total_field.len = 12;
-	total_field.print_routine = slurmdb_report_print_time;
+	total_field.print_routine = print_fields_str;
 	list_append(header_list, &total_field);
 
 	print_fields_header(header_list);
@@ -955,16 +864,19 @@ static int _run_report(int type, int argc, char **argv)
 				field = list_next(itr2);
 				switch (field->type) {
 				case PRINT_JOB_SIZE:
+					tmp_char = sreport_get_time_str(
+							job_cpu_alloc_secs,
+							acct_tres_alloc_secs);
 					field->print_routine(
 						field,
-						job_cpu_alloc_secs,
-						acct_tres_alloc_secs,
+						tmp_char,
 						0);
+					xfree(tmp_char);
 					break;
 				case PRINT_JOB_COUNT:
 					field->print_routine(
 						field,
-						job_group->count,
+						&job_group->count,
 						0);
 					break;
 				default:
@@ -986,8 +898,9 @@ static int _run_report(int type, int argc, char **argv)
 				count1 = acct_group->count;
 				count2 = cluster_group->count;
 			}
-			total_field.print_routine(&total_field,
-						  count1, count2, 1);
+			tmp_char = sreport_get_time_str(count1, count2);
+			total_field.print_routine(&total_field, tmp_char, 1);
+			xfree(tmp_char);
 			time_format = temp_format;
 			printf("\n");
 		}
