@@ -199,6 +199,10 @@ job_create_noalloc(void)
 
 	if (job != NULL)
 		job_update_io_fnames(job, opt_local);
+	if (job && (job->ntasks == NO_VAL)) {
+		job->ntasks = ai->nnodes;
+		job->cpu_count = opt_local->cpus_per_task * job->ntasks;
+	}
 
 error:
 	xfree(ai);
@@ -479,6 +483,7 @@ extern srun_job_t *job_step_create_allocation(
 	 */
 	job = _job_create_structure(ai, opt_local);
 error:
+	xfree(ai->nodelist);
    	xfree(ai);
 	return (job);
 
@@ -528,7 +533,7 @@ extern srun_job_t *job_create_allocation(
 
 static void _copy_args(List missing_argc_list, slurm_opt_t *opt_master)
 {
-	ListIterator iter;
+	list_itr_t *iter;
 	slurm_opt_t *opt_local;
 	int i;
 
@@ -551,7 +556,7 @@ static void _copy_args(List missing_argc_list, slurm_opt_t *opt_master)
  */
 static void _het_grp_test(List opt_list)
 {
-	ListIterator iter;
+	list_itr_t *iter;
 	int het_job_offset;
 	bitstr_t *master_map = NULL;
 	List missing_argv_list = NULL;
@@ -619,7 +624,7 @@ static void _het_grp_test(List opt_list)
 static void _match_job_name(List opt_list)
 {
 	int cnt;
-	ListIterator iter;
+	list_itr_t *iter;
 	slurm_opt_t *opt_local;
 
 	if (!opt_list)
@@ -846,12 +851,13 @@ static int _create_job_step(srun_job_t *job, bool use_all_cpus,
 			    List srun_job_list, uint32_t het_job_id,
 			    char *het_job_nodelist)
 {
-	ListIterator opt_iter = NULL, job_iter;
+	list_itr_t *opt_iter = NULL, *job_iter;
 	slurm_opt_t *opt_local = &opt;
 	uint32_t node_offset = 0, het_job_nnodes = 0, step_id = NO_VAL;
 	uint32_t het_job_ntasks = 0, task_offset = 0;
 	bool update_het_nnodes = false;
 	uint32_t updated_het_nnodes;
+	uint32_t updated_het_ntasks = 0;
 
 	job_step_create_response_msg_t *step_resp;
 	char *resv_ports = NULL;
@@ -882,7 +888,10 @@ static int _create_job_step(srun_job_t *job, bool use_all_cpus,
 				job->step_id.step_het_comp = NO_VAL;
 
 			het_job_nnodes += job->nhosts;
-			het_job_ntasks += job->ntasks;
+			if (job->ntasks == NO_VAL)
+				het_job_ntasks = NO_VAL;
+			else if (het_job_ntasks != NO_VAL)
+				het_job_ntasks += job->ntasks;
 		}
 
 		updated_het_nnodes = het_job_nnodes;
@@ -936,6 +945,9 @@ static int _create_job_step(srun_job_t *job, bool use_all_cpus,
 				update_het_nnodes = true;
 				updated_het_nnodes -= old_nhosts - job->nhosts;
 			}
+
+			if (het_job_ntasks == NO_VAL)
+				updated_het_ntasks += job->ntasks;
 		}
 
 		if (update_het_nnodes) {
@@ -944,6 +956,12 @@ static int _create_job_step(srun_job_t *job, bool use_all_cpus,
 				job->het_job_nnodes = updated_het_nnodes;
 			}
 		}
+		if (updated_het_ntasks) {
+			list_iterator_reset(job_iter);
+			while ((job = list_next(job_iter))) {
+				job->het_job_ntasks = updated_het_ntasks;
+			}
+		}
 
 		FREE_NULL_HOSTLIST(exclude_hl);
 
@@ -995,7 +1013,7 @@ static int _create_job_step(srun_job_t *job, bool use_all_cpus,
 static void _cancel_steps(List srun_job_list)
 {
 	srun_job_t *job;
-	ListIterator job_iter;
+	list_itr_t *job_iter;
 	slurm_msg_t req;
 	step_complete_msg_t msg;
 	int rc = 0;
@@ -1043,7 +1061,7 @@ static char *_compress_het_job_nodelist(List used_resp_list)
 	resource_allocation_response_msg_t *resp;
 	het_job_resp_struct_t *het_job_resp;
 	List het_job_resp_list;
-	ListIterator resp_iter;
+	list_itr_t *resp_iter;
 	char *aliases = NULL, *save_ptr = NULL, *tok, *tmp;
 	char *het_job_nodelist = NULL, *node_name;
 	hostset_t *hs;
@@ -1200,7 +1218,7 @@ extern void create_srun_job(void **p_job, bool *got_alloc)
 	resource_allocation_response_msg_t *resp;
 	List job_resp_list = NULL, srun_job_list = NULL;
 	List used_resp_list = NULL;
-	ListIterator opt_iter, resp_iter;
+	list_itr_t *opt_iter, *resp_iter;
 	srun_job_t *job = NULL;
 	int i, max_list_offset, max_het_job_offset, het_job_offset = -1,
 		het_step_offset = -1;
@@ -1208,9 +1226,6 @@ extern void create_srun_job(void **p_job, bool *got_alloc)
 	char *het_job_nodelist = NULL;
 	bool begin_error_logged = false;
 	bool core_spec_error_logged = false;
-#ifdef HAVE_NATIVE_CRAY
-	bool network_error_logged = false;
-#endif
 	bool node_cnt_error_logged = false;
 	bool tres_license_error_logged = false;
 	bool x11_error_logged = false;
@@ -1358,31 +1373,15 @@ extern void create_srun_job(void **p_job, bool *got_alloc)
 					      "at job allocation time.");
 					core_spec_error_logged = true;
 				}
-#ifdef HAVE_NATIVE_CRAY
-				if (opt_local->network &&
-				    !network_error_logged) {
-					if (slurm_option_set_by_env(opt_local,
-								    LONG_OPT_NETWORK)) {
-						debug2("Ignoring SLURM_NETWORK value for a "
-						       "job step within an existing job. "
-						       "Using what was set at job "
-						       "allocation time.  Most likely this "
-						       "variable was set by sbatch or salloc.");
-					} else {
-						error("Ignoring --network value for a job step "
-						      "within an existing job. Set network "
-						      "options at job allocation time.");
-					}
-					network_error_logged = true;
-				}
-				xfree(opt_local->network);
+
 				/*
 				 * Here we send the het job groups to the
 				 * slurmctld to set up the interconnect
 				 * correctly.  We only ever need to send it to
 				 * the first component of the step.
+				 *
+				 * FIXME - is this still needed post-Cray?
 				 */
-#endif
 				if (g_het_grp_bits) {
 					xfree(opt_local->step_het_grps);
 					opt_local->step_het_grps =
@@ -1585,7 +1584,7 @@ extern void fini_srun(srun_job_t *job, bool got_alloc, uint32_t *global_rc)
 	if (signal_thread) {
 		srun_shutdown = true;
 		pthread_kill(signal_thread, SIGINT);
-		pthread_join(signal_thread,  NULL);
+		slurm_thread_join(signal_thread);
 	}
 
 	_run_srun_epilog(job);
@@ -1654,20 +1653,19 @@ static void _set_ntasks(allocation_info_t *ai, slurm_opt_t *opt_local)
 {
 	int cnt = 0;
 
-	if (opt_local->ntasks_set && (opt_local->ntasks_per_node == NO_VAL))
+	/* Distinction between explicit or implicit set of ntasks */
+	if (opt_local->ntasks_opt_set ||
+	    (opt_local->ntasks_set &&
+	     (opt_local->ntasks_per_node == NO_VAL)))
 		return;
 
 	if (opt_local->ntasks_per_node != NO_VAL) {
 		cnt = ai->nnodes * opt_local->ntasks_per_node;
 		opt_local->ntasks_set = true;	/* implicit */
 	} else if (opt_local->cpus_set) {
-		int i;
-
-		for (i = 0; i < ai->num_cpu_groups; i++)
-			cnt += (ai->cpu_count_reps[i] *
-				(ai->cpus_per_node[i] /
-				 opt_local->cpus_per_task));
+		opt_local->ntasks = NO_VAL;
 		opt_local->ntasks_set = true;	/* implicit */
+		return;
 	}
 
 	opt_local->ntasks = (cnt < ai->nnodes) ? ai->nnodes : cnt;
@@ -1733,9 +1731,13 @@ static srun_job_t *_job_create_structure(allocation_info_t *ainfo,
 	 * requested step (we might very well use less, especially if
 	 * --exclusive is used).  Else get the total for the allocation given.
 	 */
-	if (opt_local->cpus_set)
-		job->cpu_count = opt_local->ntasks * opt_local->cpus_per_task;
-	else {
+	if (opt_local->cpus_set) {
+		if (opt_local->ntasks == NO_VAL)
+			job->cpu_count = NO_VAL;
+		else
+			job->cpu_count = opt_local->ntasks *
+				opt_local->cpus_per_task;
+	} else {
 		for (i = 0; i < ainfo->num_cpu_groups; i++) {
 			job->cpu_count += ainfo->cpus_per_node[i] *
 				ainfo->cpu_count_reps[i];
@@ -2234,7 +2236,7 @@ static int _shepherd_spawn(srun_job_t *job, List srun_job_list, bool got_alloc)
 	}
 
 	if (srun_job_list) {
-		ListIterator job_iter;
+		list_itr_t *job_iter;
 		job_iter  = list_iterator_create(srun_job_list);
 		while ((job = list_next(job_iter))) {
 			(void) slurm_kill_job_step(job->step_id.job_id, job->step_id.step_id,
